<?php /*

 Composr
 Copyright (c) ocProducts, 2004-2016

 See text/EN/licence.txt for full licencing information.

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  ocProducts Ltd
 * @package    meta_toolkit
 */

function string_scan($lang, $do_guessing = false, $only_exists = true)
{
    require_code('lorem');
    require_code('lang_compile');

    $source_files_admin = array();
    $source_files_non_admin = array();

    if ($do_guessing) {
        $hooks = find_all_hooks('systems', 'addon_registry');
        foreach ($hooks as $hook => $place) {
            if ($place == 'sources_custom') {
                continue;
            }

            require_code('hooks/systems/addon_registry/' . filter_naughty_harsh($hook));

            $ob = object_factory('Hook_addon_registry_' . filter_naughty_harsh($hook));
            if (method_exists($ob, 'tpl_previews')) {
                $previews = $ob->tpl_previews();
                foreach ($previews as $template => $preview) {
                    $full = 'themes/default/' . $template;
                    if (file_exists(get_file_base() . '/' . $full)) {
                        if (preg_match('#^administrative__#', $preview) != 0) {
                            $source_files_admin[] = $full;
                        } else {
                            $source_files_non_admin[] = $full;
                        }
                    }
                }
            }
        }
        $source_files_admin = array_merge($source_files_admin, find_files_in_dir('adminzone'));
        $source_files_admin = array_merge($source_files_admin, find_files_in_dir('cms'));
        $source_files_admin = array_merge($source_files_admin, find_files_in_dir('collaboration'));
        $source_files_non_admin = array_merge($source_files_non_admin, find_files_in_dir('site'));
        $source_files_non_admin = array_merge($source_files_non_admin, find_files_in_dir('pages'));
        $source_files_non_admin = array_merge($source_files_non_admin, find_files_in_dir('forum'));
    }

    $admin_files = array(
        'abstract_file_manager.ini',
        'actionlog.ini',
        'addons.ini',
        'aggregate_types.ini',
        'backups.ini',
        'cleanup.ini',
        'cns_config.ini',
        'cns_multi_moderations.ini',
        'commandr.ini',
        'custom_comcode.ini',
        'debrand.ini',
        'email_log.ini',
        'errorlog.ini',
        'filedump.ini',
        'installer.ini',
        'lookup.ini',
        'profiling.ini',
        'realtime_rain.ini',
        'ssl.ini',
        'staff_checklist.ini',
        'stats.ini',
        'submitban.ini',
        'themes.ini',
        'tips.ini',
        'upgrade.ini',
        'version.ini',
        'webstandards.ini',
        'import.ini',
    );

    $admin_string_suffixes = array(
        'MODULE_TRANS_NAME_admin_',
        'MODULE_TRANS_NAME_cms_',
        'ABSTRACTION_SYMBOL__',
        'ARITHMETICAL_SYMBOL__',
        'PROGRAMMATIC_SYMBOL__',
        'DIRECTIVE__',
        'FORMATTING_SYMBOL__',
        'LOGICAL_SYMBOL__',
        'SYMBOL__',
        'PRIVILEGE_',
        'FU_',
        'FIELDTYPE_',
        'CHECKLIST_',
        'CONFIG_OPTION_',
        'CONFIG_CATEGORY_',
        'CONFIG_GROUP_',
        'DOC_',
        'TIP_',
        'WCAG_',
        'XHTML_',
        'CSS_',
        'COMCODE_TAG_',
        'CMD_',
        'BLOCK_',
        'FIELD_TYPE_',
    );

    $lang_strings_admin_initial = array_map('trim', explode("\n", trim('
        CMD_OBFUSCATE_DIRECTORY_HELP
        CMD_OBFUSCATE_DIRECTORY_HELP_PARAM_0
        CMD_OBFUSCATE_DIRECTORY_HELP_PARAM_1
        IMAGEMAGICK_PATH
        CONFIG_OPTION_imagemagick_path
        ADD_MEMBER_EMAIL_HELP
        CONFIG_OPTION_new_member_default_email_message
        CONFIG_OPTION_new_member_default_email_subject
        CONFIG_OPTION_new_member_default_temporary_password
        NEW_MEMBER_DEFAULT_EMAIL_MESSAGE
        NEW_MEMBER_DEFAULT_EMAIL_SUBJECT
        NEW_MEMBER_DEFAULT_TEMPORARY_PASSWORD
        IF_UNCHANGED_CURRENT_FILE_REMAINS
        MONIKER_TRANSLITERATION
        CONFIG_OPTION_moniker_transliteration
        BLOCKS_AT_TOP
        BLOCK_TOP_LOGIN
        BLOCK_TOP_NOTIFICATIONS
        BLOCK_TOP_PERSONAL_STATS
        BLOCK_TOP_SEARCH
        CONFIG_GROUP_DESCRIP_BLOCKS_AT_TOP
        CONFIG_OPTION_block_top_login
        CONFIG_OPTION_block_top_notifications
        CONFIG_OPTION_block_top_personal_stats
        CONFIG_OPTION_block_top_search
        NEWS_SUMMARY_REQUIRED
        BACKUP_RESTORE_SUCCESS
        CONTINUE_RESTORATION
        CONFIG_OPTION_news_summary_required
        OWNER
        DEEPER_CONFIGURATION
        LOST_DISK_CONTENT
        DESCRIPTION_LOST_DISK_CONTENT
        TICKETS_STAFF
        NO_CNS
        DOCS
        EXPLANATION_WITH_DEFAULT
        PROJECT_SPONSORS
        PROJECT_SPONSORS_LABEL
        CONFIG_OPTION_complex_lists
        COMPLEX_LISTS
        HERE_SURVEY_RESULTS
        HERE_TEST_RESULTS
        METADATA
        ABOUT_DELETE_MENU
        ABOUT_REVISIONS
        ABRUPTED_DIRECTIVE_OR_BRACE
        INVOICE_REQUIRED_AMOUNT
        ABSORB_PERMISSIONS
        WHAT_NEXT
        WHERE_NEXT
        ABSTRACTION_SYMBOL__BLOCK
        ABSTRACTION_SYMBOL__IMG
        ABSTRACTION_SYMBOL__LOAD_PAGE
        ABSTRACTION_SYMBOL__LOAD_PANEL
        ABSTRACTION_SYMBOL__MAILTO
        ABSTRACTION_SYMBOL__PAGE_LINK
        ABSTRACTION_SYMBOL__THUMBNAIL
        ABSTRACT_FILE_MANAGEMENT
        ACCEPTED_MEDIA_TYPES
        INVALID_BASE_URL
        WORDFILTER
        AUDIT
        DESCRIPTION_REORGANISE_UPLOADS
        REORGANISE_UPLOADS
        KA_CHING
        PLEASE_STATE
        VIDEO_WAS_AUTO_IMPORTED
        CONTACT_US_NOTIFICATION_MESSAGE
        CONTACT_US_NOTIFICATION_SUBJECT
        WORD_SUBSTR
        SCREEN_DEV_TOOLS
        OR_ONE_OF_THE_BELOW
        SALE_MADE
        MERGE
        MAIL_TEST_FAIL
        MAIL_TEST_PASS
        MAIL_TEST_UNKNOWN
        EDIT_LIVE_POLL
        DESCRIPTION_MAIL_DOMAIN
        RSS
        PGE_M
        PGE_B
        PGE_D
        PGE_R
        PGE_HASH
        DKIM_PRIVATE_KEY
        CONFIG_OPTION_dkim_private_key
        DKIM_SELECTOR
        CONFIG_OPTION_dkim_selector
        _EMAIL
        PHP_UPLOAD_SETTING_VERY_LOW
        ADDING_THEME_IMAGE
        DESCRIPTION_ADDING_THEME_IMAGE
        NO_SSL_SETUP
        ADD_NOTE_TITLE
        CONFIG_OPTION_points_RECOMMEND_SITE
        SUBSCRIPTION_CANCELLED_BODY
        SUBSCRIPTION_CANCELLED_SUBJECT
        NO_ISSUES_FOUND
        ADBLOCKER
        VIEW_THUMBNAILS
        USER_ONLINE_TRACKING
        ECOMMERCE_PRODUCT_MISSING
        POST_FILES
        GROUP_MEMBER_TIMEOUT_DATE
        ENTER_PAGE
        ENTER_ZONE
        ENTER_ZONE_SPZ
        WHETHER_CONVERT_COMCODE
        MERGE_POSTS
        VIEW_ALERT
        ANTISPAM_RESPONSE_SKIP
        CORRUPT_ATTACHMENT
        ATTACHMENT_REFERENCES
        FILENAME_TOO_LONG
        MERGE_SUPPORT_TICKETS
        NO_PARAMETER_SENT
        CONFIRM_MERGE_TICKETS
        DEFAULT_PERMISSION
        ACCESS_DENIED__SUBMIT_HERE
        TICKET_EMAILED_IN
        TICKET_ASSIGN
        TICKET_ASSIGNED_BODY
        TICKET_ASSIGNED_SUBJECT
        ACCESS_DENIED__ATTACHMENT_ACCESS
        ACCESS_DENIED__ZONE_ACCESS_SESSION
        ACCEPT_IMAGES
        ACCEPT_VIDEOS
        ACCESSIB_COLONS_IN_PRE_LABELS
        ACCESS_BANNER_INFO
        ACCESS_DENIED_SCREEN
        ACCESS_DENIED__ADD_COMCODE_PAGE
        ACCESS_DENIED__ADD_OR_EDIT_COMCODE_PAGES
        ACCESS_DENIED__ADMIN_ONLY
        ACCESS_DENIED__ASSOCIATE_EXISTING_FILE
        ACCESS_DENIED__EDIT_COMCODE_PAGE
        ACCESS_DENIED__I_ERROR
        ACCESS_DENIED__PRIVILEGE
        ACCESS_DENIED__STAFF_ONLY
        TICKET_ACTIVITY_OVERVIEW
        PARTNER_SITES
        REFERRER
        SUPPORT_TICKETS_MAIL
        _COMCODE_PAGES
        ERROR_UPLOADING
        ERROR_UPLOADING_1
        ERROR_UPLOADING_2
        ERROR_UPLOADING_3
        ERROR_UPLOADING_4
        ERROR_UPLOADING_6
        ERROR_UPLOADING_7
        VOTE_CHEAT
        ACCESS_FOR
        MISSING_CONFIRM_CODE
        INVALID_DATE_GIVEN
        ACTIONLOG_NOTIFICATION_MAIL
        ACTIONLOG_NOTIFICATION_MAIL_SUBJECT
        ACTIONLOG_USERCOUNT
        ACTIONLOG_USERCOUNT_UNI
        ACTIVITY_ADD_CLUB
        ACTIVITY_ADD_DOWNLOAD
        ACTIVITY_ADD_IMAGE
        ACTIVITY_ADD_NEWS
        ACTIVITY_ADD_NEWS_BLOG
        ACTIVITY_ADD_POLL
        ACTIVITY_ADD_QUIZ
        ACTIVITY_ADD_VIDEO
        ACTIVITY_CALENDAR_EVENT
        ACTIVITY_CATALOGUE_GENERIC_ADD
        ACTIVITY_CHOOSE_POLL
        ACTIVITY_FILEDUMP_UPLOAD
        ACTIVITY_GIVE_AWARD
        ACTIVITY_VALIDATE_CALENDAR_EVENT
        ACTIVITY_VALIDATE_CATALOGUE_GENERIC
        ACTIVITY_VALIDATE_DOWNLOAD
        ACTIVITY_VALIDATE_IMAGE
        ACTIVITY_VALIDATE_NEWS
        ACTIVITY_VALIDATE_NEWS_BLOG
        ACTIVITY_VALIDATE_QUIZ
        ACTIVITY_VALIDATE_VIDEO
        ADDING_CONTENT_IN_LANGUAGE
        ADDING_CONTENT_IN_LANGUAGE_STAFF
        ADDITIONAL_ACCESS
        TICKET_ACTIVITY_BODY
        GUEST_TICKET_REPLY_LINK
        BANNER_DESCRIPTION
        BANNER_SHARING_END
        BANNER_SHARING_START
        BANNER_MISSING_SOURCE
        BANNERS_A
        BANNERS_ACTIVATE
        BANNERS_B
        BANNERS_C
        BANNERS_D
        BANNER_ADS
        ADDON
        ADDONS
        ADDONS_NOT_INSTALLED_IN_SETUPWIZARD
        ADDONS_SCREEN
        ADDON_CREATED
        ADDON_FILES
        ADDON_FILE_IS_COMCODE_PAGE
        ADDON_FILE_IS_PHP
        ADDON_FILE_NORMAL
        ADDON_FILE_WILL_OVERWRITE
        ADDON_FILE_WILL_OVERWRITE_BACKUP
        ADDON_IMPORTED
        ADDON_NOT_TAR
        ADDON_WARNING_GENERAL
        ADDON_WARNING_INCOMPATIBILITIES
        ADDON_WARNING_MISSING_DEPENDENCIES
        ADDON_WARNING_OVERWRITE
        ADDON_WARNING_PHP
        ADDON_WARNING_PRESENT_DEPENDENCIES
        _ERROR_CHECKING_FOR_SPAMMERS
        ERROR_CHECKING_FOR_SPAMMERS
        ADD_AGGREGATE_TYPE_INSTANCE
        ADD_AWARD_TYPE
        ADD_BANNER
        ADD_BANNER_TEMPLATING
        ADD_BANNER_TYPE
        ADD_BRANCH
        ADD_CATALOGUE
        ADD_CATALOGUE_CATEGORY
        ADD_CATALOGUE_FOR_CONTENT_TYPE
        ADD_CHATROOM
        ADD_CHILD_PAGE
        ADD_CUSTOM_COMCODE_TAG
        ADD_CUSTOM_PROFILE_FIELD
        ADD_DOWNLOAD_CATEGORY
        ADD_DOWNLOAD_LICENCE
        ADD_EMOTICON
        ADD_EMPTY_THEME
        ADD_EVENT_TYPE
        ADD_FORUM
        ADD_FORUM_GROUPING
        ADD_GALLERY_SELECTION
        ADD_GROUP
        ADD_MEMBER
        ADD_MEMBER_TO_GROUP
        ADD_MENU
        ADD_MENU_ITEM
        ADD_MULTI_MODERATION
        ADD_NEWSLETTER
        ADD_NEWS_CATEGORY
        ADD_NEW_CUSTOM_PRODUCT
        ADD_NEW_FORWARDING_DOMAIN
        ADD_NEW_PERMISSION_PRODUCT
        ADD_NEW_POP3_DOMAIN
        ADD_NOTE
        ADD_NOTE_APPEND_TEXT
        ADD_ONE_AT_A_TIME
        ADD_PERIODIC_NEWSLETTER
        ADD_POST_TEMPLATE
        ADD_STAFF
        ADD_THEME
        ADD_THEME_IMAGE
        ADD_TICKET_AS
        ADD_TICKET_TYPE
        ADD_TO_OWN_ACCOUNT
        ADD_USERGROUP_SUBSCRIPTION
        ADD_USER_GROUP_FIRST
        ADD_WARNING
        ADD_WELCOME_EMAIL
        ADD_WORDFILTER
        ADD_ZONE
        POST_ACCESS_IS_RESTRICTED
        ADMIN
        ADMINGUIDE
        ADMINGUIDE_DEFAULT_TRAINING
        ADMINSTART_PAGE
        ADMINZONE_SEARCH_RESULTS
        ADMINZONE_SEARCH_TIP
        ENABLE_NOTIFICATIONS_RESOURCE_LONG_EXP
        ADMIN_BANNERS
        ADMIN_MODULES
        ADMIN_SCREENS
        ADMIN_USERNAME
        ADMIN_USERS_PASSWORD
        ADMIN_ZONE_LINK
        ADMIN_ZONE_SEARCH_RESULTS
        ADVANCED_BELOW
        ADVANCED_DATABASE_SETUP
        ADVERTISE_HERE
        ADVERTISE_ON_COMPOSR
        AFA_NOTIFICATION_MAIL
        AFA_NOTIFICATION_MAIL_SUBJECT
        AGGREGATE_TYPE
        AGGREGATE_TYPES
        AGGREGATE_TYPE_INSTANCE
        AGGREGATE_TYPE_INSTANCES
        ALLOWED_FILES
        ALLOWED_POST_SUBMITTERS
        ALLOWS_ANONYMOUS_POSTS
        ALLOW_ALPHA_SEARCH
        ALLOW_AUDIO_VIDEOS
        ALLOW_AUTO_NOTIFICATIONS
        ALLOW_COMMENTS
        ALLOW_COMMENTS_ONLY
        ALLOW_EMAIL_DISABLE
        ALLOW_EMAIL_FROM_STAFF_DISABLE
        ALLOW_EXT_IMAGES
        ALLOW_INTERNATIONAL
        ALLOW_OPTING_OUT_OF_TAX
        ALLOW_OWN_RATE
        ALLOW_RATING
        ALLOW_REVIEWS
        ALLOW_THEME_IMAGE_SELECTOR
        ALLOW_TRACKBACKS
        ALL_TIPS_READ
        ALREADY_EXISTS
        ALSO_ADMIN_ZONE_SEARCH
        ALSO_AUDIO
        ALSO_SEE_ADMIN
        ALSO_SEE_AUDIT
        ALSO_SEE_CMS
        ALSO_SEE_SETUP
        AMERICAN_ENGLISH
        ANOMALOUS_FORUM_ACCESS
        ANOTHER_TIP
        ANSWER_X
        ANTISPAM_CONFIDENCE
        ANTISPAM_CONFIDENCE_NA
        ANTISPAM_RESPONSE_ACTIVE
        ANTISPAM_RESPONSE_ACTIVE_UNKNOWN_STALE
        ANTISPAM_RESPONSE_ERROR
        ANTISPAM_RESPONSE_STALE
        ANTISPAM_RESPONSE_UNLISTED
        NOTIFICATION_TYPE_content_reviews
        NOTIFICATION_TYPE_content_reviews__own
        ANTI_LEECH
        API_ID
        ARCHIVE_PAGE_BACKUPS
        ARE_YOU_SURE_REVERSE
        ARITHMETICAL_SYMBOL__ADD
        ARITHMETICAL_SYMBOL__DIV
        ARITHMETICAL_SYMBOL__MAX
        ARITHMETICAL_SYMBOL__MIN
        ARITHMETICAL_SYMBOL__MULT
        ARITHMETICAL_SYMBOL__REM
        ARITHMETICAL_SYMBOL__ROUND
        ARITHMETICAL_SYMBOL__SUBTRACT
        SPECIFYING_NEW_PAGE
        ASCII_ENTITY_URL_HACK
        ASSIGN_RANDOM_AVATARS
        ATTACHMENT_CLEANUP
        AUDIO_BITRATE
        AUDIO_CAPTCHA
        AUDIO_TYPES
        AUTHOR_MANAGE
        AUTHOR_URL
        AUTODETECT
        AUTOMATIC_META_EXTRACTION
        AUTO_BAN_DOS_MESSAGE
        AUTO_BAN_HACK_MESSAGE
        AUTO_BAN_SUBJECT
        AUTO_EVENT_TITLE
        AUTO_FILL
        AUTO_IN
        AUTO_INSTALL
        AUTO_MERGED
        AUTO_RECUR
        AUTO_SPACER_STUB
        AUTO_SPACER_TAKE_RESPONSIBILITY
        AUTO_SUBMIT_SITEMAP
        AUTO_TRANSLATE
        AUTO_VALIDATE_ALL_POSTS
        AVERAGE_GAMBLE_MULTIPLIER
        AVOIDING_BANNING_SELF
        AWARDED_ITEMS_PER_PAGE
        AWARD_ALLOCATEHELP
        AWARD_PERMISSION_HELP
        AWARD_UPDATE_TIME_HOURS
        BACKGROUND_THEME_IMAGE
        BACKUP
        BACKUPS
        BACKUP_FILE_COMPRESSED
        BACKUP_FILE_LOG
        DECLINE_FROM_GROUP
        ACCEPT_INTO_GROUP
        BACKUP_FILE_UNCOMPRESSED
        BACKUP_FILE_UNFINISHED
        BACKUP_FINISHED
        BACKUP_INFO_1
        BACKUP_OVERWRITE
        BACKUP_README
        BACKUP_REGULARITY
        BACKUP_SERVER_HOSTNAME
        BACKUP_SERVER_PASSWORD
        BACKUP_SERVER_PATH
        BACKUP_SERVER_PORT
        BACKUP_SERVER_USER
        BACKUP_TYPE
        BAD_ACCESS_DOMAIN
        BAD_ARCHIVE_FORMAT
        BAD_CODENAME
        BAD_IMPORT_PATH
        BAD_INPUT
        BAD_PATH
        BAD_ZONE
        CURRENT_BANNER_IS
        SOURCE_MEDIA
        BANNED_ADDRESSES
        BANNERS
        BANNERS_DESCRIPTION
        BANNER_ALREADY
        BANNER_AUTOSIZE
        BANNER_CAMPAIGN
        BANNER_CLICKTHROUGH
        BANNER_CLICK_SCHEME
        BANNER_DELETED_REMAKE
        BANNER_DIRECT_CODE
        BANNER_FALLBACK
        BANNER_HITS_FROM
        BANNER_HITS_LEFT
        BANNER_HITS_TO
        BANNER_HTML_NOT_RUN
        TREE_LIST_FEEDBACK
        TREE_LIST_HELP
        TREE_LIST_HELP_DESELECT
        TREE_LIST_HELP_MASS_EXPAND
        TREE_LIST_SELECT
        GREETING
        BREADCRUMB_OVERRIDES
        IMPORTANCE_BUY
        IMPORTANCE_MODULUS
        USER_INTERACTION
        USER_INTERACTION_COMMENTING
        USER_INTERACTION_RATING
        IDEAL_DENSITY
        BANNER_INFORMATION
        BANNER_IS_TEXTUAL
        BANNER_LOOKS
        BANNER_PERMANENT
        BANNER_PHP_NOT_RUN
        BANNER_RES_BAD
        BANNER_STATISTICS
        BANNER_TITLE_TEXT
        BANNER_TOO_LARGE
        BANNER_TOO_LARGE_2
        BANNER_TYPE
        BANNER_TYPES
        BANNER_TYPE_LINE
        BANNER_TYPE_LINE_TEXTUAL
        BANNER_UPGRADED
        BANNER_UPGRADE_CONFIRM
        BANNER_UPGRADE_LINE
        BANNER_VIEWS_FROM
        BANNER_VIEWS_TO
        BAN_IP_DESCRIPTION
        BAN_MEMBER
        BAN_MEMBER_DESCRIPTION
        BAN_MEMBER_DOUBLE_DESCRIPTION
        BAN_SUBMITTER_DESCRIPTION
        BASE_CONFIGURATION
        BASE_URL
        BASE_URL_REDIRECTS_WITHOUT_WWW
        BASE_URL_REDIRECTS_WITH_WWW
        BASE_URL_TEXT
        BASE_URL_TEXT_FORUM
        BASIC_CONFIG
        BATCH_IMPORT_ARCHIVE_CONTENTS
        BCC
        BEFORE_MOD_REWRITE
        BEING_INSTALLED
        BENEFITS
        BLANK_EM
        BLOCK
        BLOCKING_REDIRECT_IN_PLACE
        BLOCKS
        BLOCKS_TYPE_ADDON
        BLOCKS_TYPE_bottom
        BLOCKS_TYPE_main
        BLOCKS_TYPE_side
        BLOCK_CACHE
        BLOCK_CONFIGURATION__PANEL_LEFT
        BLOCK_CONFIGURATION__PANEL_NO
        BLOCK_CONFIGURATION__PANEL_NONE
        BLOCK_CONFIGURATION__PANEL_RIGHT
        BLOCK_CONFIGURATION__PANEL_YES
        BLOCK_CONFIGURATION__PANEL_YES_CELL
        BLOCK_HELPER
        RESOURCE_NEW
        BLOCK_HELPER_1
        BLOCK_HELPER_1X
        BLOCK_HELPER_2
        BLOCK_HELPER_NO_PARAMETERS
        BLOCK_IND_ADVANCED
        BLOCK_IND_DEFAULT
        BLOCK_IND_EITHER
        BLOCK_IND_HOOKTYPE
        BLOCK_IND_NUMERIC
        BLOCK_IND_STRIPPABLE_1
        BLOCK_IND_SUPPORTS_COMCODE
        BLOCK_IND_WHETHER
        BLOCK_LIST_IN_PLACE
        BLOCK_MEMBER
        BLOCK_MEMBER_MANUAL
        BLOCK_PARAM_block_id
        BLOCK_PARAM_cache
        BLOCK_PARAM_defer
        BLOCK_PARAM_failsafe
        BLOCK_PARAM_quick_cache
        BLOCK_TAG
        BLOCK_THEM
        BLOCK_TRANS_NAME_bottom_forum_news
        BLOCK_TRANS_NAME_bottom_news
        BLOCK_TRANS_NAME_bottom_rss
        BLOCK_TRANS_NAME_main_awards
        BLOCK_TRANS_NAME_main_banner_wave
        BLOCK_TRANS_NAME_main_bottom_bar
        BLOCK_TRANS_NAME_main_cc_embed
        BLOCK_TRANS_NAME_main_cns_involved_topics
        BLOCK_TRANS_NAME_main_comcode_page_children
        BLOCK_TRANS_NAME_main_comments
        BLOCK_TRANS_NAME_main_contact_catalogues
        BLOCK_TRANS_NAME_main_contact_simple
        BLOCK_TRANS_NAME_main_contact_us
        BLOCK_TRANS_NAME_main_content
        BLOCK_TRANS_NAME_main_content_filtering
        BLOCK_TRANS_NAME_main_count
        BLOCK_TRANS_NAME_main_countdown
        BLOCK_TRANS_NAME_main_custom_comcode_tags
        BLOCK_TRANS_NAME_main_custom_gfx
        BLOCK_TRANS_NAME_main_db_notes
        BLOCK_TRANS_NAME_main_emoticon_codes
        BLOCK_TRANS_NAME_main_forum_news
        BLOCK_TRANS_NAME_main_forum_topics
        BLOCK_TRANS_NAME_main_friends_list
        BLOCK_TRANS_NAME_main_gallery_embed
        BLOCK_TRANS_NAME_main_greeting
        BLOCK_TRANS_NAME_main_image_fader
        BLOCK_TRANS_NAME_main_image_fader_news
        BLOCK_TRANS_NAME_main_include_module
        BLOCK_TRANS_NAME_main_leader_board
        BLOCK_TRANS_NAME_main_member_bar
        BLOCK_TRANS_NAME_main_members
        BLOCK_TRANS_NAME_main_multi_content
        BLOCK_TRANS_NAME_main_news
        BLOCK_TRANS_NAME_main_newsletter_signup
        BLOCK_TRANS_NAME_main_notes
        BLOCK_TRANS_NAME_main_only_if_match
        BLOCK_TRANS_NAME_main_personal_galleries_list
        BLOCK_TRANS_NAME_main_poll
        BLOCK_TRANS_NAME_main_pt_notifications
        BLOCK_TRANS_NAME_main_quotes
        BLOCK_TRANS_NAME_main_rating
        BLOCK_TRANS_NAME_main_rss
        BLOCK_TRANS_NAME_main_screen_actions
        BLOCK_TRANS_NAME_main_search
        BLOCK_TRANS_NAME_main_staff_actions
        BLOCK_TRANS_NAME_main_staff_checklist
        BLOCK_TRANS_NAME_main_staff_new_version
        BLOCK_TRANS_NAME_main_staff_tips
        BLOCK_TRANS_NAME_main_top_sites
        BLOCK_TRANS_NAME_main_trackback
        BLOCK_TRANS_NAME_menu
        BLOCK_TRANS_NAME_side_calendar
        BLOCK_TRANS_NAME_side_cns_private_topics
        NEXT_ITEM_BACK
        NEXT_ITEM_back
        NEXT_ITEM_cms
        NEXT_ITEM_edit_this
        NEXT_ITEM_start
        NEXT_ITEM_view_archive
        NEXT_ITEM_view_this
        BLOCK_TRANS_NAME_side_forum_news
        BLOCK_TRANS_NAME_side_friends
        BLOCK_TRANS_NAME_side_galleries
        BLOCK_TRANS_NAME_side_language
        BLOCK_TRANS_NAME_side_network
        BLOCK_TRANS_NAME_side_news
        BLOCK_TRANS_NAME_side_news_archive
        BLOCK_TRANS_NAME_side_news_categories
        BLOCK_TRANS_NAME_side_personal_stats
        BLOCK_TRANS_NAME_side_printer_friendly
        BLOCK_TRANS_NAME_side_rss
        BLOCK_TRANS_NAME_side_search
        BLOCK_TRANS_NAME_side_shoutbox
        BLOCK_TRANS_NAME_side_stats
        BLOCK_TRANS_NAME_side_tag_cloud
        BLOCK_TRANS_NAME_side_users_online
        BLOCK_TRANS_NAME_top_notifications
        BLOCK_USED_BY
        BLOCK_bottom_forum_news_DESCRIPTION
        BLOCK_bottom_forum_news_PARAM_date_key
        BLOCK_bottom_forum_news_PARAM_date_key_TITLE
        BLOCK_bottom_forum_news_PARAM_forum
        BLOCK_bottom_forum_news_PARAM_forum_TITLE
        BLOCK_bottom_forum_news_PARAM_param
        BLOCK_bottom_forum_news_PARAM_param_TITLE
        BLOCK_bottom_forum_news_USE
        BLOCK_bottom_news_DESCRIPTION
        BLOCK_bottom_news_PARAM_as_guest
        BLOCK_bottom_news_PARAM_as_guest_TITLE
        BLOCK_bottom_news_PARAM_blogs
        BLOCK_bottom_news_PARAM_blogs_TITLE
        BLOCK_bottom_news_PARAM_param
        BLOCK_bottom_news_PARAM_param_TITLE
        BLOCK_bottom_news_PARAM_select
        BLOCK_bottom_news_PARAM_select_TITLE
        BLOCK_bottom_news_PARAM_select_and
        BLOCK_bottom_news_PARAM_select_and_TITLE
        BLOCK_bottom_news_PARAM_zone
        BLOCK_bottom_news_PARAM_zone_TITLE
        BLOCK_bottom_news_USE
        BLOCK_bottom_rss_DESCRIPTION
        BLOCK_bottom_rss_PARAM_max_entries
        BLOCK_bottom_rss_PARAM_max_entries_TITLE
        BLOCK_bottom_rss_PARAM_param
        BLOCK_bottom_rss_PARAM_param_TITLE
        BLOCK_bottom_rss_USE
        BLOCK_main_awards_DESCRIPTION
        BLOCK_main_awards_PARAM_give_context
        BLOCK_main_awards_PARAM_give_context_TITLE
        BLOCK_main_awards_PARAM_guid
        BLOCK_main_awards_PARAM_guid_TITLE
        BLOCK_main_awards_PARAM_include_breadcrumbs
        BLOCK_main_awards_PARAM_include_breadcrumbs_TITLE
        BLOCK_main_awards_PARAM_param
        BLOCK_main_awards_PARAM_param_TITLE
        BLOCK_main_awards_PARAM_zone
        BLOCK_main_awards_PARAM_zone_TITLE
        BLOCK_main_awards_USE
        BLOCK_main_banner_wave_DESCRIPTION
        BLOCK_main_banner_wave_PARAM_max
        BLOCK_main_banner_wave_PARAM_max_TITLE
        BLOCK_main_banner_wave_PARAM_name
        BLOCK_main_banner_wave_PARAM_name_TITLE
        BLOCK_main_banner_wave_PARAM_param
        BLOCK_main_banner_wave_PARAM_param_TITLE
        BLOCK_main_banner_wave_PARAM_region
        BLOCK_main_banner_wave_PARAM_region_TITLE
        BLOCK_main_banner_wave_USE
        BLOCK_main_bottom_bar_DESCRIPTION
        BLOCK_main_bottom_bar_USE
        BLOCK_main_cc_embed_DESCRIPTION
        BLOCK_main_cc_embed_PARAM_as_guest
        BLOCK_main_cc_embed_PARAM_as_guest_TITLE
        BLOCK_main_cc_embed_PARAM_display_type
        BLOCK_main_cc_embed_PARAM_display_type_TITLE
        BLOCK_main_cc_embed_PARAM_filter
        BLOCK_main_cc_embed_PARAM_filter_TITLE
        BLOCK_main_cc_embed_PARAM_max
        BLOCK_main_cc_embed_PARAM_max_TITLE
        BLOCK_main_cc_embed_PARAM_pagination
        BLOCK_main_cc_embed_PARAM_pagination_TITLE
        BLOCK_main_cc_embed_PARAM_param
        BLOCK_main_cc_embed_PARAM_param_TITLE
        BLOCK_main_cc_embed_PARAM_root
        BLOCK_main_cc_embed_PARAM_root_TITLE
        BLOCK_main_cc_embed_PARAM_select
        BLOCK_main_cc_embed_PARAM_select_TITLE
        BLOCK_main_cc_embed_PARAM_sort
        BLOCK_main_cc_embed_PARAM_sort_TITLE
        BLOCK_main_cc_embed_PARAM_sorting
        BLOCK_main_cc_embed_PARAM_sorting_TITLE
        BLOCK_main_cc_embed_PARAM_start
        BLOCK_main_cc_embed_PARAM_start_TITLE
        BLOCK_main_cc_embed_PARAM_template_set
        BLOCK_main_cc_embed_PARAM_template_set_TITLE
        BLOCK_main_cc_embed_USE
        BLOCK_main_cns_involved_topics_DESCRIPTION
        BLOCK_main_cns_involved_topics_PARAM_max
        BLOCK_main_cns_involved_topics_PARAM_max_TITLE
        BLOCK_main_cns_involved_topics_PARAM_member_id
        BLOCK_main_cns_involved_topics_PARAM_member_id_TITLE
        BLOCK_main_cns_involved_topics_PARAM_start
        BLOCK_main_cns_involved_topics_PARAM_start_TITLE
        BLOCK_main_cns_involved_topics_USE
        BLOCK_main_comcode_page_children_DESCRIPTION
        BLOCK_main_comcode_page_children_PARAM_param
        BLOCK_main_comcode_page_children_PARAM_param_TITLE
        BLOCK_main_comcode_page_children_PARAM_zone
        BLOCK_main_comcode_page_children_PARAM_zone_TITLE
        BLOCK_main_comcode_page_children_USE
        BLOCK_main_comments_DESCRIPTION
        BLOCK_main_comments_PARAM_explicit_allow
        BLOCK_main_comments_PARAM_explicit_allow_TITLE
        BLOCK_main_comments_PARAM_extra_param_from
        BLOCK_main_comments_PARAM_extra_param_from_TITLE
        BLOCK_main_comments_PARAM_forum
        BLOCK_main_comments_PARAM_forum_TITLE
        BLOCK_main_comments_PARAM_invisible_if_no_comments
        BLOCK_main_comments_PARAM_invisible_if_no_comments_TITLE
        BLOCK_main_comments_PARAM_max
        BLOCK_main_comments_PARAM_max_TITLE
        BLOCK_main_comments_PARAM_page
        BLOCK_main_comments_PARAM_page_TITLE
        BLOCK_main_comments_PARAM_param
        BLOCK_main_comments_PARAM_param_TITLE
        BLOCK_main_comments_PARAM_reverse
        BLOCK_main_comments_PARAM_reverse_TITLE
        BLOCK_main_comments_PARAM_reviews
        BLOCK_main_comments_PARAM_reviews_TITLE
        BLOCK_main_comments_PARAM_title
        BLOCK_main_comments_PARAM_title_TITLE
        BLOCK_main_comments_USE
        BLOCK_main_contact_catalogues_DESCRIPTION
        BLOCK_main_contact_catalogues_PARAM_body_prefix
        BLOCK_main_contact_catalogues_PARAM_body_prefix_TITLE
        BLOCK_main_contact_catalogues_PARAM_body_suffix
        BLOCK_main_contact_catalogues_PARAM_body_suffix_TITLE
        BLOCK_main_contact_catalogues_PARAM_guid
        BLOCK_main_contact_catalogues_PARAM_guid_TITLE
        BLOCK_main_contact_catalogues_PARAM_param
        BLOCK_main_contact_catalogues_PARAM_param_TITLE
        BLOCK_main_contact_catalogues_PARAM_redirect
        BLOCK_main_contact_catalogues_PARAM_redirect_TITLE
        BLOCK_main_contact_catalogues_PARAM_subject
        BLOCK_main_contact_catalogues_PARAM_subject_TITLE
        BLOCK_main_contact_catalogues_PARAM_subject_prefix
        BLOCK_main_contact_catalogues_PARAM_subject_prefix_TITLE
        BLOCK_main_contact_catalogues_PARAM_subject_suffix
        BLOCK_main_contact_catalogues_PARAM_subject_suffix_TITLE
        BLOCK_main_contact_catalogues_PARAM_to
        BLOCK_main_contact_catalogues_PARAM_to_TITLE
        BLOCK_main_contact_catalogues_USE
        BLOCK_main_contact_simple_DESCRIPTION
        BLOCK_main_contact_simple_PARAM_body_prefix
        BLOCK_main_contact_simple_PARAM_body_prefix_TITLE
        BLOCK_main_contact_simple_PARAM_body_suffix
        BLOCK_main_contact_simple_PARAM_body_suffix_TITLE
        BLOCK_main_contact_simple_PARAM_email_optional
        BLOCK_main_contact_simple_PARAM_email_optional_TITLE
        BLOCK_main_contact_simple_PARAM_guid
        BLOCK_main_contact_simple_PARAM_guid_TITLE
        BLOCK_main_contact_simple_PARAM_param
        BLOCK_main_contact_simple_PARAM_param_TITLE
        BLOCK_main_contact_simple_PARAM_private
        BLOCK_main_contact_simple_PARAM_private_TITLE
        BLOCK_main_contact_simple_PARAM_redirect
        BLOCK_main_contact_simple_PARAM_redirect_TITLE
        BLOCK_main_contact_simple_PARAM_subject_prefix
        BLOCK_main_contact_simple_PARAM_subject_prefix_TITLE
        BLOCK_main_contact_simple_PARAM_subject_suffix
        BLOCK_main_contact_simple_PARAM_subject_suffix_TITLE
        BLOCK_main_contact_simple_PARAM_title
        BLOCK_main_contact_simple_PARAM_title_TITLE
        BLOCK_main_contact_simple_USE
        BLOCK_main_contact_us_DESCRIPTION
        BLOCK_main_contact_us_PARAM_body_prefix
        BLOCK_main_contact_us_PARAM_body_prefix_TITLE
        BLOCK_main_contact_us_PARAM_body_suffix
        BLOCK_main_contact_us_PARAM_body_suffix_TITLE
        BLOCK_main_contact_us_PARAM_email_optional
        BLOCK_main_contact_us_PARAM_email_optional_TITLE
        BLOCK_main_contact_us_PARAM_guid
        BLOCK_main_contact_us_PARAM_guid_TITLE
        BLOCK_main_contact_us_PARAM_param
        BLOCK_main_contact_us_PARAM_param_TITLE
        BLOCK_main_contact_us_PARAM_redirect
        BLOCK_main_contact_us_PARAM_redirect_TITLE
        BLOCK_main_contact_us_PARAM_subject_prefix
        BLOCK_main_contact_us_PARAM_subject_prefix_TITLE
        BLOCK_main_contact_us_PARAM_subject_suffix
        BLOCK_main_contact_us_PARAM_subject_suffix_TITLE
        BLOCK_main_contact_us_PARAM_title
        BLOCK_main_contact_us_PARAM_title_TITLE
        BLOCK_main_contact_us_USE
        BLOCK_main_content_DESCRIPTION
        BLOCK_main_content_PARAM_as_guest
        BLOCK_main_content_PARAM_as_guest_TITLE
        BLOCK_main_content_PARAM_efficient
        BLOCK_main_content_PARAM_efficient_TITLE
        BLOCK_main_content_PARAM_give_context
        BLOCK_main_content_PARAM_give_context_TITLE
        BLOCK_main_content_PARAM_guid
        BLOCK_main_content_PARAM_guid_TITLE
        BLOCK_main_content_PARAM_id
        BLOCK_main_content_PARAM_id_TITLE
        BLOCK_main_content_PARAM_include_breadcrumbs
        BLOCK_main_content_PARAM_include_breadcrumbs_TITLE
        BLOCK_main_content_PARAM_no_links
        BLOCK_main_content_PARAM_no_links_TITLE
        BLOCK_main_content_PARAM_param
        BLOCK_main_content_PARAM_param_TITLE
        BLOCK_main_content_PARAM_render_if_empty
        BLOCK_main_content_PARAM_render_if_empty_TITLE
        BLOCK_main_content_PARAM_select
        BLOCK_main_content_PARAM_select_TITLE
        BLOCK_main_content_PARAM_select_b
        BLOCK_main_content_PARAM_select_b_TITLE
        BLOCK_main_content_PARAM_title
        BLOCK_main_content_PARAM_title_TITLE
        BLOCK_main_content_PARAM_zone
        BLOCK_main_content_PARAM_zone_TITLE
        BLOCK_main_content_USE
        BLOCK_main_content_filtering_DESCRIPTION
        BLOCK_main_content_filtering_PARAM_content_type
        BLOCK_main_content_filtering_PARAM_content_type_TITLE
        BLOCK_main_content_filtering_PARAM_labels
        BLOCK_main_content_filtering_PARAM_labels_TITLE
        BLOCK_main_content_filtering_PARAM_links
        BLOCK_main_content_filtering_PARAM_links_TITLE
        BLOCK_main_content_filtering_PARAM_param
        BLOCK_main_content_filtering_PARAM_param_TITLE
        BLOCK_main_content_filtering_PARAM_types
        BLOCK_main_content_filtering_PARAM_types_TITLE
        BLOCK_main_content_filtering_USE
        BLOCK_main_count_DESCRIPTION
        BLOCK_main_count_PARAM_hit_count
        BLOCK_main_count_PARAM_hit_count_TITLE
        BLOCK_main_count_PARAM_param
        BLOCK_main_count_PARAM_param_TITLE
        BLOCK_main_count_PARAM_start
        BLOCK_main_count_PARAM_start_TITLE
        BLOCK_main_count_USE
        BLOCK_main_countdown_DESCRIPTION
        MANUAL_CHECKS_TOO
        BLOCK_main_countdown_PARAM_param
        BLOCK_main_countdown_PARAM_param_TITLE
        BLOCK_main_countdown_PARAM_precision
        BLOCK_main_countdown_PARAM_precision_TITLE
        BLOCK_main_countdown_PARAM_tailing
        BLOCK_main_countdown_PARAM_tailing_TITLE
        BLOCK_main_countdown_USE
        BLOCK_main_custom_comcode_tags_DESCRIPTION
        BLOCK_main_custom_comcode_tags_USE
        BLOCK_main_custom_gfx_DESCRIPTION
        BLOCK_main_custom_gfx_PARAM_center
        BLOCK_main_custom_gfx_PARAM_center_TITLE
        BLOCK_main_custom_gfx_PARAM_color
        BLOCK_main_custom_gfx_PARAM_color_TITLE
        BLOCK_main_custom_gfx_PARAM_data
        BLOCK_main_custom_gfx_PARAM_data_TITLE
        BLOCK_main_custom_gfx_PARAM_font
        BLOCK_main_custom_gfx_PARAM_font_TITLE
        BLOCK_main_custom_gfx_PARAM_font_size
        BLOCK_main_custom_gfx_PARAM_font_size_TITLE
        BLOCK_main_custom_gfx_PARAM_param
        BLOCK_main_custom_gfx_PARAM_param_TITLE
        BLOCK_main_custom_gfx_PARAM_x
        BLOCK_main_custom_gfx_PARAM_x_TITLE
        BLOCK_main_custom_gfx_PARAM_y
        BLOCK_main_custom_gfx_PARAM_y_TITLE
        BLOCK_main_custom_gfx_USE
        BLOCK_main_db_notes_DESCRIPTION
        BLOCK_main_db_notes_PARAM_param
        BLOCK_main_db_notes_PARAM_param_TITLE
        BLOCK_main_db_notes_PARAM_scrolls
        BLOCK_main_db_notes_PARAM_scrolls_TITLE
        BLOCK_main_db_notes_PARAM_title
        BLOCK_main_db_notes_PARAM_title_TITLE
        BLOCK_main_db_notes_USE
        BLOCK_main_emoticon_codes_DESCRIPTION
        BLOCK_main_emoticon_codes_PARAM_num_columns
        BLOCK_main_emoticon_codes_PARAM_num_columns_TITLE
        BLOCK_main_emoticon_codes_USE
        BLOCK_main_forum_news_DESCRIPTION
        BLOCK_main_forum_news_PARAM_date_key
        BLOCK_main_forum_news_PARAM_date_key_TITLE
        BLOCK_main_forum_news_PARAM_forum
        BLOCK_main_forum_news_PARAM_forum_TITLE
        BLOCK_main_forum_news_PARAM_member_based
        BLOCK_main_forum_news_PARAM_member_based_TITLE
        BLOCK_main_forum_news_PARAM_optimise
        BLOCK_main_forum_news_PARAM_optimise_TITLE
        BLOCK_main_forum_news_PARAM_param
        BLOCK_main_forum_news_PARAM_param_TITLE
        BLOCK_main_forum_news_PARAM_title
        BLOCK_main_forum_news_PARAM_title_TITLE
        BLOCK_main_forum_news_USE
        BLOCK_main_forum_topics_DESCRIPTION
        BLOCK_main_forum_topics_PARAM_check
        BLOCK_main_forum_topics_PARAM_check_TITLE
        BLOCK_main_forum_topics_PARAM_date_key
        BLOCK_main_forum_topics_PARAM_date_key_TITLE
        BLOCK_main_forum_topics_PARAM_hot
        BLOCK_main_forum_topics_PARAM_hot_TITLE
        BLOCK_main_forum_topics_PARAM_limit
        BLOCK_main_forum_topics_PARAM_limit_TITLE
        BLOCK_main_forum_topics_PARAM_param
        BLOCK_main_forum_topics_PARAM_param_TITLE
        BLOCK_main_forum_topics_PARAM_title
        BLOCK_main_forum_topics_PARAM_title_TITLE
        BLOCK_main_forum_topics_PARAM_username_key
        BLOCK_main_forum_topics_PARAM_username_key_TITLE
        BLOCK_main_forum_topics_USE
        BLOCK_main_friends_list_DESCRIPTION
        BLOCK_main_friends_list_PARAM_max
        BLOCK_main_friends_list_PARAM_max_TITLE
        BLOCK_main_friends_list_PARAM_member_id
        BLOCK_main_friends_list_PARAM_member_id_TITLE
        BLOCK_main_friends_list_PARAM_mutual
        BLOCK_main_friends_list_PARAM_mutual_TITLE
        BLOCK_main_friends_list_PARAM_start
        BLOCK_main_friends_list_PARAM_start_TITLE
        BLOCK_main_friends_list_USE
        BLOCK_main_gallery_embed_DESCRIPTION
        BLOCK_main_gallery_embed_PARAM_as_guest
        BLOCK_main_gallery_embed_PARAM_as_guest_TITLE
        BLOCK_main_gallery_embed_PARAM_days
        BLOCK_main_gallery_embed_PARAM_days_TITLE
        BLOCK_main_gallery_embed_PARAM_filter
        BLOCK_main_gallery_embed_PARAM_filter_TITLE
        BLOCK_main_gallery_embed_PARAM_filtercode
        BLOCK_main_gallery_embed_PARAM_guid
        BLOCK_main_gallery_embed_PARAM_guid_TITLE
        BLOCK_main_gallery_embed_PARAM_max
        BLOCK_main_gallery_embed_PARAM_max_TITLE
        BLOCK_main_gallery_embed_PARAM_pagination
        BLOCK_main_gallery_embed_PARAM_pagination_TITLE
        BLOCK_main_gallery_embed_PARAM_param
        BLOCK_main_gallery_embed_PARAM_param_TITLE
        BLOCK_main_gallery_embed_PARAM_render_if_empty
        BLOCK_main_gallery_embed_PARAM_render_if_empty_TITLE
        BLOCK_main_gallery_embed_PARAM_root
        BLOCK_main_gallery_embed_PARAM_root_TITLE
        BLOCK_main_gallery_embed_PARAM_select
        BLOCK_main_gallery_embed_PARAM_select_TITLE
        BLOCK_main_gallery_embed_PARAM_sort
        BLOCK_main_gallery_embed_PARAM_sort_TITLE
        BLOCK_main_gallery_embed_PARAM_start
        BLOCK_main_gallery_embed_PARAM_start_TITLE
        BLOCK_main_gallery_embed_PARAM_title
        BLOCK_main_gallery_embed_PARAM_title_TITLE
        BLOCK_main_gallery_embed_PARAM_video_filter
        BLOCK_main_gallery_embed_PARAM_video_filter_TITLE
        BLOCK_main_gallery_embed_PARAM_video_select
        BLOCK_main_gallery_embed_PARAM_video_select_TITLE
        BLOCK_main_gallery_embed_PARAM_zone
        BLOCK_main_gallery_embed_PARAM_zone_TITLE
        BLOCK_main_gallery_embed_USE
        BLOCK_main_greeting_DESCRIPTION
        BLOCK_main_greeting_USE
        BLOCK_main_image_fader_DESCRIPTION
        BLOCK_main_image_fader_PARAM_as_guest
        BLOCK_main_image_fader_PARAM_as_guest_TITLE
        BLOCK_main_image_fader_PARAM_order
        BLOCK_main_image_fader_PARAM_order_TITLE
        BLOCK_main_image_fader_PARAM_param
        BLOCK_main_image_fader_PARAM_param_TITLE
        BLOCK_main_image_fader_PARAM_time
        BLOCK_main_image_fader_PARAM_time_TITLE
        BLOCK_main_image_fader_PARAM_zone
        BLOCK_main_image_fader_PARAM_zone_TITLE
        BLOCK_main_image_fader_USE
        BLOCK_main_image_fader_news_DESCRIPTION
        BLOCK_main_image_fader_news_PARAM_as_guest
        BLOCK_main_image_fader_news_PARAM_as_guest_TITLE
        BLOCK_main_image_fader_news_PARAM_blogs
        BLOCK_main_image_fader_news_PARAM_blogs_TITLE
        BLOCK_main_image_fader_news_PARAM_max
        BLOCK_main_image_fader_news_PARAM_max_TITLE
        BLOCK_main_image_fader_news_PARAM_param
        BLOCK_main_image_fader_news_PARAM_param_TITLE
        BLOCK_main_image_fader_news_PARAM_time
        BLOCK_main_image_fader_news_PARAM_time_TITLE
        BLOCK_main_image_fader_news_PARAM_title
        BLOCK_main_image_fader_news_PARAM_title_TITLE
        BLOCK_main_image_fader_news_PARAM_zone
        BLOCK_main_image_fader_news_PARAM_zone_TITLE
        BLOCK_main_image_fader_news_USE
        BLOCK_main_include_module_DESCRIPTION
        BLOCK_main_include_module_PARAM_leave_page_and_zone
        BLOCK_main_include_module_PARAM_leave_page_and_zone_TITLE
        BLOCK_main_include_module_PARAM_merge_parameters
        BLOCK_main_include_module_PARAM_merge_parameters_TITLE
        BLOCK_main_include_module_PARAM_only_if_permissions
        BLOCK_main_include_module_PARAM_only_if_permissions_TITLE
        BLOCK_main_include_module_PARAM_param
        BLOCK_main_include_module_PARAM_param_TITLE
        BLOCK_main_include_module_PARAM_strip_title
        BLOCK_main_include_module_PARAM_strip_title_TITLE
        BLOCK_main_include_module_PARAM_use_attached_messages
        BLOCK_main_include_module_PARAM_use_attached_messages_TITLE
        BLOCK_main_include_module_PARAM_use_breadcrumbs
        BLOCK_main_include_module_PARAM_use_breadcrumbs_TITLE
        BLOCK_main_include_module_PARAM_use_helper_panel
        BLOCK_main_include_module_PARAM_use_helper_panel_TITLE
        BLOCK_main_include_module_PARAM_use_http_status
        BLOCK_main_include_module_PARAM_use_http_status_TITLE
        BLOCK_main_include_module_PARAM_use_metadata
        BLOCK_main_include_module_PARAM_use_metadata_TITLE
        BLOCK_main_include_module_PARAM_use_refreshes
        BLOCK_main_include_module_PARAM_use_refreshes_TITLE
        BLOCK_main_include_module_USE
        BLOCK_main_leader_board_DESCRIPTION
        BLOCK_main_leader_board_PARAM_zone
        BLOCK_main_leader_board_PARAM_zone_TITLE
        BLOCK_main_leader_board_USE
        BLOCK_main_member_bar_DESCRIPTION
        BLOCK_main_member_bar_USE
        BLOCK_main_members_DESCRIPTION
        BLOCK_main_members_PARAM_display_mode
        BLOCK_main_members_PARAM_display_mode_TITLE
        BLOCK_main_members_PARAM_filter
        BLOCK_main_members_PARAM_filter_TITLE
        BLOCK_main_members_PARAM_filters_row_a
        BLOCK_main_members_PARAM_filters_row_a_TITLE
        BLOCK_main_members_PARAM_filters_row_b
        BLOCK_main_members_PARAM_filters_row_b_TITLE
        BLOCK_main_members_PARAM_guid
        BLOCK_main_members_PARAM_guid_TITLE
        BLOCK_main_members_PARAM_include_form
        BLOCK_main_members_PARAM_include_form_TITLE
        BLOCK_main_members_PARAM_max
        BLOCK_main_members_PARAM_max_TITLE
        BLOCK_main_members_PARAM_must_have_avatar
        BLOCK_main_members_PARAM_must_have_avatar_TITLE
        BLOCK_main_members_PARAM_must_have_photo
        BLOCK_main_members_PARAM_must_have_photo_TITLE
        BLOCK_main_members_PARAM_pagination
        BLOCK_main_members_PARAM_pagination_TITLE
        BLOCK_main_members_PARAM_parent_gallery
        BLOCK_main_members_PARAM_parent_gallery_TITLE
        BLOCK_main_members_PARAM_per_row
        BLOCK_main_members_PARAM_per_row_TITLE
        BLOCK_main_members_PARAM_select
        BLOCK_main_members_PARAM_select_TITLE
        BLOCK_main_members_PARAM_sort
        BLOCK_main_members_PARAM_sort_TITLE
        BLOCK_main_members_PARAM_start
        BLOCK_main_members_PARAM_start_TITLE
        BLOCK_main_members_PARAM_usergroup
        BLOCK_main_members_PARAM_usergroup_TITLE
        BLOCK_main_members_USE
        BLOCK_main_multi_content_DESCRIPTION
        BLOCK_main_multi_content_PARAM_as_guest
        BLOCK_main_multi_content_PARAM_as_guest_TITLE
        BLOCK_main_multi_content_PARAM_attach_to_url_filter
        BLOCK_main_multi_content_PARAM_attach_to_url_filter_TITLE
        BLOCK_main_multi_content_PARAM_days
        BLOCK_main_multi_content_PARAM_days_TITLE
        BLOCK_main_multi_content_PARAM_efficient
        BLOCK_main_multi_content_PARAM_efficient_TITLE
        BLOCK_main_multi_content_PARAM_filter
        BLOCK_main_multi_content_PARAM_filter_TITLE
        BLOCK_main_multi_content_PARAM_give_context
        BLOCK_main_multi_content_PARAM_give_context_TITLE
        BLOCK_main_multi_content_PARAM_guid
        BLOCK_main_multi_content_PARAM_guid_TITLE
        BLOCK_main_multi_content_PARAM_include_breadcrumbs
        BLOCK_main_multi_content_PARAM_include_breadcrumbs_TITLE
        BLOCK_main_multi_content_PARAM_lifetime
        BLOCK_main_multi_content_PARAM_lifetime_TITLE
        BLOCK_main_multi_content_PARAM_max
        BLOCK_main_multi_content_PARAM_max_TITLE
        BLOCK_main_multi_content_PARAM_no_links
        BLOCK_main_multi_content_PARAM_no_links_TITLE
        BLOCK_main_multi_content_PARAM_pagination
        BLOCK_main_multi_content_PARAM_pagination_TITLE
        BLOCK_main_multi_content_PARAM_param
        BLOCK_main_multi_content_PARAM_param_TITLE
        BLOCK_main_multi_content_PARAM_pinned
        BLOCK_main_multi_content_PARAM_pinned_TITLE
        BLOCK_main_multi_content_PARAM_render_if_empty
        BLOCK_main_multi_content_PARAM_render_if_empty_TITLE
        BLOCK_main_multi_content_PARAM_root
        BLOCK_main_multi_content_PARAM_root_TITLE
        BLOCK_main_multi_content_PARAM_select
        BLOCK_main_multi_content_PARAM_select_TITLE
        BLOCK_main_multi_content_PARAM_select_b
        BLOCK_main_multi_content_PARAM_select_b_TITLE
        BLOCK_main_multi_content_PARAM_sort
        BLOCK_main_multi_content_PARAM_sort_TITLE
        BLOCK_main_multi_content_PARAM_start
        BLOCK_main_multi_content_PARAM_start_TITLE
        BLOCK_main_multi_content_PARAM_title
        BLOCK_main_multi_content_PARAM_title_TITLE
        BLOCK_main_multi_content_PARAM_zone
        BLOCK_main_multi_content_PARAM_zone_TITLE
        BLOCK_main_multi_content_USE
        BLOCK_main_news_DESCRIPTION
        CONFIG_OPTION_autogrow
        AUTOGROW
        BLOCK_main_news_PARAM_as_guest
        BLOCK_main_news_PARAM_as_guest_TITLE
        BLOCK_main_news_PARAM_attach_to_url_filter
        BLOCK_main_news_PARAM_attach_to_url_filter_TITLE
        BLOCK_main_news_PARAM_blogs
        BLOCK_main_news_PARAM_blogs_TITLE
        BLOCK_main_news_PARAM_fallback_archive
        BLOCK_main_news_PARAM_fallback_archive_TITLE
        BLOCK_main_news_PARAM_fallback_full
        BLOCK_main_news_PARAM_fallback_full_TITLE
        BLOCK_main_news_PARAM_filter
        BLOCK_main_news_PARAM_filter_TITLE
        BLOCK_main_news_PARAM_historic
        BLOCK_main_news_PARAM_historic_TITLE
        BLOCK_main_news_PARAM_member_based
        BLOCK_main_news_PARAM_member_based_TITLE
        BLOCK_main_news_PARAM_multiplier
        BLOCK_main_news_PARAM_multiplier_TITLE
        BLOCK_main_news_PARAM_no_links
        BLOCK_main_news_PARAM_no_links_TITLE
        BLOCK_main_news_PARAM_optimise
        BLOCK_main_news_PARAM_optimise_TITLE
        BLOCK_main_news_PARAM_pagination
        BLOCK_main_news_PARAM_pagination_TITLE
        BLOCK_main_news_PARAM_param
        BLOCK_main_news_PARAM_param_TITLE
        BLOCK_main_news_PARAM_render_if_empty
        BLOCK_main_news_PARAM_render_if_empty_TITLE
        BLOCK_main_news_PARAM_select
        BLOCK_main_news_PARAM_select_TITLE
        BLOCK_main_news_PARAM_select_and
        BLOCK_main_news_PARAM_select_and_TITLE
        BLOCK_main_news_PARAM_show_in_full
        BLOCK_main_news_PARAM_show_in_full_TITLE
        BLOCK_main_news_PARAM_start
        BLOCK_main_news_PARAM_start_TITLE
        BLOCK_main_news_PARAM_title
        BLOCK_main_news_PARAM_title_TITLE
        BLOCK_main_news_PARAM_zone
        BLOCK_main_news_PARAM_zone_TITLE
        BLOCK_main_news_USE
        BLOCK_main_newsletter_signup_DESCRIPTION
        BLOCK_main_newsletter_signup_PARAM_param
        BLOCK_main_newsletter_signup_PARAM_param_TITLE
        BLOCK_main_newsletter_signup_PARAM_path
        BLOCK_main_newsletter_signup_PARAM_path_TITLE
        BLOCK_main_newsletter_signup_PARAM_subject
        BLOCK_main_newsletter_signup_PARAM_subject_TITLE
        BLOCK_main_newsletter_signup_PARAM_to
        BLOCK_main_newsletter_signup_PARAM_to_TITLE
        BLOCK_main_newsletter_signup_PARAM_button_only
        BLOCK_main_newsletter_signup_PARAM_button_only_TITLE
        BLOCK_main_newsletter_signup_USE
        BLOCK_main_notes_DESCRIPTION
        BLOCK_main_notes_PARAM_lang_none
        BLOCK_main_notes_PARAM_lang_none_TITLE
        BLOCK_main_notes_PARAM_param
        BLOCK_main_notes_PARAM_param_TITLE
        BLOCK_main_notes_PARAM_scrolls
        BLOCK_main_notes_PARAM_scrolls_TITLE
        BLOCK_main_notes_PARAM_title
        BLOCK_main_notes_PARAM_title_TITLE
        BLOCK_main_notes_USE
        NO_DOWNLOAD_ACCESS
        BLOCK_main_only_if_match_DESCRIPTION
        BLOCK_main_only_if_match_PARAM_caption
        BLOCK_main_only_if_match_PARAM_caption_TITLE
        BLOCK_main_only_if_match_PARAM_param
        BLOCK_main_only_if_match_PARAM_param_TITLE
        BLOCK_main_only_if_match_USE
        BLOCK_main_personal_galleries_list_DESCRIPTION
        BLOCK_main_personal_galleries_list_PARAM_max
        BLOCK_main_personal_galleries_list_PARAM_max_TITLE
        BLOCK_main_personal_galleries_list_PARAM_member_id
        BLOCK_main_personal_galleries_list_PARAM_member_id_TITLE
        BLOCK_main_personal_galleries_list_PARAM_start
        BLOCK_main_personal_galleries_list_PARAM_start_TITLE
        BLOCK_main_personal_galleries_list_USE
        BLOCK_main_poll_DESCRIPTION
        BLOCK_main_poll_PARAM_param
        BLOCK_main_poll_PARAM_param_TITLE
        BLOCK_main_poll_PARAM_zone
        BLOCK_main_poll_PARAM_zone_TITLE
        BLOCK_main_poll_USE
        BLOCK_main_pt_notifications_DESCRIPTION
        BLOCK_main_pt_notifications_USE
        BLOCK_main_quotes_DESCRIPTION
        BLOCK_main_quotes_PARAM_param
        BLOCK_main_quotes_PARAM_param_TITLE
        BLOCK_main_quotes_PARAM_title
        BLOCK_main_quotes_PARAM_title_TITLE
        BLOCK_main_quotes_USE
        BLOCK_main_rating_DESCRIPTION
        BLOCK_main_rating_PARAM_extra_param_from
        BLOCK_main_rating_PARAM_extra_param_from_TITLE
        BLOCK_main_rating_PARAM_display_tpl
        BLOCK_main_rating_PARAM_display_tpl_TITLE
        BLOCK_main_rating_PARAM_page
        BLOCK_main_rating_PARAM_page_TITLE
        BLOCK_main_rating_PARAM_param
        BLOCK_main_rating_PARAM_param_TITLE
        BLOCK_main_rating_PARAM_title
        BLOCK_main_rating_PARAM_title_TITLE
        BLOCK_main_rating_USE
        BLOCK_main_rss_DESCRIPTION
        BLOCK_main_rss_PARAM_copyright
        BLOCK_main_rss_PARAM_copyright_TITLE
        BLOCK_main_rss_PARAM_max_entries
        BLOCK_main_rss_PARAM_max_entries_TITLE
        BLOCK_main_rss_PARAM_param
        BLOCK_main_rss_PARAM_param_TITLE
        BLOCK_main_rss_PARAM_title
        BLOCK_main_rss_PARAM_title_TITLE
        BLOCK_main_rss_USE
        BLOCK_main_screen_actions_DESCRIPTION
        BLOCK_main_screen_actions_PARAM_title
        BLOCK_main_screen_actions_PARAM_title_TITLE
        BLOCK_main_screen_actions_PARAM_url
        BLOCK_main_screen_actions_PARAM_url_TITLE
        BLOCK_main_screen_actions_USE
        BLOCK_main_search_DESCRIPTION
        BLOCK_main_search_PARAM_author
        BLOCK_main_search_PARAM_author_TITLE
        BLOCK_main_search_PARAM_boolean_search
        BLOCK_main_search_PARAM_boolean_search_TITLE
        BLOCK_main_search_PARAM_conjunctive_operator
        BLOCK_main_search_PARAM_conjunctive_operator_TITLE
        BLOCK_main_search_PARAM_days
        BLOCK_main_search_PARAM_days_TITLE
        BLOCK_main_search_PARAM_direction
        BLOCK_main_search_PARAM_direction_TITLE
        BLOCK_main_search_PARAM_extra
        BLOCK_main_search_PARAM_extra_TITLE
        BLOCK_main_search_PARAM_input_fields
        BLOCK_main_search_PARAM_input_fields_TITLE
        BLOCK_main_search_PARAM_limit_to
        BLOCK_main_search_PARAM_limit_to_TITLE
        BLOCK_main_search_PARAM_only_search_meta
        BLOCK_main_search_PARAM_only_search_meta_TITLE
        BLOCK_main_search_PARAM_only_titles
        BLOCK_main_search_PARAM_only_titles_TITLE
        BLOCK_main_search_PARAM_search_under
        BLOCK_main_search_PARAM_search_under_TITLE
        BLOCK_main_search_PARAM_sort
        BLOCK_main_search_PARAM_sort_TITLE
        BLOCK_main_search_PARAM_title
        BLOCK_main_search_PARAM_title_TITLE
        BLOCK_main_search_PARAM_zone
        BLOCK_main_search_PARAM_zone_TITLE
        BLOCK_main_search_USE
        BLOCK_main_staff_actions_DESCRIPTION
        BLOCK_main_staff_actions_PARAM_max
        BLOCK_main_staff_actions_PARAM_max_TITLE
        BLOCK_main_staff_actions_USE
        BLOCK_main_staff_checklist_DESCRIPTION
        BLOCK_main_staff_checklist_USE
        BLOCK_main_staff_links_DESCRIPTION
        BLOCK_main_staff_links_USE
        BLOCK_main_staff_new_version_DESCRIPTION
        BLOCK_main_staff_new_version_USE
        BLOCK_main_staff_tips_DESCRIPTION
        BLOCK_main_staff_tips_USE
        BLOCK_main_staff_website_monitoring_DESCRIPTION
        BLOCK_main_staff_website_monitoring_USE
        BLOCK_main_top_sites_DESCRIPTION
        BLOCK_main_top_sites_PARAM_param
        BLOCK_main_top_sites_PARAM_param_TITLE
        BLOCK_main_top_sites_USE
        BLOCK_main_trackback_DESCRIPTION
        BLOCK_main_trackback_PARAM_id
        BLOCK_main_trackback_PARAM_id_TITLE
        BLOCK_main_trackback_PARAM_page
        BLOCK_main_trackback_PARAM_page_TITLE
        BLOCK_main_trackback_PARAM_param
        BLOCK_main_trackback_PARAM_param_TITLE
        BLOCK_main_trackback_USE
        BLOCK_menu_DESCRIPTION
        BLOCK_menu_PARAM_javascript_highlighting
        BLOCK_menu_PARAM_javascript_highlighting_TITLE
        BLOCK_menu_PARAM_param
        BLOCK_menu_PARAM_param_TITLE
        BLOCK_menu_PARAM_silent_failure
        BLOCK_menu_PARAM_silent_failure_TITLE
        BLOCK_menu_PARAM_title
        BLOCK_menu_PARAM_title_TITLE
        BLOCK_menu_PARAM_tray_status
        BLOCK_menu_PARAM_tray_status_TITLE
        BLOCK_menu_PARAM_type
        BLOCK_menu_PARAM_type_TITLE
        BLOCK_menu_USE
        INDEX_DOWNLOAD
        SPAM_URLS
        BLOCK_side_tag_cloud_PARAM_apply_permissions_TITLE
        BLOCK_side_tag_cloud_PARAM_apply_permissions
        BLOCK_side_calendar_DESCRIPTION
        BLOCK_side_calendar_PARAM_as_guest
        BLOCK_side_calendar_PARAM_as_guest_TITLE
        BLOCK_side_calendar_PARAM_days
        BLOCK_side_calendar_PARAM_days_TITLE
        BLOCK_side_calendar_PARAM_filter
        BLOCK_side_calendar_PARAM_filter_TITLE
        BLOCK_side_calendar_PARAM_param
        BLOCK_side_calendar_PARAM_param_TITLE
        BLOCK_side_calendar_PARAM_private
        BLOCK_side_calendar_PARAM_private_TITLE
        BLOCK_side_calendar_PARAM_title
        BLOCK_side_calendar_PARAM_title_TITLE
        BLOCK_side_calendar_PARAM_zone
        BLOCK_side_calendar_PARAM_zone_TITLE
        BLOCK_side_calendar_USE
        BLOCK_side_cns_private_topics_DESCRIPTION
        BLOCK_side_cns_private_topics_USE
        BLOCK_side_forum_news_DESCRIPTION
        BLOCK_side_forum_news_PARAM_date_key
        BLOCK_side_forum_news_PARAM_date_key_TITLE
        BLOCK_side_forum_news_PARAM_forum
        BLOCK_side_forum_news_PARAM_forum_TITLE
        BLOCK_side_forum_news_PARAM_param
        BLOCK_side_forum_news_PARAM_param_TITLE
        BLOCK_side_forum_news_PARAM_title
        BLOCK_side_forum_news_PARAM_title_TITLE
        BLOCK_side_forum_news_USE
        BLOCK_side_friends_DESCRIPTION
        BLOCK_side_friends_PARAM_max
        BLOCK_side_friends_PARAM_max_TITLE
        BLOCK_side_friends_USE
        BLOCK_side_galleries_DESCRIPTION
        BLOCK_side_galleries_PARAM_depth
        BLOCK_side_galleries_PARAM_depth_TITLE
        BLOCK_side_galleries_PARAM_param
        BLOCK_side_galleries_PARAM_param_TITLE
        BLOCK_side_galleries_PARAM_show_empty
        BLOCK_side_galleries_PARAM_show_empty_TITLE
        BLOCK_side_galleries_PARAM_zone
        BLOCK_side_galleries_PARAM_zone_TITLE
        BLOCK_side_galleries_USE
        BLOCK_side_language_DESCRIPTION
        BLOCK_side_language_USE
        BLOCK_side_network_DESCRIPTION
        BLOCK_side_network_USE
        BLOCK_side_news_DESCRIPTION
        BLOCK_side_news_PARAM_as_guest
        BLOCK_side_news_PARAM_as_guest_TITLE
        BLOCK_side_news_PARAM_blogs
        BLOCK_side_news_PARAM_blogs_TITLE
        BLOCK_side_news_PARAM_historic
        BLOCK_side_news_PARAM_historic_TITLE
        BLOCK_side_news_PARAM_param
        BLOCK_side_news_PARAM_param_TITLE
        BLOCK_side_news_PARAM_select
        BLOCK_side_news_PARAM_select_TITLE
        BLOCK_side_news_PARAM_select_and
        BLOCK_side_news_PARAM_select_and_TITLE
        BLOCK_side_news_PARAM_ticker
        BLOCK_side_news_PARAM_title
        BLOCK_side_news_PARAM_title_TITLE
        BLOCK_side_news_PARAM_zone
        BLOCK_side_news_PARAM_zone_TITLE
        BLOCK_side_news_USE
        BLOCK_side_news_archive_DESCRIPTION
        BLOCK_side_news_archive_PARAM_select
        BLOCK_side_news_archive_PARAM_select_TITLE
        BLOCK_side_news_archive_PARAM_title
        BLOCK_side_news_archive_PARAM_title_TITLE
        BLOCK_side_news_archive_PARAM_zone
        BLOCK_side_news_archive_PARAM_zone_TITLE
        BLOCK_side_news_archive_USE
        BLOCK_side_news_categories_DESCRIPTION
        BLOCK_side_news_categories_USE
        BLOCK_side_personal_stats_DESCRIPTION
        BLOCK_side_personal_stats_USE
        BLOCK_side_printer_friendly_DESCRIPTION
        BLOCK_side_printer_friendly_USE
        BLOCK_side_rss_DESCRIPTION
        BLOCK_side_rss_PARAM_copyright
        BLOCK_side_rss_PARAM_copyright_TITLE
        BLOCK_side_rss_PARAM_max_entries
        BLOCK_side_rss_PARAM_max_entries_TITLE
        BLOCK_side_rss_PARAM_param
        BLOCK_side_rss_PARAM_param_TITLE
        BLOCK_side_rss_PARAM_ticker
        BLOCK_side_rss_PARAM_ticker_TITLE
        BLOCK_side_rss_PARAM_title
        BLOCK_side_rss_PARAM_title_TITLE
        BLOCK_side_rss_USE
        BLOCK_side_shoutbox_DESCRIPTION
        BLOCK_side_shoutbox_PARAM_max
        BLOCK_side_shoutbox_PARAM_max_TITLE
        BLOCK_side_shoutbox_PARAM_param
        BLOCK_side_shoutbox_PARAM_param_TITLE
        BLOCK_side_shoutbox_USE
        BLOCK_side_stats_DESCRIPTION
        BLOCK_side_stats_USE
        BLOCK_side_tag_cloud_DESCRIPTION
        BLOCK_side_tag_cloud_PARAM_max
        BLOCK_side_tag_cloud_PARAM_max_TITLE
        BLOCK_side_tag_cloud_PARAM_param
        BLOCK_side_tag_cloud_PARAM_param_TITLE
        BLOCK_side_tag_cloud_PARAM_title
        BLOCK_side_tag_cloud_PARAM_title_TITLE
        BLOCK_side_tag_cloud_PARAM_zone
        ADD_TIME
        BLOCK_side_tag_cloud_PARAM_zone_TITLE
        BLOCK_side_tag_cloud_USE
        BLOCK_side_users_online_DESCRIPTION
        BLOCK_side_users_online_USE
        BLOCK_top_login_DESCRIPTION
        BLOCK_top_login_USE
        BLOCK_top_notifications_DESCRIPTION
        BLOCK_top_notifications_PARAM_max
        BLOCK_top_notifications_PARAM_max_TITLE
        BLOCK_top_notifications_USE
        BLOCK_top_personal_stats_DESCRIPTION
        BLOCK_top_personal_stats_USE
        BLOCK_top_search_DESCRIPTION
        BLOCK_top_search_PARAM_author
        BLOCK_top_search_PARAM_author_TITLE
        BLOCK_top_search_PARAM_boolean_search
        BLOCK_top_search_PARAM_boolean_search_TITLE
        BLOCK_top_search_PARAM_conjunctive_operator
        BLOCK_top_search_PARAM_conjunctive_operator_TITLE
        BLOCK_top_search_PARAM_days
        BLOCK_top_search_PARAM_days_TITLE
        BLOCK_top_search_PARAM_direction
        BLOCK_top_search_PARAM_direction_TITLE
        BLOCK_top_search_PARAM_extra
        BLOCK_top_search_PARAM_extra_TITLE
        BLOCK_top_search_PARAM_input_fields_TITLE
        BLOCK_top_search_PARAM_limit_to
        BLOCK_top_search_PARAM_limit_to_TITLE
        BLOCK_top_search_PARAM_only_search_meta
        BLOCK_top_search_PARAM_only_search_meta_TITLE
        BLOCK_top_search_PARAM_only_titles
        BLOCK_top_search_PARAM_only_titles_TITLE
        BLOCK_top_search_PARAM_search_under
        BLOCK_top_search_PARAM_search_under_TITLE
        BLOCK_top_search_PARAM_sort
        BLOCK_top_search_PARAM_sort_TITLE
        BLOCK_top_search_PARAM_title
        BLOCK_top_search_PARAM_title_TITLE
        BLOCK_top_search_PARAM_zone
        BLOCK_top_search_PARAM_zone_TITLE
        BLOCK_top_search_USE
        BLOG_NEWS_ARTICLE
        ORDER_AUTOMATED_CRITERIA
        NOTIFICATION_TYPE_cns_username_changed_staff
        BLOG_NEWS_SUMMARY
        BLOG_REGULARITY
        BOARD_INSTALL_NUMBER
        BODY
        BOTTOM_LEFT
        BOTTOM_RIGHT
        BOT_STATS
        BOUNCE_FILTER
        BOUNCE_WHICH
        BRANCH
        BRANCHES
        BRANCHES_DESCRIPTION
        BRANCH_TYPE
        BREADCRUMB_CROP_LENGTH
        BROKEN_URLS
        BROKEN_URLS_404
        BROKEN_URLS_FILES
        BROKEN_WIKI_CHILD_LINK
        BROKEN_XHTML_FIXED
        BROWSER_SHARE
        BROWSE_ID_INSERT
        BRUTEFORCE_LOGIN_HACK
        BRUTE_FORCE
        BRUTE_FORCE_INSTANT_BAN
        BRUTE_FORCE_LOGIN_MINUTES
        BRUTE_FORCE_THRESHOLD
        BUT_EXPIRED
        BYPASS_COMCODE_PAGE_VALIDATION
        BYPASS_DOWNLOAD_VALIDATION
        BYPASS_NEWS_BLOG_VALIDATION
        BYPASS_NEWS_VALIDATION
        BYPASS_POST_VALIDATION
        BYPASS_TOPIC_VALIDATION
        BYPASS_VALIDATION_BANNER
        BYPASS_VALIDATION_CALENDAR_EVENT
        BYPASS_VALIDATION_CALENDAR_EVENT__NON_PUBLIC
        BYPASS_VALIDATION_CATALOGUE_ENTRY
        BYPASS_VALIDATION_HACK
        BYPASS_VALIDATION_MEDIA
        BYPASS_VALIDATION_POLL
        BYPASS_VALIDATION_QUIZ
        BYPASS_WIKI_VALIDATION
        CACHES
        CACHES_PAGE
        CACHES_PAGE_CACHES
        CACHES_PAGE_EXP_CACHES
        CACHES_PAGE_EXP_OPTIMISERS
        CACHE_FORUMS
        FEEDS_TO_OVERLAY
        PURCHASE_WRONG_CURRENCY
        PURCHASE_WRONG_PRICE
        CACHE_MEMBERS
        CACHE_NOT_READY_YET
        CACHE_TOPICS
        CALCULATE
        CALCULATED_SIZE
        CALCULATE_SIZE
        CALLBACK_PASSWORD
        CALL_HOME
        CANCEL
        CANCELLED
        CANCEL_MANUAL_SUBSCRIPTION_CONFIRM
        CANNOT_ACCESS_URL
        CANNOT_CATEGORISE_INVITED
        CANNOT_CONNECT_HOME
        CANNOT_COPY_TO_SERVER
        CANNOT_CRITICISE_BASE_LANG
        CANNOT_DELETE_FORUM_OPTION
        CANNOT_DELETE_ROOT_FORUM
        CANNOT_MERGE_CATALOGUES
        CANT_TAKE_RESPONSIBILITY
        CANT_TRACK
        CAN_MASS_UPLOAD
        CAPTCHAFAIL_HACK
        CAPTCHA_NOISE
        CAPTCHA_ON_FEEDBACK
        CAPTCHA_SINGLE_GUESS
        CAPTION_LONG
        CAREFUL
        CART_HOLD_HOURS
        CASH_FLOW
        INCORRECT_COOKIE_DOMAIN
        INCORRECT_COOKIE_PATH
        CATALOGUES_IMPORT_MISSING_KEY_FIELD
        CATALOGUES_IMPORT_MISSING_META_DESCRIPTION_FIELD
        CATALOGUES_IMPORT_MISSING_META_KEYWORDS_FIELD
        CATALOGUES_IMPORT_MISSING_NOTES_FIELD
        CATALOGUES_SUBCAT_NARROWIN
        CATALOGUE_CATEGORY
        CATALOGUE_CSV_DELETE_HANDLING
        CATALOGUE_CSV_IMPORT_KEY_FIELD
        CATALOGUE_CSV_IMPORT_META_DESCRIPTION_FIELD
        CATALOGUE_CSV_IMPORT_META_KEYWORDS_FIELD
        CATALOGUE_CSV_IMPORT_NOTES_FIELD
        CATALOGUE_CSV_NEW_HANDLING
        CATALOGUE_CSV_UPDATE_HANDLING
        CATALOGUE_ENTRIES_PER_PAGE
        CATALOGUE_EXPORT
        CATALOGUE_FIELD
        CATALOGUE_GENERIC_ADD
        CATALOGUE_GENERIC_ADD_CATEGORY
        CATALOGUE_GENERIC_DELETE
        CATALOGUE_GENERIC_DELETE_CATEGORY
        CATALOGUE_GENERIC_EDIT
        CATALOGUE_GENERIC_EDIT_CATEGORY
        CATALOGUE_IMPORT
        CATALOGUE_IMPORT_TEXT
        CATALOGUE_NO_DELETE_ROOT
        CATALOGUE_SUBCATS_PER_PAGE
        CAT_ECOMMERCE
        CAT_NOT_FOUND
        CCP_ATTRIBUTE_ERROR
        CCP_BROKEN_END
        CCP_DUPLICATE_ATTRIBUTES
        CCP_ERROR_STUB
        CCP_LIST_JUMPYNESS
        CCP_MENU_JUMPYNESS
        CCP_NO_CLOSE
        CCP_NO_CLOSE_MATCH
        CCP_TAG_CLOSE_ANOMALY
        CCP_TAG_OPEN_ANOMALY
        CC_ADDRESS
        CDN
        CHANGE
        CHANGED_USERGROUP
        CHANGE_BLOCK_BACKGROUND
        EDIT_TIME
        CHANGE_FIXED_WIDTH
        CHANGE_FONT
        CHANGE_HEADER_IMAGE
        CHANGE_INNER_BACKGROUND
        CHANGE_MICRO
        CHANGE_OUTER_BACKGROUND
        CHANGE_REGULARITY
        CHANGE_USERGROUP_TO
        CHARGED_POINTS
        CHARGE_MEMBER
        CHATROOMS
        CHATROOM_LANG
        CHATROOM_OWNER
        CHATTING
        CHAT_BAN
        CHAT_BAN_GUEST
        CHAT_BAN_YOURSELF
        CHAT_MAX_MESSAGES_TO_SHOW
        CHAT_MESSAGE_CHECK_INTERVAL
        CHAT_MODERATION
        CHAT_MODERATION_INTRO
        CHAT_OPTIONS_COLOUR_DESCRIPTION
        CHAT_OPTIONS_TEXT_DESCRIPTION
        CHAT_TRANSITORY_ALERT_TIME
        CHAT_UNBAN
        CHECKLIST_INITIAL_TASK_CONTENT
        CHECKLIST_INITIAL_TASK_FAVICON
        CHECKLIST_INITIAL_TASK_GOOGLE_WEBMASTER_TOOLS
        CHECKLIST_INITIAL_TASK_MAIL
        CHECKLIST_INITIAL_TASK_MAIL_LOGO
        CHECKLIST_INITIAL_TASK_STRUCTURE
        CHECKLIST_INITIAL_TASK_THEME
        CHECKLIST_INITIAL_TASK_UPTIME_MONITOR
        CHECKLIST_INITIAL_TASK_WEBCLIP
        CHECK_BROKEN_URLS
        CHECK_LIST
        CHECK_LIST_ADD
        CHECK_LIST_DELETE
        CHECK_LIST_MARK_DONE
        CHECK_LIST_MARK_UNDONE
        CHMOD_FAIL
        CHMOD_PASS
        CHOICE
        CHOOSE_A_CONFIG_CATEGORY
        CHOOSE_CATALOGUE
        CHOOSE_CRITICISE_LIST_LANG_FILE
        CHOOSE_CSS
        CHOOSE_DELURK_CRITERIA
        CHOOSE_EDIT_LIST
        CHOOSE_EDIT_LIST_EXTRA
        CHOOSE_EDIT_LIST_LANG_FILE
        CHOOSE_EDIT_TABLE
        CHOOSE_EDIT_TABLE_EXTRA
        CHOOSE_EDIT_TABLE_EXTRA_COMCODE_PAGES
        CHOOSE_ENTRY_POINT
        CHOOSE_FORUM_EDIT
        CHOOSE_FORUM_EDIT_2
        CHOOSE_LANG_DESCRIP
        CHOOSE_LANG_DESCRIP_ADD_TO_MAIN_LANG_FIRST
        CHOOSE_MASTER_PASSWORD
        CHOOSE_OPEN_NEW_WINDOW
        CHOOSE_POLL
        CHOOSE_SAVED_WARNING
        CHOOSE_SEND_TO
        PAYMENT_GATEWAY
        DEFAULT_POST_CONTENT
        DEFAULT_POST_TITLE
        CHOOSE_TEMPLATES
        CHOOSE_THEME_IMAGE
        CHOOSE_VIEW_LIST
        CHOOSE_WINNERS
        CHOOSE_ZONE_OF_MODULES
        CLASSIFIED_ADS
        CLEANED_X_FIELDS
        CLEANUP_FILES
        CLEANUP_TOOLS
        CLEAR_EXISTING_FORUMS_ON_INSTALL
        CLEAR_STATISTICS
        CLICK_ENTRY_POINT_TO_USE
        CLICK_ENTRY_POINT_TO_USE_2
        CLONE
        CLOSED_SITE
        CLOSED_SITE_OPTION
        CLOSED_SITE_SCREEN
        CLOSE_TIME
        CLOSING_BALANCE
        CLUB_FORUM_PARENT_FORUM
        CLUB_FORUM_PARENT_FORUM_GROUPING
        CMD_SEND_CHATMESSAGE_HELP
        CMD_SEND_CHATMESSAGE_HELP_PARAM_0
        CMD_SEND_CHATMESSAGE_HELP_PARAM_1
        CMD_FIND_COMMANDR_FS_FILENAME_VIA_ID_HELP
        CMD_FIND_COMMANDR_FS_FILENAME_VIA_ID_HELP_PARAM_0
        CMD_FIND_COMMANDR_FS_FILENAME_VIA_ID_HELP_PARAM_1
        CMD_FIND_ID_VIA_COMMANDR_FS_FILENAME_HELP
        CMD_FIND_ID_VIA_COMMANDR_FS_FILENAME_HELP_PARAM_0
        CMD_FIND_ID_VIA_COMMANDR_FS_FILENAME_HELP_PARAM_1
        CMD_ALIEN_CHECK_HELP
        CMD_ANTISPAM_CHECK_HELP
        CMD_ANTISPAM_CHECK_HELP_PARAM_0
        CMD_APPEND_HELP
        CMD_APPEND_HELP_PARAM_0
        CMD_APPEND_HELP_PARAM_1
        CMD_BAN_IP_HELP
        CMD_BAN_IP_HELP_PARAM_0
        CMD_BAN_IP_HELP_PARAM_1
        CMD_BAN_IP_HELP_U
        CMD_BAN_MEMBER_HELP
        CMD_BAN_MEMBER_HELP_PARAM_0
        CMD_BAN_MEMBER_HELP_U
        CMD_BSOD_HELP
        CMD_CALL_HELP
        CMD_CALL_HELP_PARAM_0
        CMD_CAT_HELP
        CMD_CAT_HELP_L
        CMD_CAT_HELP_PARAM_0
        CMD_CD_HELP
        CMD_CD_HELP_PARAM_0
        CMD_CHECK_PERMS_HELP
        CMD_CLEAR_CACHES_HELP
        CMD_CLEAR_CACHES_HELP_PARAM_0
        CMD_CLEAR_HELP
        CMD_CLOSED_HELP
        CMD_CLOSED_HELP_C
        CMD_CLOSED_HELP_O
        CMD_CLOSED_HELP_PARAM_0
        CMD_COMMANDRCHAT_HELP
        CMD_COMMANDRCHAT_HELP_PARAM_0
        CMD_COMMANDRCHAT_HELP_PARAM_1
        CMD_COMMANDS_HELP
        CMD_CONTINUE_IMPORT_HELP
        CMD_CONTINUE_IMPORT_HELP_PARAM_0
        CMD_CONTINUE_IMPORT_HELP_PARAM_1
        CMD_CPDIR_HELP
        CMD_CPDIR_HELP_PARAM_0
        CMD_CPDIR_HELP_PARAM_1
        CMD_CP_HELP
        CMD_CP_HELP_PARAM_0
        CMD_CP_HELP_PARAM_1
        CMD_DATABASE_UPGRADE_HELP
        CMD_DATE_HELP
        CMD_DB_SEARCH_HELP
        CMD_DB_SEARCH_HELP_PARAM_0
        CMD_DB_SEARCH_HELP_PARAM_1
        CMD_DEEP_CLEAN_HELP
        CMD_DEEP_CLEAN_HELP_PARAM_0
        CMD_DEEP_CLEAN_HELP_PARAM_1
        CMD_DEEP_CLEAN_HELP_PARAM_2
        CMD_DIRECTORY_SIZES_HELP
        CMD_ECHO_HELP
        CMD_ECHO_HELP_PARAM_0
        CMD_EDIT_HELP
        CMD_EDIT_HELP_PARAM_0
        CMD_EXIT_HELP
        CMD_FEEDBACK_HELP
        CMD_FEEDBACK_HELP_PARAM_0
        CMD_FEED_DISPLAY_HELP
        CMD_FEED_DISPLAY_HELP_M
        CMD_FEED_DISPLAY_HELP_PARAM_0
        CMD_FIND_CODES_HELP
        CMD_FIND_CODES_HELP_PARAM_0
        CMD_FIND_COMMANDRFS_FILENAME_VIA_ID_HELP
        CMD_FIND_COMMANDRFS_FILENAME_VIA_ID_HELP_PARAM_0
        CMD_FIND_COMMANDRFS_FILENAME_VIA_ID_HELP_PARAM_1
        CMD_FIND_ENTRY_POINTS_HELP
        CMD_FIND_ENTRY_POINTS_HELP_PARAM_0
        CMD_FIND_FUNCTION_HELP
        CMD_FIND_FUNCTION_HELP_PARAM_0
        CMD_FIND_GUID_VIA_ID_HELP
        CMD_FIND_GUID_VIA_ID_HELP_PARAM_0
        CMD_FIND_GUID_VIA_ID_HELP_PARAM_1
        CMD_FIND_HELP
        CMD_FIND_HELP_D
        CMD_FIND_HELP_F
        CMD_FIND_HELP_P
        CMD_FIND_HELP_PARAM_0
        CMD_FIND_HELP_PARAM_1
        CMD_FIND_HELP_R
        CMD_FIND_ID_VIA_COMMANDRFS_FILENAME_HELP
        CMD_FIND_ID_VIA_COMMANDRFS_FILENAME_HELP_PARAM_0
        CMD_FIND_ID_VIA_COMMANDRFS_FILENAME_HELP_PARAM_1
        CMD_FIND_ID_VIA_GUID_HELP
        CMD_FIND_ID_VIA_GUID_HELP_PARAM_0
        CMD_FIND_ID_VIA_LABEL_HELP
        CMD_FIND_ID_VIA_LABEL_HELP_PARAM_0
        CMD_FIND_ID_VIA_LABEL_HELP_PARAM_1
        CMD_FIND_ID_VIA_LABEL_HELP_PARAM_2
        CMD_FIND_LABEL_VIA_ID_HELP
        CMD_FIND_LABEL_VIA_ID_HELP_PARAM_0
        CMD_FIND_LABEL_VIA_ID_HELP_PARAM_1
        CMD_FIX_PERMS_HELP
        CMD_FIX_PERMS_HELP_PARAM_0
        CMD_FIX_PERMS_HELP_PARAM_1
        CMD_FIX_PERMS_HELP_PARAM_2
        CMD_GIVE_HELP
        CMD_GIVE_HELP_A
        CMD_GIVE_HELP_PARAM_0
        CMD_GIVE_HELP_PARAM_1
        CMD_GIVE_HELP_PARAM_2
        CMD_GREP_HELP
        CMD_GREP_HELP_PARAM_0
        CMD_GREP_HELP_PARAM_1
        CMD_INTEGRITY_CHECK_HELP
        CMD_LS_HELP
        CMD_LS_HELP_PARAM_0
        CMD_MKDIR_HELP
        CMD_MKDIR_HELP_PARAM_0
        CMD_MVDIR_HELP
        CMD_MVDIR_HELP_PARAM_0
        CMD_MVDIR_HELP_PARAM_1
        CMD_MVDIR_HELP_PARAM_2
        CMD_MV_HELP
        CMD_MV_HELP_PARAM_0
        CMD_MV_HELP_PARAM_1
        CMD_DB_TABLE_SIZES_HELP
        CMD_DB_TABLE_SIZES_HELP_PARAM_0
        NOT_SUPPORTED_ON_DB
        CMD_SQL_DUMP_HELP
        CMD_SQL_DUMP_HELP_PARAM_0
        CMD_SQL_DUMP_HELP_PARAM_1
        CMD_SQL_DUMP_HELP_PARAM_2
        SQL_DUMP_SAVED_TO
        CMD_PASSWD_HELP
        CMD_PASSWD_HELP_PARAM_0
        CMD_PASSWD_HELP_U
        CMD_PHPINFO_HELP
        CMD_PWD_HELP
        CMD_READ_HELP
        CMD_READ_HELP_PARAM_0
        CMD_RESET_HELP
        CMD_RMDIR_HELP
        CMD_RMDIR_HELP_F
        CMD_RMDIR_HELP_PARAM_0
        CMD_RM_HELP
        CMD_RM_HELP_PARAM_0
        CMD_SEND_CHATMESSAGE_HELP
        CMD_SEND_CHATMESSAGE_HELP_PARAM_0
        CMD_SEND_CHATMESSAGE_HELP_PARAM_1
        CMD_SET_COMMENT_FORUM_HELP
        CMD_SET_COMMENT_FORUM_HELP_PARAM_0
        CMD_SET_COMMENT_FORUM_HELP_PARAM_1
        CMD_SET_COMMENT_FORUM_HELP_PARAM_2
        CMD_THEMEWIZARD_COMPUTE_EQUATION_HELP
        CMD_THEMEWIZARD_COMPUTE_EQUATION_HELP_PARAM_0
        CMD_THEMEWIZARD_COMPUTE_EQUATION_HELP_PARAM_1
        CMD_THEMEWIZARD_FIND_COLOR_HELP
        CMD_THEMEWIZARD_FIND_COLOR_HELP_PARAM_0
        CMD_THEMEWIZARD_FIND_COLOR_HELP_PARAM_1
        CMD_TIME_HELP
        CMD_UNTAR_HELP
        CMD_UNTAR_HELP_PARAM_0
        CMD_USERS_ONLINE_HELP
        CMD_VERSION_HELP
        CMD_VERSION_HELP_F
        CMD_VERSION_HELP_G
        CMD_VERSION_HELP_M
        CMD_VERSION_HELP_T
        CMD_VERSION_HELP_V
        CMD_WATCH_CHATROOM_HELP
        CMD_WATCH_CHATROOM_HELP_PARAM_0
        CMD_WATCH_CHATROOM_HELP_U
        CMD_WHOAMI_HELP
        CMD_WHOIS_HELP
        CMD_WHOIS_HELP_F
        CMD_WHOIS_HELP_M
        CMD_WHOIS_HELP_O
        CMD_WHOIS_HELP_PARAM_0
        CMD_WHOIS_HELP_S
        CMD_WRITE_HELP
        CMD_WRITE_HELP_PARAM_0
        CMD_WRITE_HELP_PARAM_1
        CMS_ZONE
        CNS_CONVERTED_INFO
        CODE_EDITOR
        COLLAPSE_USER_ZONES
        COLUMNS
        COMCODE
        COMCODE_CACHE
        COMCODE_GROUP_CUSTOM
        COMCODE_INLINE
        COMCODE_PAGE
        COMCODE_PAGES
        COMCODE_PAGE_ADD
        COMCODE_PAGE_CACHE
        COMCODE_PAGE_DEFAULT_REVIEW_FREQ
        COMCODE_PAGE_EDIT
        COMCODE_PAGE_EDIT_THIS
        COMCODE_PAGE_MANAGEMENT
        COMCODE_PAGE_OWN_EDIT
        COMCODE_PAGE_VIEW_THIS
        COMCODE_PREVIEW
        COMCODE_REPLACE
        COMCODE_TAG
        COMCODE_TAG_MODIFIER_ADD
        COMCODE_TAG_MODIFIER_CHOICE_TEXT
        COMCODE_TAG_MODIFIER_DELETE
        COMCODE_TAG_MODIFIER_EDIT
        COMCODE_TAG_PARAM_dimensions
        COMCODE_TAG_abbr_DESCRIPTION
        COMCODE_TAG_abbr_EMBED
        COMCODE_TAG_abbr_PARAM_param
        COMCODE_TAG_abbr_PARAM_param_TITLE
        COMCODE_TAG_address_DESCRIPTION
        COMCODE_TAG_address_EMBED
        COMCODE_TAG_align_DESCRIPTION
        COMCODE_TAG_align_EMBED
        COMCODE_TAG_align_PARAM_param
        COMCODE_TAG_align_PARAM_param_TITLE
        COMCODE_TAG_attachment_DESCRIPTION
        COMCODE_TAG_attachment_EMBED
        COMCODE_TAG_attachment_PARAM_click_url
        COMCODE_TAG_attachment_PARAM_click_url_TITLE
        COMCODE_TAG_attachment_PARAM_description
        COMCODE_TAG_attachment_PARAM_description_TITLE
        COMCODE_TAG_attachment_PARAM_filename
        COMCODE_TAG_attachment_PARAM_filename_TITLE
        COMCODE_TAG_attachment_PARAM_filesize
        COMCODE_TAG_attachment_PARAM_filesize_TITLE
        COMCODE_TAG_attachment_PARAM_float
        COMCODE_TAG_attachment_PARAM_float_TITLE
        COMCODE_TAG_attachment_PARAM_framed
        COMCODE_TAG_attachment_PARAM_framed_TITLE
        COMCODE_TAG_attachment_PARAM_height
        COMCODE_TAG_attachment_PARAM_height_TITLE
        COMCODE_TAG_attachment_PARAM_length
        COMCODE_TAG_attachment_PARAM_length_TITLE
        COMCODE_TAG_attachment_PARAM_mime_type
        COMCODE_TAG_attachment_PARAM_mime_type_TITLE
        COMCODE_TAG_attachment_PARAM_thumb
        COMCODE_TAG_attachment_PARAM_thumb_TITLE
        COMCODE_TAG_attachment_PARAM_thumb_url
        COMCODE_TAG_attachment_PARAM_thumb_url_TITLE
        COMCODE_TAG_attachment_PARAM_type
        COMCODE_TAG_attachment_PARAM_type_TITLE
        COMCODE_TAG_attachment_PARAM_width
        COMCODE_TAG_attachment_PARAM_width_TITLE
        COMCODE_TAG_attachment_PARAM_wysiwyg_editable
        COMCODE_TAG_attachment_PARAM_wysiwyg_editable_TITLE
        COMCODE_TAG_attachment_safe
        COMCODE_TAG_attachment_safe_DESCRIPTION
        COMCODE_TAG_attachment_simplified_DESCRIPTION
        COMCODE_TAG_b_DESCRIPTION
        COMCODE_TAG_b_EMBED
        COMCODE_TAG_big_tabs_DESCRIPTION
        COMCODE_TAG_big_tabs_EMBED
        COMCODE_TAG_big_tabs_PARAM_default
        COMCODE_TAG_big_tabs_PARAM_name
        COMCODE_TAG_big_tabs_PARAM_switch_time
        COMCODE_TAG_block_DESCRIPTION
        COMCODE_TAG_block_EMBED
        COMCODE_TAG_box_DESCRIPTION
        COMCODE_TAG_box_EMBED
        COMCODE_TAG_box_PARAM_class
        COMCODE_TAG_box_PARAM_class_TITLE
        COMCODE_TAG_box_PARAM_float
        COMCODE_TAG_box_PARAM_float_TITLE
        COMCODE_TAG_box_PARAM_links
        COMCODE_TAG_box_PARAM_links_TITLE
        COMCODE_TAG_box_PARAM_meta
        COMCODE_TAG_box_PARAM_meta_TITLE
        COMCODE_TAG_box_PARAM_options
        COMCODE_TAG_box_PARAM_options_TITLE
        COMCODE_TAG_box_PARAM_param
        COMCODE_TAG_box_PARAM_param_TITLE
        COMCODE_TAG_box_PARAM_type
        COMCODE_TAG_box_PARAM_type_TITLE
        COMCODE_TAG_box_PARAM_width
        COMCODE_TAG_box_PARAM_width_TITLE
        COMCODE_TAG_carousel_DESCRIPTION
        COMCODE_TAG_carousel_EMBED
        COMCODE_TAG_carousel_PARAM_param
        COMCODE_TAG_carousel_PARAM_param_TITLE
        COMCODE_TAG_center_DESCRIPTION
        COMCODE_TAG_center_EMBED
        COMCODE_TAG_cite_DESCRIPTION
        COMCODE_TAG_cite_EMBED
        COMCODE_TAG_code_DESCRIPTION
        COMCODE_TAG_code_EMBED
        COMCODE_TAG_code_PARAM_numbers
        COMCODE_TAG_code_PARAM_numbers_TITLE
        COMCODE_TAG_code_PARAM_param
        COMCODE_TAG_code_PARAM_param_TITLE
        COMCODE_TAG_code_PARAM_scroll
        COMCODE_TAG_code_PARAM_scroll_TITLE
        COMCODE_TAG_codebox_DESCRIPTION
        COMCODE_TAG_codebox_EMBED
        COMCODE_TAG_codebox_PARAM_numbers
        COMCODE_TAG_codebox_PARAM_numbers_TITLE
        COMCODE_TAG_codebox_PARAM_param
        COMCODE_TAG_codebox_PARAM_param_TITLE
        COMCODE_TAG_color_DESCRIPTION
        COMCODE_TAG_color_EMBED
        COMCODE_TAG_color_PARAM_param
        COMCODE_TAG_color_PARAM_param_TITLE
        COMCODE_TAG_concept_DESCRIPTION
        COMCODE_TAG_concept_EMBED
        COMCODE_TAG_concept_EMBED_TITLE
        COMCODE_TAG_concept_PARAM_param
        COMCODE_TAG_concept_PARAM_param_TITLE
        COMCODE_TAG_concepts_DESCRIPTION
        COMCODE_TAG_concepts_EMBED
        COMCODE_TAG_concepts_EMBED_TITLE
        COMCODE_TAG_concepts_PARAM_x_key
        COMCODE_TAG_concepts_PARAM_x_value
        COMCODE_TAG_contents_DESCRIPTION
        COMCODE_TAG_contents_EMBED
        COMCODE_TAG_contents_EMBED_TITLE
        COMCODE_TAG_contents_PARAM_base
        COMCODE_TAG_contents_PARAM_base_TITLE
        COMCODE_TAG_contents_PARAM_files
        COMCODE_TAG_contents_PARAM_files_TITLE
        COMCODE_TAG_contents_PARAM_levels
        COMCODE_TAG_contents_PARAM_levels_TITLE
        COMCODE_TAG_contents_PARAM_zone
        COMCODE_TAG_contents_PARAM_zone_TITLE
        COMCODE_TAG_currency_DESCRIPTION
        COMCODE_TAG_currency_EMBED
        COMCODE_TAG_currency_PARAM_bracket
        COMCODE_TAG_currency_PARAM_bracket_TITLE
        COMCODE_TAG_currency_PARAM_param
        COMCODE_TAG_currency_PARAM_param_TITLE
        COMCODE_TAG_del_DESCRIPTION
        COMCODE_TAG_del_EMBED
        COMCODE_TAG_del_PARAM_cite
        COMCODE_TAG_del_PARAM_cite_TITLE
        COMCODE_TAG_del_PARAM_datetime
        COMCODE_TAG_del_PARAM_datetime_TITLE
        COMCODE_TAG_dfn_DESCRIPTION
        COMCODE_TAG_dfn_EMBED
        COMCODE_TAG_email_DESCRIPTION
        COMCODE_TAG_email_EMBED
        COMCODE_TAG_email_EMBED_TITLE
        COMCODE_TAG_email_PARAM_body
        COMCODE_TAG_email_PARAM_body_TITLE
        COMCODE_TAG_email_PARAM_param
        COMCODE_TAG_email_PARAM_param_TITLE
        COMCODE_TAG_email_PARAM_subject
        COMCODE_TAG_email_PARAM_subject_TITLE
        COMCODE_TAG_email_PARAM_title
        COMCODE_TAG_email_PARAM_title_TITLE
        COMCODE_TAG_flash_DESCRIPTION
        COMCODE_TAG_flash_EMBED
        COMCODE_TAG_flash_PARAM_param
        COMCODE_TAG_flash_PARAM_param_TITLE
        COMCODE_TAG_font_DESCRIPTION
        COMCODE_TAG_font_EMBED
        COMCODE_TAG_font_PARAM_color
        COMCODE_TAG_font_PARAM_color_TITLE
        COMCODE_TAG_font_PARAM_param
        COMCODE_TAG_font_PARAM_param_TITLE
        COMCODE_TAG_font_PARAM_size
        COMCODE_TAG_font_PARAM_size_TITLE
        COMCODE_TAG_hide_DESCRIPTION
        COMCODE_TAG_hide_EMBED
        COMCODE_TAG_hide_PARAM_param
        COMCODE_TAG_hide_PARAM_param_TITLE
        COMCODE_TAG_highlight_DESCRIPTION
        COMCODE_TAG_highlight_EMBED
        COMCODE_TAG_html_DESCRIPTION
        COMCODE_TAG_html_EMBED
        COMCODE_TAG_i_DESCRIPTION
        COMCODE_TAG_i_EMBED
        COMCODE_TAG_if_in_group_DESCRIPTION
        COMCODE_TAG_if_in_group_EMBED
        COMCODE_TAG_if_in_group_PARAM_param
        COMCODE_TAG_if_in_group_PARAM_param_TITLE
        COMCODE_TAG_if_in_group_PARAM_type
        COMCODE_TAG_if_in_group_PARAM_type_TITLE
        COMCODE_TAG_img_DESCRIPTION
        COMCODE_TAG_img_EMBED
        COMCODE_TAG_img_PARAM_align
        COMCODE_TAG_img_PARAM_align_TITLE
        COMCODE_TAG_img_PARAM_float
        COMCODE_TAG_img_PARAM_float_TITLE
        COMCODE_TAG_img_PARAM_param
        COMCODE_TAG_img_PARAM_param_TITLE
        COMCODE_TAG_img_PARAM_refresh_time
        COMCODE_TAG_img_PARAM_refresh_time_TITLE
        COMCODE_TAG_img_PARAM_rollover
        COMCODE_TAG_img_PARAM_rollover_TITLE
        COMCODE_TAG_img_PARAM_title
        COMCODE_TAG_img_PARAM_title_TITLE
        COMCODE_TAG_include_DESCRIPTION
        COMCODE_TAG_include_EMBED
        COMCODE_TAG_include_EMBED_TITLE
        COMCODE_TAG_include_PARAM_param
        COMCODE_TAG_include_PARAM_param_TITLE
        COMCODE_TAG_indent_DESCRIPTION
        COMCODE_TAG_indent_EMBED
        COMCODE_TAG_indent_PARAM_param
        COMCODE_TAG_indent_PARAM_param_TITLE
        COMCODE_TAG_ins_DESCRIPTION
        COMCODE_TAG_ins_EMBED
        COMCODE_TAG_ins_PARAM_cite
        COMCODE_TAG_ins_PARAM_cite_TITLE
        COMCODE_TAG_ins_PARAM_datetime
        COMCODE_TAG_ins_PARAM_datetime_TITLE
        COMCODE_TAG_jumping_DESCRIPTION
        COMCODE_TAG_jumping_EMBED
        COMCODE_TAG_jumping_PARAM_string
        COMCODE_TAG_left_DESCRIPTION
        COMCODE_TAG_left_EMBED
        COMCODE_TAG_list_DESCRIPTION
        COMCODE_TAG_list_EMBED
        COMCODE_TAG_list_PARAM_param
        COMCODE_TAG_list_PARAM_param_TITLE
        COMCODE_TAG_media_DESCRIPTION
        COMCODE_TAG_media_EMBED
        COMCODE_TAG_media_EMBED__library
        COMCODE_TAG_media_EMBED__url
        COMCODE_TAG_media_NAME_OF_EMBED__library
        COMCODE_TAG_media_NAME_OF_EMBED__url
        COMCODE_TAG_media_PARAM_click_url
        COMCODE_TAG_media_PARAM_click_url_TITLE
        COMCODE_TAG_media_PARAM_description
        COMCODE_TAG_media_PARAM_description_TITLE
        COMCODE_TAG_media_PARAM_filename
        COMCODE_TAG_media_PARAM_filename_TITLE
        COMCODE_TAG_media_PARAM_filesize
        COMCODE_TAG_media_PARAM_filesize_TITLE
        COMCODE_TAG_media_PARAM_float
        COMCODE_TAG_media_PARAM_float_TITLE
        COMCODE_TAG_media_PARAM_framed
        COMCODE_TAG_media_PARAM_framed_TITLE
        COMCODE_TAG_media_PARAM_height
        COMCODE_TAG_media_PARAM_height_TITLE
        COMCODE_TAG_media_PARAM_length
        COMCODE_TAG_media_PARAM_length_TITLE
        COMCODE_TAG_media_PARAM_mime_type
        COMCODE_TAG_media_PARAM_mime_type_TITLE
        COMCODE_TAG_media_PARAM_thumb
        COMCODE_TAG_media_PARAM_thumb_TITLE
        COMCODE_TAG_media_PARAM_thumb_url
        COMCODE_TAG_media_PARAM_thumb_url_TITLE
        COMCODE_TAG_media_PARAM_thumb_url__library
        COMCODE_TAG_media_PARAM_thumb_url__library_TITLE
        COMCODE_TAG_media_PARAM_thumb_url__url
        COMCODE_TAG_media_PARAM_thumb_url__url_TITLE
        COMCODE_TAG_media_PARAM_type
        COMCODE_TAG_media_PARAM_type_TITLE
        DEV_MODE_ON
        COMCODE_TAG_media_PARAM_width
        COMCODE_TAG_media_PARAM_width_TITLE
        COMCODE_TAG_media_PARAM_wysiwyg_editable
        COMCODE_TAG_media_PARAM_wysiwyg_editable_TITLE
        COMCODE_TAG_media_set_DESCRIPTION
        COMCODE_TAG_media_set_EMBED
        COMCODE_TAG_media_set_PARAM_height
        COMCODE_TAG_media_set_PARAM_height_TITLE
        COMCODE_TAG_media_set_PARAM_width
        COMCODE_TAG_media_set_PARAM_width_TITLE
        COMCODE_TAG_menu_DESCRIPTION
        COMCODE_TAG_menu_EMBED
        COMCODE_TAG_menu_PARAM_param
        COMCODE_TAG_menu_PARAM_param_TITLE
        COMCODE_TAG_menu_PARAM_type
        COMCODE_TAG_menu_PARAM_type_TITLE
        COMCODE_TAG_no_parse_DESCRIPTION
        COMCODE_TAG_no_parse_EMBED
        COMCODE_TAG_overlay_DESCRIPTION
        COMCODE_TAG_overlay_EMBED
        COMCODE_TAG_overlay_PARAM_height
        COMCODE_TAG_overlay_PARAM_height_TITLE
        COMCODE_TAG_overlay_PARAM_param
        COMCODE_TAG_overlay_PARAM_param_TITLE
        COMCODE_TAG_overlay_PARAM_timein
        COMCODE_TAG_overlay_PARAM_timein_TITLE
        COMCODE_TAG_overlay_PARAM_timeout
        COMCODE_TAG_overlay_PARAM_timeout_TITLE
        COMCODE_TAG_overlay_PARAM_width
        COMCODE_TAG_overlay_PARAM_width_TITLE
        COMCODE_TAG_overlay_PARAM_x
        COMCODE_TAG_overlay_PARAM_x_TITLE
        COMCODE_TAG_overlay_PARAM_y
        COMCODE_TAG_overlay_PARAM_y_TITLE
        COMCODE_TAG_page_DESCRIPTION
        COMCODE_TAG_page_EMBED
        COMCODE_TAG_page_EMBED_TITLE
        COMCODE_TAG_page_PARAM_external
        COMCODE_TAG_page_PARAM_external_TITLE
        COMCODE_TAG_page_PARAM_param
        COMCODE_TAG_page_PARAM_param_TITLE
        COMCODE_TAG_post_DESCRIPTION
        COMCODE_TAG_post_EMBED
        COMCODE_TAG_post_EMBED_TITLE
        COMCODE_TAG_post_PARAM_forum
        COMCODE_TAG_post_PARAM_forum_TITLE
        COMCODE_TAG_post_PARAM_param
        COMCODE_TAG_post_PARAM_param_TITLE
        COMCODE_TAG_pulse_DESCRIPTION
        COMCODE_TAG_pulse_EMBED
        COMCODE_TAG_pulse_PARAM_max
        COMCODE_TAG_pulse_PARAM_max_TITLE
        COMCODE_TAG_pulse_PARAM_min
        COMCODE_TAG_pulse_PARAM_min_TITLE
        COMCODE_TAG_pulse_PARAM_param
        COMCODE_TAG_pulse_PARAM_param_TITLE
        COMCODE_TAG_q_DESCRIPTION
        COMCODE_TAG_q_EMBED
        COMCODE_TAG_quote_DESCRIPTION
        COMCODE_TAG_quote_EMBED
        COMCODE_TAG_quote_PARAM_cite
        COMCODE_TAG_quote_PARAM_cite_TITLE
        COMCODE_TAG_quote_PARAM_param
        COMCODE_TAG_quote_PARAM_param_TITLE
        COMCODE_TAG_quote_PARAM_saidless
        COMCODE_TAG_quote_PARAM_saidless_TITLE
        COMCODE_TAG_random_DESCRIPTION
        COMCODE_TAG_random_EMBED
        COMCODE_TAG_random_EMBED_TITLE
        COMCODE_TAG_random_PARAM_X
        COMCODE_TAG_random_PARAM_X_TITLE
        COMCODE_TAG_random_PARAM_string
        COMCODE_TAG_random_PARAM_string_TITLE
        COMCODE_TAG_reference_DESCRIPTION
        COMCODE_TAG_reference_EMBED
        COMCODE_TAG_reference_EMBED_TITLE
        COMCODE_TAG_reference_PARAM_param
        COMCODE_TAG_reference_PARAM_param_TITLE
        COMCODE_TAG_reference_PARAM_type
        COMCODE_TAG_reference_PARAM_type_TITLE
        COMCODE_TAG_right_DESCRIPTION
        COMCODE_TAG_right_EMBED
        COMCODE_TAG_s_DESCRIPTION
        COMCODE_TAG_s_EMBED
        COMCODE_TAG_samp_DESCRIPTION
        COMCODE_TAG_samp_EMBED
        COMCODE_TAG_sections_DESCRIPTION
        COMCODE_TAG_sections_EMBED
        COMCODE_TAG_sections_PARAM_default
        COMCODE_TAG_sections_PARAM_name
        COMCODE_TAG_semihtml_DESCRIPTION
        COMCODE_TAG_semihtml_EMBED
        COMCODE_TAG_shocker_DESCRIPTION
        COMCODE_TAG_shocker_EMBED
        COMCODE_TAG_shocker_PARAM_left
        COMCODE_TAG_shocker_PARAM_left_TITLE
        COMCODE_TAG_shocker_PARAM_max
        COMCODE_TAG_shocker_PARAM_max_TITLE
        COMCODE_TAG_shocker_PARAM_min
        COMCODE_TAG_shocker_PARAM_min_TITLE
        COMCODE_TAG_shocker_PARAM_right
        COMCODE_TAG_shocker_PARAM_right_TITLE
        COMCODE_TAG_size_DESCRIPTION
        COMCODE_TAG_size_EMBED
        COMCODE_TAG_size_PARAM_param
        COMCODE_TAG_size_PARAM_param_TITLE
        COMCODE_TAG_snapback_DESCRIPTION
        COMCODE_TAG_snapback_EMBED
        COMCODE_TAG_snapback_EMBED_TITLE
        COMCODE_TAG_snapback_PARAM_forum
        COMCODE_TAG_snapback_PARAM_forum_TITLE
        COMCODE_TAG_snapback_PARAM_param
        COMCODE_TAG_snapback_PARAM_param_TITLE
        COMCODE_TAG_staff_note_DESCRIPTION
        COMCODE_TAG_staff_note_EMBED
        COMCODE_TAG_sub_DESCRIPTION
        COMCODE_TAG_sub_EMBED
        COMCODE_TAG_sup_DESCRIPTION
        COMCODE_TAG_sup_EMBED
        COMCODE_TAG_surround_DESCRIPTION
        COMCODE_TAG_surround_EMBED
        COMCODE_TAG_surround_PARAM_param
        COMCODE_TAG_surround_PARAM_param_TITLE
        COMCODE_TAG_surround_PARAM_style
        COMCODE_TAG_surround_PARAM_style_TITLE
        COMCODE_TAG_tabs_DESCRIPTION
        COMCODE_TAG_tabs_EMBED
        COMCODE_TAG_tabs_PARAM_default
        COMCODE_TAG_tabs_PARAM_name
        COMCODE_TAG_thumb_DESCRIPTION
        COMCODE_TAG_thumb_EMBED
        COMCODE_TAG_thumb_PARAM_align
        COMCODE_TAG_thumb_PARAM_align_TITLE
        COMCODE_TAG_thumb_PARAM_caption
        COMCODE_TAG_thumb_PARAM_caption_TITLE
        COMCODE_TAG_thumb_PARAM_float
        COMCODE_TAG_thumb_PARAM_float_TITLE
        COMCODE_TAG_thumb_PARAM_param
        COMCODE_TAG_thumb_PARAM_param_TITLE
        COMCODE_TAG_ticker_DESCRIPTION
        COMCODE_TAG_ticker_EMBED
        COMCODE_TAG_ticker_PARAM_param
        COMCODE_TAG_ticker_PARAM_param_TITLE
        COMCODE_TAG_ticker_PARAM_speed
        COMCODE_TAG_ticker_PARAM_speed_TITLE
        COMCODE_TAG_title_DESCRIPTION
        COMCODE_TAG_title_EMBED
        COMCODE_TAG_title_PARAM_base
        COMCODE_TAG_title_PARAM_base_TITLE
        COMCODE_TAG_title_PARAM_number
        COMCODE_TAG_title_PARAM_number_TITLE
        COMCODE_TAG_title_PARAM_param
        COMCODE_TAG_title_PARAM_param_TITLE
        COMCODE_TAG_title_PARAM_sub
        COMCODE_TAG_title_PARAM_sub_TITLE
        COMCODE_TAG_tooltip_DESCRIPTION
        COMCODE_TAG_tooltip_EMBED
        COMCODE_TAG_tooltip_PARAM_param
        COMCODE_TAG_tooltip_PARAM_param_TITLE
        COMCODE_TAG_topic_DESCRIPTION
        COMCODE_TAG_topic_EMBED
        COMCODE_TAG_topic_EMBED_TITLE
        COMCODE_TAG_topic_PARAM_forum
        COMCODE_TAG_topic_PARAM_forum_TITLE
        COMCODE_TAG_topic_PARAM_param
        COMCODE_TAG_topic_PARAM_param_TITLE
        COMCODE_TAG_tt_DESCRIPTION
        COMCODE_TAG_tt_EMBED
        COMCODE_TAG_u_DESCRIPTION
        COMCODE_TAG_u_EMBED
        COMCODE_TAG_url_DESCRIPTION
        COMCODE_TAG_url_EMBED
        COMCODE_TAG_url_EMBED_TITLE
        COMCODE_TAG_url_PARAM_param
        COMCODE_TAG_url_PARAM_param_TITLE
        COMCODE_TAG_url_PARAM_rel
        COMCODE_TAG_url_PARAM_rel_TITLE
        COMCODE_TAG_url_PARAM_target
        COMCODE_TAG_url_PARAM_target_TITLE
        COMCODE_TAG_url_PARAM_title
        COMCODE_TAG_url_PARAM_title_TITLE
        COMCODE_TAG_var_DESCRIPTION
        COMCODE_TAG_var_EMBED
        COMMANDR
        HAVE_BLOG
        COMMANDR_BUTTON
        COMMANDR_CHAT_ANNOUNCE
        COMMANDR_DESCRIPTIVE_TITLE
        COMMAND_PROMPT
        COMMENT
        COMMENTS_FORUM_NAME
        COMMENTS_TO_SHOW_IN_THREAD
        COMMENT_FORM_TEXT
        COMMENT_TOPIC_SUBJECT
        COMMON_CSS_PROPERTIES
        COMPANY_NAME
        COMPLETED
        COMPLETION_PERCENTAGE
        COMPLEX_FIRST_PARAMETER
        COMPLEX_PRIVACY_OPTIONS
        COMPLEX_UPLOADER
        COMPOSR_SETTINGS
        COMPOSR_SITE_MERGER
        COMPRESSING
        CONCEDED_MODE_LINK
        CONFIGURATION
        CONFIGURE
        CONFIG_CATEGORY_ACCESSIBILITY
        CONFIG_CATEGORY_ADMIN
        CONFIG_CATEGORY_BLOCKS
        CONFIG_CATEGORY_COMPOSR_APIS
        CONFIG_CATEGORY_DESCRIPTION__ACCESSIBILITY
        CONFIG_CATEGORY_DESCRIPTION__ADMIN
        CONFIG_CATEGORY_DESCRIPTION__BLOCKS
        CONFIG_CATEGORY_DESCRIPTION__COMPOSR_APIS
        CONFIG_CATEGORY_DESCRIPTION__ECOMMERCE
        CONFIG_CATEGORY_DESCRIPTION__FEATURE
        CONFIG_CATEGORY_DESCRIPTION__FORUMS
        CONFIG_CATEGORY_DESCRIPTION__GALLERY
        CONFIG_CATEGORY_DESCRIPTION__PERFORMANCE
        CONFIG_CATEGORY_DESCRIPTION__POINTS
        CONFIG_CATEGORY_DESCRIPTION__POINTSTORE
        CONFIG_CATEGORY_DESCRIPTION__PRIVACY
        CONFIG_CATEGORY_DESCRIPTION__SECURITY
        CONFIG_CATEGORY_DESCRIPTION__SERVER
        CONFIG_CATEGORY_DESCRIPTION__SITE
        CONFIG_CATEGORY_DESCRIPTION__THEME
        CONFIG_CATEGORY_DESCRIPTION__USERS
        CONFIG_CATEGORY_ECOMMERCE
        CONFIG_CATEGORY_FEATURE
        CONFIG_CATEGORY_FORUMS
        CONFIG_CATEGORY_GALLERY
        CONFIG_CATEGORY_PERFORMANCE
        CONFIG_CATEGORY_POINTS
        CONFIG_CATEGORY_POINTSTORE
        CONFIG_CATEGORY_PRIVACY
        CONFIG_CATEGORY_SECURITY
        CONFIG_CATEGORY_SERVER
        CONFIG_CATEGORY_SITE
        CONFIG_CATEGORY_THEME
        CONFIG_CATEGORY_USERS
        CONFIG_GROUP_DEFAULT_DESCRIP_PERSONAL_BLOCK
        CONFIG_GROUP_DEFAULT_DESCRIP_STATISTICS
        CONFIG_GROUP_DEFAULT_DESCRIP_USERS_ONLINE_BLOCK
        CONFIG_GROUP_DESCRIP_CHECK_LIST
        CONFIG_GROUP_DESCRIP_CONTACT_US_MESSAGING
        CONFIG_GROUP_DESCRIP_IMAP
        CONFIG_GROUP_DESCRIP_LDAP
        CONFIG_GROUP_DESCRIP_PERSONAL_BLOCK
        CONFIG_GROUP_DESCRIP_SPECIAL_CPF_TO_SHOW
        CONFIG_GROUP_DESCRIP_STATISTICS
        CONFIG_GROUP_DESCRIP_SUPPORT_TICKETS
        CONFIG_GROUP_DESCRIP_TRANSCODING
        CONFIG_GROUP_DESCRIP_UPLOAD
        CONFIG_OPTION_activity_show_stats_count_users_online
        CONFIG_OPTION_activity_show_stats_count_users_online_forum
        CONFIG_OPTION_activity_show_stats_count_users_online_record
        CONFIG_OPTION_admin_banners
        CONFIG_OPTION_allow_alpha_search
        CONFIG_OPTION_allow_audio_videos
        CONFIG_OPTION_allow_auto_notifications
        CONFIG_OPTION_allow_email_disable
        CONFIG_OPTION_allow_email_from_staff_disable
        CONFIG_OPTION_allow_ext_images
        CONFIG_OPTION_allow_international
        CONFIG_OPTION_allow_opting_out_of_tax
        CONFIG_OPTION_allow_own_rate
        CONFIG_OPTION_allow_theme_image_selector
        CONFIG_OPTION_allowed_post_submitters
        CONFIG_OPTION_anti_leech
        CONFIG_OPTION_attachment_cleanup
        CONFIG_OPTION_attachment_default_height
        CONFIG_OPTION_attachment_default_width
        CONFIG_OPTION_audio_bitrate
        CONFIG_OPTION_audio_captcha
        CONFIG_OPTION_auto_submit_sitemap
        CONFIG_OPTION_autoban
        CONFIG_OPTION_automatic_meta_extraction
        CONFIG_OPTION_average_gamble_multiplier
        CONFIG_OPTION_awarded_items_per_page
        CONFIG_OPTION_backup_overwrite
        CONFIG_OPTION_backup_server_hostname
        CONFIG_OPTION_backup_server_password
        CONFIG_OPTION_backup_server_path
        CONFIG_OPTION_backup_server_port
        CONFIG_OPTION_backup_server_user
        CONFIG_OPTION_backup_time
        CONFIG_OPTION_banner_autosize
        CONFIG_OPTION_banner_hit
        CONFIG_OPTION_banner_imp
        CONFIG_OPTION_banner_setup
        CONFIG_OPTION_bcc
        CONFIG_OPTION_blog_update_time
        CONFIG_OPTION_bot_stats
        CONFIG_OPTION_bottom_show_commandr_button
        CONFIG_OPTION_bottom_show_feedback_link
        CONFIG_OPTION_bottom_show_privacy_link
        CONFIG_OPTION_bottom_show_realtime_rain_button
        CONFIG_OPTION_bottom_show_rules_link
        CONFIG_OPTION_bottom_show_sitemap_button
        CONFIG_OPTION_bottom_show_top_button
        CONFIG_OPTION_breadcrumb_crop_length
        CONFIG_OPTION_brute_force_instant_ban
        CONFIG_OPTION_brute_force_login_minutes
        CONFIG_OPTION_brute_force_threshold
        CONFIG_OPTION_call_home
        CONFIG_OPTION_callback_password
        CONFIG_OPTION_captcha_noise
        CONFIG_OPTION_captcha_on_feedback
        CONFIG_OPTION_captcha_single_guess
        CONFIG_OPTION_cart_hold_hours
        CONFIG_OPTION_catalogue_entries_per_page
        CONFIG_OPTION_catalogue_subcats_per_page
        CONFIG_OPTION_catalogues_subcat_narrowin
        CONFIG_OPTION_cc_address
        CONFIG_OPTION_cdn
        CONFIG_OPTION_chat_default_post_colour
        CONFIG_OPTION_chat_default_post_font
        CONFIG_OPTION_chat_flood_timelimit
        CONFIG_OPTION_chat_max_messages_to_show
        CONFIG_OPTION_chat_message_check_interval
        CONFIG_OPTION_chat_private_room_deletion_time
        CONFIG_OPTION_chat_transitory_alert_time
        CONFIG_OPTION_check_broken_urls
        CONFIG_OPTION_cleanup_files
        DELETE_POSTS_DESCRIPTION
        DELETE_POSTS_FROM_IP
        DELETE_POSTS_FROM_IP_DESCRIPTION
        CONFIG_OPTION_closed
        CONFIG_OPTION_club_forum_parent_forum
        CONFIG_OPTION_club_forum_parent_forum_grouping
        CONFIG_OPTION_cms_show_personal_last_visit
        CONFIG_OPTION_collapse_user_zones
        CONFIG_OPTION_comcode_page_default_review_freq
        CONFIG_OPTION_commandr_chat_announce
        CONFIG_OPTION_comment_text
        CONFIG_OPTION_comment_topic_subject
        CONFIG_OPTION_comments_forum_name
        CONFIG_OPTION_comments_to_show_in_thread
        CONFIG_OPTION_complex_privacy_options
        CONFIG_OPTION_complex_uploader
        CONFIG_OPTION_cookie_notice
        CONFIG_OPTION_copyright
        CONFIG_OPTION_cpf_enable_city
        CONFIG_OPTION_cpf_enable_country
        CONFIG_OPTION_cpf_enable_county
        CONFIG_OPTION_cpf_enable_name
        CONFIG_OPTION_cpf_enable_phone
        CONFIG_OPTION_cpf_enable_post_code
        CONFIG_OPTION_cpf_enable_state
        CONFIG_OPTION_cpf_enable_street_address
        CONFIG_OPTION_crypt_ratchet
        CONFIG_OPTION_csrf_token_expire_fresh
        CONFIG_OPTION_csrf_token_expire_new
        CONFIG_OPTION_css_captcha
        CURRENCY_API_KEY
        CONFIG_OPTION_currency_api_key
        CONFIG_OPTION_currency
        CONFIG_OPTION_decryption_key
        CONFIG_OPTION_deeper_admin_breadcrumbs
        CONFIG_OPTION_default_comment_sort_order
        CONFIG_OPTION_default_comment_sort_order_VALUE_newest
        CONFIG_OPTION_default_comment_sort_order_VALUE_oldest
        CONFIG_OPTION_default_comment_sort_order_VALUE_relevance
        CONFIG_OPTION_default_lang
        CONFIG_OPTION_default_preview_guests
        CONFIG_OPTION_default_video_height
        CONFIG_OPTION_default_video_width
        CONFIG_OPTION_delete_time_limit
        CONFIG_OPTION_delete_trashed_pts
        CONFIG_OPTION_description
        CONFIG_OPTION_detect_javascript
        CONFIG_OPTION_detect_lang_browser
        CONFIG_OPTION_detect_lang_forum
        CONFIG_OPTION_display_name_generator
        CONFIG_OPTION_display_php_errors
        CONFIG_OPTION_dload_search_index
        CONFIG_OPTION_dobs
        CONFIG_OPTION_download_entries_per_page
        CONFIG_OPTION_download_gallery_root
        CONFIG_OPTION_download_subcats_per_page
        CONFIG_OPTION_downloads_default_sort_order
        CONFIG_OPTION_downloads_default_sort_order_VALUE_add_date__ASC
        CONFIG_OPTION_downloads_default_sort_order_VALUE_add_date__DESC
        CONFIG_OPTION_downloads_default_sort_order_VALUE_average_rating__DESC
        CONFIG_OPTION_downloads_default_sort_order_VALUE_compound_rating__DESC
        CONFIG_OPTION_downloads_default_sort_order_VALUE_file_size__ASC
        CONFIG_OPTION_downloads_default_sort_order_VALUE_file_size__DESC
        CONFIG_OPTION_downloads_default_sort_order_VALUE_fixed_random__ASC
        CONFIG_OPTION_downloads_default_sort_order_VALUE_num_downloads__DESC
        CONFIG_OPTION_downloads_default_sort_order_VALUE_title__ASC
        CONFIG_OPTION_downloads_subcat_narrowin
        CONFIG_OPTION_dual_format_newsletters
        CONFIG_OPTION_dynamic_firewall
        CONFIG_OPTION_eager_wysiwyg
        CONFIG_OPTION_ecommerce_test_mode
        CONFIG_OPTION_edit_time_limit
        CONFIG_OPTION_edit_under
        CONFIG_OPTION_editarea
        CONFIG_OPTION_email_confirm_join
        CONFIG_OPTION_enable_add_topic_btn_in_topic
        CONFIG_OPTION_enable_animations
        CONFIG_OPTION_enable_birthdays
        CONFIG_OPTION_enable_boolean_search
        CONFIG_OPTION_enable_csv_recommend
        CONFIG_OPTION_enable_ecards
        CONFIG_OPTION_enable_feedback
        CONFIG_OPTION_enable_forum_dupe_buttons
        CONFIG_OPTION_enable_highlight_name
        CONFIG_OPTION_enable_keyword_density_check
        CONFIG_OPTION_enable_mark_forum_read
        CONFIG_OPTION_enable_mark_topic_unread
        CONFIG_OPTION_enable_markup_webstandards
        CONFIG_OPTION_enable_multi_quote
        CONFIG_OPTION_enable_post_emphasis
        CONFIG_OPTION_enable_previews
        CONFIG_OPTION_enable_privacy_tab
        CONFIG_OPTION_enable_pt_filtering
        CONFIG_OPTION_enable_pt_restrict
        CONFIG_OPTION_enable_secondary_news
        CONFIG_OPTION_enable_seo_fields
        CONFIG_OPTION_enable_seo_fields_VALUE_no
        CONFIG_OPTION_enable_seo_fields_VALUE_only_on_edit
        CONFIG_OPTION_enable_seo_fields_VALUE_yes
        CONFIG_OPTION_enable_skip_sig
        CONFIG_OPTION_enable_spell_check
        CONFIG_OPTION_enable_staff_notes
        CONFIG_OPTION_enable_sunk
        CONFIG_OPTION_enable_theme_img_buttons
        CONFIG_OPTION_enable_user_online_groups
        CONFIG_OPTION_enable_views_sigs_option
        CONFIG_OPTION_encryption_key
        CONFIG_OPTION_enveloper_override
        CONFIG_OPTION_ffmpeg_path
        CONFIG_OPTION_filetype_icons
        CONFIG_OPTION_filter_regions
        CONFIG_OPTION_finish_profile
        CONFIG_OPTION_fixed_width
        CONFIG_OPTION_force_guest_names
        CONFIG_OPTION_force_local_temp_dir
        CONFIG_OPTION_force_meta_refresh
        CONFIG_OPTION_forced_preview_option
        CONFIG_OPTION_forum_in_portal
        CONFIG_OPTION_forum_in_portal__cns
        CONFIG_OPTION_forum_posts_per_page
        CONFIG_OPTION_forum_topics_per_page
        CONFIG_OPTION_forw_url
        CONFIG_OPTION_fractional_editing
        CONFIG_OPTION_galleries_default_sort_order
        CONFIG_OPTION_galleries_default_sort_order_VALUE_add_date__ASC
        CONFIG_OPTION_galleries_default_sort_order_VALUE_add_date__DESC
        CONFIG_OPTION_galleries_default_sort_order_VALUE_average_rating__DESC
        CONFIG_OPTION_galleries_default_sort_order_VALUE_compound_rating__DESC
        CONFIG_OPTION_galleries_default_sort_order_VALUE_fixed_random__ASC
        CONFIG_OPTION_galleries_default_sort_order_VALUE_url__ASC
        CONFIG_OPTION_galleries_default_sort_order_VALUE_url__DESC
        CONFIG_OPTION_galleries_subcat_narrowin
        CONFIG_OPTION_gallery_entries_flow_per_page
        CONFIG_OPTION_gallery_entries_regular_per_page
        CONFIG_OPTION_gallery_feedback_fields
        CONFIG_OPTION_gallery_media_title_required
        CONFIG_OPTION_gallery_member_synced
        CONFIG_OPTION_gallery_mode_is
        CONFIG_OPTION_gallery_mode_is_VALUE_choice
        CONFIG_OPTION_gallery_mode_is_VALUE_flow
        CONFIG_OPTION_gallery_mode_is_VALUE_regular
        CONFIG_OPTION_gallery_name_order
        CONFIG_OPTION_gallery_permissions
        CONFIG_OPTION_gallery_rep_image
        CONFIG_OPTION_gallery_selectors
        CONFIG_OPTION_gallery_watermarks
        CONFIG_OPTION_general_safety_listing_limit
        CONFIG_OPTION_gift_reward_amount
        CONFIG_OPTION_gift_reward_chance
        CONFIG_OPTION_global_donext_icons
        CONFIG_OPTION_google_analytics
        CONFIG_OPTION_google_geocode_api_key
        CONFIG_OPTION_google_translate_api_key
        CONFIG_OPTION_group_private_chatrooms
        CONFIG_OPTION_gzip_output
        CONFIG_OPTION_hack_ban_threshold
        CONFIG_OPTION_has_low_memory_limit
        CONFIG_OPTION_header_menu_call_string
        CONFIG_OPTION_highlight_name
        CONFIG_OPTION_honeypot_phrase
        CONFIG_OPTION_honeypot_url
        CONFIG_OPTION_hot_topic_definition
        CONFIG_OPTION_httpauth_is_enabled
        CONFIG_OPTION_imap_folder
        CONFIG_OPTION_imap_host
        CONFIG_OPTION_imap_password
        CONFIG_OPTION_imap_port
        CONFIG_OPTION_imap_username
        CONFIG_OPTION_immediate_downloads
        CONFIG_OPTION_implied_spammer_confidence
        CONFIG_OPTION_important_groups_per_page
        CONFIG_OPTION_infinite_scrolling
        CONFIG_OPTION_initial_banner_hits
        CONFIG_OPTION_initial_quota
        CONFIG_OPTION_inline_pp_advertise
        CONFIG_OPTION_interest_levels
        CONFIG_OPTION_intro_forum_id
        CONFIG_OPTION_invites_per_day
        CONFIG_OPTION_ip_forwarding
        CONFIG_OPTION_ip_strict_for_sessions
        CONFIG_OPTION_ipn
        CONFIG_OPTION_ipn_digest
        CONFIG_OPTION_ipn_password
        CONFIG_OPTION_ipn_test
        CONFIG_OPTION_is_on_anonymous_posts
        CONFIG_OPTION_is_on_automatic_mark_topic_read
        CONFIG_OPTION_is_on_banner_buy
        CONFIG_OPTION_is_on_block_cache
        CONFIG_OPTION_is_on_comcode_page_cache
        CONFIG_OPTION_is_on_comcode_page_children
        CONFIG_OPTION_is_on_comments
        CONFIG_OPTION_is_on_coppa
        CNS_WAITING_CONFIRM_MAIL_COPPA
        COPPA_JOIN_SUBJECT
        COPPA_MAIL
        PREPARE_EMAIL
        PREPARE_EMAIL_DESCRIPTION
        PREPARE_EMAIL_SUBJECT
        PREPARE_EMAIL_BODY
        HANDLE_POST__POST_STANDALONE_AT_END
        HANDLE_POST__POST_STANDALONE_IN_MIDDLE
        HANDLE_POST__POST_AS_TOPIC_FULL
        HANDLE_POST__POST_AS_TOPIC_STARTER
        HANDLE_POST__NOTHING
        HANDLE_POST__DELETE_POST
        HANDLE_POST__DELETE_POST_WITH_GAP
        HANDLE_POST__DELETE_POST_AND_FOLLOWING
        HANDLE_POST__DELETE_TOPIC
        HANDLE_POST__DELETE_TOPIC_WITH_REPLIES
        CONFIG_OPTION_is_on_emoticon_choosers
        CONFIG_OPTION_is_on_folder_create
        CONFIG_OPTION_is_on_forw_buy
        CONFIG_OPTION_is_on_gambling_buy
        CONFIG_OPTION_is_on_highlight_name_buy
        CONFIG_OPTION_is_on_invisibility
        CONFIG_OPTION_is_on_invites
        CONFIG_OPTION_is_on_lang_cache
        CONFIG_OPTION_is_on_pop3_buy
        CONFIG_OPTION_is_on_post_map
        CONFIG_OPTION_is_on_post_titles
        CONFIG_OPTION_is_on_rating
        CONFIG_OPTION_is_on_rss
        CONFIG_OPTION_is_on_show_online
        CONFIG_OPTION_is_on_staff_filter
        CONFIG_OPTION_is_on_strong_forum_tie
        CONFIG_OPTION_is_on_sync_staff
        CONFIG_OPTION_is_on_template_cache
        CONFIG_OPTION_is_on_timezone_detection
        CONFIG_OPTION_is_on_topic_descriptions
        CONFIG_OPTION_is_on_topic_emoticons
        CONFIG_OPTION_is_on_topic_pin_buy
        CONFIG_OPTION_is_on_trackbacks
        CONFIG_OPTION_is_rss_advertised
        CONFIG_OPTION_jpeg_quality
        CONFIG_OPTION_js_captcha
        CONFIG_OPTION_js_overlays
        CONFIG_OPTION_keyset_pagination
        CONFIG_OPTION_keywords
        CONFIG_OPTION_lax_comcode
        CONFIG_OPTION_ldap_allow_joining
        CONFIG_OPTION_ldap_base_dn
        CONFIG_OPTION_ldap_bind_password
        CONFIG_OPTION_ldap_bind_rdn
        CONFIG_OPTION_ldap_group_class
        CONFIG_OPTION_ldap_group_search_qualifier
        CONFIG_OPTION_ldap_hostname
        CONFIG_OPTION_ldap_is_enabled
        CONFIG_OPTION_ldap_is_windows
        CONFIG_OPTION_ldap_login_qualifier
        CONFIG_OPTION_ldap_member_class
        CONFIG_OPTION_ldap_member_property
        CONFIG_OPTION_ldap_member_search_qualifier
        CONFIG_OPTION_ldap_none_bind_logins
        CONFIG_OPTION_ldap_version
        CONFIG_OPTION_leader_board_show_staff
        CONFIG_OPTION_leader_board_size
        CONFIG_OPTION_leader_board_start_date
        CONFIG_OPTION_likes
        CONFIG_OPTION_log_php_errors
        CONFIG_OPTION_logo_map
        CONFIG_OPTION_long_google_cookies
        CONFIG_OPTION_low_space_check
        CONFIG_OPTION_mail_queue
        CONFIG_OPTION_mail_queue_debug
        CONFIG_OPTION_mail_server
        CONFIG_OPTION_mails_per_send
        CONFIG_OPTION_main_forum_name
        CONFIG_OPTION_manual_gallery_codename
        CONFIG_OPTION_manual_gallery_media_types
        CONFIG_OPTION_manual_gallery_parent
        CONFIG_OPTION_manual_subscription_expiry_notice
        CONFIG_OPTION_max_chat_lobby_friends
        CONFIG_OPTION_max_download_size
        CONFIG_OPTION_max_forum_detail
        CONFIG_OPTION_max_forum_inspect
        CONFIG_OPTION_max_ip_addresses_per_subscriber
        CONFIG_OPTION_max_member_title_length
        CONFIG_OPTION_max_moniker_length
        CONFIG_OPTION_max_newsletter_whatsnew
        CONFIG_OPTION_max_personal_gallery_images_high
        CONFIG_OPTION_max_personal_gallery_images_low
        CONFIG_OPTION_max_personal_gallery_videos_high
        CONFIG_OPTION_max_personal_gallery_videos_low
        CONFIG_OPTION_max_quota
        CONFIG_OPTION_max_thread_depth
        CONFIG_OPTION_maximum_autocomplete_suggestions
        CONFIG_OPTION_maximum_download
        CONFIG_OPTION_maximum_gamble_amount
        CONFIG_OPTION_maximum_gamble_multiplier
        CONFIG_OPTION_maximum_image_size
        CONFIG_OPTION_maximum_password_length
        CONFIG_OPTION_maximum_username_length
        CONFIG_OPTION_maximum_users
        CONFIG_OPTION_md_default_sort_order
        CONFIG_OPTION_md_default_sort_order_VALUE_m_cache_num_posts__DESC
        CONFIG_OPTION_md_default_sort_order_VALUE_m_join_time__ASC
        CONFIG_OPTION_md_default_sort_order_VALUE_m_join_time__DESC
        CONFIG_OPTION_md_default_sort_order_VALUE_m_last_visit_time__DESC
        CONFIG_OPTION_md_default_sort_order_VALUE_m_username__ASC
        CONFIG_OPTION_members_per_page
        CONFIG_OPTION_message_received_emails
        CONFIG_OPTION_messaging_forum_name
        CONFIG_OPTION_minimum_autocomplete_length
        CONFIG_OPTION_minimum_autocomplete_past_search
        CONFIG_OPTION_minimum_gamble_amount
        CONFIG_OPTION_minimum_password_length
        CONFIG_OPTION_minimum_password_strength
        CONFIG_OPTION_minimum_password_strength_VALUE_1
        CONFIG_OPTION_minimum_password_strength_VALUE_10
        CONFIG_OPTION_minimum_password_strength_VALUE_2
        CONFIG_OPTION_minimum_password_strength_VALUE_3
        CONFIG_OPTION_minimum_password_strength_VALUE_4
        CONFIG_OPTION_minimum_password_strength_VALUE_5
        CONFIG_OPTION_minimum_password_strength_VALUE_6
        CONFIG_OPTION_minimum_password_strength_VALUE_7
        CONFIG_OPTION_minimum_password_strength_VALUE_8
        CONFIG_OPTION_minimum_password_strength_VALUE_9
        CONFIG_OPTION_minimum_username_length
        CONFIG_OPTION_minutes_between_sends
        CONFIG_OPTION_mobile_support
        CONFIG_OPTION_modal_user
        CONFIG_OPTION_network_links
        CONFIG_OPTION_news_categories_per_page
        CONFIG_OPTION_news_entries_per_page
        CONFIG_OPTION_news_update_time
        CONFIG_OPTION_newsletter_text
        CONFIG_OPTION_newsletter_title
        CONFIG_OPTION_newsletter_update_time
        CONFIG_OPTION_normal_groups_per_page
        CONFIG_OPTION_notification_desktop_alerts
        CONFIG_OPTION_notification_enable_digests
        CONFIG_OPTION_notification_enable_private_topics
        CONFIG_OPTION_notification_keep_days
        CONFIG_OPTION_notification_poll_frequency
        CONFIG_OPTION_oembed_html_whitelist
        CONFIG_OPTION_oembed_manual_patterns
        CONFIG_OPTION_oembed_max_size
        CONFIG_OPTION_one_per_email_address
        CONFIG_OPTION_output_streaming
        CONFIG_OPTION_overt_whisper_suggestion
        CONFIG_OPTION_password_change_days
        CONFIG_OPTION_password_cookies
        CONFIG_OPTION_password_expiry_days
        CONFIG_OPTION_password_reset_process
        CONFIG_OPTION_password_reset_process_VALUE_emailed
        CONFIG_OPTION_password_reset_process_VALUE_temporary
        CONFIG_OPTION_password_reset_process_VALUE_ultra
        CONFIG_OPTION_payment_gateway
        CONFIG_OPTION_pd_address
        CONFIG_OPTION_pd_email
        CONFIG_OPTION_pd_number
        CONFIG_OPTION_personal_sub_links
        CONFIG_OPTION_personal_under_members
        CONFIG_OPTION_ping_url
        CONFIG_OPTION_point_logs_per_page
        CONFIG_OPTION_points_ADD_BANNER
        CONFIG_OPTION_points_ADD_DOWNLOAD
        CONFIG_OPTION_points_ADD_IMAGE
        CONFIG_OPTION_points_ADD_NEWS
        CONFIG_OPTION_points_ADD_POLL
        CONFIG_OPTION_points_ADD_QUIZ
        CONFIG_OPTION_points_ADD_VIDEO
        CONFIG_OPTION_points_CHOOSE_POLL
        CONFIG_OPTION_points_COMCODE_PAGE_ADD
        CONFIG_OPTION_points_chat
        CONFIG_OPTION_points_if_liked
        CONFIG_OPTION_points_joining
        CONFIG_OPTION_points_per_currency_unit
        CONFIG_OPTION_points_per_daily_visit
        CONFIG_OPTION_points_per_day
        CONFIG_OPTION_points_points_RECOMMEND_SITE
        CONFIG_OPTION_points_posting
        CONFIG_OPTION_points_rating
        CONFIG_OPTION_points_voting
        CONFIG_OPTION_points_wiki
        CONFIG_OPTION_poll_update_time
        CONFIG_OPTION_pop_url
        CONFIG_OPTION_post_read_history_days
        CONFIG_OPTION_primary_members_per_page
        CONFIG_OPTION_primary_paypal_email
        CONFIG_OPTION_privacy_fax
        CONFIG_OPTION_privacy_postal_address
        CONFIG_OPTION_private_topics_per_page
        CONFIG_OPTION_probation_usergroup
        CONFIG_OPTION_prohibit_password_whitespace
        CONFIG_OPTION_prohibit_username_whitespace
        CONFIG_OPTION_proxy
        CONFIG_OPTION_proxy_password
        CONFIG_OPTION_proxy_port
        CONFIG_OPTION_proxy_user
        CONFIG_OPTION_pt_notifications_as_web
        CONFIG_OPTION_quota
        CONFIG_OPTION_quota_url
        CONFIG_OPTION_random_avatars
        CONFIG_OPTION_remember_me_by_default
        CONFIG_OPTION_reported_posts_forum
        CONFIG_OPTION_require_new_member_validation
        CONFIG_OPTION_restricted_usernames
        CONFIG_OPTION_reverse_thumb_order
        CONFIG_OPTION_root_zone_login_theme
        CONFIG_OPTION_rss_update_time
        CONFIG_OPTION_search_results_per_page
        CONFIG_OPTION_search_with_date_range
        CONFIG_OPTION_secondary_members_per_page
        CONFIG_OPTION_security_token_exceptions
        CONFIG_OPTION_send_error_emails_ocproducts
        CONFIG_OPTION_seq_post_ids
        CONFIG_OPTION_session_expiry_time
        CONFIG_OPTION_session_prudence
        CONFIG_OPTION_shipping_cost_factor
        CONFIG_OPTION_show_conceded_mode_link
        CONFIG_OPTION_show_content_tagging
        CONFIG_OPTION_show_content_tagging_inline
        CONFIG_OPTION_show_docs
        CONFIG_OPTION_show_empty_cpfs
        CONFIG_OPTION_show_empty_galleries
        CONFIG_OPTION_show_first_join_page
        CONFIG_OPTION_show_gallery_counts
        CONFIG_OPTION_show_inline_stats
        CONFIG_OPTION_show_personal_adminzone_link
        CONFIG_OPTION_show_post_validation
        CONFIG_OPTION_show_screen_actions
        CONFIG_OPTION_show_staff_page_actions
        CONFIG_OPTION_show_su
        CONFIG_OPTION_simplified_attachments_ui
        CONFIG_OPTION_simplified_donext
        CONFIG_OPTION_simplify_wysiwyg_by_permissions
        CONFIG_OPTION_site_closed
        CONFIG_OPTION_site_name
        CONFIG_OPTION_site_scope
        CONFIG_OPTION_sitewide_im
        CONFIG_OPTION_sms_api_id
        CONFIG_OPTION_sms_high_limit
        CONFIG_OPTION_sms_high_trigger_limit
        CONFIG_OPTION_sms_low_limit
        CONFIG_OPTION_sms_low_trigger_limit
        CONFIG_OPTION_sms_password
        CONFIG_OPTION_sms_username
        CONFIG_OPTION_smtp_from_address
        CONFIG_OPTION_smtp_sockets_host
        CONFIG_OPTION_smtp_sockets_password
        CONFIG_OPTION_smtp_sockets_port
        CONFIG_OPTION_smtp_sockets_use
        CONFIG_OPTION_smtp_sockets_username
        CONFIG_OPTION_spam_approval_threshold
        CONFIG_OPTION_spam_ban_threshold
        CONFIG_OPTION_spam_blackhole_detection
        CONFIG_OPTION_spam_block_lists
        CONFIG_OPTION_spam_block_threshold
        CONFIG_OPTION_spam_cache_time
        CONFIG_OPTION_spam_check_exclusions
        CONFIG_OPTION_spam_check_level
        CONFIG_OPTION_spam_check_level_VALUE_ACTIONS
        CONFIG_OPTION_spam_check_level_VALUE_EVERYTHING
        CONFIG_OPTION_spam_check_level_VALUE_GUESTACTIONS
        CONFIG_OPTION_spam_check_level_VALUE_JOINING
        CONFIG_OPTION_spam_check_level_VALUE_NEVER
        CONFIG_OPTION_spam_check_usernames
        CONFIG_OPTION_spam_stale_threshold
        CONFIG_OPTION_ssw
        CONFIG_OPTION_staff_address
        CONFIG_OPTION_staff_text
        CONFIG_OPTION_stats_store_time
        CONFIG_OPTION_stats_when_closed
        CONFIG_OPTION_stopforumspam_api_key
        CONFIG_OPTION_store_revisions
        CONFIG_OPTION_subgallery_link_limit
        CONFIG_OPTION_super_logging
        CONFIG_OPTION_supermembers_text
        CONFIG_OPTION_support_operator
        CONFIG_OPTION_tasks_background
        CONFIG_OPTION_threaded_buttons
        CONFIG_OPTION_threaded_comments
        INCORRECT_PASSWORD_RESET_CODE
        CONFIG_OPTION_threaded_topics_default
        CONFIG_OPTION_thumb_width
        CONFIG_OPTION_ticket_auto_assign
        CONFIG_OPTION_ticket_email_from
        CONFIG_OPTION_ticket_forum_name
        CONFIG_OPTION_ticket_mail_on
        CONFIG_OPTION_ticket_mail_password
        CONFIG_OPTION_ticket_mail_server
        CONFIG_OPTION_ticket_mail_server_port
        CONFIG_OPTION_ticket_mail_server_type
        CONFIG_OPTION_ticket_mail_username
        CONFIG_OPTION_ticket_member_forums
        CONFIG_OPTION_ticket_text
        CONFIG_OPTION_ticket_type_forums
        CONFIG_OPTION_topic_pin
        CONFIG_OPTION_topic_pin_max_days
        CONFIG_OPTION_tornevall_api_password
        CONFIG_OPTION_tornevall_api_username
        CONFIG_OPTION_transcoding_server
        CONFIG_OPTION_transcoding_zencoder_api_key
        CONFIG_OPTION_transcoding_zencoder_ftp_path
        CONFIG_OPTION_tree_lists
        CONFIG_OPTION_unzip_cmd
        CONFIG_OPTION_unzip_dir
        CONFIG_OPTION_url_monikers_enabled
        CONFIG_OPTION_url_scheme
        CONFIG_OPTION_url_scheme_VALUE_HTM
        CONFIG_OPTION_url_scheme_VALUE_PG
        CONFIG_OPTION_url_scheme_VALUE_RAW
        CONFIG_OPTION_url_scheme_VALUE_SIMPLE
        CONFIG_OPTION_use_banner_permissions
        CONFIG_OPTION_use_captchas
        CONFIG_OPTION_use_contextual_dates
        CONFIG_OPTION_use_joindate
        CONFIG_OPTION_use_lastondate
        CONFIG_OPTION_use_local_payment
        CONFIG_OPTION_use_true_from
        CONFIG_OPTION_use_true_from_VALUE_0
        CONFIG_OPTION_use_true_from_VALUE_1
        CONFIG_OPTION_use_true_from_VALUE_2
        EMAIL_LOG_DAYS
        CONFIG_OPTION_email_log_days
        CONFIG_OPTION_user_postsize_errors
        CONFIG_OPTION_username_click_im
        CONFIG_OPTION_username_profile_links
        CONFIG_OPTION_users_online_time
        CONFIG_OPTION_valid_audios
        CONFIG_OPTION_valid_email_domains
        CONFIG_OPTION_valid_images
        CONFIG_OPTION_valid_types
        CONFIG_OPTION_valid_videos
        CONFIG_OPTION_video_bitrate
        CONFIG_OPTION_video_height_setting
        CONFIG_OPTION_video_width_setting
        CONFIG_OPTION_vote_member_ip_restrict
        CONFIG_OPTION_vpn_password
        CONFIG_OPTION_vpn_username
        CONFIG_OPTION_web_notifications_enabled
        CONFIG_OPTION_website_email
        CONFIG_OPTION_webstandards_compat
        CONFIG_OPTION_webstandards_css
        CONFIG_OPTION_webstandards_ext_files
        CONFIG_OPTION_webstandards_javascript
        CONFIG_OPTION_webstandards_wcag
        CONFIG_OPTION_webstandards_xhtml
        CONFIG_OPTION_welcome_message
        CONFIG_OPTION_width_left
        CONFIG_OPTION_width_right
        CONFIG_OPTION_wiki_enable_children
        CONFIG_OPTION_wiki_enable_content_posts
        CONFIG_OPTION_wysiwyg
        CONFIG_OPTION_yeehaw
        CONFIRM
        CONFIRM_CODE
        CONFIRM_DELETE_ALL_CHATROOMS
        CONFIRM_DELETE_ALL_MESSAGES
        CONFIRM_DELETE_LINK
        CONFIRM_DELETE_LINK_NICE
        CONFIRM_MASTER_PASSWORD
        CONFIRM_MOVE
        CONFIRM_MOVE_LINK_NICE
        CONFIRM_REMOVE_PERIODIC
        CONFLICTING_EMOTICON_CODE
        CONFLICTING_PAGE_NAME
        CONFLICTING_ZONE_NAME
        CONFLICTING_ZONE_NAME__PAGE
        CONFLICTING_PAGE_NAME_MODULE
        CONFLICTING_PAGE_NAME_REDIRECT
        CONNECT_DB_ERROR
        CONNECT_ERROR
        CONTACT_US_MESSAGING
        CONTENT_DATE_AND_TIME
        CONTENT_NEEDING_REVIEWING
        CONTENT_OWNER
        CONTENT_REVIEWS
        CONTENT_REVIEW_AUTO_ACTION
        CONTENT_REVIEW_AUTO_ACTION_delete
        CONTENT_REVIEW_AUTO_ACTION_leave
        CONTENT_REVIEW_AUTO_ACTION_unvalidate
        CONTENT_TYPE
        CONTEXTUAL_CSS_EDITING
        CONTEXTUAL_CSS_EDITING_GLOBAL
        CONTEXTUAL_EDITING_SCREEN
        CONTRACTED_BRANCH
        CONTROL_FUNCTIONS
        COOKIE
        COOKIE_CONFLICT_DELETE_COOKIES
        COOKIE_DAYS
        COOKIE_DAYS_EXAMPLE
        COOKIE_DAYS_TEXT
        COOKIE_DOMAIN
        COOKIE_DOMAIN_CANT_USE
        COOKIE_DOMAIN_EXAMPLE
        COOKIE_DOMAIN_MUST_MATCH
        COOKIE_DOMAIN_MUST_START_DOT
        COOKIE_DOMAIN_TEXT
        COOKIE_EXAMPLE
        COOKIE_NOTICE_OPTION
        COOKIE_PASSWORD
        COOKIE_PASSWORD_EXAMPLE
        COOKIE_PASSWORD_TEXT
        COOKIE_PATH
        COOKIE_PATH_EXAMPLE
        COOKIE_PATH_MUST_MATCH
        COOKIE_PATH_TEXT
        COOKIE_SETTINGS
        COOKIE_TEXT
        COPPA
        COPPA_ENABLED
        COPYRIGHT
        COPYRIGHT_ATTRIBUTION
        COPY_MEMBERS_INTO
        COPY_THEME
        COPY_TO_EDITABLE_MENU
        COPY_TO_SERVER
        CORRECT_MYSQL_SCHEMA_ISSUES
        CORRUPT_FILE
        CORRUPT_FILES_CROP
        CORRUPT_FILES_LOWERCASE
        CORRUPT_INSTALLATION_FILE
        CORRUPT_TAR
        COST_banner_hit
        COST_banner_imp
        COST_banner_setup
        COST_highlight_name
        COST_quota
        COST_topic_pin
        COULD_NOT_SAVE_FILE
        CPF_ENABLE_CITY
        CPF_ENABLE_COUNTRY
        CPF_ENABLE_COUNTY
        CPF_ENABLE_NAME
        CPF_ENABLE_PHONE
        CPF_ENABLE_POST_CODE
        CPF_ENABLE_STATE
        CPF_ENABLE_STREET_ADDRESS
        CREATED_BY
        CREATED_CUSTOM_PROFILE_FIELDS
        CREATE_INVOICE
        CREATE_PERIODIC_FOR
        CREATE_PERIODIC_FOR_ALL
        CREATE_PERIODIC_FOR_FUTURE
        CRITICISE_LANGUAGE_PACK
        CRITICISM_EXTRA_STRING
        CRITICISM_MISSING_FILE
        CRITICISM_MISSING_STRING
        CRITICISM_PARAMETER_COUNT_MISMATCH
        CRON_NEEDED_TO_WORK
        CRYPT_RATCHET
        CSRF_TOKEN_EXPIRE_FRESH
        CSRF_TOKEN_EXPIRE_NEW
        CSS
        CSS_BAD_PROPERTY_VALUE
        CSS_BAD_PSEUDO_CLASS
        CSS_CAPTCHA
        CSS_CHOOSE_TO_EDIT
        CSS_EDITOR_SELECTOR_TIPS
        CSS_EQUATION_HELPER
        CSS_FILE
        CSS_FILE_UPGRADED
        CSS_INLINE_STYLES
        CSS_NON_IE_PROPERTIES
        CSS_PX_FONT
        CSS_RULE_OVERMATCHED_css_append
        CSS_RULE_OVERMATCHED_css_prepend
        CSS_RULE_OVERMATCHED_css_replace
        CSS_RULE_UNMATCHED_css_append
        CSS_RULE_UNMATCHED_css_prepend
        CSS_RULE_UNMATCHED_css_replace
        CSS_UNEXPECTED_CHARACTER
        CSS_UNEXPECTED_CHARACTER_CLASS
        CSS_UNEXPECTED_TERMINATION
        CSS_UNEXPECTED_TERMINATION_PROPERTY
        CSS_UNKNOWN_PROPERTY
        CSV_UPLOAD_DESC
        CT_NOTIFICATION_MAIL
        CT_NOTIFICATION_MAIL_SUBJECT
        CURRENCY
        CURRENTLY_HAS_KEEP_FATALISTIC
        CURRENTLY_HAS_KEEP_SAFE_MODE
        CUSTOMER
        CUSTOMER_NOTES
        CUSTOM_COMCODE
        CUSTOM_FIELDS_FOR
        CUSTOM_PRODUCT_DESCRIPTION
        CUSTOM_PRODUCT_INTEREST
        CUSTOM_PRODUCT_OTHER
        CUSTOM_PRODUCT_TAX
        CUSTOM_PRODUCT_USERGROUP
        CUSTOM_PRODUCT_WAGE
        CUSTOM_PRODUCT_WORK
        CUSTOM_PROFILE_FIELDS
        CUSTOM_PROFILE_FIELD_STATS
        CUSTOM_TASKS
        CUTOFF_DATE
        DANGEROUS_TAG
        DARK_THEME
        DASHBOARD
        DATABASE_HOST
        DATABASE_HOST_TEXT
        DATABASE_NAME
        DATABASE_PASSWORD
        DATABASE_SETTINGS
        DATABASE_USERNAME
        DATABASE_VALID
        DATA_ENCRYPTED
        DATA_FILE_CONFLICT
        DB_CHOICE
        DECRYPT
        DECRYPTION_ERROR
        DECRYPTION_KEY
        DECRYPT_DATA
        DECRYPT_DESCRIPTION
        DECRYPT_LABEL
        DECRYPT_TITLE
        DEEPER_ADMIN_BREADCRUMBS
        DEFAULT
        DEFAULT_COMMENT_SORT_ORDER
        DEFAULT_GROUP
        DEFAULT_IMAGE
        DEFAULT_PAGE
        DEFAULT_PASSWORD
        DEFAULT_PERMISSIONS_FROM
        DEFAULT_POST_TEMPLATE_bug_text
        DEFAULT_POST_TEMPLATE_bug_title
        DEFAULT_POST_TEMPLATE_fault_text
        DEFAULT_POST_TEMPLATE_fault_title
        DEFAULT_POST_TEMPLATE_task_text
        DEFAULT_POST_TEMPLATE_task_title
        DEFAULT_PREVIEW_GUESTS
        DEFAULT_THEME_BY_DEFAULT
        DEFAULT_THEME_BY_FORUM
        DEFAULT_THEME_BY_SITENAME
        DEFAULT_THEME_INHERITANCE
        DEFAULT_VALUE
        DEFAULT_VIDEO_HEIGHT
        DEFAULT_VIDEO_WIDTH
        DEFAULT__CATALOGUE_VIEW_REPORT
        DEFAULT__CATALOGUE_VIEW_REPORT_LINE
        DEFAULT__CATALOGUE_VIEW_REPORT_SUBJECT
        DEFER_TIME
        DEFINES_ORDER
        DEFINE_AUTHOR
        DELETED
        DELETE_AGGREGATE_MATCHES
        DELETE_AGGREGATE_TYPE_INSTANCE
        DELETE_ALL
        DELETE_ALL_CHATROOMS
        DELETE_ALL_MESSAGES
        DELETE_ATTACHMENTS
        DELETE_AUTHOR
        DELETE_AWARD_TYPE
        DELETE_BANNER
        DELETE_BANNER_TYPE
        DELETE_BAN_TRACKBACK
        DELETE_CALENDAR_EVENT
        DELETE_CALENDAR_EVENT__NON_PUBLIC
        DELETE_CATALOGUE
        DELETE_CATALOGUE_CATEGORY
        DELETE_CATALOGUE_ENTRY
        DELETE_CHATROOM
        DELETE_CUSTOM_COMCODE_TAG
        DELETE_CUSTOM_PROFILE_FIELD
        DELETE_DATA_AVAILABLE
        DELETE_DOWNLOAD
        DELETE_DOWNLOAD_CATEGORY
        DELETE_DOWNLOAD_LICENCE
        DELETE_EMOTICON
        DELETE_EVENT_TYPE
        DELETE_FORUM
        DELETE_FORUM_GROUPING
        DELETE_FULL
        DELETE_GALLERY
        DELETE_GROUP
        DELETE_HANDLING_DELETE
        DELETE_HANDLING_LEAVE
        DELETE_IMAGE
        DELETE_INVOICE
        DELETE_LIVE_POLL
        DELETE_LURKERS
        DELETE_MEDIA
        DELETE_MEMBER
        DELETE_MENU
        DELETE_MENU_ITEM
        DELETE_MESSAGE
        DELETE_MULTI_MODERATION
        DELETE_NEWS
        DELETE_NEWSLETTER
        DELETE_NEWS_BLOG
        DELETE_NEWS_CATEGORY
        DELETE_NUM
        DELETE_OWN_AUTHOR
        DELETE_OWN_BANNER
        DELETE_OWN_CALENDAR_EVENT
        DELETE_OWN_CALENDAR_EVENT__NON_PUBLIC
        DELETE_OWN_CATALOGUE_ENTRY
        DELETE_OWN_DOWNLOAD
        DELETE_OWN_GALLERY
        DELETE_OWN_LIVE_POLL
        DELETE_OWN_MEDIA
        DELETE_OWN_NEWS
        DELETE_OWN_NEWS_BLOG
        DELETE_OWN_NEWS_CATEGORY
        DELETE_OWN_POLL
        DELETE_OWN_POST
        DELETE_OWN_QUIZ
        DELETE_OWN_TOPIC
        DELETE_PAGES
        DELETE_PARTIAL
        DELETE_PERIODIC_NEWSLETTER
        DELETE_POLL
        DELETE_POST_TEMPLATE
        DELETE_CONTENTS
        DELETE_PRIVATE_CALENDAR_EVENT
        DELETE_PUBLIC_CALENDAR_EVENT
        DELETE_QUIZ
        DELETE_QUIZ_RESULTS
        DELETE_REVISION
        DELETE_SAVED_WARNING
        DELETE_SEARCH_STATS
        DELETE_SELECTED
        DELETE_SOME_MESSAGES
        DELETE_STATUS
        DELETE_THEME
        DELETE_THEME_IMAGE
        DELETE_TICKET_TYPE
        DELETE_TIME_LIMIT
        DELETE_TRACKBACK
        DELETE_TRACKBACKS
        DELETE_TRASHED_PTS
        DELETE_UN_PC
        DELETE_USERGROUP_SUBSCRIPTION
        DELETE_VIDEO
        DELETE_WELCOME_EMAIL
        DELETE_WITHOUT_MERGING
        DELETE_WORDFILTER
        DELETE_ZONE
        DELETE_ZONE_ERROR
        DELURK_CONFIRM
        DELURK_MAX_LOGGED_ACTIONS
        DELURK_MAX_LOGGED_ACTIONS_DESCRIPTION
        DELURK_MAX_POINTS
        DELURK_MAX_POINTS_DESCRIPTION
        DELURK_MAX_POSTS
        DELURK_MAX_POSTS_DESCRIPTION
        DELURK_MIN_DAYS_SINCE_JOIN
        DELURK_MIN_DAYS_SINCE_JOIN_DESCRIPTION
        DELURK_MIN_DAYS_SINCE_LOGIN
        DELURK_MIN_DAYS_SINCE_LOGIN_DESCRIPTION
        DELURK_USERGROUPS_DESCRIPTION
        DEMOGRAPHICS
        DEMOGRAPHICS_STATS_RANGE
        DENSITY
        DEPENDENCIES
        DEPLOYMENT_AGREEMENT
        DEPLOYMENT_DETERMINATION
        DESCRIPTION_ACCEPT_IMAGES
        DESCRIPTION_ACCEPT_VIDEOS
        DESCRIPTION_ACCESS_FOR
        DESCRIPTION_ADDITIONAL_INFO
        DESCRIPTION_ADDON_CATEGORY
        DESCRIPTION_ADDON_LICENCE
        DESCRIPTION_ADD_THEME_MANUAL
        DESCRIPTION_ADD_TO_OWN_ACCOUNT
        DESCRIPTION_ADMINGUIDE
        DESCRIPTION_ADMINSTART_PAGE
        DESCRIPTION_ADMIN_USERNAME
        DESCRIPTION_ADMIN_USERS_PASSWORD
        DESCRIPTION_ALLOWS_ANONYMOUS_POSTS
        DESCRIPTION_ALLOW_COMMENTS
        DESCRIPTION_ALLOW_RATING
        DESCRIPTION_ALLOW_TRACKBACKS
        DESCRIPTION_ALL_DAY_EVENT
        DESCRIPTION_ALTERNATE_URL_FILEDUMP
        DESCRIPTION_ANSWER
        DESCRIPTION_ARCHIVE_IMAGES
        DESCRIPTION_ARCHIVE_MEDIA
        DESCRIPTION_ARCHIVE_PAGE_BACKUPS
        DESCRIPTION_ARCHIVE_TEXT_FILES
        DESCRIPTION_ARCHIVE_TEXT_FILES_PT
        DESCRIPTION_AUTHOR
        DESCRIPTION_AUTHOR_THEME
        DESCRIPTION_AUTHOR_URL
        DESCRIPTION_AUTO_FILL
        DESCRIPTION_AUTO_RECUR
        DESCRIPTION_AWARD_POINTS
        DESCRIPTION_AWARD_UPDATE_TIME_HOURS
        DESCRIPTION_BANNED_ADDRESSES_A
        DESCRIPTION_BANNED_ADDRESSES_B
        DESCRIPTION_BANNED_MEMBER
        DESCRIPTION_BANNER_DESCRIPTION
        DESCRIPTION_BANNER_DESCRIPTION_SIMPLE
        DESCRIPTION_BANNER_DIMENSIONS
        DESCRIPTION_BANNER_DIRECT_CODE
        DESCRIPTION_BANNER_FILE_SIZE
        DESCRIPTION_BANNER_IS_TEXTUAL
        DESCRIPTION_BANNER_NAME
        DESCRIPTION_BANNER_TITLE_TEXT
        DESCRIPTION_BANNER_TYPE
        DESCRIPTION_BANNER_TYPE_2
        DESCRIPTION_BANNER_URL
        DESCRIPTION_BLOCK_CACHE
        DESCRIPTION_BLOCK_TAG
        DESCRIPTION_BRAND_BASE_URL
        DESCRIPTION_BROKEN_URLS
        DESCRIPTION_BROWSER_SHARE
        DESCRIPTION_CACHE_FORUMS
        DESCRIPTION_CACHE_MEMBERS
        DESCRIPTION_CACHE_TOPICS
        DESCRIPTION_CATALOGUE
        DESCRIPTION_CATALOGUE_CSV_DELETE_HANDLING
        DESCRIPTION_CATALOGUE_CSV_IMPORT_KEY_FIELD
        DESCRIPTION_CATALOGUE_CSV_IMPORT_META_DESCRIPTION_FIELD
        DESCRIPTION_CATALOGUE_CSV_IMPORT_META_KEYWORDS_FIELD
        DESCRIPTION_CATALOGUE_CSV_IMPORT_NOTES_FIELD
        DESCRIPTION_CATALOGUE_CSV_NEW_HANDLING
        DESCRIPTION_CATALOGUE_CSV_UPDATE_HANDLING
        DESCRIPTION_CATALOGUE_DESCRIPTION
        DESCRIPTION_CATALOGUE_REVIEW_FREQ
        DESCRIPTION_CATEGORY_TREE
        DESCRIPTION_CAT_ECOMMERCE
        DESCRIPTION_CHANGE_USERGROUP_TO
        DESCRIPTION_CHARGED_POINTS
        DESCRIPTION_CLEAR_ALL
        DESCRIPTION_CLEAR_EXISTING_FORUMS_ON_INSTALL
        DESCRIPTION_CLEAR_GRAPHS
        DESCRIPTION_CLEAR_STATISTICS
        DESCRIPTION_CLOSE_TIME
        DESCRIPTION_CODENAME
        DESCRIPTION_CODENAME_SHOULDNT
        DESCRIPTION_CODENAME_THEME
        DESCRIPTION_CODENAME_THEME_HELPER
        DESCRIPTION_COMCODE_CACHE
        DESCRIPTION_COMCODE_EXAMPLE
        DESCRIPTION_COMCODE_INLINE
        DESCRIPTION_COMCODE_PAGE_CACHE
        DESCRIPTION_COMCODE_PARAMETERS
        DESCRIPTION_COMCODE_REPLACE
        DESCRIPTION_COMCODE_TAG
        DESCRIPTION_COMPLETION_PERCENTAGE
        DESCRIPTION_CONTENT_TYPE
        DESCRIPTION_COPYRIGHT_ATTRIBUTION
        DESCRIPTION_COPY_MEMBERS_INTO
        DESCRIPTION_COPY_THEME
        DESCRIPTION_COPY_TO_SERVER
        DESCRIPTION_CORRECT_MYSQL_SCHEMA_ISSUES
        DESCRIPTION_CUSTOM_EXPIRY_DATE
        DESCRIPTION_CUTOFF_DATE
        DESCRIPTION_DANGEROUS_TAG
        DESCRIPTION_DARK_THEME
        DESCRIPTION_DECRYPT_DATA
        DESCRIPTION_DEFAULT_IMAGE
        DESCRIPTION_DEFAULT_PAGE
        DESCRIPTION_DEFAULT_PASSWORD
        DESCRIPTION_DEFAULT_PERMISSIONS_FROM
        DESCRIPTION_DEFAULT_PERMISSIONS_FROM_NEW
        DESCRIPTION_DEFER_TIME
        DESCRIPTION_DEFINES_ORDER
        DESCRIPTION_DELETE_AGGREGATE_MATCHES
        DESCRIPTION_DELETE_CHATROOM
        DESCRIPTION_DELETE_DAYS
        DESCRIPTION_DELETE_LOSE_CONTENTS
        DESCRIPTION_DELETE_MESSAGE
        DESCRIPTION_DELETE_PARENT_CONTENTS
        DESCRIPTION_DELETE_SEARCH_STATS
        DESCRIPTION_DELETE_STATUS
        DESCRIPTION_DELETE_THEME_IMAGE
        DESCRIPTION_DELETE_TOPICS
        DESCRIPTION_DELETE_UN_PC
        DESCRIPTION_DELETE_USERGROUP_SUB_DANGER
        DESCRIPTION_DEMOGRAPHICS
        DESCRIPTION_DEPENDENCIES
        DESCRIPTION_DESCRIPTION
        DESCRIPTION_DESCRIPTION_ACCESSIBILITY
        DESCRIPTION_DESCRIPTION_FILES
        DESCRIPTION_DESTINATION_FORUM
        DESCRIPTION_DESTINATION_PATH
        DESCRIPTION_DESTINATION_ZONE
        DESCRIPTION_DISPLAY_REVIEW_STATUS
        DESCRIPTION_DISPLAY_TYPE
        DESCRIPTION_DOWNLOADS_STATISTICS
        DESCRIPTION_DOWNLOAD_AS_CSV
        DESCRIPTION_DOWNLOAD_COMPOSR_HOMESITE
        DESCRIPTION_DOWNLOAD_COST
        DESCRIPTION_DOWNLOAD_IMAGES
        DESCRIPTION_DOWNLOAD_IMAGE_SHORTCUT
        DESCRIPTION_DOWNLOAD_LICENCE
        DESCRIPTION_DO_TIMEZONE_CONV
        DESCRIPTION_EDIT_THEME
        DESCRIPTION_EMBED_FULL_ARTICLES
        DESCRIPTION_EMOTICON_CODE
        DESCRIPTION_EMOTICON_IS_SPECIAL
        DESCRIPTION_ENCRYPTED
        DESCRIPTION_END_DATE_AND_TIME
        DESCRIPTION_ENQUIRE_ON_NEW_IPS
        DESCRIPTION_EVENT_TEXT
        DESCRIPTION_EVENT_TIMEZONE
        DESCRIPTION_EVENT_TYPE
        DESCRIPTION_EXPANDED_BY_DEFAULT
        DESCRIPTION_EXPIRY_MOVE_DAYS_HIGHER
        DESCRIPTION_EXPIRY_MOVE_DAYS_LOWER
        DESCRIPTION_EXPIRY_MOVE_TARGET
        DESCRIPTION_EXTEND_PROBATION
        DESCRIPTION_EXTERNAL_FEED
        DESCRIPTION_FAVICON
        DESCRIPTION_FIELD_DEFAULT
        DESCRIPTION_FIELD_DESCRIPTION
        DESCRIPTION_FIELD_NAME
        DESCRIPTION_FIELD_ONLY_GROUP
        DESCRIPTION_FIELD_OPTIONS
        DESCRIPTION_FIELD_ORDER_CLEVER
        DESCRIPTION_FIELD_TYPE
        DESCRIPTION_FIELD_TYPE_FIRST_TIME
        DESCRIPTION_FILEDUMP_BROKEN
        DESCRIPTION_FILES
        DESCRIPTION_FILE_SIZE
        DESCRIPTION_FILTER_BY_ALLOW
        DESCRIPTION_FILTER_REGIONS
        DESCRIPTION_FINISH_STARTED_ALREADY
        DESCRIPTION_FIX_INVALID_HTML
        DESCRIPTION_FIX_PAST_RECURRENCES
        DESCRIPTION_FLOOD_CONTROL_ACCESS_SECS
        DESCRIPTION_FLOOD_CONTROL_SUBMIT_SECS
        DESCRIPTION_FLOW_MODE_INTERFACE
        DESCRIPTION_FOLDER_NAME
        DESCRIPTION_FORCE_TEMPORARY_PASSWORD
        DESCRIPTION_FORUM_GROUPING
        DESCRIPTION_FORUM_MOVE_TARGET
        DESCRIPTION_FORUM_ORDER
        DESCRIPTION_FORUM_REDIRECTION
        DESCRIPTION_FORUM_TOPIC_ID
        DESCRIPTION_FTP_FILES_PER_GO
        DESCRIPTION_FTP_SERVER_URL
        DESCRIPTION_GAE_APPLICATION
        DESCRIPTION_GAE_BUCKET_NAME
        DESCRIPTION_GALLERY
        DESCRIPTION_GALLERY_IMPORT_TITLE
        DESCRIPTION_GIFT_POINTS_BASE
        DESCRIPTION_GIFT_POINTS_PER_DAY
        DESCRIPTION_GROUP_HIDDEN
        DESCRIPTION_GROUP_LEADER
        DESCRIPTION_GROUP_TITLE
        DESCRIPTION_GUESTS
        DESCRIPTION_GUEST_ZONE_ACCESS
        DESCRIPTION_HAVE_DEFAULT_BANNERS_ADVERTISING
        DESCRIPTION_HAVE_DEFAULT_BANNERS_DONATION
        DESCRIPTION_HAVE_DEFAULT_CATALOGUES_CONTACTS
        DESCRIPTION_HAVE_DEFAULT_CATALOGUES_FAQS
        DESCRIPTION_HAVE_DEFAULT_CATALOGUES_LINKS
        DESCRIPTION_HAVE_DEFAULT_CATALOGUES_PROJECTS
        DESCRIPTION_HAVE_DEFAULT_CPF_SET
        DESCRIPTION_HAVE_DEFAULT_FULL_EMOTICON_SET
        DESCRIPTION_HAVE_DEFAULT_RANK_SET
        DESCRIPTION_HAVE_DEFAULT_WORDFILTER
        DESCRIPTION_HEADER_TEXT
        DESCRIPTION_HIDE_AWARDEE
        DESCRIPTION_HIDE_POSTS
        DESCRIPTION_HITS_ALLOCATED
        DESCRIPTION_HOURLY_BREAKDOWNS
        DESCRIPTION_HTML_ONLY
        DESCRIPTION_ICAL
        DESCRIPTION_IMAGE
        DESCRIPTION_IMAGE_THUMBNAILS
        DESCRIPTION_IMMEDIATE_USE
        DESCRIPTION_IMMEDIATE_USE_ADD
        DESCRIPTION_IMPORTANCE_MODULUS
        DESCRIPTION_IMPORTER
        DESCRIPTION_IMPORT_BASE_URL
        DESCRIPTION_IMPORT_BLOG_COMMENTS
        DESCRIPTION_IMPORT_CSV
        DESCRIPTION_IMPORT_SESSION
        DESCRIPTION_IMPORT_STOCK_RESPONSES
        DESCRIPTION_IMPORT_STOCK_RESPONSES_PT
        DESCRIPTION_IMPORT_TO_BLOG
        DESCRIPTION_IMPORT_WORDPRESS_USER
        DESCRIPTION_INCLUDE_CMS_ADVERT
        DESCRIPTION_INCOMPATIBILITIES
        DESCRIPTION_INHERIT_CSS
        DESCRIPTION_INSTALLPROFILE
        DESCRIPTION_INTRO_ANSWER
        DESCRIPTION_INTRO_POST
        DESCRIPTION_INTRO_QUESTION
        DESCRIPTION_INVESTIGATE
        DESCRIPTION_INVOICE_FOR
        DESCRIPTION_INVOICE_NOTE
        DESCRIPTION_INVOICE_PAGE
        DESCRIPTION_INVOICE_SPECIAL
        DESCRIPTION_IP_ADDRESS_DISTRIBUTION
        DESCRIPTION_IS_DEFAULT_GROUP
        DESCRIPTION_IS_MEMBER_SYNCHED_GALLERY
        DESCRIPTION_IS_PRESENTED_AT_INSTALL
        DESCRIPTION_IS_SUPER_ADMIN
        DESCRIPTION_IS_SUPER_ADMIN_B
        DESCRIPTION_IS_SUPER_MODERATOR
        DESCRIPTION_IS_THREADED
        DESCRIPTION_IS_TREE
        DESCRIPTION_KEEP_BLOGS
        DESCRIPTION_KEEP_DEFAULT_NEWS_CATEGORIES
        DESCRIPTION_KEEP_PERSONAL_GALLERIES
        DESCRIPTION_KEEP_SUBFOLDERS
        DESCRIPTION_KEYWORDS_SHARE
        DESCRIPTION_LABEL
        DESCRIPTION_LANGUAGE
        DESCRIPTION_LANGUAGE_CACHE
        DESCRIPTION_LANGUAGE_FILE
        DESCRIPTION_LENGTH_UNITS
        DESCRIPTION_LICENCE_TEXT
        DESCRIPTION_LOAD_FILESYSTEM_FILES
        DESCRIPTION_LOAD_FTP_FILES
        DESCRIPTION_LOAD_TIMES
        DESCRIPTION_LOGOWIZARD_THEME
        DESCRIPTION_LOGO_NAME
        DESCRIPTION_MAIL_COST
        DESCRIPTION_MAIN_CATEGORY
        DESCRIPTION_MAX_ATTACHMENTS_PER_POST
        DESCRIPTION_MAX_AVATAR_DIMENSIONS
        DESCRIPTION_MAX_DAILY_UPLOAD_MB
        DESCRIPTION_MAX_POST_LENGTH_COMCODE
        DESCRIPTION_MAX_SIG_LENGTH_COMCODE
        DESCRIPTION_MEMBERS_JOINED_FROM
        DESCRIPTION_MEMBERS_JOINED_TO
        DESCRIPTION_MEMBER_AUTHOR
        DESCRIPTION_MEMBER_CALENDAR
        DESCRIPTION_MEMBER_DESCRIPTION
        DESCRIPTION_MEMBER_FROM
        DESCRIPTION_MEMBER_TO
        DESCRIPTION_MENU_ENTRY_CHECK_PERMS
        DESCRIPTION_MERGING_ON_MSN
        DESCRIPTION_MESSAGE
        DESCRIPTION_MESSAGE_NEWSLETTER
        DESCRIPTION_METADATA
        DESCRIPTION_META_ADD_TIME
        DESCRIPTION_META_DESCRIPTION
        DESCRIPTION_META_EDIT_TIME
        DESCRIPTION_META_KEYWORDS
        DESCRIPTION_META_NUM_DOWNLOADS
        DESCRIPTION_META_URL_MONIKER
        DESCRIPTION_META_VIEWS
        DESCRIPTION_MM_POST_TEXT
        DESCRIPTION_MOBILE_PAGES
        DESCRIPTION_MONTHLY_SPEC_TYPE
        DESCRIPTION_MULTI_MODERATION_POST
        DESCRIPTION_MYSQL_OPTIMISE
        DESCRIPTION_NAME
        DESCRIPTION_NAME_REFERENCE
        DESCRIPTION_NEED_FTP
        DESCRIPTION_NEWSLETTER_FROM_EMAIL
        DESCRIPTION_NEWSLETTER_FROM_NAME
        DESCRIPTION_NEWSLETTER_PRIORITY
        DESCRIPTION_NEWSLETTER_TEMPLATE
        DESCRIPTION_NEWS_IMAGE_OVERRIDE
        DESCRIPTION_NEWS_SUMMARY
        DESCRIPTION_NEW_COMCODE_PAGE
        DESCRIPTION_NEW_LANG
        DESCRIPTION_NEW_UGROUP_SUB_FOR
        DESCRIPTION_NEW_USERGROUP
        DESCRIPTION_NEXT_REVIEW_DATE
        DESCRIPTION_NOSUBSCRIPTION_LEVEL
        DESCRIPTION_NOTES
        DESCRIPTION_NUM_WINNERS
        DESCRIPTION_ONE_PER_MEMBER
        DESCRIPTION_ON_PROBATION_UNTIL
        DESCRIPTION_OPEN_STATE
        DESCRIPTION_OPEN_TIME
        DESCRIPTION_ORDER
        DESCRIPTION_ORDER_NO_CATS
        DESCRIPTION_ORDER_SUB_ALPHA
        DESCRIPTION_ORGANISATION
        DESCRIPTION_ORIGINAL_FILENAME
        DESCRIPTION_ORPHANED_LANG_STRINGS
        DESCRIPTION_ORPHANED_TAGS
        DESCRIPTION_ORPHANED_UPLOADS
        DESCRIPTION_OS_SHARE
        DESCRIPTION_OUTMODE
        DESCRIPTION_OVERVIEW_STATISTICS
        DESCRIPTION_OWNER
        DESCRIPTION_OWNER_SET
        DESCRIPTION_OWNER_VIEW
        DESCRIPTION_PAGES_STATISTICS
        DESCRIPTION_PAGE_LINK_BOOKMARK
        DESCRIPTION_PAGE_NAME
        DESCRIPTION_PAGE_STATS_DELETE
        DESCRIPTION_PARENT
        DESCRIPTION_PARENT_FORUM
        DESCRIPTION_PARENT_PAGE
        DESCRIPTION_PERMISSION_HOURS
        DESCRIPTION_PERMISSION_SCOPE_category
        DESCRIPTION_PERMISSION_SCOPE_module
        DESCRIPTION_PERMISSION_SCOPE_page
        DESCRIPTION_PERMISSION_SCOPE_privilege
        DESCRIPTION_PERMISSION_SCOPE_type
        DESCRIPTION_PERMISSION_SCOPE_zone
        DESCRIPTION_PIN_STATE
        DESCRIPTION_POINTS_FOR_COMPLETING
        DESCRIPTION_POSTING_RATES
        DESCRIPTION_POSTS_DESTINATION_FORUM
        DESCRIPTION_POST_COUNT_INCREMENT
        DESCRIPTION_POST_TEMPLATE_X
        DESCRIPTION_PP_MESSAGE
        DESCRIPTION_PRIMARY_GROUP
        DESCRIPTION_PROBATION
        DESCRIPTION_PROJECT_PROGRESS
        DESCRIPTION_PROMOTION_TARGET
        DESCRIPTION_PROMOTION_THRESHOLD
        DESCRIPTION_PUBLICATION_TIME
        DESCRIPTION_PUBLIC_VIEW
        DESCRIPTION_PURCHASE_MAIL
        DESCRIPTION_PUT_IN_CATEGORY
        DESCRIPTION_PUT_IN_SEARCH
        DESCRIPTION_QUIZZES_EXPORT
        DESCRIPTION_QUIZ_END_TEXT
        DESCRIPTION_QUIZ_END_TEXT_FAIL
        DESCRIPTION_QUIZ_START_TEXT
        DESCRIPTION_QUIZ_TIMEOUT
        DESCRIPTION_QUIZ_TYPE
        DESCRIPTION_RANK_IMAGE
        DESCRIPTION_REBRAND_NAME
        DESCRIPTION_RECURRANCE_DAYS
        DESCRIPTION_RECURRENCE
        DESCRIPTION_RECURRENCES
        DESCRIPTION_RECURRENCE_PATTERN
        DESCRIPTION_REDO_TIME
        DESCRIPTION_REFERRER_SHARE
        DESCRIPTION_REFRESH_TIME
        REFRESH_TIMEOUT_REACHED
        DESCRIPTION_REGIONALITY_SHARE
        DESCRIPTION_RELEVANCE_LEVEL
        DESCRIPTION_REMEMBER_PASSWORD
        DESCRIPTION_REMINDER_TIME
        DESCRIPTION_REPLACEMENT
        DESCRIPTION_REPRESENTATIVE_IMAGE
        DESCRIPTION_REPRESENTATIVE_IMAGE_GALLERY
        DESCRIPTION_REQUIRED
        DESCRIPTION_REQUIRE_SESSION
        DESCRIPTION_RESET_CATEGORY_PERMISSIONS
        DESCRIPTION_RESET_INTRO_ACCEPTANCE
        DESCRIPTION_REVEAL_ANSWERS
        DESCRIPTION_ROLE
        DESCRIPTION_RSS_FEED
        DESCRIPTION_RULES
        DESCRIPTION_SAVE_WARNING_DETAILS
        DESCRIPTION_SCHEDULE_TIME
        DESCRIPTION_SEARCHABLE
        DESCRIPTION_SEARCH_STATISTICS
        DESCRIPTION_SECONDARY_CATEGORIES
        DESCRIPTION_SECONDARY_GROUP
        DESCRIPTION_SEED_COLOUR
        DESCRIPTION_SEG_RECURRENCES
        DESCRIPTION_SELF_LEARNING_CACHE
        DESCRIPTION_SEND_NOTIFICATION
        DESCRIPTION_SEND_TIME
        DESCRIPTION_SEND_TRACKBACKS
        DESCRIPTION_SHOPPING_FORCE_DELETE
        DESCRIPTION_SHOW_AS_EDITED
        DESCRIPTION_SHOW_DOCS
        DESCRIPTION_SHOW_IN_POSTS
        DESCRIPTION_SHOW_IN_POST_PREVIEWS
        DESCRIPTION_SHOW_ON_JOIN_FORM
        DESCRIPTION_SHUFFLE_ANSWERS
        DESCRIPTION_SHUFFLE_QUESTIONS
        DESCRIPTION_SIGN_UP_REMINDER
        DESCRIPTION_SIGN_UP_REMINDER_GROUPS
        DESCRIPTION_SILENCE_FROM_FORUM
        DESCRIPTION_SILENCE_FROM_TOPIC
        DESCRIPTION_SINK_STATE
        DESCRIPTION_SKILLS
        DESCRIPTION_SMART_TOPIC_NOTIFICATION
        DESCRIPTION_SOURCE
        DESCRIPTION_SOURCE_THEME
        DESCRIPTION_STATS_CACHE
        DESCRIPTION_SUBMISSION_STATISTICS
        DESCRIPTION_SUBMITTER_GETS_POINTS
        DESCRIPTION_SUBMIT_POINTS
        DESCRIPTION_SUBSCRIPTION_BODY
        DESCRIPTION_SUBSCRIPTION_LEVEL
        DESCRIPTION_SUBSCRIPTION_LEVEL_2
        DESCRIPTION_SUBSCRIPTION_LEVEL_3
        DESCRIPTION_SUBSCRIPTION_REF_POINT
        DESCRIPTION_SUBSCRIPTION_REF_POINT_OFFSET
        DESCRIPTION_SUBSCRIPTION_SUBJECT
        DESCRIPTION_SUPER_MEMBERS
        DESCRIPTION_SUPER_MODERATORS
        DESCRIPTION_SUPPORTS_WIDE
        DESCRIPTION_SYNDICATE_TO_STOPFORUMSPAM
        DESCRIPTION_SYSTEM_AVATAR
        DESCRIPTION_TAG_TITLE
        DESCRIPTION_TEMPLATES
        DESCRIPTION_TEMPLATES_SEARCH
        DESCRIPTION_TEXTUAL_TAG
        DESCRIPTION_THEME
        DESCRIPTION_THEMEGEN_ALGORITHM
        DESCRIPTION_THEME_IMAGES_CACHE
        DESCRIPTION_THEME_IMAGE_FOR_MENU_ITEM
        DESCRIPTION_THEME_IMAGE_NAME
        DESCRIPTION_THEME_MAPPING
        DESCRIPTION_THUMBNAIL
        DESCRIPTION_TICKET_GUEST_EMAILS_MANDATORY
        DESCRIPTION_TICKET_SEARCH_FAQ
        DESCRIPTION_TICKET_TYPE
        DESCRIPTION_TIED_TO_NEWSLETTER
        DESCRIPTION_TIMEZONE_SITE
        DESCRIPTION_TITLE
        DESCRIPTION_TITLE_SUFFIX
        DESCRIPTION_TOPIC_MOVE_TARGET
        DESCRIPTION_TOPIC_ORDER
        DESCRIPTION_UPLOAD
        DESCRIPTION_UPLOAD_CSV
        DESCRIPTION_UPLOAD_CSV_2
        DESCRIPTION_URL
        DESCRIPTION_URLS_CACHE
        DESCRIPTION_URL_BANNER
        DESCRIPTION_URL_REDIRECT
        DESCRIPTION_USERGROUP_SUBSCRIPTION_COST
        DESCRIPTION_USERGROUP_SUBSCRIPTION_DESCRIPTION
        DESCRIPTION_USERGROUP_SUBSCRIPTION_ENABLED
        DESCRIPTION_USERGROUP_SUBSCRIPTION_GROUP
        DESCRIPTION_USERGROUP_SUBSCRIPTION_LENGTH
        DESCRIPTION_USERGROUP_SUBSCRIPTION_MAIL_END
        DESCRIPTION_USERGROUP_SUBSCRIPTION_MAIL_START
        DESCRIPTION_USERGROUP_SUBSCRIPTION_MAIL_UHOH
        DESCRIPTION_USERGROUP_SUBSCRIPTION_TITLE
        DESCRIPTION_USERGROUP_TITLE
        DESCRIPTION_USERS_ONLINE_STATISTICS
        DESCRIPTION_USES_PRIMARY
        DESCRIPTION_USE_ALL_LANGS
        DESCRIPTION_USE_ALL_THEMES
        DESCRIPTION_USE_ON_ZONES
        DESCRIPTION_USE_TOPICS
        DESCRIPTION_VALIDATED_SIMPLE
        DESCRIPTION_VALIDATE_ALL_POSTS
        DESCRIPTION_VERSION
        DESCRIPTION_VIDEO_DIMENSIONS
        DESCRIPTION_VIDEO_LENGTH
        DESCRIPTION_VIEWS_PER_DAY
        DESCRIPTION_VIEWS_PER_HOUR
        DESCRIPTION_VIEWS_PER_MONTH
        DESCRIPTION_VIEWS_PER_WEEK
        DESCRIPTION_VIEW_REPORTS
        DESCRIPTION_VISIBLE
        DESCRIPTION_WATERMARK
        DESCRIPTION_WEBCLIPICON
        DESCRIPTION_WEBSTANDARDS_ON_PREVIEW_0
        DESCRIPTION_WEBSTANDARDS_ON_PREVIEW_1
        DESCRIPTION_WEBSTANDARDS_ON_PREVIEW_2
        DESCRIPTION_WELCOME_EMAIL_NEWSLETTER
        DESCRIPTION_WELCOME_EMAIL_SUBJECT
        DESCRIPTION_WELCOME_EMAIL_TEXT
        DESCRIPTION_WELCOME_EMAIL_USERGROUP
        DESCRIPTION_WELCOME_EMAIL_USERGROUP_TYPE
        DESCRIPTION_WELCOME_MESSAGE
        DESCRIPTION_WHATSNEW_CATEGORIES_SELECT
        DESCRIPTION_WHETHER_BANNED_IP
        DESCRIPTION_WHETHER_CONVERT_COMCODE
        DESCRIPTION_WHETHER_MAKE_WARNING
        DESCRIPTION_WORD
        DESCRIPTION_WORDPRESS_DB_NAME
        DESCRIPTION_WORDPRESS_DB_PASSWORD
        DESCRIPTION_WORDPRESS_DB_USERNAME
        DESCRIPTION_WORDPRESS_HOST_NAME
        DESCRIPTION_WORDPRESS_TABLE_PREFIX
        DESCRIPTION_WORD_SUBSTR
        DESCRIPTION_ZONE_BASE_URL
        DESCRIPTION_ZONE_RENAME
        DESCRIPTION_ZONE_RENAME_DEFAULT_ZONE
        DESCRIP_CSS_EQUATION_HELPER
        DESTINATION_FORUM
        DESTINATION_PATH
        DESTINATION_TOPIC
        DESTINATION_TOPIC_ID
        DESTINATION_URL
        DETECT_JAVASCRIPT
        DETECT_LANG_BROWSER
        DETECT_LANG_FORUM
        DEVELOPMENT_VIEWS
        DEV_DATABASE_SETTINGS
        DEV_DATABASE_SETTINGS_HELP
        DIFF
        DIFF_NONE
        DIFF_TOO_MUCH
        DIMENSIONS
        DIRECTIVE__BOX
        DIRECTIVE__IF
        DIRECTIVE__IF_EMPTY
        DIRECTIVE__IF_NON_EMPTY
        DIRECTIVE__LOOP
        DIRECTIVE__SET
        DIRECTIVE__WHILE
        DIRECTORY
        DIRECTORY_NOT_FOUND
        DIRECTORY_RELATIVE_INSTALL
        DISABLED_FUNCTION
        DISCUSS_BELOW
        DISK_USAGE
        DISMISS_TIP
        DISPATCH
        DISPATCH_CONFIRMATION_MESSAGE
        DISPATCH_MAIL_PREVIEW
        DISPATCH_MAIL_PREVIEW_DESCRIPTION
        DISPATCH_STATUS
        DISPLAYING_BANNER_ROTATION
        DISPLAY_ELEMENTS
        DISPLAY_NAME_GENERATOR
        DISPLAY_PHP_ERRORS
        DISPLAY_REVIEW_STATUS
        DISPLAY_TYPE
        DLOAD_SEARCH_INDEX
        DOBS
        DOC_ABSORB_PERMISSIONS
        DOC_ACTIONLOG
        DOC_ADDONS
        DOC_ADD_MEMBER
        DOC_ADMIN_ZONE
        DOC_AGGREGATE_TYPES
        DOC_AUDIT
        DOC_AUTHORS
        DOC_AWARDS
        DOC_BACKUPS
        DOC_BACKUPS_2
        DOC_BANNERS
        DOC_BASE_CONFIGURATION
        DOC_BLOGS
        DOC_BREADCRUMB_OVERRIDES
        DOC_CALENDAR
        DOC_CATALOGUES
        DOC_CHAT
        DOC_CLEANUP_TOOLS
        DOC_CLUBS
        DOC_CMS
        DOC_CODE_EDITOR
        DOC_COMCODE_PAGE_EDIT
        DOC_COMMANDR
        DOC_CONFIGURATION
        DOC_CONTENT_REVIEWS
        DOC_CRITICISE_LANGUAGE_PACK
        DOC_CSS
        DOC_CUSTOM_COMCODE
        DOC_CUSTOM_PROFILE_FIELDS
        DOC_CUSTOM_PROFILE_FIELDS_STATS
        DOC_DASHBOARD
        DOC_DELETE_LURKERS
        DOC_DOWNLOADS
        DOC_DOWNLOAD_MEMBER_CSV
        DOC_ECOMMERCE
        DOC_EDIT_MEMBER
        DOC_EMAIL_LOG
        DOC_EMOTICONS
        DOC_ERROR_LOG
        DOC_EXPORT_POINTS
        DOC_FIELD_FILTERS
        DOC_FILEDUMP
        DOC_FIND_LANG_STRING_TIP
        DOC_FORUMS
        DOC_FORUM_GROUPINGS
        DOC_FRONTEND_ICONS
        DOC_GALLERIES
        DOC_GALLERY_IMPORT
        DOC_GROUPS
        DOC_HELPER_PANEL
        DOC_IMPORT
        DOC_IMPORT_MEMBER_CSV
        DOC_INSTALL_GEOLOCATION_DATA
        DOC_INVESTIGATE_USER
        DOC_IP_BAN
        DOC_LDAP
        DOC_LOAD_TIMES
        DOC_LOGOWIZARD
        DOC_MANAGE_GROUP_MEMBER_TIMEOUTS
        DOC_MARKUP
        DOC_MEMBERS
        DOC_MENUS
        DOC_MERGE_MEMBERS
        DOC_MESSAGING
        DOC_MULTI_MODERATIONS
        DOC_NETLINK
        DOC_NETLINK_1
        DOC_NEWS
        DOC_NEWSLETTER
        DOC_NOTIFICATIONS
        DOC_PAGES
        DOC_PAGE_MATCH_KEY_ACCESS
        DOC_PAGE_STATISTICS
        DOC_PERMISSIONS_TREE
        DOC_PHPINFO
        DOC_POINTS
        DOC_POINTSTORE
        DOC_POLLS
        DOC_POST_TEMPLATES
        DOC_PRIVILEGES
        DOC_QUIZZES
        DOC_QUOTES
        DOC_REALTIME_RAIN
        DOC_REDIRECTS
        DOC_REVISIONS
        DOC_SECONDARY_GROUP_MEMBERSHIP
        DOC_SECURITY
        DOC_SECURITY_LOG
        DOC_SETUP
        DOC_SETUPWIZARD
        DOC_SITEMAP_EDITOR
        DOC_SSL
        DOC_STAFF
        DOC_STATISTICS
        DOC_STRUCTURE
        DOC_STYLE
        DOC_SUBMISSION_STATISTICS
        DOC_SUPERDEBRAND
        DOC_TEMPLATES
        DOC_THEMES
        DOC_THEMEWIZARD
        DOC_TICKETS
        DOC_TOOLS
        DOC_TOP_REFERRERS
        DOC_TOP_SEARCH_KEYWORDS
        DOC_TRACKBACKS
        DOC_TRANSLATE
        DOC_TRANSLATE_CONTENT
        DOC_UNVALIDATED
        DOC_USERGROUP_SUBSCRIPTION
        DOC_USERS_ONLINE_STATISTICS
        DOC_WELCOME_EMAILS
        DOC_WELCOME_EMAIL_PREVIEW
        DOC_WHATSNEW
        DOC_WIKI
        DOC_WORDFILTER
        DOC_WRITING
        DOC_ZONES
        DOC_ZONE_EDITOR
        DODGY_GET_HACK
        DOES_RECUR
        DOMAIN
        DOMAIN_EXAMPLE
        DOMAIN_STATISTICS
        DONT_CLICK_TRACKBACK
        DONT_MAKE_PERIODIC_NEWSLETTER
        DOUBLE_INSTALL_DO
        DOWNLOADS_DEFAULT_SORT_ORDER
        DOWNLOADS_SUBCAT_NARROWIN
        DOWNLOAD_ADDED
        DOWNLOAD_AS_CSV
        DOWNLOAD_ENTRIES_PER_PAGE
        DOWNLOAD_GALLERY_ROOT
        DOWNLOAD_IMAGES
        DOWNLOAD_MEMBER_CSV
        DOWNLOAD_STATS_BACKUP
        DOWNLOAD_STATS_RANGE
        DOWNLOAD_SUBCATS_PER_PAGE
        DO_NOT_FILL_ME_SPAMMER_BLACKHOLE
        DO_TIMEZONE_CONV
        DPLU_SECONDS
        DRAWS_FROM
        DT_FIELDMAPS
        DT_GRID
        DT_TABULAR
        DT_TITLELIST
        DUAL_FORMAT_NEWSLETTERS
        DUE_NOT
        DUE_NOW
        DUE_TIME
        DUE_TIME_AGO
        DUPLICATE
        DUPLICATE_AGGREGATE_INSTANCE
        DUPLICATE_CNS
        DUPLICATE_PAGE_REDIRECT
        DYNAMIC_FIREWALL
        EAGER_WYSIWYG
        ECOMMERCE_TEST_MODE
        ECOM_ADDED_SUBSCRIP
        ECOM_CATD_image
        ECOM_CATD_price_pre_tax
        ECOM_CATD_reduction_end
        ECOM_CATD_reduction_start
        ECOM_CATD_sku
        ECOM_CATD_stock_level
        ECOM_CATD_stock_level_maintain
        ECOM_CATD_stock_level_warn_at
        ECOM_CATD_tax_type
        ECOM_CATD_weight
        ECOM_CAT_description
        ECOM_CAT_image
        ECOM_CAT_price_pre_tax
        ECOM_CAT_product_title
        ECOM_CAT_reduction_end
        ECOM_CAT_reduction_start
        ECOM_CAT_sku
        ECOM_CAT_stock_level
        ECOM_CAT_stock_level_maintain
        ECOM_CAT_stock_level_warn_at
        ECOM_CAT_tax_type
        ECOM_CAT_weight
        ECOM_ORDER
        EDITAREA
        EDITING
        EDITING_CONTENT_IN_LANGUAGE_STAFF
        EDITING_CUSTOM_FIELDS_HELP
        EDITING_ON_WRONG_MSN
        EDIT_AGGREGATE_TYPES
        EDIT_AGGREGATE_TYPE_INSTANCE
        EDIT_AND_SEND
        EDIT_AWARD_TYPE
        EDIT_BANNER
        EDIT_BANNER_TYPE
        EDIT_CALENDAR_EVENT
        EDIT_CALENDAR_EVENT__NON_PUBLIC
        EDIT_CATALOGUE
        EDIT_CATALOGUE_CATEGORY
        EDIT_CATALOGUE_ENTRY
        EDIT_CATALOGUE_FOR_CONTENT_TYPE
        EDIT_CHATROOM
        EDIT_CLUB
        EDIT_CONFLICT_WARNING
        EDIT_CSS
        EDIT_CUSTOM_COMCODE_TAG
        EDIT_CUSTOM_FIELDS
        EDIT_CUSTOM_PRODUCT
        EDIT_CUSTOM_PROFILE_FIELD
        EDIT_DEFAULT_THEME_WARNING
        EDIT_DESCRIPTIONS
        EDIT_DOWNLOAD
        EDIT_DOWNLOAD_CATEGORY
        EDIT_DOWNLOAD_LICENCE
        EDIT_EMOTICON
        EDIT_EVENT_TYPE
        EDIT_FORUM
        EDIT_FORUM_GROUPING
        EDIT_FORWARDING_DOMAIN
        EDIT_GALLERY
        EDIT_GROUP
        EDIT_IMAGE
        EDIT_IN_FULL_PAGE_EDITOR
        EDIT_MEDIA
        EDIT_MEMBER
        EDIT_MENU_ITEM
        EDIT_MERGE_AUTHORS
        EDIT_MESSAGE
        EDIT_MULTI_MODERATION
        EDIT_MY_AUTHOR_PROFILE
        EDIT_NEWS
        EDIT_NEWSLETTER
        EDIT_NEWS_BLOG
        EDIT_NEWS_CATEGORY
        EDIT_OR_CHOOSE_POLL
        EDIT_OWN_AUTHOR
        EDIT_OWN_BANNER
        EDIT_OWN_CALENDAR_EVENT
        EDIT_OWN_CALENDAR_EVENT__NON_PUBLIC
        EDIT_OWN_CATALOGUE_ENTRY
        EDIT_OWN_DOWNLOAD
        EDIT_OWN_GALLERY
        EDIT_OWN_LIVE_POLL
        EDIT_OWN_MEDIA
        EDIT_OWN_NEWS
        EDIT_OWN_NEWS_BLOG
        EDIT_OWN_NEWS_CATEGORY
        EDIT_OWN_POLL
        EDIT_OWN_QUIZ
        EDIT_PERIODIC_NEWSLETTER
        EDIT_PERMISSION_PRODUCT
        EDIT_POLL
        EDIT_POP3_DOMAIN
        EDIT_POST_TEMPLATE
        EDIT_PRIVATE_CALENDAR_EVENT
        EDIT_PRIVATE_THIS_CALENDAR_EVENT
        EDIT_PROFILE
        EDIT_PS_CONFIG
        EDIT_PUBLIC_CALENDAR_EVENT
        EDIT_PUBLIC_THIS_CALENDAR_EVENT
        EDIT_QUIZ
        EDIT_SELECTED_FIELD
        EDIT_STAFF
        EDIT_TEMPLATES
        EDIT_TEXT
        EDIT_THEME
        EDIT_THEME_IMAGE
        EDIT_THEME_IMAGES
        EDIT_THIS_AGGREGATE_TYPE_INSTANCE
        EDIT_THIS_AWARD_TYPE
        EDIT_THIS_BANNER
        EDIT_THIS_BANNER_TYPE
        EDIT_THIS_CALENDAR_EVENT
        EDIT_THIS_CHATROOM
        EDIT_THIS_CLUB
        EDIT_THIS_CUSTOM_COMCODE_TAG
        EDIT_THIS_CUSTOM_PROFILE_FIELD
        EDIT_THIS_EMOTICON
        EDIT_THIS_EVENT_TYPE
        EDIT_THIS_FORUM
        EDIT_THIS_FORUM_GROUPING
        EDIT_THIS_GALLERY
        EDIT_THIS_GROUP
        EDIT_THIS_MULTI_MODERATION
        EDIT_THIS_NEWSLETTER
        EDIT_THIS_POLL
        EDIT_THIS_POST_TEMPLATE
        EDIT_THIS_QUIZ
        EDIT_THIS_USERGROUP_SUBSCRIPTION
        EDIT_THIS_WELCOME_EMAIL
        EDIT_TIME_LIMIT
        EDIT_UNDER
        EDIT_USERGROUP_SUBSCRIPTION
        EDIT_VIDEO
        EDIT_WARNING
        EDIT_WELCOME_EMAIL
        EDIT_WIKI_PAGE_BODY
        EDIT_WIKI_PAGE_SUBJECT
        EDIT_WIKI_POST_BODY
        EDIT_WIKI_POST_SUBJECT
        EDIT_ZONE
        EDIT_ZONE_EDITOR
        EMAIL_CONFIRM_JOIN
        EMAIL_LOG
        EMAIL_NEWSLETTER
        EMBED_FULL_ARTICLES
        EMOTICON_IS_SPECIAL
        EMOTICON_RELEVANCE_LEVEL_0
        EMOTICON_RELEVANCE_LEVEL_1
        EMOTICON_RELEVANCE_LEVEL_2
        EMOTICON_RELEVANCE_LEVEL_3
        EMOTICON_RELEVANCE_LEVEL_4
        ENABLED
        ENABLE_ADD_TOPIC_BTN_IN_TOPIC
        ENABLE_ANIMATIONS
        ENABLE_AUTOBAN
        ENABLE_BIRTHDAYS
        ENABLE_BOOLEAN_SEARCH
        ENABLE_CSV_RECOMMEND
        ENABLE_ECARDS
        ENABLE_FEEDBACK
        ENABLE_FORUM_DUPE_BUTTONS
        ENABLE_HIGHLIGHT_NAME
        ENABLE_KEYWORD_DENSITY_CHECK
        ENABLE_LAX_COMCODE
        ENABLE_LIKES
        ENABLE_MARKUP_WEBSTANDARDS
        ENABLE_MARK_FORUM_READ
        ENABLE_MARK_TOPIC_UNREAD
        ENABLE_MULTI_QUOTE
        ENABLE_POST_EMPHASIS
        ENABLE_PREVIEWS
        ENABLE_PRIVACY_TAB
        ENABLE_PT_FILTERING
        ENABLE_PT_RESTRICT
        ENABLE_PURCHASE
        ENABLE_RSS
        ENABLE_RSS_ADVERTISING
        ENABLE_SECONDARY_NEWS
        ENABLE_SEO_FIELDS
        ENABLE_SKIP_SIG
        ENABLE_SPELL_CHECK
        ENABLE_STAFF_NOTES
        ENABLE_SUNK
        ENABLE_THEME_IMG_BUTTONS
        ENABLE_USER_ONLINE_GROUPS
        ENABLE_VIEWS_SIGS_OPTION
        ENCRYPTED
        ENCRYPTION_ERROR
        ENCRYPTION_KEY
        ENCRYPTION_KEY_ERROR
        END_DATE_AND_TIME
        ENQUIRE_ON_NEW_IPS
        ENSURE_FINISH_PROFILE
        ENTER_IMAP_DETAILS
        ENTER_MENU_CAPTION
        ENTER_MENU_NAME
        ENTRY_POINT
        ENTRY_POINTS_DESCRIPTION
        ENVELOPER_OVERRIDE
        ERRORS_IN_ERROR_LOG
        ERRORS_IN_PERMISSIONS_LOG
        ERROR_FRACTIONAL_EDIT
        ERROR_HANDLING_RSS_FEED
        ERROR_INACCESSIBLE_DIR
        ERROR_LOG
        ERROR_LOGGING_PROBABLY_BROKEN
        ERROR_MAIL
        ERROR_NON_TERMINAL
        ERROR_NOT_ACCEPT_CONTAINER
        ERROR_NOT_ACCEPT_IMAGES
        ERROR_NOT_ACCEPT_VIDEOS
        ERROR_NOT_CORRECT_DATABASE
        ERROR_NOT_CORRECT_FILES
        ERROR_NOT_CORRECT_LINKING
        ERROR_NOT_CORRECT_LINKING_POSSIBLY
        ERROR_NOT_CORRECT_VERSION
        ERROR_OCCURRED_SUBJECT
        EVAL_ERROR
        EVAL_ERROR_QUOTE_ISSUE
        EVAL_HACK
        EVENTS
        EVENT_CANNOT_AROUND
        EVENT_CONFLICT_DETECTED
        EVENT_TEXT
        EVENT_TYPES
        EVIL_POSTED_FORM_EXPIRED_TOKEN_HACK
        EVIL_POSTED_FORM_HACK
        EVIL_POSTED_FORM_MISMATCHED_TOKEN_HACK
        EVIL_POSTED_FORM_NO_TOKEN_HACK
        EVIL_POSTED_FORM_UNKNOWN_TOKEN_HACK
        EXAMPLE_COPYRIGHT
        EXAMPLE_DESCRIPTION
        EXAMPLE_HEADER_TEXT
        EXAMPLE_KEYWORDS
        EXAMPLE_QUESTIONS
        EXAMPLE_SITE_NAME
        EXAMPLE_SITE_SCOPE
        EXCEPT_IN_USERGROUPS
        EXECUTE_TIME
        EXISTING_MENU
        EXISTING_REDIRECTS
        EXISTING_THEMES
        EXPANDED_BRANCH
        EXPANDED_BY_DEFAULT
        EXPENSES
        EXPIRY_DATE
        EXPIRY_MOVE_DAYS_HIGHER
        EXPIRY_MOVE_DAYS_LOWER
        EXPIRY_MOVE_TARGET
        EXPLANATORY_TEXT
        EXPORT
        EXPORT_ADDON
        EXPORT_ADDON_TITLE
        EXPORT_CATALOGUE_ENTRIES
        EXPORT_CSV_BANNERS
        EXPORT_FILES
        EXPORT_ICAL
        EXPORT_ICAL_TEXT
        EXPORT_LANGUAGE
        EXPORT_ORDER_LIST
        EXPORT_ORDER_LIST_TEXT
        EXPORT_POINTS
        EXPORT_QUIZ
        EXPORT_QUIZ_TEXT
        EXPORT_STATS_TO_CSV
        EXPORT_THEME
        EXTENDED_GROUP_TITLE_NORMAL
        EXTENDED_GROUP_TITLE_RANK
        EXTENDED_NEWS_CATEGORIES_SET
        EXTEND_PROBATION
        EXTERNALLY_BANNED_ADDRESSES
        EXTERNAL_FEED
        EXTERNAL_LINKS
        EXTRACTING_MESSAGE
        EXTRA_BCC_ADDRESSES
        EXTRA_CC_ADDRESSES
        EXTRA_HITS
        EXTRA_HITS_DESCRIPTION
        EXTRA_SUBSCRIPTION_MAIL
        FAILED_ENTRY
        FAILED_LOGINS
        FAILED_TO_UPLOAD_BACKUP_BODY
        FAILED_TO_UPLOAD_BACKUP_SUBJECT
        FAVICON
        FAX_NUMBER
        FEEDBACK_AND_NOTES
        FEEDBACK_LINK
        FFMPEG_PATH
        FIELDS_EXISTING
        FIELDS_NEW
        FIELDS_NEW_HELP
        FIELDS_UNMATCH
        FIELD_FILTERS
        FIELD_OPTIONS
        FIELD_TYPES__CHOICES
        FIELD_TYPES__MAGIC
        FIELD_TYPES__NUMBERS
        FIELD_TYPES__OTHER
        FIELD_TYPES__REFERENCES
        FIELD_TYPES__TEXT
        FIELD_TYPES__UPLOADSANDURLS
        FIELD_TYPE_author
        FIELD_TYPE_codename
        FIELD_TYPE_color
        FIELD_TYPE_content_link_multi_x
        FIELD_TYPE_content_link_x
        FIELD_TYPE_date
        FIELD_TYPE_email
        FIELD_TYPE_float
        FIELD_TYPE_guid
        FIELD_TYPE_integer
        FIELD_TYPE_isbn
        FIELD_TYPE_just_date
        FIELD_TYPE_just_time
        FIELD_TYPE_list
        FIELD_TYPE_list_multi
        FIELD_TYPE_long_text
        FIELD_TYPE_long_trans
        FIELD_TYPE_member
        FIELD_TYPE_member_multi
        FIELD_TYPE_page_link
        FIELD_TYPE_password
        FIELD_TYPE_picture
        FIELD_TYPE_picture_multi
        FIELD_TYPE_posting_field
        FIELD_TYPE_reference_multi_x
        FIELD_TYPE_reference_x
        FIELD_TYPE_short_text
        FIELD_TYPE_short_text_multi
        FIELD_TYPE_short_trans
        FIELD_TYPE_short_trans_multi
        FIELD_TYPE_tel
        FIELD_TYPE_theme_image_x
        FIELD_TYPE_tick
        FIELD_TYPE_upload
        FIELD_TYPE_upload_multi
        FIELD_TYPE_url
        FIELD_TYPE_url_multi
        FIELD_TYPE_video
        FIELD_TYPE_video_multi
        FILEDUMP
        FILEDUMP_BROKEN
        FILEDUMP_COUNT_FILES
        FILEDUMP_CREATE_FOLDER
        FILEDUMP_DELETE
        FILEDUMP_DELETE_FILE
        FILEDUMP_DELETE_FOLDER
        FILEDUMP_DISK_USAGE
        FILEDUMP_EDIT
        FILEDUMP_EMBED
        FILEDUMP_EXISTING_COUNT
        FILEDUMP_FOLDER_MATCHING
        FILEDUMP_FOLDER_NON_MATCHING
        FILEDUMP_IMAGE_URLS
        FILEDUMP_IMAGE_URLS_LARGE
        FILEDUMP_IMAGE_URLS_MEDIUM
        FILEDUMP_IMAGE_URLS_SMALL
        FILEDUMP_MOVE
        FILEDUMP_NOTIFICATION_MAIL
        FILEDUMP_NOTIFICATION_MAIL_SUBJECT
        FILEDUMP_SIZE
        FILEDUMP_UPLOAD
        FILEDUMP_ZIP
        FILESYSTEM_DOWNLOADS
        FILES_TRANSFERRED
        FILETYPE_ICONS
        FILE_BASE
        FILE_IN_ARCHIVE_TWICE
        FILE_MOVED
        FILE_MOVE_ERROR
        FILE_PERM_GOOD
        FILE_TOO_BIG
        FILE_TYPES
        FILL_IN_DATA_PARAM
        FILTERCODE_OP_CO
        FILTERCODE_OP_EQ
        FILTERCODE_OP_EQE
        FILTERCODE_OP_FT
        FILTERCODE_OP_GT
        FILTERCODE_OP_GTE
        FILTERCODE_OP_LT
        FILTERCODE_OP_LTE
        FILTERCODE_OP_NE
        FILTERCODE_OP_NEE
        FILTERCODE_OP_RANGE
        FILTERCODE_UNKNOWN_FIELD
        FILTER_BY_ALLOW
        FILTER_BY_ZONE
        FINAL_INSTRUCTIONS_A
        FINAL_INSTRUCTIONS_A_SUP
        FINAL_INSTRUCTIONS_B
        FINAL_INSTRUCTIONS_C
        FIND_BROKEN_FILEDUMP_LINKS
        FIND_IN_SITEMAP
        FIND_IN_SITEMAP_EDITOR
        FIND_WINNER
        FIND_WINNERS
        FIXED_WIDTH
        FIX_BROKEN_FILEDUMP_LINKS
        FIX_INVALID_HTML
        FIX_PAST_RECURRENCES
        EDIT_ALL_RECURRENCES
        FLOOD_CONTROL
        FLOOD_CONTROL_ACCESS_SECS
        FLOOD_CONTROL_SCREEN
        FLOOD_CONTROL_SUBMIT_SECS
        FLOOD_TIMELIMIT
        FLOW_MODE_INTERFACE
        FOLDER_CREATE
        FOLDER_DELETE_ERROR
        FOLDER_OVERWRITE_ERROR
        FOOTER_LINKS
        FORCED_PREVIEW_OPTION
        FORCE_GUEST_NAMES
        FORCE_LOCAL_TEMP_DIR
        FORCE_META_REFRESH
        FORCE_TEMPORARY_PASSWORD
        FORMATTING_SYMBOL__AT
        FORMATTING_SYMBOL__LCASE
        FORMATTING_SYMBOL__LENGTH
        FORMATTING_SYMBOL__REPLACE
        FORMATTING_SYMBOL__SUBSTR
        FORMATTING_SYMBOL__TRUNCATE_LEFT
        FORMATTING_SYMBOL__TRUNCATE_SPREAD
        FORMATTING_SYMBOL__UCASE
        FORMATTING_SYMBOL__WCASE
        FORMATTING_SYMBOL__WORDWRAP
        FORUM
        FORUMS_AND_MEMBERS
        LOGIN_ERROR_SECRECY
        CONFIG_OPTION_login_error_secrecy
        FORUM_BASE_URL_EXAMPLE
        FORUM_BASE_URL_INVALID
        FORUM_BASE_URL_UNCHANGED
        FORUM_CACHE_CLEAR
        FORUM_CANNOT_BE_OWN_PARENT
        FORUM_CHOICE
        FORUM_CLASS_aef
        FORUM_CLASS_cns
        FORUM_CLASS_general
        FORUM_CLASS_ipb
        FORUM_CLASS_mybb
        FORUM_CLASS_none
        FORUM_CLASS_obb
        FORUM_CLASS_phpbb
        FORUM_CLASS_smf
        FORUM_CLASS_tf
        FORUM_CLASS_ubb
        FORUM_CLASS_vb
        FORUM_CLASS_wbb
        FORUM_CLASS_wowbb
        FORUM_CLASS_xmb
        FORUM_CURRENTLY_UNSET
        FORUM_DB
        FORUM_DRIVER_NATIVE_LOGIN
        FORUM_GROUPING
        FORUM_GROUPINGS
        FORUM_IN_PORTAL
        FORUM_NOT_FOUND
        FORUM_PATH
        FORUM_PATH_TEXT
        FORUM_POSTS_PER_PAGE
        FORUM_SOFTWARE
        FORUM_STAFF
        FORUM_TOPICS_PER_PAGE
        FORUM_VERSION
        FORWARDING_DESCRIPTION
        FORW_MAINTAIN_URL
        FOR_EXAMPLE
        FOR_PUNISHMENT
        FRACTIONAL_EDIT
        FRACTIONAL_EDITING
        FRACTIONAL_EDIT_CANCELLED
        FRACTIONAL_EDIT_CANCEL_CONFIRM
        FREE_ENTRY_ANSWER
        FROM_ADDON
        FROM_COLOUR
        FROM_EMAIL
        FROM_IMPORTING_SYSTEM
        FROM_NAME
        FRONT_END_PERFORMANCE
        FTP_DIRECTORY
        FTP_DOMAIN
        FTP_DOMAIN_NOT_LIKE_THIS
        FTP_DOWNLOADS
        FTP_ERROR
        FTP_FILES_PER_GO
        FTP_FIREWALL_ERROR
        FTP_FOLDER
        FTP_NEEDED
        FTP_PASSWORD
        FTP_SERVER_URL
        FTP_USERNAME
        FULL_ADDRESS
        FULL_BACKUP
        FULL_PERMISSION_LIST_SEE_FILE
        FUSED_LANG_STRINGS
        FU_ADDON_MANAGEMENT
        FU_ALL_CHMODDED_GOOD
        FU_AUTO_HANDLE
        FU_BUGS
        FU_CHECK_ALL
        FU_CHECK_PERMISSIONS
        FU_CHOICES
        FU_CLEAR_CACHES
        FU_CLOSED_FOR_UPGRADES
        FU_CLOSENESS
        FU_CLOSE_SITE
        FU_CUSTOMISATIONS
        FU_DATABASE_UPGRADE
        FU_DOING_DRY_RUN
        FU_DOWNLOAD
        FU_DRY_RUN
        FU_ERROR_CORRECTION
        FU_ESTIMATED_TIME
        FU_EXTRACTING_WINDOW
        FU_FILE_UPGRADE_INFO
        FU_FILE_UPGRADE_INFO_MANUAL
        FU_FIX_PERMISSIONS
        FU_FTP_INFO
        FU_FTP_NEEDED
        FU_INSTALLED_ADDON
        FU_INSTALLED_BLOCK
        FU_INSTALLED_MODULE
        FU_INTEGRITY_SCAN
        FU_INTEGRITY_SCAN_NO_CSS_MERGE
        FU_LOGIN_FORGOT_PASSWORD_Q
        FU_LOGIN_INFO
        FU_LOGIN_INFO_PASS_FORGET
        FU_MAINTENANCE
        FU_MUST_UPGRADE_CNS
        FU_NEEDS_CHMOD
        FU_NEEDS_UNCHMOD
        FU_NOT_FOR_PATCH
        FU_NO_CNS_UPGRADE
        FU_OPEN_SITE
        FU_REFER_RELEASE_NOTES
        FU_RELEASE_NOTES
        FU_REMOVE_ADDON_FILES
        FU_SAFE_MODE
        FU_STEP
        FU_TAKE_BACKUP
        FU_THEME_ERRORS
        FU_THEME_UPGRADE
        FU_THEME_UPGRADE_FINAL
        FU_TUTORIAL
        FU_UPGRADED_ADDON
        FU_UPGRADED_BLOCK
        FU_UPGRADED_CORE_TABLES
        FU_UPGRADED_MODULE
        FU_UPGRADER_INTRO
        FU_UPGRADER_TITLE
        FU_UPGRADE_CNS
        FU_UPGRADE_MODULES
        FU_UPGRADE_STEPS
        FU_UPGRADING
        FU_UP_INFO_1
        FU_UP_INFO_2
        FU_WILL_MERGE
        FXML_FIELD_NOT_IN_SET
        FXML_FIELD_NOT_IN_SET_SECRETIVE
        FXML_FIELD_PATTERN_FAIL
        FXML_FIELD_SHUNNED
        FXML_FIELD_SHUNNED_SUBSTRING
        FXML_FIELD_TOO_LONG
        FXML_FIELD_TOO_SHORT
        GAE_APPLICATION
        GAE_BUCKET_NAME
        GALLERIES_DEFAULT_SORT_ORDER
        GALLERIES_SUBCAT_NARROWIN
        GALLERY_ENTRIES_FLOW_PER_PAGE
        GALLERY_ENTRIES_REGULAR_PER_PAGE
        GALLERY_FEATURES
        GALLERY_FEEDBACK_FIELDS
        GALLERY_FOR_DOWNLOAD
        GALLERY_IMAGE_LIMIT_HIGH
        GALLERY_IMAGE_LIMIT_LOW
        GALLERY_IMPORT
        GALLERY_IMPORT_SUCCESS_EDIT_TITLES_INLINE
        GALLERY_MEDIA_TITLE_REQUIRED
        GALLERY_MEMBER_SYNCED
        GALLERY_MODE_IS
        GALLERY_NAME_ORDER
        GALLERY_PERMISSIONS
        GALLERY_REP_IMAGE
        GALLERY_SELECTORS
        GALLERY_VIDEO_LIMIT_HIGH
        GALLERY_VIDEO_LIMIT_LOW
        GALLERY_WATERMARKS
        GD_NEEDED
        GD_THUMB_ERROR
        GENERAL_HELP
        GENERAL_SAFETY_LISTING_LIMIT
        GENERAL_SETTINGS
        GENERATE_COMCODE
        GENERATE_PAGE_SITEMAP
        GENERATE_SITEMAP
        GIFT_POINTS_BASE
        GIFT_POINTS_LOG
        GIFT_POINTS_PER_DAY
        GIFT_REWARD_AMOUNT
        GIFT_REWARD_CHANCE
        GIFT_TRANSACTIONS
        GIVE_AWARD
        GLOBAL_DONEXT_ICONS
        GLOBAL_NAVIGATION
        GLOBAL_PRIVILEGES
        GO
        GOOD_PATH
        GOOGLE_ANALYTICS
        GOOGLE_GEOCODE_API_KEY
        GOOGLE_GEOCODE_API_NOT_CONFIGURED
        GOOGLE_GEOCODE_COULD_NOT_CONNECT
        GOOGLE_GEOCODE_COULD_NOT_PARSE
        GOOGLE_GEOCODE_INCOMPLETE
        GOOGLE_GEOCODE_OVER_QUERY_LIMIT
        GOOGLE_GEOCODE_REQUEST_DENIED
        GOOGLE_GEOCODE_UNKNOWN_ERROR
        GOOGLE_GEOCODE_ZERO_RESULTS
        GOOGLE_TRANSLATE_API_KEY
        GRAPH
        GROUP_LEADER
        GROUP_MEMBER_TIMEOUTS
        GROUP_PRIVATE_CHATROOMS
        GROUP_TEXT
        GUEST_ZONE_ACCESS
        GZIP_OUTPUT
        HACKED_BY
        HACKER_DETECTED
        HACK_ATTACK
        HACK_ATTACK_INFO
        HACK_ATTACK_PASSWORD_CHANGE
        HACK_ATTACK_SUBJECT
        HACK_ATTACK_USER
        HACK_BAN_THRESHOLD
        HACK_VERSION
        HANDLE_QUEUED_MESSAGE
        HAS_ALREADY_X_WARNINGS
        HAS_LOW_MEMORY_LIMIT
        HAS_THESE_SUBS
        HAVE_DEFAULT_BANNERS_ADVERTISING
        HAVE_DEFAULT_BANNERS_DONATION
        HAVE_DEFAULT_CATALOGUES_CONTACTS
        HAVE_DEFAULT_CATALOGUES_FAQS
        HAVE_DEFAULT_CATALOGUES_LINKS
        HAVE_DEFAULT_CATALOGUES_PROJECTS
        HAVE_DEFAULT_CPF_SET
        HAVE_DEFAULT_FULL_EMOTICON_SET
        HAVE_DEFAULT_RANK_SET
        HAVE_PERSONAL_GALLERIES
        HEADER_MENU_CALL_STRING
        HEADER_SPLIT_HACK
        HEADER_TEXT
        HEADER_TEXT_ADMINZONE
        HEADER_TEXT_collaboration
        HELP_ADD_NEWSLETTER
        HELP_ADD_WORDFILTER
        HELP_IMPORTING_ADDON
        HELP_INSERT_DISTINGUISHING_TEMPCODE
        HIDDEN_USERGROUP
        HIDE_AWARDEE
        HIDE_DONE
        HIDE_POSTS
        HIT
        HITS_ALLOCATED
        HOLD_ORDER
        HONEYPOT_PHRASE
        HONEYPOT_URL
        HOST
        HOT_TOPIC_DEFINITION
        HOURLY_BREAKDOWNS
        HOVER_FOR_CONTENTS
        HOW_MUCH_THIS_COSTS
        HTACCESS_SEO_PROBLEM
        HTML_ONLY
        HTML_PAGES
        HTML_PREVIEW
        HTML_VERSION
        HTM_SHORT_URLS_CARE
        HTTPAUTH_IS_ENABLED
        HTTP_DOWNLOAD_BAD_URL
        HTTP_DOWNLOAD_CUT_SHORT
        HTTP_DOWNLOAD_CONNECTION_STALLED
        HTTP_DOWNLOAD_NO_SERVER
        HTTP_DOWNLOAD_STATUS_NOT_FOUND
        HTTP_DOWNLOAD_STATUS_SERVER_ERROR
        HTTP_DOWNLOAD_STATUS_UNAUTHORIZED
        HTTP_DOWNLOAD_STATUS_UNKNOWN
        HTTP_REDIRECT_PROBLEM
        IDE_LINKAGE
        ID_ASSISTANCE_TOOL
        IMAGE_THUMBNAILS
        IMAGE_TYPES
        IMAGE_URL
        IMAP
        IMAP_ERROR
        IMAP_NEEDED
        IMMEDIATE_DOWNLOADS
        IMMEDIATE_USE
        IMPLIED_SPAMMER_CONFIDENCE
        IMPORT
        IMPORTANCE
        IMPORTANCE_UPGRADE_DESCRIPTION
        IMPORTANT_GROUPS_PER_PAGE
        IMPORTER
        IMPORT_ADDON
        IMPORT_ADDON_2
        IMPORT_ALL_FINISHED
        IMPORT_BLOG_COMMENTS
        IMPORT_CATALOGUE_ENTRIES
        IMPORT_EMOTICONS
        IMPORT_EMOTICONS_WARNING
        IMPORT_EXPORT_THEME_HELP
        IMPORT_FROM_DB
        IMPORT_ICAL
        IMPORT_ICAL_DONE
        IMPORT_ICAL_TEXT
        IMPORT_MEMBER_CSV
        IMPORT_NEWS
        IMPORT_NEWSLETTER_SUBSCRIBERS
        IMPORT_NEWS_DONE
        IMPORT_NEWS_TEXT
        IMPORT_NOT_IMPORTED
        IMPORT_OF_SKIPPED
        IMPORT_QUESTIONS_TEXT
        IMPORT_SELF_NO
        IMPORT_SESSION
        IMPORT_SESSION_CURRENT
        IMPORT_SESSION_EXISTING_REMAP
        IMPORT_SESSION_NEW
        IMPORT_SESSION_NEW_DELETE
        IMPORT_SESSION_NEW_DELETE_CNS_SATELLITE
        IMPORT_STOCK_RESPONSES
        IMPORT_STOCK_RESPONSES_PT
        IMPORT_TO_BLOG
        IMPORT_UPDATED_ADDONS
        IMPORT_WARNING
        IMPORT_WARNINGS_GIVEN
        IMPORT_WARNING_CONVERT
        IMPORT_WORDPRESS
        IMPORT_WORDPRESS_DONE
        IMPORT_WORDPRESS_USERS
        IMPOSSIBLE_TYPE_USED
        IMPROPERLY_FILLED_IN_BANNERS
        IMPROPERLY_FILLED_IN_THUMBNAIL
        IMPROPERLY_FILLED_IN_UPLOAD
        IMPROPERLY_FILLED_IN_UPLOAD_BANNERS
        INBUILT_CPF_LANG_STRING
        INCLUDE_CMS_ADVERT
        INCLUDE_SITEMAP
        INCLUDE_SITEMAP_NO
        INCLUDE_SITEMAP_OVER
        INCLUDE_SITEMAP_UNDER
        INCLUDE_SUBFOLDERS
        INCOMPATIBILITIES
        INCOMPLETED_TRANSACTION
        INCOMPLETE_ERROR
        INCONSISTENT_BANNER_SET
        INCREMENTAL_BACKUP
        INDEX
        INFINITE_SCROLLING
        INFOSITE
        INHERIT_CSS
        INLINE_PP_ADVERTISE
        INPUT_NECESSARY_PARAMETER
        INPUT_OPTIONAL_PARAMETER
        INSERT_ABSTRACTION_SYMBOL
        INSERT_ARITHMETICAL_SYMBOL
        INSERT_DIRECTIVE
        INSERT_DISTINGUISHING_TEMPCODE
        INSERT_FORMATTING_SYMBOL
        INSERT_LOGICAL_SYMBOL
        INSERT_PARAMETER
        INSERT_PROGRAMMATIC_SYMBOL
        INSERT_SYMBOL
        INSTALLED_ADDON
        INSTALLED_BLOCK
        INSTALLED_CNS
        INSTALLED_MODULE
        INSTALLER
        INSTALLER_STEP
        INSTALLPROFILE
        INSTALL_ADDON
        INSTALL_AND_UNINSTALL
        INSTALL_COMPLETE
        INSTALL_GEOLOCATION_DATA
        INSTALL_LOG
        INSTALL_LOG_BELOW
        INSTALL_SLOW_SERVER
        INSTALL_THEME_SUCCESS
        INSTALL_WRITE_ERROR
        INST_POST_ERROR
        INST_SUBSCRIPTION_LEVEL
        INTERFERING_AD_SCRIPT
        INTERNAL_ERROR
        INTERNATIONALISATION
        INTRO_ANSWER
        INTRO_FORUM_ID
        INTRO_GENERATE_PAGE_SITEMAP
        INVALID_ATTACHMENT
        INVALID_COLOUR
        INVALID_DOMAIN
        INVALID_FILE_TYPE_ADMIN
        INVALID_FORUM_DATABASE
        INVALID_LANG_CODE
        INVALID_URL
        INVALID_ZONE_BASE_URL
        INVESTIGATE_USER
        INVESTIGATE_USER_BY_IP
        INVESTIGATE_USER_BY_MEMBER_ID
        INVESTIGATE_USER_BY_USERNAME
        INVITES_ENABLED
        INVITES_PER_DAY
        INVOICES
        INVOICE_AMOUNT_TEXT
        INVOICE_SPECIAL
        IPN_ADDRESS
        IPN_ADDRESS_TEST
        IPN_BAD_TRIAL
        IPN_DIGEST
        IPN_EMAIL_ERROR
        IPN_PASSWORD
        IPN_SOCKET_ERROR
        IPN_SUB_PERIOD_WRONG
        IPN_SUB_RECURRING_WRONG
        IPN_UNVERIFIED
        IP_ADDRESSES
        IP_ADDRESS_DISTRIBUTION
        IP_ADDRESS_NOT_VALID
        IP_ADDRESS_NOT_VALID_MAKE_UNBANNABLE
        IP_BANNED
        IP_BANNING_WILDCARDS
        IP_BANS
        IP_BAN_LOG_AUTOBAN_ANTISPAM
        IP_FORWARDING
        IP_FORWARDING_CHANGE
        IP_LIST
        IP_STRICT_FOR_SESSIONS
        IP_UNBANNED
        IP_VERIFY_MAIL
        IP_VERIFY_MAIL_SUBJECT
        IS_DIRECTORY
        IS_MEMBER_SYNCHED
        IS_ON_ANONYMOUS_POSTS_PRIVATE_TOPICS
        IS_ON_AUTOMATIC_MARK_TOPIC_READ
        IS_ON_COMCODE_PAGE_CHILDREN
        IS_ON_EMOTICON_CHOOSERS
        IS_ON_INVISIBILITY
        IS_ON_POST_MAP
        IS_ON_POST_TITLES
        IS_ON_SHOW_ONLINE
        IS_ON_TIMEZONE_DETECTION
        IS_ON_TOPIC_DESCRIPTIONS
        IS_ON_TOPIC_EMOTICONS
        IS_PRESENTED_AT_INSTALL
        IS_PRIVATE_CLUB
        IS_PRIVATE_CLUB_DESCRIPTION
        IS_THREADED
        IS_TRANSPARENT_REDIRECT
        IS_TREE
        ITEMS_HERE
        JAVASCRIPT_ERROR
        JAVASCRIPT_EXECUTED
        JAVASCRIPT_REQUIRED
        JPEG_QUALITY
        REPAIR_IMAGES
        CONFIG_OPTION_repair_images
        JSON_ONLY
        JS_CAPTCHA
        JS_OVERLAYS
        JUMP_TO_FOLDER
        JUST_PRESETS
        KEEP_ADDING_QUESTION
        KEEP_BLOGS
        KEEP_PERSONAL_GALLERIES
        KEEP_SUBFOLDERS
        KEEP_WORDFILTER
        KEYSET_PAGINATION
        KEYWORDS
        KEYWORDS_SHARE
        LAME_SPAM_HACK
        LANGUAGE_CACHE
        LANGUAGE_CORRUPTION
        LANGUAGE_FILE
        LANGUAGE_FINE
        LAST_BACKUP
        LAST_BACKUP_TODAY
        LAST_BACKUP_YESTERDAY
        LAST_HERE
        LAST_RAN_AT
        LDAP
        LDAP_ALLOW_JOINING
        LDAP_BASE_DN
        LDAP_CANNOT_CHECK_PASSWORDS
        LDAP_DISABLED
        LDAP_GROUP_CLASS
        LDAP_GROUP_SEARCH_QUALIFIER
        LDAP_HOSTNAME
        LDAP_INTRO
        LDAP_IS_ENABLED
        LDAP_IS_WINDOWS
        LDAP_LOGIN_QUALIFIER
        LDAP_MEMBER_CLASS
        LDAP_MEMBER_PROPERTY
        LDAP_MEMBER_SEARCH_QUALIFIER
        LDAP_NONE_BIND_LOGINS
        LDAP_SYNC
        LDAP_SYNC_GROUPS_ADD
        LDAP_SYNC_GROUPS_DELETE
        LDAP_SYNC_MEMBERS_DELETE
        LDAP_VERSION
        LEADER_BOARD_SHOW_STAFF
        LEADER_BOARD_SIZE
        LEADER_BOARD_START_DATE
        LEAVE
        LEAVE_PERIODIC_NEWSLETTER
        LEAVE_TRACKBACK
        LEECH_BLOCK
        LENGTH_UNITS
        LENGTH_UNIT_d
        LENGTH_UNIT_m
        LENGTH_UNIT_w
        LENGTH_UNIT_y
        LICENCE
        LICENCES
        LINE
        LINK
        LINKED_ID
        LINKS
        LISTING_DIRECTORIES
        LISTING_FILES
        LIVE_DATABASE_SETTINGS
        LIVE_DATABASE_SETTINGS_HELP
        LOAD_FILESYSTEM_FILES
        LOAD_FTP_FILES
        LOAD_SAVED_WARNING
        LOAD_TIME
        LOAD_TIMES
        LOAD_TIMES_RANGE
        LOCAL_PAYMENT_NOT_SUPPORTED
        LOGGING
        LOGICAL_SYMBOL__AND
        LOGICAL_SYMBOL__EQ
        LOGICAL_SYMBOL__GT
        LOGICAL_SYMBOL__LT
        LOGICAL_SYMBOL__NEQ
        LOGICAL_SYMBOL__NOT
        LOGICAL_SYMBOL__OR
        LOGO
        LOGOWIZARD
        LOGOWIZARD_1_DESCRIBE
        LOGOWIZARD_3_DESCRIBE
        LOGO_THEME_IMAGE
        LOGS
        LONG_GOOGLE_COOKIES
        LOWLEVEL_PERMISSIONS
        LOW_DISK_SPACE_MAIL
        LOW_DISK_SPACE_SUBJECT
        LOW_MEMORY_LIMIT
        MADE_IP_BANNABLE
        MADE_IP_UNBANNABLE
        MAILS_PER_SEND
        MAIL_BAD_TAG
        MAIL_BODY
        MAIL_COST
        MAIL_DOCTYPE
        MAIL_DOMAIN
        MAIL_END
        MAIL_ERROR_CONNECT
        MAIL_ERROR_CONNECT_AUTH
        MAIL_ERROR_CONNECT_PASSWORD
        MAIL_ERROR_CONNECT_USERNAME
        MAIL_ERROR_DATA
        MAIL_ERROR_FROM
        MAIL_ERROR_TO
        MAIL_FAIL
        MAIL_FLASHWORD
        MAIL_FOLDER
        MAIL_IP_ADDRESS_REPORT_BODY
        MAIL_IP_ADDRESS_REPORT_SUBJECT
        MAIL_JAVASCRIPT
        MAIL_LOCAL_REF
        MAIL_LONG_LINE
        MAIL_POSITIONING
        MAIL_PROLOG
        MAIL_QUEUE
        MAIL_QUEUE_DEBUG
        MAIL_REQUEST_CUSTOM
        MAIL_REQUEST_CUSTOM_BODY
        MAIL_REQUEST_FORWARDING
        MAIL_REQUEST_POP3
        MAIL_REQUEST_QUOTA
        MAIL_SERVER
        MAIL_START
        MAIL_UHOH
        MAIL_UNSUBSCRIBE
        MAIL_WAS_LOGGED
        MAIL_WAS_QUEUED
        MAIL_WEB_VERSION
        MAIL_WIDTH
        MAIN_CATEGORY
        MAIN_FORUM_NAME
        MAKE_MEMBER_GROUP_LEADER
        MAKE_PERIODIC_NEWSLETTER
        MANAGE_AWARDS
        MANAGE_BANNERS
        MANAGE_BLOGS
        MANAGE_CALENDARS
        MANAGE_CATALOGUES
        MANAGE_CHATROOMS
        MANAGE_CLUBS
        MANAGE_DOWNLOADS
        MANAGE_FORUMS
        MANAGE_GALLERIES
        MANAGE_NEWS
        MANAGE_NEWSLETTER
        MANAGE_POLLS
        MANAGE_QUIZZES
        MANAGE_STAFF
        MANAGE_THEMES
        MANAGE_TICKET_TYPES
        MANAGE_TRACKBACKS
        MANAGE_USERGROUPS
        MANAGE_WIKI
        MANAGE_WORDFILTER
        MANUAL_CHMOD_TMP_FILE
        MANUAL_GALLERY_CODENAME
        MANUAL_GALLERY_MEDIA_TYPES
        MANUAL_GALLERY_PARENT
        MANUAL_META
        MANUAL_PRIVACY
        MANUAL_SUBSCRIPTIONS
        MANUAL_SUBSCRIPTION_EXPIRY_NOTICE
        MANUAL_SUBSCRIPTION_NOTIFICATION_MAIL
        MANUAL_SUBSCRIPTION_NOTIFICATION_MAIL_SUBJECT
        MANUAL_TRANSACTION
        MANUAL_TRANSACTION_IDENTIFIER
        MANUAL_TRANSACTION_TEXT
        MANUAL_WCAG_ANIMATION
        MANUAL_WCAG_SCRIPT
        MANUAL_WCAG_SEMANTIC_BLOCKQUOTE
        MANUAL_WCAG_SEMANTIC_LIST
        MANY_PANEL_TYPES
        MARK_AS_DELIVERED
        MARK_AS_SPAMMER
        MARK_TASK_DONE
        MASQUERADE_AS_MEMBER
        MASS_DELETE
        MASTER_PASSWORD
        MATCH_KEY
        MATCH_KEYS
        MATCH_KEY_MESSAGES
        MATCH_KEY_MESSAGE_FIELD
        MAXIMUM_AUTOCOMPLETE_SUGGESTIONS
        MAXIMUM_DOWNLOAD
        MAXIMUM_GAMBLE_AMOUNT
        MAXIMUM_GAMBLE_MULTIPLIER
        MAXIMUM_IMAGE_SIZE
        MAXIMUM_PASSWORD_LENGTH
        MAXIMUM_SIZE_INCLUSION
        MAXIMUM_UPLOAD_STAFF
        MAXIMUM_USERNAME_LENGTH
        MAXIMUM_USERS
        MAX_ATTACHMENTS_PER_POST
        MAX_AVATAR_DIMENSIONS
        MAX_CHAT_LOBBY_FRIENDS
        MAX_DAILY_UPLOAD_MB
        MAX_FILE_SIZE
        MAX_FORUM_DETAIL
        MAX_FORUM_INSPECT
        MAX_IP_ADDRESSES_PER_SUBSCRIBER
        MAX_MEMBER_TITLE_LENGTH
        MAX_MONIKER_LENGTH
        MAX_NEWSLETTER_WHATSNEW
        MAX_POST_LENGTH_COMCODE
        MAX_QUOTA
        MAX_SIG_LENGTH_COMCODE
        MAX_SIZE
        MAX_THREAD_DEPTH
        MAYBE_NOT_FATAL
        MD_DEFAULT_SORT_ORDER
        MEDIA_FILE_IN_SET
        MEDIA_TYPE_
        MEMBERS_ALSO_ON_IP
        MEMBERS_MAY_ALTER_THEME
        MEMBERS_PER_PAGE
        MEMBER_ACCESS
        MEMBER_ADDED_TO_GROUP
        MEMBER_BAD_STRONGHOLD
        MEMBER_BANNED
        MEMBER_CHARGED_POINTS
        MEMBER_EXPORT_ORDER
        MEMBER_FILES
        MEMBER_FILTER
        MEMBER_GIVEN_POINTS
        MEMBER_GIVEN_POINTS_FOR
        MEMBER_ID
        MEMBER_IMPORT_CPF_ADDED
        MEMBER_IMPORT_GROUP_ADDED
        MEMBER_NO_EXIST
        MEMBER_NO_GROUP
        MEMBER_PRIMARY_GROUP_CHANGED
        MEMBER_PROMOTED_AUTOMATICALLY
        MEMBER_REMOVED_FROM_GROUP
        MEMBER_SETTINGS
        MEMBER_SETTINGS_SEARCH_GROUP
        MEMBER_UNBANNED
        MEMORY_USAGE
        MENU_ENTRY_BRANCH
        MENU_ENTRY_CAPTION_LONG
        MENU_ENTRY_CHECK_PERMS
        MENU_ENTRY_MATCH_KEYS
        MENU_ENTRY_NEW_WINDOW
        MENU_ENTRY_URL
        MENU_ITEM
        MENU_ITEMS
        MENU_ITEM_COUNT
        MENU_MANAGEMENT
        MENU_UNDERSCORE_RESERVED
        MERGED_MEMBERS
        MERGE_AUTHORS
        MERGE_MEMBERS
        MERGE_MEMBERS_TEXT
        MERGE_SAME
        MERGING_ON_MSN
        MESSAGES
        MESSAGE_RECEIVED_EMAILS
        MESSAGING_FORUM_NAME
        MESSAGING_NOTIFICATION_WRAPPER
        META_DESCRIPTION
        MINIMALISTIC_SITE
        MINIMODULE
        MINIMUM_AUTOCOMPLETE_LENGTH
        MINIMUM_AUTOCOMPLETE_PAST_SEARCH
        MINIMUM_GAMBLE_AMOUNT
        MINIMUM_PASSWORD_LENGTH
        MINIMUM_PASSWORD_STRENGTH
        MINIMUM_USERNAME_LENGTH
        MINUTES_BETWEEN_SENDS
        MISSING_ADDON
        MISSING_AGGREGATE_TYPE
        MISSING_ATTACHMENT
        MISSING_AVATAR
        MISSING_BLOCK_FILE
        MISSING_CAPTION_ERROR
        MISSING_CONTENT_TYPE
        MISSING_CONTENT_TYPE_TEMPLATE
        MISSING_FILE
        MISSING_FORUM
        MISSING_IMPORT_SESSION
        MISSING_INSTALLATION_FILE
        MISSING_LANG_FILE
        MISSING_LANG_STRING
        MISSING_LANG_STRINGS
        MISSING_LANG_STRINGS_ZERO
        MISSING_MENU
        MISSING_MODULE_REFERENCED
        MISSING_OPTION
        MISSING_PAGE
        MISSING_PARAM
        MISSING_PHOTO
        MISSING_RESOURCE_COMCODE
        MISSING_SOURCE_FILE
        MISSING_SYMBOL
        MISSING_TEMPLATE_FILE
        MISSING_TEMPLATE_PARAMETER
        MISSING_TEXT_FILE
        MISSING_THEME_IMAGE_FOR_MENU
        MISSING_URL_COMCODE
        MISSING_URL_ERROR
        MISSING_UTF_SUPPORT
        MM_APPLY_TWICE
        MM_TOOLTIP_AUDIT
        MM_TOOLTIP_CMS
        MM_TOOLTIP_DASHBOARD
        MM_TOOLTIP_DOCS
        MM_TOOLTIP_SECURITY
        MM_TOOLTIP_SETUP
        MM_TOOLTIP_STRUCTURE
        MM_TOOLTIP_STYLE
        MM_TOOLTIP_TOOLS
        MOBILE_PAGES
        MOBILE_SUPPORT
        MODAL_USER
        MODERATE_CHATROOMS
        MODERATOR
        MODULE
        MODULES
        MODULE_MANAGEMENT
        MODULE_TRANS_NAME_admin
        MODULE_TRANS_NAME_admin_shopping
        MODULE_TRANS_NAME_admin_actionlog
        MODULE_TRANS_NAME_admin_addons
        MODULE_TRANS_NAME_admin_awards
        MODULE_TRANS_NAME_admin_backup
        MODULE_TRANS_NAME_admin_chat
        MODULE_TRANS_NAME_admin_cleanup
        MODULE_TRANS_NAME_admin_cns_customprofilefields
        MODULE_TRANS_NAME_admin_cns_emoticons
        MODULE_TRANS_NAME_admin_cns_forum_groupings
        MODULE_TRANS_NAME_admin_cns_forums
        MODULE_TRANS_NAME_admin_cns_groups
        MODULE_TRANS_NAME_admin_cns_history
        MODULE_TRANS_NAME_admin_cns_ldap
        MODULE_TRANS_NAME_admin_cns_members
        MODULE_TRANS_NAME_admin_cns_merge_members
        MODULE_TRANS_NAME_admin_cns_multi_moderations
        MODULE_TRANS_NAME_admin_cns_post_templates
        MODULE_TRANS_NAME_admin_cns_welcome_emails
        MODULE_TRANS_NAME_admin_commandr
        MODULE_TRANS_NAME_admin_config
        MODULE_TRANS_NAME_admin_custom_comcode
        MODULE_TRANS_NAME_admin_debrand
        MODULE_TRANS_NAME_admin_ecommerce
        MODULE_TRANS_NAME_admin_email_log
        MODULE_TRANS_NAME_admin_errorlog
        MODULE_TRANS_NAME_admin_import
        MODULE_TRANS_NAME_admin_invoices
        MODULE_TRANS_NAME_admin_ip_ban
        MODULE_TRANS_NAME_admin_lang
        MODULE_TRANS_NAME_admin_lookup
        MODULE_TRANS_NAME_admin_menus
        MODULE_TRANS_NAME_admin_messaging
        MODULE_TRANS_NAME_admin_newsletter
        MODULE_TRANS_NAME_admin_permissions
        MODULE_TRANS_NAME_admin_phpinfo
        MODULE_TRANS_NAME_admin_points
        MODULE_TRANS_NAME_admin_pointstore
        MODULE_TRANS_NAME_admin_quiz
        MODULE_TRANS_NAME_admin_redirects
        MODULE_TRANS_NAME_admin_security
        MODULE_TRANS_NAME_admin_setupwizard
        MODULE_TRANS_NAME_admin_sitemap
        MODULE_TRANS_NAME_admin_ssl
        MODULE_TRANS_NAME_admin_staff
        MODULE_TRANS_NAME_admin_stats
        MODULE_TRANS_NAME_admin_themes
        MODULE_TRANS_NAME_admin_themewizard
        MODULE_TRANS_NAME_admin_tickets
        MODULE_TRANS_NAME_admin_unvalidated
        MODULE_TRANS_NAME_admin_version
        MODULE_TRANS_NAME_admin_wordfilter
        MODULE_TRANS_NAME_admin_zones
        MODULE_TRANS_NAME_cms
        MODULE_TRANS_NAME_cms_authors
        MODULE_TRANS_NAME_cms_banners
        MODULE_TRANS_NAME_cms_calendar
        MODULE_TRANS_NAME_cms_catalogues
        MODULE_TRANS_NAME_cms_chat
        MODULE_TRANS_NAME_cms_cns_groups
        MODULE_TRANS_NAME_cms_comcode_pages
        MODULE_TRANS_NAME_cms_downloads
        MODULE_TRANS_NAME_cms_galleries
        MODULE_TRANS_NAME_cms_news
        MODULE_TRANS_NAME_cms_polls
        MODULE_TRANS_NAME_cms_quiz
        MODULE_TRANS_NAME_cms_wiki
        MODULE_TRANS_NAME_filedump
        MOD_SECURITY
        MONEY_AMOUNT_DESCRIPTION
        MONTHLY_SPEC_TYPE
        MOST_COMMON
        MOST_DEFAULT
        MOVETO_MENU
        MOVE_DOWN
        MOVE_MENU_ITEM
        MOVE_PAGES
        MOVE_POSTS
        MOVE_POSTS_A_TEXT
        MOVE_POSTS_B_TEXT
        MOVE_TO
        MOVE_TOPIC
        MOVE_TOPICS
        MOVE_UP
        MSG_JS_NEEDED
        MSN_FORUM_DB_NOT_CNS_ALREADY
        MULTILANG_OFF_OPTION
        MULTILANG_OFF_PRACTICAL
        MULTILANG_OFF_CONTENT
        MULTI_LANG_CONTENT
        MULTI_LANG_CONTENT_TEXT
        MULTI_MODERATION
        MULTI_MODERATION_WILL
        MULTI_MODERATION_WILL_CLOSE
        MULTI_MODERATION_WILL_MOVE
        MULTI_MODERATION_WILL_OPEN
        MULTI_MODERATION_WILL_PIN
        MULTI_MODERATION_WILL_POST
        MULTI_MODERATION_WILL_SINK
        MULTI_MODERATION_WILL_TITLE_SUFFIX
        MULTI_MODERATION_WILL_UNPIN
        MULTI_MODERATION_WILL_UNSINK
        MULTI_SITE_NETWORKING
        MUST_BE_EMAIL_DOMAIN
        MUST_DELETE_INSTALLER
        MUST_MOVE_POSTS_SOMEWHERE
        MYSQL_OPTIMISE
        MYSQL_QUERY_CHANGES_MAKE_1
        MYSQL_QUERY_CHANGES_MAKE_2
        MYSQL_TOO_OLD
        MY_PROFILE_LINK
        M_INTEREST_MINUS
        M_INTEREST_PLUS
        NAG_CONTENT_REVIEWS
        NAG_COPYRIGHT_DATE
        NAG_FORUMS
        NAG_MONITOR_GIFTS
        NAG_OPEN_WEBSITE
        NAG_POINTSTORE
        NAG_SETUP_CRON
        NAG_SETUP_PROFILE
        NAG_VALIDATE
        NAG_WIKI
        NEED_FTP
        NEED_GEOLOCATION_DATA
        NETWORKING
        NETWORK_LINKS
        NET_PROFIT
        NEWSLETTER_0_ALT
        NEWSLETTER_ARCHIVE
        NEWSLETTER_CNS
        NEWSLETTER_CONFIRM_MESSAGE
        NEWSLETTER_DEFERRED
        NEWSLETTER_DESCRIPTION_TITLE
        NEWSLETTER_DRIP_SEND_QUEUE
        NEWSLETTER_HASH
        NEWSLETTER_IMPORT
        NEWSLETTER_IMPORTED_THIS
        NEWSLETTER_PEOPLE_ON_LEVEL
        NEWSLETTER_PREFIX
        NEWSLETTER_PREVIEW_SUBJECT
        NEWSLETTER_REGULARITY
        NEWSLETTER_REMOVED_THIS
        NEWSLETTER_SEND
        NEWSLETTER_SEND_ID
        NEWSLETTER_SEND_TEXT
        NEWSLETTER_SUBSCRIBER
        NEWSLETTER_SUBSCRIBERS_FORM
        NEWSLETTER_TEMPLATE
        NEWSLETTER_THIS_ALSO
        NEWSLETTER_WONT_IMPORT_MEMBERS
        NEWS_ARTICLE
        NEWS_CATEGORIES_PER_PAGE
        NEWS_ENTRIES_PER_PAGE
        NEWS_FLASHES
        NEWS_REGULARITY
        NEWS_SUMMARY
        NEWS_WILL_APPEAR_HERE
        NEW_COMMANDRCHAT_MESSAGES
        NEW_FIELD
        NEW_FIELD_TITLE
        NEW_HANDLING_ADD
        NEW_HANDLING_SKIP
        NEW_MENU
        NEW_REDIRECT
        NEW_TASK_RUNNING
        NEW_TEMPLATE
        NEW_THEME
        NEW_USERGROUP
        NEXT_ITEM_add_image_to_this
        NEXT_ITEM_add_one
        NEXT_ITEM_add_one_catalogue
        NEXT_ITEM_add_one_category
        NEXT_ITEM_add_one_image
        NEXT_ITEM_add_one_video
        NEXT_ITEM_add_to_catalogue
        NEXT_ITEM_add_to_category
        NEXT_ITEM_add_video_to_this
        NEXT_ITEM_adminzone
        NEXT_ITEM_edit_one
        NEXT_ITEM_edit_one_catalogue
        NEXT_ITEM_edit_one_category
        NEXT_ITEM_edit_one_image
        NEXT_ITEM_edit_one_video
        NEXT_ITEM_edit_this_catalogue
        NEXT_ITEM_edit_this_category
        NEXT_ITEM_view_this_category
        NEXT_REVIEW_DATE
        NICE_BLOCK_NAME
        MODULE_TRANS_NAME_supermembers
        NNR
        NONPRESENT_EXTENSION_FUNCTION
        NON_COMMAND
        NON_COMMAND_MISSING_COLON
        COMMAND_NO_OUTPUT
        NON_RECOGNISED_CSS_FILE
        NORMAL_GROUPS_PER_PAGE
        NOTE
        NOTES_ABOUT_REDIRECTS
        NOTE_DESCRIPTION
        NOTHING_TO_INVOICE_FOR
        NOTHING_TO_TRANSLATE
        NOTIFICATIONS_DEFINE_LOCKDOWN
        NOTIFICATIONS_LOCKDOWN
        NOTIFICATION_BODY_CONTENT_REVIEWS
        NOTIFICATION_BODY_CONTENT_REVIEWS_delete
        NOTIFICATION_CODE_LOCKED_DOWN
        NOTIFICATION_DESKTOP_ALERTS
        NOTIFICATION_ENABLE_DIGESTS
        NOTIFICATION_ENABLE_PRIVATE_TOPICS
        NOTIFICATION_KEEP_DAYS
        NOTIFICATION_MANAGEMENT_FOR
        NOTIFICATION_POLL_FREQUENCY
        NOTIFICATION_SECTION
        NOTIFICATION_SPAM_CHECK_BLOCK_BODY_APPROVE
        NOTIFICATION_SPAM_CHECK_BLOCK_BODY_BAN
        NOTIFICATION_SPAM_CHECK_BLOCK_BODY_BLOCK
        NOTIFICATION_SPAM_CHECK_BLOCK_SUBJECT_APPROVE
        NOTIFICATION_SPAM_CHECK_BLOCK_SUBJECT_BAN
        NOTIFICATION_SPAM_CHECK_BLOCK_SUBJECT_BLOCK
        NOTIFICATION_SUBJECT_CONTENT_REVIEWS
        NOTIFICATION_SUBJECT_CONTENT_REVIEWS_delete
        NOTIFICATION_TYPE
        NOTIFICATION_TYPE_actionlog
        NOTIFICATION_TYPE_adminzone_dashboard_accessed
        NOTIFICATION_TYPE_auto_ban
        NOTIFICATION_TYPE_catalogue_view_reports
        NOTIFICATION_TYPE_checklist_task
        NOTIFICATION_TYPE_cns_choose_avatar
        NOTIFICATION_TYPE_cns_choose_photo
        NOTIFICATION_TYPE_cns_choose_signature
        NOTIFICATION_TYPE_cns_group_join_request_staff
        NOTIFICATION_TYPE_cns_member_needs_validation
        NOTIFICATION_TYPE_error_occurred
        NOTIFICATION_TYPE_error_occurred_cron
        NOTIFICATION_TYPE_error_occurred_missing_page
        NOTIFICATION_TYPE_error_occurred_missing_reference
        NOTIFICATION_TYPE_error_occurred_missing_reference_important
        NOTIFICATION_TYPE_error_occurred_missing_resource
        NOTIFICATION_TYPE_error_occurred_rss
        NOTIFICATION_TYPE_filedump
        NOTIFICATION_TYPE_hack_attack
        NOTIFICATION_TYPE_ip_address_sharing
        NOTIFICATION_TYPE_low_disk_space
        NOTIFICATION_TYPE_low_stock
        NOTIFICATION_TYPE_messaging
        NOTIFICATION_TYPE_needs_validation
        NOTIFICATION_TYPE_pointstore_request_custom
        NOTIFICATION_TYPE_pointstore_request_forwarding
        NOTIFICATION_TYPE_pointstore_request_pop3
        NOTIFICATION_TYPE_pointstore_request_quota
        NOTIFICATION_TYPE_receive_points_staff
        NOTIFICATION_TYPE_service_cancelled_staff
        NOTIFICATION_TYPE_service_paid_for_staff
        NOTIFICATION_TYPE_spam_check_block
        NOTIFICATION_TYPE_staff_checklist_notify
        NOTIFICATION_TYPE_subscription_cancelled_staff
        NOTIFICATION_TYPE_task_completed
        NOTIFICATION_TYPE_ticket_assigned_staff
        NOTIFICATION_TYPE_ticket_new_staff
        NOTIFICATION_TYPE_ticket_reply_staff
        NOT_ADDON
        NOT_A_DIR
        NOT_A_FILE
        NOT_CODENAME
        NOT_COMPLETED
        NOT_EMPTY_FORCE
        NOT_INTEGER
        NOT_INTEGER_URL_MONIKERS
        NOT_LOGGED_IN_NO_CREDIT
        NOT_ON_GOOGLE_APPENGINE
        NOT_ON_LIVE_GOOGLE_APPENGINE
        NOT_THEME_IMAGE
        NOT_USED_PREVIOUSLY
        NO_ACCEPT_EMAILS
        NO_ACCESS_FOR_TAG
        NO_ACTION_REQUIRED
        NO_COMPACT
        YES_COMPACT
        NO_DATA
        NO_DATA_IMPORTED
        NO_DATA_SPECIFIC
        NO_DEFINED_THEME_UPGRADER
        NO_DELETE_LAST_CATEGORY
        NO_DELETE_ROOT
        NO_DIRECTIVE_TYPE
        NO_ENTRY_POINT_SELECTED
        NO_EXTENDABLE_PRODUCTS_ENABLED
        NO_FIELDS
        NO_FORUM
        NO_FORUM_INSTALLED
        NO_FTP_ACCESS
        NO_FTP_CONNECT
        NO_FTP_DIR
        NO_FTP_LOGIN
        NO_GD_ON_SERVER
        NO_GD_ON_SERVER_JPEG
        NO_GD_ON_SERVER_PNG
        NO_GD_ON_SERVER_TTF
        NO_HELP
        NO_INTEGERS_TEMPLATE
        NO_JS_ADVANCED_SCREEN_PERMISSIONS
        NO_JS_ADVANCED_SCREEN_SITEMAP
        NO_JS_ADVANCED_SCREEN_ZONE_EDITOR
        NO_JS_REALTIME
        NO_LAST_BACKUP
        NO_LEVEL_1_HEADERS
        NO_LEVEL_2_HEADERS
        NO_LURKERS_FOUND
        NO_MEMBER_SYSTEM_INSTALLED
        NO_MYSQL_QUERY_CHANGES_MAKE
        NO_NOTE_GUEST
        NO_PAGES_FOUND
        NO_PARAMETER_SENT_SPECIAL
        NO_PHP_DB
        NO_PHP_FTP
        NO_PHP_IN_TEMPLATES
        NO_POP3
        NO_POSSIBLE_WINNERS
        NO_SANDBOX
        NO_SELECTABLE_THEME_IMAGES_MSN
        NO_SERVER_ISSUES_FOUND
        NO_SESSION_SECURITY_CODE
        NO_SHELL_ZIP_POSSIBLE
        NO_SHELL_ZIP_POSSIBLE2
        NO_SUCH_CONTENT_TYPE
        NO_SUCH_RENDERER
        NO_SUCH_SUBSCRIPTION
        NO_SUCH_THEME_IMAGE
        NO_SUCH_ZONE
        NO_SUICIDAL_SUPER_ADMIN_REMOVAL
        NO_TEMPLATES_SELECTED
        NO_TRACKBACKS
        NO_UPGRADE_DONE
        NO_UPLOAD
        NO_XML_ON_SERVER
        NO_ZIP_ON_SERVER
        NULL_LANG_STRING
        NUM_DOWNLOADS
        NUM_NEW_SINCE
        NUM_QUEUE
        NUM_READERS
        NUM_WINNERS
        OBSCENITY_WARNING
        OEMBED_HTML_WHITELIST
        OEMBED_MANUAL_PATTERNS
        OEMBED_MAX_SIZE
        OLD
        ONEWSLETTER_1
        ONEWSLETTER_2
        ONEWSLETTER_3
        ONEWSLETTER_4
        ONE_OFF_TASKS
        ONE_PER_EMAIL_ADDRESS
        ONE_PER_MEMBER
        ONE_PER_MEMBER_ONLY
        ONLINE_STORE
        ONLY_1000_SHOWN
        ONLY_LOCAL_HOST_FOR_TYPE
        OPENING_BALANCE
        OPEN_MEMBERSHIP
        OPEN_MEMBERSHIP_DESCRIPTION
        OPEN_STATE
        OPEN_TIME
        OPERATOR_FOR
        OPML_INDEX_DESCRIPTION
        OPTION_CATEGORIES
        OPTION_CATEGORY
        OPTION_GROUPS
        ORDER
        ORDERBY_HACK
        ORDERED_BY
        ORDERED_DATE
        ORDERED_PRODUCTS
        ORDERS
        ORDER_ACTIONS
        ORDER_DETAILS_TEXT
        ORDER_NUMBER
        ORDER_PLACED_MAIL_MESSAGE
        ORDER_PLACED_MAIL_SUBJECT
        ORDER_PLACED_ON
        ORDER_PRICE
        ORDER_STATUS
        ORDER_STATUS_FILTER_DESCRIPTION
        ORDER_STATUS_awaiting_payment
        ORDER_STATUS_cancelled
        ORDER_STATUS_dispatched
        ORDER_STATUS_onhold
        ORDER_STATUS_payment_received
        ORDER_STATUS_returned
        ORDER_SUB_ALPHA
        ORDER_TAX_OPT_OUT
        ORGANISATION
        ORIGINAL
        ORIGINAL_FILENAME
        ORIGINAL_PAGE_NO_RENAME
        ORPHANED_IMAGES
        ORPHANED_LANG_STRINGS
        ORPHANED_TAGS
        ORPHANED_UPLOADS
        OS_SHARE
        OTHER_MAINTENANCE
        OUTMODE
        OUTPUT_STREAMING
        OUTSTANDING_INVOICES
        OUT_OF_BOUNDS_TIME
        OVERRIDE
        OVERT_WHISPER_SUGGESTION
        OVERVIEW_STATISTICS
        OVERWRITE_ERROR
        OVER_SMS_LIMIT
        OWNER_SET
        OWNER_VIEW
        OWN_PARENT_ERROR
        PAGES_STATISTICS
        PAGES_STATISTICS_RANGE
        PAGE_ACCESS
        PAGE_ACTIONS
        PAGE_DRAG
        PAGE_MATCH_KEY_ACCESS
        PAGE_MATCH_KEY_ACCESS_TEXT
        PAGE_MATCH_KEY_MESSAGES_TEXT
        PAGE_OR_URL
        PAGE_STATS_DELETE
        PAGE_TEMPLATE
        PAGE_TEMPLATE_DESCRIPTION
        PAGE_TEXT
        PAGE_TYPE
        PAGE_WRITE
        PANELS
        PARAMETER
        PARAMETERS
        PARAMETER_A
        PARAMETER_B
        PARAMETER_DEFINED_NOT_USED
        PARAMETER_USED_NOT_DEFINED
        PARENT
        PARENT_PAGE
        PASSWORDS_DO_NOT_MATCH
        PASSWORD_CHANGE_DAYS
        PASSWORD_COOKIES
        PASSWORD_EXPIRY_DAYS
        PASSWORD_HASH
        PASSWORD_RESET_PROCESS
        PATH_HACK
        PAYMENT_GATEWAY_TESTING_MODE
        PAYMENT_GATEWAY_manual
        PAYMENT_GATEWAY_paypal
        PAYMENT_GATEWAY_secpay
        PAYMENT_GATEWAY_worldpay
        PAYMENT_STATE_active
        PAYMENT_STATE_cancelled
        PAYMENT_STATE_delivered
        PAYMENT_STATE_new
        PAYMENT_STATE_paid
        PAYMENT_STATE_pending
        PAYPAL
        PEAK
        PENDING_REASON
        PERFORM_MULTI_MODERATION
        PERIODIC_BIWEEKLY_ON
        PERIODIC_CREATE
        PERIODIC_CREATE_HELP
        PERIODIC_MONTHLY_ON
        PERIODIC_NEWSLETTER_AMEND
        PERIODIC_NEWSLETTER_EMPTY
        PERIODIC_NEWSLETTER_EXISTS
        PERIODIC_NEWSLETTER_HELP
        PERIODIC_NEWSLETTER_SETTINGS
        PERIODIC_NO_EDIT
        PERIODIC_REMOVED
        PERIODIC_REMOVED_TEXT
        PERIODIC_REPLACE
        PERIODIC_REPLACE_HELP
        PERIODIC_SUBJECT_HELP
        PERIODIC_SUCCESS_MESSAGE_ADD
        PERIODIC_SUCCESS_MESSAGE_EDIT
        PERIODIC_WEEKLY_ON
        PERIODIC_WHEN
        PERIODIC_WHEN_CHOICE
        PERIODIC_WHEN_HELP
        PERMISSIONS
        PERMISSIONS_FROM
        PERMISSIONS_TO
        PERMISSIONS_TREE
        PERMISSIONS_TREE_EDITOR_ABOUT_BARRIERS
        PERMISSIONS_TREE_EDITOR_ABOUT_GROUP
        PERMISSIONS_TREE_EDITOR_ABOUT_HOVER_FOR_PERMISSIONS
        PERMISSIONS_TREE_EDITOR_ABOUT_MULTI_SELECT
        PERMISSIONS_TREE_EDITOR_ABOUT_PRIVILEGE
        PERMISSIONS_TREE_EDITOR_HOW_WORK
        PERMISSIONS_TREE_EDITOR_ICON_LABEL
        PERMISSIONS_TREE_EDITOR_ICON_SPECIFIC
        PERMISSIONS_TREE_EDITOR_INHERIT
        PERMISSIONS_TREE_EDITOR_MULTI_SELECTED
        PERMISSIONS_TREE_EDITOR_NONE_SELECTED
        PERMISSIONS_TREE_EDITOR_ONE_SELECTED
        PERMISSIONS_TREE_EDITOR_PERMISSIONS_FOR_SELECTION
        PERMISSIONS_TREE_EDITOR_SAVED
        PERMISSION_BANNER_SKIP
        PERMISSION_CELL
        PERMISSION_HOURS
        PERMISSION_INHERITANCE_HOVER
        PERMISSION_PRODUCT_DESCRIPTION
        PERMISSION_SCOPE_category
        PERMISSION_SCOPE_module
        PERMISSION_SCOPE_page
        PERMISSION_SCOPE_privilege
        PERMISSION_SCOPE_type
        PERMISSION_SCOPE_zone
        PERMISSION_TRAGEDY_PREVENTED
        PERM_TYPE_member_category_access
        PERM_TYPE_member_page_access
        PERM_TYPE_member_privileges
        PERM_TYPE_member_zone_access
        PERSONAL_SUB_LINKS
        PERSONAL_UNDER_MEMBERS
        PHONE_NUMBER
        PHPINFO
        PHP_DOWNLOAD_INNOCENT
        PHP_OLD
        PING_URL
        PINTERACE_HELP
        PINTERFACE_LEVEL_0
        PINTERFACE_LEVEL_1
        PINTERFACE_LEVEL_2
        PINTERFACE_LEVEL_3
        PINTERFACE_LEVEL_CUSTOM
        PINTERFACE_LEVEL_GLOBAL
        PINTERFACE_PRESETS
        PINTERFACE_VIEW
        PINTERFACE_VIEW_NO
        PIN_STATE
        PLEASE_CHOOSE_LANG
        PLEASE_TAKE_RESPONSIBILITY
        POINTSTORE_ADD_MAIL_FORWARDER
        POINTSTORE_ADD_MAIL_POP3
        POINTSTORE_AMEND_CUSTOM_PERMISSIONS
        POINTSTORE_AMEND_CUSTOM_PRODUCTS
        POINTSTORE_CHANGED_PRICES
        POINTSTORE_LOG_TEXT
        POINTSTORE_MANAGE_INVENTORY
        POINTSTORE_MANAGE_SALES
        POINTSTORE_PRICE_PAGE_TEXT
        POINTS_FOR_COMPLETING
        POINTS_IF_LIKED
        POINTS_PER_CURRENCY_UNIT
        POINTS_PER_DAILY_VISIT
        POINTS_PER_DAY
        POINTS_TO_ROOT_SCARY
        POINT_LOGS_PER_PAGE
        POLL_REGULARITY
        POP3_DESCRIPTION
        POP3_MAINTAIN_URL
        PORT
        PORTFOLIO
        POSTAL_ADDRESS
        POSTING_RATES
        POSTING_TICKET_AS
        POST_COUNT_INCREMENT
        POST_DATA
        POST_DATA_EXPLANATION
        POST_READ_HISTORY_DAYS
        POST_REQUIRING_VALIDATION_MAIL
        POST_REQUIRING_VALIDATION_MAIL_SUBJECT
        POST_TEMPLATES
        POWERED_BY
        PRESET
        PREVIEW_THEME
        PRIMARY_CORE_INSTALLED
        PRIMARY_MEMBERS_PER_PAGE
        PRIMARY_PAYPAL_EMAIL
        PRIORITY_1
        PRIORITY_2
        PRIORITY_3
        PRIORITY_4
        PRIORITY_5
        PRIVACY_LINK
        PRIVATE_CHATROOM_DELETION_TIME
        PRIVATE_TOPICS_PER_PAGE
        PRIVILEGES
        PRIVILEGE_SECTIONS
        PRIVILEGE_USED_IN
        PRIVILEGE_USED_IN_SLASHED
        PRIVILEGE_access_closed_site
        PRIVILEGE_access_ecommerce_in_test_mode
        PRIVILEGE_access_overrun_site
        PRIVILEGE_add_public_events
        PRIVILEGE_allow_html
        PRIVILEGE_assume_any_member
        PRIVILEGE_autocomplete_keyword_catalogue_category
        PRIVILEGE_autocomplete_keyword_comcode_page
        PRIVILEGE_autocomplete_keyword_download
        PRIVILEGE_autocomplete_keyword_download_category
        PRIVILEGE_autocomplete_keyword_event
        PRIVILEGE_autocomplete_keyword_gallery
        PRIVILEGE_autocomplete_keyword_image
        PRIVILEGE_autocomplete_keyword_news
        PRIVILEGE_autocomplete_keyword_poll
        PRIVILEGE_autocomplete_keyword_quiz
        PRIVILEGE_autocomplete_keyword_videos
        PRIVILEGE_autocomplete_past_search
        PRIVILEGE_autocomplete_title_catalogue_category
        PRIVILEGE_autocomplete_title_comcode_page
        PRIVILEGE_autocomplete_title_download
        PRIVILEGE_autocomplete_title_download_category
        PRIVILEGE_autocomplete_title_event
        PRIVILEGE_autocomplete_title_gallery
        PRIVILEGE_autocomplete_title_image
        PRIVILEGE_autocomplete_title_news
        PRIVILEGE_autocomplete_title_poll
        PRIVILEGE_autocomplete_title_quiz
        PRIVILEGE_autocomplete_title_videos
        PRIVILEGE_ban_chatters_from_rooms
        PRIVILEGE_banner_free
        PRIVILEGE_bypass_bandwidth_restriction
        PRIVILEGE_bypass_dob
        PRIVILEGE_bypass_dob_if_already_empty
        PRIVILEGE_bypass_email_address
        PRIVILEGE_bypass_email_address_if_already_empty
        PRIVILEGE_bypass_flood_control
        PRIVILEGE_bypass_quiz_repeat_time_restriction
        PRIVILEGE_bypass_quiz_timer
        PRIVILEGE_bypass_required_cpfs
        PRIVILEGE_bypass_required_cpfs_if_already_empty
        PRIVILEGE_bypass_validation_highrange_content
        PRIVILEGE_bypass_validation_lowrange_content
        PRIVILEGE_bypass_validation_midrange_content
        PRIVILEGE_bypass_wordfilter
        PRIVILEGE_calendar_add_to_others
        PRIVILEGE_can_submit_to_others_categories
        PRIVILEGE_change_newsletter_subscriptions
        PRIVILEGE_choose_poll
        PRIVILEGE_close_own_topics
        PRIVILEGE_comcode_dangerous
        PRIVILEGE_comcode_nuisance
        PRIVILEGE_comment
        PRIVILEGE_control_usergroups
        PRIVILEGE_create_private_room
        PRIVILEGE_decide_comment_type
        PRIVILEGE_delete_account
        PRIVILEGE_delete_anything_filedump
        PRIVILEGE_delete_cat_highrange_content
        PRIVILEGE_delete_cat_lowrange_content
        PRIVILEGE_delete_cat_midrange_content
        PRIVILEGE_delete_highrange_content
        PRIVILEGE_delete_lowrange_content
        PRIVILEGE_delete_midrange_content
        PRIVILEGE_delete_own_cat_highrange_content
        PRIVILEGE_delete_own_cat_lowrange_content
        PRIVILEGE_delete_own_cat_midrange_content
        PRIVILEGE_delete_own_highrange_content
        PRIVILEGE_delete_own_lowrange_content
        PRIVILEGE_delete_own_midrange_content
        PRIVILEGE_delete_private_topic_posts
        PRIVILEGE_delete_revisions
        PRIVILEGE_disable_lost_passwords
        PRIVILEGE_double_post
        PRIVILEGE_download
        PRIVILEGE_draw_to_server
        PRIVILEGE_edit_cat_highrange_content
        PRIVILEGE_edit_cat_lowrange_content
        PRIVILEGE_edit_cat_midrange_content
        PRIVILEGE_edit_highrange_content
        PRIVILEGE_edit_lowrange_content
        PRIVILEGE_edit_meta_fields
        PRIVILEGE_edit_midrange_content
        PRIVILEGE_edit_own_cat_highrange_content
        PRIVILEGE_edit_own_cat_lowrange_content
        PRIVILEGE_edit_own_cat_midrange_content
        PRIVILEGE_edit_own_highrange_content
        PRIVILEGE_edit_own_lowrange_content
        PRIVILEGE_edit_own_midrange_content
        PRIVILEGE_edit_own_polls
        PRIVILEGE_edit_owned_events
        PRIVILEGE_edit_private_topic_posts
        PRIVILEGE_edit_viewable_events
        PRIVILEGE_exceed_filesize_limit
        PRIVILEGE_exceed_post_delete_time_limit
        PRIVILEGE_exceed_post_edit_time_limit
        PRIVILEGE_feature
        PRIVILEGE_full_banner_setup
        PRIVILEGE_give_negative_points
        PRIVILEGE_give_points_self
        PRIVILEGE_have_negative_gift_points
        PRIVILEGE_have_personal_category
        PRIVILEGE_high_catalogue_entry_timeout
        PRIVILEGE_high_personal_gallery_limit
        PRIVILEGE_jump_to_unvalidated
        PRIVILEGE_mass_delete_from_ip
        PRIVILEGE_mass_import
        PRIVILEGE_may_choose_custom_title
        PRIVILEGE_may_download_gallery
        PRIVILEGE_may_enable_staff_notifications
        PRIVILEGE_may_report_post
        PRIVILEGE_may_unblind_own_poll
        PRIVILEGE_member_maintenance
        PRIVILEGE_moderate_my_private_rooms
        PRIVILEGE_moderate_private_topic
        PRIVILEGE_multi_delete_topics
        PRIVILEGE_no_personal_gallery_limit
        PRIVILEGE_open_virtual_roots
        PRIVILEGE_own_avatars
        PRIVILEGE_perform_keyword_check
        PRIVILEGE_perform_webstandards_check_by_default
        PRIVILEGE_probate_members
        PRIVILEGE_pt_anyone
        PRIVILEGE_rate
        PRIVILEGE_remove_page_split
        PRIVILEGE_rename_self
        PRIVILEGE_reuse_others_attachments
        PRIVILEGE_run_multi_moderations
        PRIVILEGE_scheduled_publication_times
        PRIVILEGE_search_engine_links
        PRIVILEGE_see_hidden_groups
        PRIVILEGE_see_ip
        PRIVILEGE_see_php_errors
        PRIVILEGE_see_software_docs
        PRIVILEGE_see_stack_dump
        PRIVILEGE_see_unvalidated
        PRIVILEGE_see_warnings
        PRIVILEGE_sees_javascript_error_alerts
        PRIVILEGE_sense_personal_conflicts
        PRIVILEGE_set_content_review_settings
        PRIVILEGE_set_own_author_profile
        PRIVILEGE_set_reminders
        PRIVILEGE_show_user_browsing
        PRIVILEGE_sms_higher_limit
        PRIVILEGE_sms_higher_trigger_limit
        PRIVILEGE_start_im
        PRIVILEGE_submit_cat_highrange_content
        PRIVILEGE_submit_cat_lowrange_content
        PRIVILEGE_submit_cat_midrange_content
        PRIVILEGE_submit_highrange_content
        PRIVILEGE_submit_lowrange_content
        PRIVILEGE_submit_midrange_content
        PRIVILEGE_support_operator
        PRIVILEGE_trace_anonymous_gifts
        PRIVILEGE_undo_revisions
        PRIVILEGE_unfiltered_input
        PRIVILEGE_upload_anything_filedump
        PRIVILEGE_upload_filedump
        PRIVILEGE_use_html_banner
        PRIVILEGE_use_php_banner
        PRIVILEGE_use_points
        PRIVILEGE_use_pt
        PRIVILEGE_use_quick_reply
        PRIVILEGE_use_sms
        PRIVILEGE_use_special_emoticons
        PRIVILEGE_use_very_dangerous_comcode
        PRIVILEGE_view_any_profile_field
        PRIVILEGE_view_anyones_banner_stats
        PRIVILEGE_view_calendar
        PRIVILEGE_view_charge_log
        PRIVILEGE_view_event_subscriptions
        PRIVILEGE_view_member_photos
        PRIVILEGE_view_other_pt
        PRIVILEGE_view_others_quiz_results
        PRIVILEGE_view_others_tickets
        PRIVILEGE_view_poll_results_before_voting
        PRIVILEGE_view_private_content
        PRIVILEGE_view_profiles
        PRIVILEGE_view_profiling_modes
        PRIVILEGE_view_revision_history
        PRIVILEGE_view_revisions
        PRIVILEGE_vote_in_polls
        PRIVILEGE_warn_members
        PRIVILEGE_wiki_manage_tree
        PROBATION_USERGROUP
        PROBLEM_ACCESSING_RESPONSE
        PROBLEM_RETRIEVING_XML
        PRODUCT_NAME
        PROFIT_LOSS
        PROGRAMMATIC_SYMBOL__CYCLE
        PROGRAMMATIC_SYMBOL__DEC
        PROGRAMMATIC_SYMBOL__GET
        PROGRAMMATIC_SYMBOL__INC
        PROGRAMMATIC_SYMBOL__INIT
        PROGRAMMATIC_SYMBOL__RAND
        PROGRAMMATIC_SYMBOL__SET
        PROGRAMMATIC_SYMBOL__SET_RAND
        PROHIBIT_PASSWORD_WHITESPACE
        PROHIBIT_USERNAME_WHITESPACE
        PROMOTION_TARGET
        PROXY
        PROXY_PASSWORD
        PROXY_PORT
        PROXY_USER
        PT_NOTIFICATIONS_AS_WEB
        PUBLICATION_TIME
        PUBLIC_VIEW
        PUBLISH_NEWS
        PUNISHMENT_UNDOING
        PUNITIVE_ACTIONS
        PUNITIVE_HISTORY_TEXT
        PURCHASE
        PURCHASE_ORDER
        PUT_IN_CATEGORY
        PUT_IN_SEARCH
        PUT_ON_PROBATION
        QUERIES_WITHOUT_CACHE
        QUERY
        QUERY_FAILED
        QUERY_FAILED_TOO_BIG
        QUERY_NULL
        QUICK_CSS_CHANGE_LINKS
        QUIZ_ANSWER_RESULT
        QUIZ_END_TEXT
        QUIZ_END_TEXT_FAIL
        QUIZ_INVALID_LONG_HAS_ANSWERS
        QUIZ_INVALID_MULTI_NO_ANSWERS
        QUIZ_INVALID_UNMARKED_HAS_ANSWERS
        QUIZ_START_TEXT
        QUIZ_WAS_ENTERED_AS_FOLLOWS
        QUOTA
        QUOTA_MAINTAIN_URL
        Q_SURE_BAN
        Q_SURE_MOVE
        Q_SURE_UNBAN
        RANK
        RANK_IMAGE
        RANK_IMAGE_PRI_ONLY
        RANK_IMAGE_PRI_ONLY_DESCRIPTION
        RAW_PREVIEW
        REALTIME_RAIN
        REALTIME_RAIN_BUTTON
        REBRAND_BASE_URL
        REBRAND_DASHBOARD
        REBRAND_NAME
        RECEIVER_DETAILS
        RECEIVER_EMAIL
        RECOMMENDED
        RECURRANCE_DAYS
        RECURRENCES
        RECURRENCE_PATTERN
        RECURSIVE_SERVER
        RECURSIVE_TREE_CHAIN
        RECUR_EVERY
        REDIRECTED_TO_CACHE_MODULES
        REDIRECTING
        REDIRECTS
        REDIRECT_FROM_PAGE
        REDIRECT_FROM_ZONE
        REDIRECT_PAGE_TO
        REDIRECT_TO_PAGE
        REDIRECT_TO_ZONE
        REDIRECT_TRANS_SHORT
        REDUCE_PROBATION
        REFERER
        REFERRER_IFRAME_HACK
        REFERRER_SHARE
        REFRESH_TIME
        REFRESH_TO_FINISH
        REGIONALITY
        REGIONALITY_SHARE
        REGULAR_TASKS
        REINSTALL_MODULE
        RELATED_SCREENS
        RELEVANCE_LEVEL
        REMEMBER_ME_BY_DEFAULT
        REMEMBER_PASSWORD
        REMOVE_FROM_PROBATION
        REMOVE_PERIODIC
        REMOVE_PERIODIC_NEWSLETTER
        REMOVE_PROBATION_DAYS
        REMOVE_STAFF
        RENAME_THEME
        REORDER_FORUMS
        REPEAT_PERMISSION
        REPEAT_PERMISSION_NOTICE
        REPLACEMENT
        REPLACE_PERIODIC
        REPORTED_POSTS_FORUM
        REPRESENTATIVE_IMAGE
        REQUIRES_MORE_INFO
        REQUIRES_TTF
        REQUIRE_NEW_MEMBER_VALIDATION
        REQUIRE_SESSION
        RESET_BANNER_STATS
        RESET_CATEGORY_PERMISSIONS
        RESET_INTRO_ACCEPTANCE
        RESOURCE_FS_PATH_PRIOR_TO_REVISION
        RESOURCE_FS_RECORD_PRIOR_TO_REVISION
        RESTORE_POINTS
        RESTRICTED_USERNAMES
        RESTRICTIONS
        RESTRICTION_CELL
        RESTRICT_PAGE_VISIBILITY
        RESULT
        RETURN_PRODUCT
        REVEAL_ANSWERS
        REVERSE
        REVERSE_THUMB_ORDER
        REVERSE_TITLE
        REVERSE_TITLE_DESCRIPTION
        REVIEW_FREQ
        REVISION_UNDO_DESCRIPTION
        RIGHT_CLICK_SAVE_AS
        RM_HINT
        ROOT_ZONE_LOGIN_THEME
        RSS_UNKNOWN_VERSION
        RSS_XML_ERROR
        RSS_XML_MISSING
        RTEV_BOT
        RTEV_EMAILS
        RTEV_NEWS
        RTEV_PHONE
        RULES_LINK
        SALT
        SAVED_WARNINGS
        SAVE_ALL
        SAVE_AND_STAY
        SAVE_WARNING_DETAILS
        SCHEDULE_TIME
        SCOPE
        SCREENS
        SCREEN_PREVIEW
        SCREEN_PREVIEWS
        SCREEN_TITLE
        SCREEN_TITLE_DESC
        SCRIPT_UPLOAD_HACK
        SCRIPT_URL_HACK
        SCRIPT_URL_HACK_2
        SD_WARNING_BELOW
        SEARCHABLE
        SEARCH_ORDERS
        SEARCH_RESULTS_PER_PAGE
        SEARCH_STATISTICS
        SEARCH_STATISTICS_RANGE
        SEARCH_STATS_ADVANCED
        SEARCH_TUTORIALS
        SEARCH_WITH_DATE_RANGE
        SECONDARY_CORE_INSTALLED
        SECONDARY_MEMBERS_PER_PAGE
        SECTION
        SECTION_DEMOGRAPHICS_RANGE
        SECTION_DOWNLOADS_RANGE
        SECTION_POSTING_RATES_RANGE
        SECURITY
        SECURITY_ALERTS
        SECURITY_LOG
        SECURITY_PAGE_CLEANUP
        SECURITY_TOKEN_EXCEPTIONS
        SEED_COLOUR
        SEED_IS
        SEG_RECURRENCES
        SELECTION_DETAILS
        SELECTORS_PARENT_PAGE
        SELECT_AGGREGATE_TYPES_FOR_SYNC
        SELECT_AN_ACTION
        SELECT_A_MESSAGE
        SELECT_CATEGORIES_WANTED
        SELECT_COLUMNS_TO_INCLUDE
        SELECT_PAGES_DELETE
        SELECT_PAGES_MOVE
        SELECT_STATS_RANGE
        SELECT_TO_IMPORT
        SELECT_USERGROUPS_TO_FILTER
        SELF_LEARNING_CACHE
        SENDING_NEWSLETTER
        SEND_ALL
        SEND_DISPATCH_NOTIFICATION
        SEND_ERROR_EMAILS_OCPRODUCTS
        SEND_TIME
        SEND_TRACKBACKS
        SENT_NUM
        SEO
        SEQ_POST_IDS
        SESSION_EXPIRY_TIME
        SESSION_PRUDENCE
        SET
        SETUP
        SETUPWIZARD
        SETUPWIZARD_10_DESCRIBE
        SETUPWIZARD_1_DESCRIBE
        SETUPWIZARD_1_DESCRIBE_ALT
        SETUPWIZARD_2_DESCRIBE
        SETUPWIZARD_2_DESCRIBE_EXPECTATIONS
        SETUPWIZARD_3_DESCRIBE
        SETUPWIZARD_4_DESCRIBE
        SETUPWIZARD_5_DESCRIBE
        SETUPWIZARD_5x_DESCRIBE
        SETUPWIZARD_6_DESCRIBE
        SETUPWIZARD_7_DESCRIBE
        SETUPWIZARD_8_DESCRIBE
        SETUPWIZARD_9_DESCRIBE
        SETUPWIZARD_NOT_RUN
        SETUPWIZARD_RULES_balanced
        SETUPWIZARD_RULES_corporate
        SETUPWIZARD_RULES_liberal
        SETUPWIZARD_SAFE_MODE
        SETUPWIZARD_STEP
        SET_ALL_PERMISSIONS_ON_ROW
        SET_REDIRECTS
        SHARED_INSTALL_PROHIBIT
        SHIPPING_COST_FACTOR
        SHOPPING_FORCE_DELETE
        SHOULD_HAVE_BEEN_POSTED
        SHOULD_HAVE_BEEN_POSTED_FILE_ERROR
        SHOW_ALL
        SHOW_CONTENT_TAGGING
        SHOW_CONTENT_TAGGING_INLINE
        SHOW_DOCS
        SHOW_EMPTY_CPFS
        SHOW_EMPTY_GALLERIES
        SHOW_FIRST_JOIN_PAGE
        SHOW_GALLERY_COUNTS
        SHOW_INLINE_STATS
        SHOW_IN_POSTS
        SHOW_IN_POST_PREVIEWS
        SHOW_NEWEST_MEMBER
        SHOW_ON_JOIN_FORM
        SHOW_ORDERS
        SHOW_POST_VALIDATION
        SHOW_SCREEN_ACTIONS
        SHOW_UNDISPATCHED_ORDERS
        SHUFFLE_ANSWERS
        SHUFFLE_QUESTIONS
        SH_COMMAND
        SIGN_UP_REMINDER
        SIGN_UP_REMINDER_GROUPS
        SILENCE_FROM_FORUM
        SILENCE_FROM_TOPIC
        SILLY_INPUT
        SIMPLIFIED_ATTACHMENTS_UI
        SIMPLIFIED_DONEXT
        SIMPLIFY_WYSIWYG_BY_PERMISSIONS
        SINK_STATE
        SITEMAP_EDITOR
        SITEMAP_LINK
        SITEMAP_TOOLS
        SITEWIDE_IM
        SITE_DB
        SITE_NAME
        SITE_SCOPE
        SITE_SPECIAL_ACCESS
        SITE_SPECIAL_ACCESS_SU
        SITE_STATISTICS
        SITE_WATCHLIST
        SIZE_CHANGE
        SKU
        SLOW_DOWN
        SLOW_SERVER
        SMS_HIGH_LIMIT
        SMS_HIGH_TRIGGER_LIMIT
        SMS_LOW_LIMIT
        SMS_LOW_TRIGGER_LIMIT
        SMTP
        SOFTWARE_CHAT_EXTRA
        SOFTWARE_CHAT_STANDALONE
        SOMETHING_NEWSLETTER_SUBSCRIBERS
        SOME_ADDONS_UPDATED
        SOME_NEWSLETTER_TARGETS_KNOWN
        SOURCE_THEME
        SPAMMER_DETECTION
        SPAM_APPROVAL_THRESHOLD
        SPAM_AUTO_BAN_TIMEOUT
        SPAM_BAN_THRESHOLD
        SPAM_BLACKHOLE_DETECTION
        SPAM_BLOCK_LISTS
        SPAM_BLOCK_THRESHOLD
        SPAM_CACHE_TIME
        SPAM_CHECK_EXCLUSIONS
        SPAM_CHECK_LEVEL
        SPAM_CHECK_USERNAMES
        SPAM_REPORT_NO_EMAIL_OR_USERNAME
        SPAM_REPORT_SITE_FLOODING
        SPAM_REPORT_TRIGGERED_SPAM_HEURISTICS
        SPAM_STALE_THRESHOLD
        SPECIAL_CLICK_TO_EDIT
        SPECIAL_CPF_TO_SHOW
        SPECIFY_NEW_PAGE
        SPEED_UP
        SQL_BACKUP
        SQL_INJECTION_HACK
        SSL_CONFIGURATION
        SSL_PAGE_SELECT
        SSW
        STACK_TRACE
        STACK_TRACE_DENIED_ERROR_NOTIFICATION
        STACK_TRACE_INFORMATION
        STAFF_CHECKLIST_MAIL_BODY
        STAFF_CHECKLIST_MAIL_SUBJECT
        STAFF_EMAIL
        STAFF_FILTER_OFF
        STAFF_FILTER_ON
        STAFF_LINKS
        STAFF_NEW_TICKET_AS
        STAFF_USERNAME_CHANGED_MAIL
        STAFF_USERNAME_CHANGED_MAIL_SUBJECT
        STANDARD_BOX
        START_PROBATION
        STAR_ALREADY_DEFINED
        STATS_CACHE
        STATS_WHEN_CLOSED
        STATUS_CURRENT
        STATUS_INSTALLED
        STATUS_NOT_INSTALLED
        STATUS_OUTOFDATE
        STATUS_TO_HACK
        STATUS_TO_UPGRADE
        STOCK
        STOCK_LEVEL_MAIL_SUBJECT
        STOCK_MAINTENANCE_WARN_MAIL
        STOCK_RESPONSE
        STOCK_RESPONSE_PT
        STOPFORUMSPAM_API_KEY
        STOPPED_RECURSIVE_RESOURCE_INCLUDE
        STOP_PROBATION
        STORE_REVISIONS
        STORE_TIME
        STRONG_FORUM_TIE
        STRUCTURE
        STYLE
        ST_END_PERIOD
        ST_END_PERIOD_DESCRIPTION
        ST_START_PERIOD
        ST_START_PERIOD_DESCRIPTION
        SU
        SUBGALLERY_LINK_LIMIT
        SUBMISSION_STATISTICS
        SUBMITTER_BAN
        SUBMITTER_BANNED
        SUBMITTER_GETS_POINTS
        SUBMITTER_UNBANNED
        SUBMIT_POINTS
        SUBMIT_THANKYOU
        SUBMIT_UNVALIDATED
        SUBSCRIPTION_EVENT_EMAILS
        SUBSCRIPTION_LENGTH
        SUBSCRIPTION_LEVEL
        SUBSCRIPTION_REF_POINT
        SUBSCRIPTION_REF_POINT_OFFSET
        SUBSCRIPTION_REF_POINT_OFFSET_NEGATIVE_ERROR
        SUBSCRIPTION_STATUS
        SUCCESSFULLY_SCHEDULED_BACKUP
        SUCCESS_ADDED_DOWNLOADS
        SUCCESS_NOW_NEXT_SCREEN
        SUEXEC_SERVER
        SUGGEST_ADD_CATEGORY_NEXT
        SUHOSIN_MAX_VALUE_TOO_SHORT
        SUHOSIN_MAX_VARS_TOO_LOW
        SUPERMEMBERS_TEXT
        SUPER_ADMIN
        SUPER_DEBRAND
        SUPER_LOGGING
        SEND_PM
        SUPER_MEMBERS
        SUPER_MODERATOR
        SUPPORTS_WIDE
        SUPPORT_OPERATOR
        SU_2
        SU_CHATTING_AS
        SVG_EXPLANATION
        SWITCH_MODULE_WARNING
        SWITCH_SECTION_WARNING
        SWITCH_TO_ADVANCED_MODE
        SWITCH_TO_CNS
        SWITCH_TO_SIMPLE_MODE
        SWITCH_ZONE_WARNING
        SYMBOLS_AND_DIRECTIVES
        SYMBOL__BASE_URL
        SYMBOL__DATE
        SYMBOL__DATE_AND_TIME
        SYMBOL__IS_STAFF
        SYMBOL__LANG
        SYMBOL__MATCH_KEY_MATCH
        SYMBOL__MEMBER
        SYMBOL__MOBILE
        SYMBOL__SITE_NAME
        SYMBOL__STAFF_ADDRESS
        SYMBOL__THEME
        SYMBOL__TIME
        SYMBOL__USERNAME
        SYNCHRONISATION
        SYNCHRONISE_AGGREGATE_TYPES
        SYNDICATED_IP_BAN
        SYNDICATE_TO_STOPFORUMSPAM
        SYSTEM_AVATAR
        SYSTEM_CONTROLLED
        TABLES_CREATED
        TABLE_ERROR
        TABLE_FIXED
        TABLE_PREFIX
        TABLE_PREFIX_TEXT
        TAKE_RESPONSIBILITY
        TARGET
        TASKS_BACKGROUND
        TASK_LENGTH
        TASK_LENGTH_UNITS
        TAX_GENERAL
        TAX_PAID
        TEMPCODE_NOT_ARRAY
        TEMPCODE_PARAMETER
        TEMPCODE_TESTER
        TEMPCODE_TESTER_HELP
        TEMPCODE_TOO_MANY_CLOSES
        TEMPCODE_VALUE
        TEMPLATE
        TEMPLATES
        TEMPLATES_CSS
        TEMPLATES_HTML
        TEMPLATES_JAVASCRIPT
        TEMPLATES_TEXT
        TEMPLATES_WITH_EDIT_LINKS_PARAMETERS
        TEMPLATES_WITH_EDIT_LINKS
        TEMPLATES_WITH_HTML_COMMENT_MARKERS
        TEMPLATES_XML
        TEMPLATE_ALTERED
        TEMPLATE_CACHE
        TEMPLATE_CHOOSE_TO_EDIT
        TEMPLATE_RENAMED
        TEMPLATE_TREE
        TEMPLATE_WILL_NEED_RESTORING
        TERMINAL_PROBLEM_ACCESSING_RESPONSE
        TERMS
        TEXTUAL_TAG
        TEXT_ABSTRACT_FILE_MANAGEMENT
        TEXT_PRIOR_TO_REVISION
        TEXT_REDIRECTS
        TEXT_VERSION
        THANKS
        THEME
        THEMEGEN_ALGORITHM
        THEMEGEN_ALGORITHM_EQUATIONS
        THEMEGEN_ALGORITHM_HSV
        THEMES
        THEMES_AND_ZONES
        THEMEWIZARD
        THEMEWIZARD_1_DESCRIBE
        THEMEWIZARD_2_DOMINANT
        THEMEWIZARD_2_LIGHT_DARK
        THEMEWIZARD_2_USE
        THEMEWIZARD_4_DESCRIBE
        THEME_ALSO_INCLUDE_PAGES
        THEME_DEFAULT_FOR_SITE
        THEME_EXPORT
        THEME_IMAGE
        THEME_IMAGES
        THEME_IMAGE_CURRENTLY_LIKE
        THEME_IMAGE_CURRENTLY_UNMODIFIED
        THEME_IMAGE_EDITING
        THEME_IMAGE_NEW
        THEME_IMAGE_RENAMED
        THEME_LOGO
        THEME_MADE_LIVE
        THEME_MAPPING
        THEME_TO_SAVE_INTO
        THEME_USED_ON
        THESE_PRODUCTS_DEFINED
        THIS_CLUB_FORUM
        THIS_HAS_BEEN_READ_BY
        THIS_WITH
        THREADED_BUTTONS
        THUMB_WIDTH
        TICKET_AUTO_ASSIGN
        TICKET_CANNOT_BIND_MAIL
        TICKET_CANNOT_BIND_SUBJECT
        TICKET_EMAIL_FROM
        TICKET_FORUM_NAME
        TICKET_GUEST_EMAILS_MANDATORY
        TICKET_MAIL_ON
        TICKET_MAIL_PASSWORD
        TICKET_MAIL_SERVER
        TICKET_MAIL_SERVER_PORT
        TICKET_MAIL_SERVER_TYPE
        TICKET_MAIL_USERNAME
        TICKET_MEMBER_FORUMS
        TICKET_NEW_MESSAGE_FOR_STAFF
        TICKET_NEW_STAFF
        TICKET_OTHERS_HACK
        TICKET_REPLY_STAFF
        TICKET_SEARCH_FAQ
        TICKET_SPLIT_POST
        TICKET_STAFF_ONLY
        TICKET_STAFF_ONLY_DESCRIPTION
        TICKET_STAFF_ONLY_REPLY
        TICKET_SYSTEM_WEIRD
        TICKET_TYPE_FORUMS
        TICKET_UNASSIGN
        TICKET_UNASSIGNED_BODY
        TICKET_UNASSIGNED_SUBJECT
        TIED_TO_NEWSLETTER
        TIPS
        TIP_0a
        TIP_1a
        TIP_1b
        TIP_1c
        TIP_1d
        TIP_1e
        TIP_1f
        TIP_1g
        TIP_1h
        TIP_1i
        TIP_1j
        TIP_1k
        TIP_1l
        TIP_1m
        TIP_1n
        TIP_2a
        TIP_2b
        TIP_2c
        TIP_2d
        TIP_2e
        TIP_2f
        TIP_2g
        TIP_2h
        TIP_2i
        TIP_2j
        TIP_2k
        TIP_2l
        TIP_2m
        TIP_2n
        TIP_2o
        TIP_2p
        TIP_2q
        TIP_2r
        TIP_2s
        TIP_2t
        TIP_2u
        TIP_3a
        TIP_3b
        TIP_3c
        TIP_3d
        TIP_3e
        TIP_3f
        TIP_3g
        TIP_3h
        TIP_3i
        TIP_3j
        TIP_3k
        TIP_3l
        TIP_3m
        TIP_3n
        TIP_3o
        TIP_4a
        TIP_4b
        TIP_4c
        TIP_4d
        TIP_4e
        TIP_4f
        TITLE_PRIOR_TO_REVISION
        TITLE_SUFFIX
        TOGGLE
        TOGGLE_DOCKED_FIELD_EDITING
        TOOLS
        TOO_MANY_GALLERY_IMAGES
        TOO_MANY_GALLERY_VIDEOS
        TOO_MANY_IMPORT_SESSIONS
        TOO_MANY_LURKERS
        TOO_MANY_QUERIES
        TOO_MANY_TO_CHOOSE_FROM
        TOO_MANY_USERS
        TOO_MANY_WIKI_CHILDREN
        TOO_MANY_WIKI_POSTS
        TOO_MUCH_CHOOSE__ALPHABETICAL
        TOO_MUCH_CHOOSE__RECENT_ONLY
        TOO_MUCH_CHOOSE__TOP_ONLY
        TOO_MUCH_DATA
        TOO_MUCH_DOWNLOAD
        TOO_MUCH_FILE_DATA
        TOPIC_ORDER
        TOPIC_PIN_MAX_DAYS
        TOP_LEFT
        TOP_LINK
        TOP_REFERRERS
        TOP_REFERRERS_RANGE
        TOP_RIGHT
        TOP_SEARCH_KEYWORDS
        TOP_SEARCH_KEYWORDS_RANGE
        TORNEVALL_API_PASSWORD
        TORNEVALL_API_USERNAME
        TOTAL_DOWNLOADS_IN_ARCHIVE
        TOTAL_NEWS_ENTRIES
        TOTAL_TAX_PAID
        TO_COLOUR
        TO_EMAIL
        TO_NAME
        TRACKBACK_ERROR
        TRACKBACK_SPAM
        TRANSACTION
        TRANSACTIONS
        TRANSACTION_FEES
        TRANSACTION_ID
        TRANSACTION_NOTES
        TRANSCODING
        TRANSCODING_ERROR
        TRANSCODING_ERROR_2
        TRANSCODING_LOCALHOST
        TRANSCODING_SERVER
        TRANSCODING_ZENCODER_API_KEY
        TRANSCODING_ZENCODER_FTP_PATH
        TRANSLATE_CODE
        TRANSLATE_CONTENT
        TRANSLATE_CONTENT_SCREEN
        TRANSLATE_TOO_MANY
        TRANSLATION_GUIDE
        TRANSLATION_PROGRESS
        TRASH
        TRASH_VERB
        TREE_LISTS
        TROUBLESOME_WINDOWS_SERVER
        TRY_QUIZ
        TRY_TO_DOWNLOAD_SCRIPT
        TXMT_PROTOCOL_EXPLAIN
        UNBANNABLE_IP_ADDRESSES
        UNBAN_IP
        UNBAN_IP_DESCRIPTION
        UNBAN_MEMBER
        UNBAN_MEMBER_DESCRIPTION
        UNBAN_MEMBER_MAIL
        UNBAN_MEMBER_MAIL_SUBJECT
        UNBAN_SUBMITTER_DESCRIPTION
        UNBLOCK_MEMBER
        UNCLOSED_DIRECTIVE_OR_BRACE
        UNDELIVERED_INVOICES
        UNDER_CONSTRUCTION
        UNDER_CONSTRUCTION_MARKERS
        UNDO
        UNDO_CHARGE
        UNDO_CHARGE_FOR
        UNDO_PROBATION
        UNDO_REVISION
        UNINSTALL
        UNINSTALLED
        UNINSTALL_ADDON
        UNINSTALL_MODULE
        UNINSTALL_WARNING
        UNKNOWN_AUTH_SCHEME_IN_DB
        UNKNOWN_DIRECTIVE
        UNKNOWN_EM
        UNKNOWN_PRIVILEGE_PRESET
        UNPIN_TOPIC
        UNRECOGNISED_CURRENCY
        UNRESOLVABLE_COLOURS
        UNSET
        UNSILENCE_FORUM
        UNSILENCE_TOPIC
        UNUSED_CPF
        UNVALIDATED_ENTRIES
        UNVALIDATED_MAIL_A
        UNVALIDATED_MAIL_B
        UNVALIDATED_MAIL_C
        UNVALIDATED_PAGE_TEXT
        UNVALIDATED_RESOURCES
        UNZIP_CMD
        UNZIP_DIR
        UPDATE_HANDLING_DELETE
        UPDATE_HANDLING_FRESHEN
        UPDATE_HANDLING_OVERWRITE
        UPDATE_HANDLING_SKIP
        UPDATE_TIME
        UPGRADE_MODULE
        UPGRADE_TO
        UPLOADED_FILES
        UPLOADED_FILE_NOT_DOWNLOADABLE_YET
        URL
        URLS_CACHE
        URL_BEGIN_FTP
        URL_MONIKER
        URL_MONIKERS_ENABLED
        URL_MONIKER_CONFLICT_PAGE
        URL_MONIKER_CONFLICT_ZONE
        URL_MONIKER_TAKEN
        URL_SCHEME
        USED_PREVIOUSLY
        URL_REDIRECT
        USERGROUPS_WITH_NO_PERMISSIONS
        USERGROUP_APPLICATIONS
        USERGROUP_SETTINGS_SEARCH_GROUP
        USERNAMES_AND_PASSWORDS
        USERNAME_CLICK_IM
        USERNAME_PROFILE_LINKS
        USERS_ONLINE_STATISTICS
        USERS_ONLINE_TIME
        USER_POSTSIZE_ERRORS
        USER_SYSTEM
        USES_PRIMARY
        USE_ALL
        USE_ALL_LANGS
        USE_ALL_THEMES
        USE_AS_DEFAULT_ON_APPLICABLE_FORUMS
        USE_CAPTCHAS
        USE_CONTEXTUAL_DATES
        USE_DEFAULT
        USE_INTEREST_LEVELS
        USE_JOINDATE
        USE_LASTONDATE
        USE_LOCAL_PAYMENT
        USE_MSN
        USE_MULTI_DB
        USE_ON_ZONES
        USE_PERSISTENT
        USE_PERSISTENT_TEXT
        USE_TOPICS
        USE_TRUE_FROM
        USING_SU
        U_EXTRACTING_MESSAGE
        U_PACKING_MESSAGE
        U_SKIPPING_MESSAGE
        VALIDATED
        VALIDATED_SHORT
        VALIDATE_NEW_MEMBER_MAIL
        VALIDATE_NEW_MEMBER_SUBJECT
        VALIDATE_POST
        VALIDATE_TOPICS
        VALIDATION
        VALID_EMAIL_DOMAINS
        VB_UNIQUE_ID
        VB_UNIQUE_ID_DESCRIP
        VERSION
        VERSION_ABOUT
        VERSION_NUM
        VIDEO_BITRATE
        VIDEO_HEIGHT_SETTING
        VIDEO_TYPES
        VIDEO_WIDTH_SETTING
        VIEWING_SOURCE
        VIEWS_PER_DAY
        VIEWS_PER_HOUR
        VIEWS_PER_MONTH
        VIEWS_PER_WEEK
        VIEW_ACTIONLOGS
        VIEW_CATALOGUE
        VIEW_LISTING
        VIEW_NEWSLETTER_SUBSCRIBERS
        VIEW_PAGE_QUERIES
        VIEW_REPORTS
        VIEW_SENT_MESSAGE
        VIEW_SUBSCRIBERS
        VOTE_MEMBER_IP_RESTRICT
        VOTING
        VPN_PASSWORD
        VPN_USERNAME
        VR_DAILY
        VR_MONTHLY
        VR_NEVER
        VR_QUARTERLY
        VR_WEEKLY
        WAGES
        WARNINGS
        WARNING_DB_OVERWRITE
        WARNING_DISK_SPACE
        WARNING_FILE_ADDON
        WARNING_FILE_ALIEN
        WARNING_FILE_CHMOD
        WARNING_FILE_FROM_UNINSTALLED_ADDON
        WARNING_FILE_FUTURE_FILES
        WARNING_FILE_MISSING_FILE_ENTIRELY
        WARNING_FILE_MISSING_ORIGINAL_BUT_HAS_OVERRIDE
        WARNING_FILE_OUTDATED
        WARNING_FILE_OUTDATED_ORIGINAL
        WARNING_FILE_OUTDATED_ORIGINAL_AND_OVERRIDE
        WARNING_MAX_EXECUTION_TIME
        WARNING_MBSTRING_FUNC_OVERLOAD
        WARNING_MODULES
        WARNING_MOVED_MODULES
        WARNING_RESET_BANNER_STATS
        WARNING_SUPER_DEBRAND_MAJOR_CHANGES
        WARNING_UNINSTALL
        WARNING_UNINSTALL_GENERAL
        WARN_MEMBER
        WARN_MEMBER_AS_SPAMMER
        WARN_TO_STACK_TRACE
        WARN_TO_STACK_TRACE_2
        WARN_TO_STACK_TRACE_3
        WATERMARK
        WATERMARKING
        WCAG_ACCESSKEY_UNIQUE
        WCAG_ADJACENT_LINKS
        WCAG_AREA_EQUIV
        WCAG_ATTRIBUTE_TOO_LONG
        WCAG_AUTO_SUBMIT_LIST
        WCAG_BAD_LAYOUT_TABLE
        WCAG_BLANK
        WCAG_DODGY_LINK
        WCAG_DODGY_LINK_2
        WCAG_HD_SPECIAL
        WCAG_HEADING_ORDER
        WCAG_LONGTEXT_DLINK
        WCAG_MISSING_TH
        WCAG_MOUSE_EVENT_UNMATCHED
        WCAG_NO_INPUT_LABEL
        WCAG_TABINDEX_UNIQUE
        WCAG_TH_TOO_LONG
        WEBCLIPICON
        WEBSITE_EMAIL
        WEBSTANDARDS
        WEBSTANDARDS_COMPAT
        WEBSTANDARDS_CSS
        WEBSTANDARDS_ERROR
        WEBSTANDARDS_ERRORS_HERE
        WEBSTANDARDS_ERROR_AT
        WEBSTANDARDS_EXT_FILES
        WEBSTANDARDS_IGNORE
        WEBSTANDARDS_IGNORE_2
        WEBSTANDARDS_JAVASCRIPT
        WEBSTANDARDS_MESSAGE
        WEBSTANDARDS_WCAG
        WEBSTANDARDS_XHTML
        WEB_NOTIFICATIONS_ENABLED
        WELCOME_A
        WELCOME_B
        WELCOME_EMAIL
        WELCOME_EMAILS
        WELCOME_EMAIL_MEMBERS
        WELCOME_EMAIL_USERGROUP_TYPE
        WELCOME_EMAIL_USERGROUP_TYPE_BOTH
        WELCOME_EMAIL_USERGROUP_TYPE_PRIMARY
        WELCOME_EMAIL_USERGROUP_TYPE_SECONDARY
        WELCOME_MESSAGE
        WELCOME_TO_COMMANDR
        WERE_YOU_LOOKING_FOR
        WHATSNEW
        WHATSNEW_CATEGORIES_SELECT
        WHATSNEW_TEXT
        WHAT_TO_EXPECT
        WHETHER_BANNED_IP
        WHETHER_MAKE_WARNING
        WHITE
        WIKI_ADD_PAGE
        WIKI_DELETE_OWN_POST
        WIKI_DELETE_PAGE
        WIKI_DELETE_POST
        WIKI_EDIT_OWN_POST
        WIKI_EDIT_PAGE
        WIKI_EDIT_PAGE_TEXT
        WIKI_EDIT_POST
        WIKI_EDIT_TREE
        WIKI_ENABLE_CHILDREN
        WIKI_ENABLE_CONTENT_POSTS
        WIKI_MANAGE_TREE_TEXT
        WIKI_MANAGE_TREE_TEXT_ADVANCED
        WIKI_MANAGE_TREE_TEXT_ADVANCED_LABEL
        WIKI_MERGE_TEXT
        WIKI_MOVE_POST
        WIKI_ORPHANED
        WIKI_POSTS
        WILL_BE_VALIDATED_WHEN_SAVING
        WILL_COST_THEM
        WILL_NEED_VALIDATING
        WINNER
        WINNERS_FOUND_AS_FOLLOWS
        WINNERS_HANDLING
        WINNER_FIRST
        WINNER_SECOND
        WINNER_THIRD
        WONT_BAN_SELF
        WONT_BAN_SERVER
        WORD
        WORDPRESS_DB_NAME
        WORDPRESS_DB_PASSWORD
        WORDPRESS_DB_USERNAME
        WORDPRESS_HOST_NAME
        WORDPRESS_TABLE_PREFIX
        WOULD_NOT_OVERWRITE_BUT_SUCCESS
        WRITE_ERROR
        WRITE_ERROR_CREATE
        WRITE_ERROR_DIRECTORY
        WRITE_ERROR_DIRECTORY_REPAIR
        WRITE_ERROR_MISSING_DIRECTORY
        WRONG_RESET_SESSION
        WROTE_CONFIGURATION
        XHTML_ANCESTOR_BLOCK_INLINE
        XHTML_ANCESTOR_INLINE_NORMAL
        XHTML_A_NAME
        XHTML_A_SPACES
        XHTML_BAD_ATTRIBUTE_VALUE
        XHTML_BAD_CHILD
        XHTML_BAD_ENTITY
        XHTML_BAD_PARENT
        XHTML_BAD_ROOT
        XHTML_BROKEN_URL
        XHTML_CASE_ATTRIBUTE
        XHTML_CASE_TAG
        XHTML_CDATA
        XHTML_CEMPTY_TAG
        XHTML_CONTENTTYPE
        XHTML_DEPRECATED_ATTRIBUTE
        XHTML_DEPRECATED_TAG
        XHTML_DESCRIPTION
        XHTML_DOCTYPE
        XHTML_DUPLICATED_ID
        XHTML_EMPTY_TAG
        XHTML_EXPECTING
        XHTML_FILE_VALUE
        XHTML_FORM_ENCODING
        XHTML_FORM_ENCODING_2
        XHTML_FORM_TYPE
        XHTML_ID_UNBOUND
        XHTML_IE_ONCHANGE
        XHTML_KEYWORDS
        XHTML_LOCAL_BROKEN_URL
        XHTML_MISSING_ANCESTOR
        XHTML_MISSING_ATTRIBUTE
        XHTML_MISSING_TAG
        XHTML_NAME_ID_DEPRECATED
        XHTML_NONEMPTY_TAG
        XHTML_NO_SIZE_FOR
        XHTML_ONLY_ONE_ALLOWED
        XHTML_PLACEHOLDER
        XHTML_PROHIBITION
        XHTML_SCRIPT_COMMENTING
        XHTML_SPAM
        XHTML_SPELLING
        XHTML_TEXT_NO_BLOCK
        XHTML_UNKNOWN_ATTRIBUTE
        XHTML_UNKNOWN_TAG
        XHTML_UNSAFE_CHAR
        XHTML_WIDTH
        XHTML_WRONG_COMMENTING
        XML_ATTRIBUTE_ERROR
        XML_BROKEN_END
        XML_JS_TAG_ESCAPE
        XML_MORE_CLOSE_THAN_OPEN
        XML_NEEDED
        XML_NO_CLOSE
        XML_NO_CLOSE_MATCH
        XML_PARSING_NOT_SUPPORTED
        XML_TAG_BAD_ATTRIBUTE
        XML_TAG_CLOSE_ANOMALY
        XML_TAG_DUPLICATED_ATTRIBUTES
        XML_TAG_OPEN_ANOMALY
        XML_UPLOAD_METHOD
        X_ENTRIES_REMAINING
        X_PER_DAY
        X_PER_MONTH
        X_PER_WEEK
        X_PER_YEAR
        YOUR_NEW_ZONE_PAGE
        YOU_MUST_KEEP_ONE_NEWS_CAT
        ZE_CLICK_TO_EDIT
        ZE_HOW_TO_SAVE
        ZIP_ERROR
        ZIP_ERROR_MZIP
        ZIP_NOT_ENABLED
        ZONE
        ZONES
        ZONE_ACCESS
        ZONE_ADD_TEXT
        ZONE_BASE_URL
        ZONE_COLON_PAGE
        ZONE_EDITOR
        ZONE_OPTIONS
        ZONE_OR_BLOCKS
        ZONE_WRITE
        _ADDON_WARNING_MISSING_DEPENDENCIES
        _ADDON_WARNING_PRESENT_DEPENDENCIES
        _BAD_INPUT
        _BLOCKS
        _CANCEL_MANUAL_SUBSCRIPTION
        _CHOOSE_EDIT_LIST_EXTRA
        _CHOOSE_EDIT_TABLE_EXTRA
        _COMCODE_HELPER
        _COMCODE_PAGE_EDIT
        _CONTENT_NEEDING_REVIEWING
        _CORRUPT_INSTALLATION_FILE
        _COUNT_BANDWIDTH
        _COUNT_DOWNLOADS
        _DELETE_MEMBER_ADMIN
        _DELETE_MEMBER_MERGE
        _DELETE_MEMBER_SEARCH
        _DESCRIPTION_ANSWER
        _DESCRIPTION_BANNER_TYPE
        _DESCRIPTION_MAIL_COST
        _DESCRIPTION_NAME
        _DESCRIPTION_REPRESENTATIVE_IMAGE
        _DESCRIPTION_THEME
        _DESCRIPTION_THEME_CNS
        _DESCRIPTION_THUMBNAIL
        _DESCRIPTION_WATERMARK
        _EDIT_CSS
        _EDIT_CUSTOM_PRODUCT
        _EDIT_FORWARDING_DOMAIN
        _EDIT_MENU
        _EDIT_PERMISSION_PRODUCT
        _EDIT_POP3_DOMAIN
        _EDIT_TEMPLATE
        _EDIT_THEME
        _EVENTS_THIS_MONTH
        _EVENTS_THIS_WEEK
        _EVENTS_THIS_YEAR
        _FAILED_TO_OPEN_POPUP
        _FILEDUMP
        _FILEDUMP_EMBED
        _FORUM_PATH
        _FORUM_SETTINGS
        _FROM_IMPORTING_SYSTEM
        _GIVE_AWARD
        _GROUP_NO_EXIST
        _HTTP_DOWNLOAD_NO_SERVER
        _HTTP_REDIRECT_PROBLEM_INSTALLING
        _HTTP_REDIRECT_PROBLEM_RUNNING
        _IMPORT_ALL_FINISHED
        _IP_FORWARDING_DISABLED
        _IP_FORWARDING_ENABLED
        _IP_FORWARDING_INSTALLING
        _LENGTH_UNIT_d
        _LENGTH_UNIT_m
        _LENGTH_UNIT_w
        _LENGTH_UNIT_y
        _LOGOWIZARD
        _MEMORY_USAGE
        _MERGE
        _MERGE_WIKI_POSTS
        _MESSAGING_FORUM_NAME
        _MISSING_INSTALLATION_FILE
        _MODIFIED
        _MUST_BE_EMAIL_DOMAIN
        _NEW_COMMANDRCHAT_MESSAGES
        _PAGE_STATISTICS
        _PASSWORD_RESET_TEXT_emailed
        _PASSWORD_RESET_TEXT_temporary
        _PASSWORD_RESET_TEXT_ultra
        _PERFORM_MULTI_MODERATION
        _PRIVILEGES
        _REALTIME_RAIN
        _SCREEN_PREVIEWS
        _SELECT_STATS_RANGE
        _SET_TICKET_EXTRA_ACCESS
        _SIZE
        _SUBSCRIPTION_EXPIRY_TIME
        _SUBSCRIPTION_START_TIME
        _SUBSCRIPTION_TERM_END_TIME
        _SUBSCRIPTION_TERM_START_TIME
        _SUHOSIN_MAX_VARS_TOO_LOW
        _TAKE_RESPONSIBILITY
        _TASK_FAILED_SUBJECT
        _THEMEWIZARD
        _TRANSLATE_CODE
        _WATERMARK
        _WIKI_EDIT_TREE
        _WITHOUT_WWW
        _WITH_WWW
        _ZONE_EDITOR
        __SUHOSIN_MAX_VARS_TOO_LOW
        __TRANSLATE_CODE
        __TRANSLATE_CONTENT
        __WARN_MEMBER
        __WIKI_EDIT_TREE
        DESCRIPTION_DEFAULT_VALUE_CPF
        MANUAL
        MULTI_MODERATIONS
        CONFIG_OPTION_one_per_email_address_VALUE_0
        CONFIG_OPTION_one_per_email_address_VALUE_1
        CONFIG_OPTION_one_per_email_address_VALUE_2
        CONFIRM_REALLY
        PASSWORDS_DO_NOT_REUSE
        PASSWORD_INSECURE
    ')));

    $non_admin_files = array(
    );

    $non_admin_string_suffixes = array(
        'ACCESS_DENIED__',
        'ACTIVITY_',
        'NOTIFICATION_TYPE_',
    );

    $lang_strings_non_admin_initial = array_map('trim', explode("\n", trim('
        takes_lots_of_space
        DESCRIPTION_AWARD_OVERVIEW
        EMAILS
        AVATARS_CARTOONS
        AVATARS_THEMATIC
        AVATARS_MISC
        ADD_AUTHOR
        ADD_CALENDAR_EVENT
        ADD_CATALOGUE_ENTRY
        ADD_CLUB
        ADD_DOWNLOAD
        ADD_GALLERY
        ADD_IMAGE
        ADD_NEWS
        ADD_NEWS_BLOG
        ADD_POLL
        ADD_PRIVATE_CALENDAR_EVENT
        ADD_PUBLIC_CALENDAR_EVENT
        ADD_QUIZ
        ADD_VIDEO
        COPYRIGHTED
        ECOMMERCE
        DESCRIPTION_DOTW
        DEFAULT_CPF_about_NAME
        NO_DAYLIGHT_SAVING
        NOTIFICATION_TYPE_payment_received
        TASK_COMPLETED
        TASK_COMPLETED_BODY
        TASK_COMPLETED_BODY_ATTACHMENT
        TASK_COMPLETED_BODY_SIMPLE
        TASK_COMPLETED_SUBJECT
        TASK_FAILED_BODY
        TASK_FAILED_SUBJECT
        VIEWS
        ATTACHMENT_SAVED
        LOGIN_TEXT
        FILES
        MEMBERS_ACTIVE_THIS_MONTH
        MEMBERS_ACTIVE_THIS_WEEK
        MEMBERS_ACTIVE_TODAY
        MEMBERS_NEW_THIS_MONTH
        MEMBERS_NEW_THIS_WEEK
        MEMBERS_NEW_TODAY
        PAGE_VIEWS_THIS_MONTH
        PAGE_VIEWS_THIS_WEEK
        PAGE_VIEWS_TODAY
        PAGES
        COUNT_CHATPOSTS
        COUNT_CHATTERS
        COUNT_FILES
        COUNT_GIFT_POINTS_USED
        COUNT_ONFORUMS
        COUNT_ONSITE
        COUNT_ONSITE_RECORD
        COUNT_POINTS_EVER
        COUNT_POINTS_LEFT
        COUNT_POINTS_USED
        COUNT_POSTSCOUNT
        COUNT_POSTSTODAY
        COUNT_TOPICSCOUNT
        CONTENTS
        FILE
        NEWSLETTER_SUBSCRIBER_DEFAULT_NAME
        ABOUT
        ABOUT_BLOCKING
        ABOUT_POP3
        ABOUT_QUOTA
        ABOUT_TO_APPLY_FREE_ACCESS
        ABOUT_TO_APPLY_LEADER
        ABOUT_TO_APPLY_STAFF
        ACCEPTED_MESSAGE
        ACCESS_DENIED
        ACCESS_DENIED__ACCESS_DENIED
        ACCESS_DENIED__CATALOGUE_ACCESS
        ACCESS_DENIED__CATEGORY_ACCESS
        ACCESS_DENIED__CATEGORY_ACCESS_LEVEL
        ACCESS_DENIED__CHATROOM_UNAUTHORISED
        ACCESS_DENIED__NOT_AS_GUEST
        ACCESS_DENIED__NOT_AS_GUEST_CALENDAR_PERFORMANCE
        ACCESS_DENIED__PAGE_ACCESS
        ACCESS_DENIED__PRIVACY_BREACH
        ACCESS_DENIED__REUSE_ATTACHMENT
        ACCESS_DENIED__SESSION
        ACCESS_DENIED__ZONE_ACCESS
        ACCOUNT
        ACTION
        ACTIONS
        ACTIVATE_ACCOUNT
        ACTIVATE_POP3_ACCOUNT
        ACTIVE
        ACTIVE_TOPICS_IN
        ACTIVITY
        ACTIVITY_ADD_POST_IN
        ACTIVITY_ADD_TOPIC
        ACTIVITY_ENTERED_COMPETITION
        ACTIVITY_FILLED_SURVEY
        ACTIVITY_GIVE_POINTS
        ACTIVITY_LIKES
        ACTIVITY_LIKES_UNTITLED
        ACTIVITY_PASSED_TEST
        ACTIVITY_SUBSCRIBED_EVENT
        ADD
        NO_PRIVATE_SELF
        ADDED
        ADDED_COMCODE_ONLY
        ADDED_COMCODE_ONLY_ATTACHMENT
        ADDED_COMCODE_ONLY_SAFE_ATTACHMENT
        ADDED_COMCODE_ONLY_SAFE_ATTACHMENT_INSTANT
        ADDED_COMMENT_ON
        ADDED_COMMENT_ON_UNTITLED
        ADDED_COMMENT_ON_UNTYPED
        ADDED_SIMPLE
        ADDED_TO_CART
        ADDITIONAL_CUSTOM
        ADDITIONAL_INFO
        ADDITIONAL_PT_POSTS
        ADDRESS
        ADDRESSES_ABOUT
        ADDRESS_CURRENT
        ADDRESS_DESIRED
        ADDRESS_DESIRED_DOMAIN
        ADDRESS_DESIRED_STUB
        ADD_ATTACHMENT
        ADD_ATTACHMENTS_MEDIA_LIBRARY
        ADD_BLOCK
        ADD_BOOKMARK
        ADD_DATE
        ADD_FRIEND_ACTION_DESCRIPTION
        ADD_IMAGES
        NEW_INLINE_PERSONAL_POST
        ADD_MEDIA
        ADD_NEW_PAGE
        ADD_POST
        ADD_PRIVATE_TOPIC
        ADD_SCHEDULED_TOPIC
        ADD_TICKET
        ADD_TOPIC
        ADD_TOPIC_POLL
        ADD_TO_CART
        ADD_TO_DIGG
        ADD_TO_FACEBOOK
        ADD_TO_FAVORITES
        ADD_TO_TWITTER
        ADD_WIKI_PAGE_BODY
        ADD_WIKI_PAGE_SUBJECT
        ADD_WIKI_POST_BODY
        ADD_WIKI_POST_SUBJECT
        ADMINISTRATORS
        ADMIN_ZONE
        ADVANCED
        AGE
        AGE_RANGE
        AGREEMENT
        AGREEMENT_PROCESS
        ALERT
        ALL
        ALLOW_EMAILS
        ALLOW_EMAILS_FROM_STAFF
        ALLOW_LIST
        VIEW_PAST_WINNERS
        ALLOW_LIST_GROUPS
        ALL_DAY_EVENT
        ALL_EM
        ALPHABETICAL_BACKWARD
        ALPHABETICAL_FORWARD
        ALREADY_APPLIED_FOR_GROUP
        ALREADY_CONFIRMED
        ALREADY_CONFIRMED_THIS
        ALREADY_EMAIL_ADDRESS
        ALREADY_FRIENDS
        ALREADY_HAVE
        ALREADY_HAVE_THIS
        ALREADY_HAVE_THIS_SINGLE
        ALREADY_IN_GROUP
        ALREADY_MEMBER_LOGIN
        ALREADY_VALIDATED
        ALT_COPY_EXISTING_POLL
        ALT_FIELD
        AMOUNT
        ANIMATION
        ANON
        ANSWER
        ANY_PROBLEMS_CAN
        APPLY_TO_GROUP
        ARCHIVES
        ARE_YOU_SURE_DELETE
        ASCENDING
        ASSIGNED_TO
        ASSIGN_TO
        ATOZ
        ATTACHMENT
        ATTACHMENTS
        ATTACHMENTS_OF
        ATTACHMENT_HELP
        ATTACHMENT_HELP_2
        ATTACHMENT_MULTI
        ATTACHMENT_POPUP
        ATTACHMENT_WOULD_NOT_UPLOAD
        AUDIO
        AUDIOS
        AUDIO_VERSION
        AUTHOR
        AUTHORS
        AUTHOR_ABOUT
        AUTHOR_FUNCTIONS
        AUTHOR_HOMEPAGE
        AUTHOR_NULL
        AUTHOR_POINTS
        AUTHOR_PROFILE
        AUTO_NOTIFICATION_CONTRIB_CONTENT
        AUTO_SAVING
        AVATAR
        AVATAR_CHANGE
        AVERAGE
        AVERAGED
        AWAITING_GROUP_LEADER
        AWAITING_MEMBER_VALIDATION
        AWARD
        AWARDED_TO
        AWARDS
        AWARDS_AFTER_VALIDATION
        AWARDS_WON
        AWARD_HISTORY
        AWARD_ON
        AWARD_OVERVIEW
        AWARD_WINNER
        A_FORUM
        A_TOPIC
        BACK_TO_TOP
        BAD_CARD_DATE
        BAD_HTML_COLOUR
        BAD_SECURITY_CODE
        BANNED
        BANNED_FROM_CHAT
        BANNER
        BANNER_CLICKED
        BAN_MEMBER_MAIL
        BAN_MEMBER_MAIL_SUBJECT
        BE_VISIBLE
        BIRTHDAY
        BIRTHDAYS
        BIRTHDAY_NOTIFICATION_MAIL
        BIRTHDAY_NOTIFICATION_MAIL_SUBJECT
        BIRTHDAY_OF
        BIRTHDAY_OF_AGE
        BLACK
        BLOG
        BLOGS
        BLOGS_POSTS
        BLOG_NEWS_ARCHIVE
        BLOG_NEWS_UNDER_THESE
        BLOG_NOTIFICATION_MAIL
        BLOG_NOTIFICATION_MAIL_SUBJECT
        BLOG_NO_NEWS
        BLOG_OLDER_NEWS
        BLOG__NEWS
        BLUE
        BODY_CONTENT
        BODY_TICKET_REROUTED
        BOOKMARKS
        BOOKMARK_THIS
        BOOLEAN_HELP
        BOOLEAN_SEARCH
        BREADCRUMBS
        BROWSE
        BROWSE_SENTENCE
        BROWSING_GALLERIES
        BUTTON_CART_EMPTY
        BUTTON_CART_ITEMS
        BUY
        BUY_NOW
        BY
        BY_SIMPLE
        BY_SIMPLE_LOWER
        CALENDAR
        CALENDAR_EVENT
        CALENDAR_EVENT_NOTIFICATION_MAIL
        CALENDAR_EVENT_NOTIFICATION_MAIL_SUBJECT
        CALENDAR_EVENT_VCAL
        CALENDAR_MONTHLY_RECURRENCE_CONCRETE_day_of_month
        CALENDAR_MONTHLY_RECURRENCE_CONCRETE_day_of_month_backwards
        CALENDAR_MONTHLY_RECURRENCE_CONCRETE_dow_of_month
        CALENDAR_MONTHLY_RECURRENCE_CONCRETE_dow_of_month_backwards
        CALENDAR_MONTHLY_RECURRENCE_day_of_month
        CALENDAR_MONTHLY_RECURRENCE_day_of_month_backwards
        CALENDAR_MONTHLY_RECURRENCE_dow_of_month
        CALENDAR_MONTHLY_RECURRENCE_dow_of_month_backwards
        CALENDAR_SPECIFIC
        CALENDAR_SPECIFIC_WEEK
        CANCEL_MANUAL_SUBSCRIPTION
        CANCEL_ORDER_CONFIRMATION_MESSAGE
        CANNOT_BEFRIEND_ONESELF
        CANNOT_REUSE_PASSWORD
        CANNOT_WILDCARD_START
        CANT_AFFORD
        CAPTCHA
        CAPTCHA_NO_SESSION
        CAPTION
        CARDHOLDER_NAME
        CARD_CV2
        CARD_EXPIRY_DATE
        CARD_ISSUE_NUMBER
        CARD_NUMBER
        CARD_START_DATE
        CARD_TYPE
        CARE_PLEASE
        CART_EMPTIED
        CART_EMPTY
        CART_ORDER
        CART_UPDATED
        CART_VALIDATION_REQUIRE_NUMBER
        CASCADE_TOPIC
        CASCADING
        CATALOGUE
        CATALOGUES
        CATALOGUE_CATEGORIES
        CATALOGUE_ENTRIES
        CATALOGUE_ENTRY
        CATALOGUE_ENTRY_NOTIFICATION_MAIL
        CATALOGUE_ENTRY_NOTIFICATION_MAIL_SUBJECT
        CATALOGUE_GENERIC
        CATALOGUE_GENERIC_CATEGORY
        CATEGORIES
        CATEGORISE_PTS
        CATEGORISE_PTS_AS
        CATEGORY
        CATEGORY_SUBORDINATE
        CATEGORY_SUBORDINATE_2
        CCP_ERROR
        CHANGE_HERE
        CHANGE_SETTINGS_BY_RESUBSCRIBING
        CHANGING_IN
        CHARGE
        CHARGES
        CHARGE_INFO
        CHAT
        DIGEST_ITEM_OMITTED
        INSTEAD_OF_SMS
        USERGROUP_SEARCH_COMBO
        CHATCODE_HELP
        CHATROOM
        CHATROOMS_LOBBY_TITLE
        CHATROOM_NAME
        CHATROOM_STATUS_PRIVATE
        CHATROOM_STATUS_PUBLIC
        CHAT_CHANGE_OPTIONS
        CHAT_DOWNLOAD_LOGS
        CHAT_DOWNLOAD_LOGS_CHATROOM_NAME
        CHAT_DOWNLOAD_LOGS_FINISH_DATE
        CHAT_DOWNLOAD_LOGS_FINISH_DATE_DESCRIPTION
        CHAT_DOWNLOAD_LOGS_START_DATE
        CHAT_DOWNLOAD_LOGS_START_DATE_DESCRIPTION
        CHAT_EFFECT_contact_off
        CHAT_EFFECT_contact_on
        CHAT_EFFECT_invited
        CHAT_EFFECT_message_background
        CHAT_EFFECT_message_initial
        CHAT_EFFECT_message_received
        CHAT_EFFECT_message_sent
        CHAT_EFFECT_you_connect
        CHAT_HELP_BOTMSG
        CHAT_INVITE_TEXT
        CHAT_INVITE_TEXT_REPLY
        CHAT_LOBBY
        CHAT_LOBBY_END_CHAT
        CHAT_LOGS
        CHAT_OPTIONS_COLOUR_NAME
        CHAT_OPTIONS_DESCRIPTION
        CHAT_OPTIONS_TEXT_NAME
        CHAT_PRIVATE_CHATROOM_DESCRIPTION
        CHAT_PRIVATE_DEFAULT_CHATROOM_NAME
        CHAT_PRIVATE_TITLE
        CHAT_SET_EFFECTS
        CHAT_TIME_BOTMSG
        CHAT_TOGGLE_COMCODE_BOX
        CHAT_USERSONLINE_BOTMSG
        CHAT_WEBSITE_HELPER_BOT
        CHECKOUT_NOTIFICATION_TEXT
        CHECK_OUT
        CHILD_PAGES
        CHILD_PLU
        CHOOSE
        CHOOSE_AUTHOR
        CHOOSE_AVATAR_BODY
        CHOOSE_AVATAR_SUBJECT
        CHOOSE_JOIN_USERGROUP
        CHOOSE_PHOTO_BODY
        CHOOSE_PHOTO_SUBJECT
        CHOOSE_SIGNATURE_BODY
        CHOOSE_SIGNATURE_SUBJECT
        CHOOSE_SOUND_EFFECTS
        CLEAR
        CLEAR_COLOR_SELECTION
        CLICK_DATE_FOR_MORE
        CLICK_TO_EDIT
        CLICK_TO_ENLARGE
        CLOSE
        CLOSED
        CLOSE_TICKET
        CLOSE_TOPIC
        CLOSE_VIA_END_CHAT_BUTTON
        CLOSING_HELP_PANEL_CONFIRM
        CLUB
        CLUBS
        CLUB_WITH_MEMBERS_APPROVAL
        CLUB_WITH_MEMBERS_OPEN
        CMS
        CMS_COMMUNITY_HELP
        CNS_DESCRIPTION_PUBLICATION_TIME
        CNS_LOGIN_AUTO
        CNS_LOGIN_INSTANT
        CNS_PROMOTION_INFO
        CNS_PUBLICATION_TIME
        CNS_SIGNUP_TEXT
        CNS_WAITING_CONFIRM_MAIL
        CNS_WAITING_CONFIRM_MAIL_INSTANT
        CNS_WAITING_CONFIRM_MAIL_VALIDATED
        CODE
        CODENAME
        CODE_IN_LANGUAGE
        COLLABORATION
        COLOUR
        COLUMNED_TABLE
        COMBINED_BIRTHDAY_NOTIFICATION_MAIL
        COMBINED_BIRTHDAY_NOTIFICATION_MAIL_ITEM
        COMBINED_BIRTHDAY_NOTIFICATION_MAIL_SUBJECT
        COMCODE_ATTACHMENT_WILL_HAVE_MARKER
        COMCODE_EDITABLE_BLOCK
        COMCODE_EDITABLE_TAG
        COMCODE_ERROR
        COMCODE_ERROR_HELP_A
        COMCODE_ERROR_HELP_B
        COMCODE_ERROR_TITLE
        COMCODE_GROUP_display_code
        COMCODE_GROUP_dynamic_back_end
        COMCODE_GROUP_dynamic_front_end
        COMCODE_GROUP_dynamic_front_end_adv
        COMCODE_GROUP_execute_code
        COMCODE_GROUP_formatting
        COMCODE_GROUP_linking
        COMCODE_GROUP_media
        COMCODE_GROUP_semantic
        COMCODE_GROUP_structure
        COMCODE_GROUP_structure_adv
        COMCODE_HELP
        COMCODE_HELPER
        COMCODE_HELPER_2
        COMCODE_MESSAGE
        COMCODE_REPLACEMENT
        COMCODE_TAGS
        COMING_SOON
        COMMENTS
        COMMENTS_DEFAULT_TEXT
        COMMENT_FORUM_NAME
        COMMUNITY
        COMPETITIONS
        COMPETITION_THANKYOU
        CONCEDED_MODE
        CONFIRM_DELETE
        CONFIRM_EMAIL_ADDRESS
        CONFIRM_EMAIL_SUBJECT
        CONFIRM_PASSWORD
        CONFIRM_TEXT
        CONFIRM_TOPIC_PIN
        CONTACT
        CONTACTS
        CONTACT_CITY
        CONTACT_COMPANY
        CONTACT_FIRST_NAME
        CONTACT_HOMEADDRESS
        CONTACT_HOMEPAGE
        CONTACT_HOMEPHONE
        CONTACT_IM
        CONTACT_LAST_NAME
        CONTACT_PHOTO
        CONTACT_STAFF_TO_JOIN_IF_IMPAIRED
        CONTACT_US
        CONTACT_WORKPHONE
        CONTENT
        CONTENT_BY
        CONTENT_IS_OF_TYPE
        CONTENT_LIKED
        CONTENT_LIKED_NOTIFICATION_MAIL
        CONTENT_LIKED_NOTIFICATION_MAIL_SUBJECT
        CONTENT_VALIDATED_NOTIFICATION_MAIL
        CONTENT_VALIDATED_NOTIFICATION_MAIL_SUBJECT
        CONTINUE_SHOPPING
        CONTRACT
        COOKIES_AS_GUEST
        COOKIE_NOTICE
        COPY_EXISTING_POLL
        CORRECT
        CORRECT_ANSWER
        COST
        COST_THEM_LINK
        COULD_NOT_UNSUBSCRIBE
        COUNTDOWN_MINUS
        COUNTDOWN_PLUS
        COUNTDOWN_SEP
        COUNTDOWN_SEP_ZERO
        COUNT_ARCHIVE
        COUNT_BANDWIDTH
        COUNT_DOWNLOADS
        COUNT_GIFT_POINTS_LEFT
        COUNT_MEMBERS
        COUNT_PAGES
        COUNT_POINTS
        COUNT_POINTS_GIVEN
        COUNT_POSTS
        COUNT_RESULTS
        COUNT_TOPICS
        COUNT_TOTAL
        COUNT_VIEWS
        COUNT_VISITS
        COUNT_VOTINGS
        AUDIO_CAPTCHA_HELP
        CREATE_BIRTHDAY_TOPIC
        CREATE_HERE
        CREATE_PRIVATE_CHATROOM
        CREATE_SUPPORT_TICKET
        CREATE_TICKET_MAKE_POST
        CURRENT
        CURRENTLY_AWARDED_TO
        CURRENT_POINTS
        CUSTOMER_OPTING_OUT_OF_TAX
        CUSTOMER_OPTING_OUT_OF_TAX_SHORT
        CUSTOM_ITEM_A
        CUSTOM_PROFILE_FIELD
        CUSTOM_UPLOAD
        DAILY
        DATE
        DATE_IN_PAST
        DATE_OF_BIRTH
        DATE_RANGE_LABEL
        DATE_TIME
        HOURS
        DAY
        DAYS
        DAYS_AGO
        DAYS_LAST_VISIT
        JANUARY
        FEBRUARY
        MARCH
        APRIL
        MAY
        JUNE
        JULY
        AUGUST
        SEPTEMBER
        OCTOBER
        NOVEMBER
        DECEMBER
        JANUARY_SHORT
        FEBRUARY_SHORT
        MARCH_SHORT
        APRIL_SHORT
        MAY_SHORT
        JUNE_SHORT
        JULY_SHORT
        AUGUST_SHORT
        SEPTEMBER_SHORT
        OCTOBER_SHORT
        NOVEMBER_SHORT
        DECEMBER_SHORT
        DECIDE_PER_CATEGORY_NOTIFICATIONS
        DECISION_TREE_SCREEN_NEEDS_PARAMETER
        DECLARE_EVENT_INTEREST
        DECLINED_MESSAGE
        DEFAULT_CALENDAR_TYPE__anniversary
        DEFAULT_CALENDAR_TYPE__appointment
        DEFAULT_CALENDAR_TYPE__birthday
        DEFAULT_CALENDAR_TYPE__commitment
        DEFAULT_CALENDAR_TYPE__free_time
        DEFAULT_CALENDAR_TYPE__general
        DEFAULT_CALENDAR_TYPE__public_holiday
        DEFAULT_CALENDAR_TYPE__system_command
        DEFAULT_CALENDAR_TYPE__vacation
        DEFAULT_CATALOGUE_CONTACTS_DESCRIPTION
        DEFAULT_CATALOGUE_FAQS_DESCRIPTION
        DEFAULT_CATALOGUE_FAQS_TITLE
        DEFAULT_CATALOGUE_LINKS_DESCRIPTION
        DEFAULT_CATALOGUE_LINKS_TITLE
        DEFAULT_CATALOGUE_PRODUCTS_DESCRIPTION
        DEFAULT_CATALOGUE_PRODUCTS_TITLE
        DEFAULT_CATALOGUE_PROJECTS_DESCRIPTION
        DEFAULT_CATALOGUE_PROJECTS_TITLE
        DEFAULT_CHATROOM_GENERAL_CHAT
        DEFAULT_CPF_about_DESCRIPTION
        DEFAULT_CPF_gender_DESCRIPTION
        DEFAULT_CPF_gender_NAME
        DEFAULT_CPF_im_jabber_DESCRIPTION
        DEFAULT_CPF_im_jabber_NAME
        DEFAULT_CPF_im_skype_DESCRIPTION
        DEFAULT_CPF_im_skype_NAME
        DEFAULT_CPF_interests_DESCRIPTION
        DEFAULT_CPF_interests_NAME
        DEFAULT_CPF_location_DESCRIPTION
        DEFAULT_CPF_location_NAME
        DEFAULT_CPF_occupation_DESCRIPTION
        DEFAULT_CPF_occupation_NAME
        DEFAULT_CPF_sn_facebook_DESCRIPTION
        DEFAULT_CPF_sn_facebook_NAME
        DEFAULT_CPF_sn_google_DESCRIPTION
        DEFAULT_CPF_sn_google_NAME
        DEFAULT_CPF_sn_twitter_DESCRIPTION
        DEFAULT_CPF_sn_twitter_NAME
        DEFAULT_CPF_staff_notes_DESCRIPTION
        DEFAULT_CPF_staff_notes_NAME
        DEFAULT_CPF_website_DESCRIPTION
        DEFAULT_CPF_website_NAME
        DEFAULT_FORUM_TITLE
        DEFAULT_GROUPING_TITLE
        DEFAULT_RANK_0
        DEFAULT_RANK_1
        DEFAULT_RANK_2
        DEFAULT_RANK_3
        DEFAULT_RANK_4
        DEFAULT__CATALOGUE_CATEGORY
        DEFAULT__CATALOGUE_CATEGORY_ATOZ
        DEFAULT__CATALOGUE_ENTRY
        DEFAULT__CATALOGUE_INDEX
        DELETE
        DELETE_IF_EMPTY
        DELETE_PHOTO
        DELETE_POST
        DELETE_POSTS
        DELETE_SELECTION
        DELETE_TOPIC
        DELETE_TOPICS
        DELETE_TOPICS_AND_POSTS
        DELETE_TOPIC_POLL
        DELETE_TOPIC_TEXT
        DELIVER
        DESCENDING
        DESCRIPTION
        DESCRIPTION_ADDITIONAL_ACCESS
        DESCRIPTION_ADDITIONAL_ACCESS_RAW
        DESCRIPTION_ADD_TOPIC_POLL
        DESCRIPTION_ADMINISTRATORS
        DESCRIPTION_ALLOW_EMAILS
        DESCRIPTION_ALLOW_EMAILS_FROM_STAFF
        DESCRIPTION_ALLOW_LIST
        DESCRIPTION_ALLOW_LIST_GROUPS
        DESCRIPTION_ALLOW_LIST_GROUPS_SIMPLE
        DESCRIPTION_ALTERNATE_URL
        DESCRIPTION_AUTO_NOTIFICATION_CONTRIB_CONTENT
        DESCRIPTION_CAPTCHA
        DESCRIPTION_CAPTCHA_2
        DESCRIPTION_CAPTCHA_3
        DESCRIPTION_CARDHOLDER_NAME
        DESCRIPTION_CARD_CV2
        DESCRIPTION_CARD_EXPIRY_DATE
        DESCRIPTION_CARD_ISSUE_NUMBER
        DESCRIPTION_CARD_NUMBER
        DESCRIPTION_CARD_START_DATE
        DESCRIPTION_CASCADING
        DESCRIPTION_CATEGORY
        DESCRIPTION_CHATROOM_LANG
        DESCRIPTION_CHATROOM_NAME
        DESCRIPTION_CHATROOM_OWNER
        DESCRIPTION_CLOSE_TICKET
        DESCRIPTION_CONCEDED_MODE
        DESCRIPTION_DELETE
        DESCRIPTION_DELETE_IF_EMPTY
        DESCRIPTION_DESTINATION_TOPIC
        DESCRIPTION_DISALLOW_LIST
        DESCRIPTION_DISALLOW_LIST_GROUPS
        DESCRIPTION_DOWNLOAD_ASSOCIATED_MEDIA
        DESCRIPTION_EMAIL_BCC_ADDRESS
        DESCRIPTION_EMAIL_CC_ADDRESS
        DESCRIPTION_EMPHASISED
        DESCRIPTION_ENABLE_AUTO_MARK_READ
        DESCRIPTION_EXPIRY_DATE
        DESCRIPTION_EXPLANATION
        DESCRIPTION_FEEDS_TO_OVERLAY
        DESCRIPTION_FRIENDS
        DESCRIPTION_GAMBLE_AMOUNT
        DESCRIPTION_HIGHLIGHTED_NAME
        DESCRIPTION_HIGHLIGHTED_NAME_P
        DESCRIPTION_INTERESTS
        DESCRIPTION_INTERESTS_2
        DESCRIPTION_I_AGREE_RULES
        DESCRIPTION_MAINTAINER
        DESCRIPTION_MAINTENANCE_PASSWORD
        DESCRIPTION_MARK_UNREAD
        DESCRIPTION_MAXIMUM_SELECTIONS
        DESCRIPTION_MEMBERS
        DESCRIPTION_MEMBER_BANNED
        DESCRIPTION_MEMBER_TITLE
        DESCRIPTION_MEMBER_VALIDATED
        DESCRIPTION_MINIMUM_SELECTIONS
        DESCRIPTION_NEW_BOOKMARK_FOLDER
        DESCRIPTION_OLD_BOOKMARK_FOLDER
        DESCRIPTION_OPEN
        DESCRIPTION_PASSWORD
        DESCRIPTION_PASSWORD_EDIT
        DESCRIPTION_PASSWORD_TEMPORARY
        DESCRIPTION_PINNED
        DESCRIPTION_POLL_IS_OPEN
        DESCRIPTION_POLL_IS_PRIVATE
        DESCRIPTION_POLL_REQUIRES_REPLY
        DESCRIPTION_POST_SHOW_AS_EDITED
        DESCRIPTION_POST_TEMPLATE
        DESCRIPTION_PREVIEW_POSTS
        DESCRIPTION_PRIVATE_TOPICS
        DESCRIPTION_QUESTION
        DESCRIPTION_REASON
        DESCRIPTION_REDIRECT_TO_TOPIC
        DESCRIPTION_REVEAL_AGE
        DESCRIPTION_SET_TICKET_EXTRA_ACCESS
        DESCRIPTION_SKIP_SIGNATURE
        DESCRIPTION_SUBSCRIBE_ADDRESS
        DESCRIPTION_SUNK
        DESCRIPTION_TIMEZONE_MEMBER
        DESCRIPTION_TOPICS_POSTED_IN
        DESCRIPTION_UPLOAD_CSV_FILE
        DESCRIPTION_USERNAME
        DESCRIPTION_VALIDATED
        DESCRIPTION_VIEWS_SIGNATURES
        DESCRIPTION_VISIBLE_TO
        DESCRIPTION_VISIBLE_TO_FRIENDS
        DESCRIPTION_VISIBLE_TO_GROUPS
        DESCRIPTION_VISIBLE_TO_GUESTS
        DESCRIPTION_VISIBLE_TO_MEMBERS
        DESCRIPTION_WHISPER
        DESCRIPTON_QUESTION
        DESKTOP_NOTIFICATION_SUBJECT
        DESTINATION
        DETAILS
        DETAILS_LIST
        DID_YOU_MEAN
        DIGEST_EMAIL_INDIVIDUAL_MESSAGE_WRAP
        DIGEST_EMAIL_MESSAGE_WRAP
        DIGEST_EMAIL_SUBJECT_16
        DIGEST_EMAIL_SUBJECT_4
        DIGEST_EMAIL_SUBJECT_8
        DIRECTION
        DISABLED
        DISABLED_FORM_FIELD
        DISABLED_FORM_FIELD_ENCHANCEDMSG
        DISABLED_FORM_FIELD_ENCHANCEDMSG_UPLOAD
        DISABLE_COMMENT_NOTIFICATIONS
        DISABLE_NOTIFICATIONS
        DISABLE_NOTIFICATION_SOUND
        DISABLE_WYSIWYG
        DISALLOW_LIST
        DISALLOW_LIST_GROUPS
        DISCARD_WYSIWYG_CHANGES
        DISCARD_WYSIWYG_CHANGES_LINE
        DISCARD_WYSIWYG_CHANGES_LINE_CONVERT
        DISCARD_WYSIWYG_CHANGES_NICE
        DISCUSSION
        DISMISS
        DISPLAYED_REVIEW_STATUS
        DOCUMENT
        DONATION
        DONE
        DOTW
        DOUBLE_POST_PREVENTED
        DOUBLE_SUBMISSION_PREVENTED
        DOWNLOAD
        DOWNLOADED_THIS
        DOWNLOADS_HOME
        DOWNLOADS_RELEASED
        DOWNLOADS_SO_FAR
        DOWNLOAD_ASSOCIATED_MEDIA
        DOWNLOAD_ATTACHMENT
        DOWNLOAD_CATEGORIES
        DOWNLOAD_CATEGORY
        DOWNLOAD_GALLERY_CONTENTS
        DOWNLOAD_GATEWAY
        DOWNLOAD_IN_PROGRESS
        DOWNLOAD_LICENCE
        DOWNLOAD_NOTIFICATION_MAIL
        DOWNLOAD_NOTIFICATION_MAIL_SUBJECT
        DOWNLOAD_NOUN
        DOWNLOAD_NOW
        DOWNLOAD_TITLE
        DPLU_DAYS
        DPLU_HOURS
        DPLU_MINUTES
        DPLU_MONTHS
        DPLU_YEARS
        DRAFT_SAVE
        DRAFT_SAVED_WITHOUT_VALIDATION
        DRAFT_SAVED_WITH_VALIDATION
        DUMP_FRIEND
        DUMP_FRIENDS
        DUPLICATE_JOIN_AUTH
        DYNAMIC_NOTICE_inform
        DYNAMIC_NOTICE_notice
        DYNAMIC_NOTICE_warning
        D_BEFORE_PROCEED_AGREE
        EACH
        ECARD_FOR_YOU
        ECARD_FOR_YOU_SUBJECT
        ECOMMERCE_PRODUCT_ALREADY_HAS
        ECOMMERCE_PRODUCT_DISABLED
        ECOMMERCE_PRODUCT_OUT_OF_STOCK
        ECOMMERCE_PRODUCT_PROHIBITED
        EDIT
        EDITED
        EDITING_FIRST_TOPIC_POST
        EDITING_TOPIC_NOT_POST
        EDIT_BOOKMARK
        EDIT_EM
        EDIT_LINK
        EDIT_MEMBER_PROFILE
        EDIT_MENU
        EDIT_OWN_POST
        EDIT_OWN_TOPIC
        EDIT_PAGE
        EDIT_POST
        EDIT_TICKET
        EDIT_TICKET_TYPE
        EDIT_TOPIC
        EDIT_TOPIC_POLL
        EMAIL
        EMAIL_ACCOUNTS
        EMAIL_ADDRESS
        EMAIL_ADDRESS_IN_USE
        EMAIL_ADDRESS_MISMATCH
        EMAIL_ATTACHMENTS
        EMAIL_BCC_ADDRESS
        EMAIL_CC_ADDRESS
        EMAIL_FOOT
        EMAIL_HEAD
        EMAIL_MEMBER
        EMAIL_MEMBER_SUBJECT
        EMAIL_MEMBER_TEXT
        EMAIL_TAKEN
        EMAIL_TYPES
        EMAIL_TYPES_DESCRIPTION
        EMOTICON
        EMOTICONS
        EMOTICONS_POPUP
        EMPHASISED
        EMPTY
        EMPTY_CART
        EMPTY_CONFIRM
        ENABLE_AUTO_MARK_READ
        ENABLE_COMMENT_NOTIFICATIONS
        ENABLE_NOTIFICATIONS
        ENABLE_NOTIFICATIONS_DAILY_EMAIL_DIGEST
        ENABLE_NOTIFICATIONS_INSTANT_EMAIL
        ENABLE_NOTIFICATIONS_INSTANT_PT
        ENABLE_NOTIFICATIONS_INSTANT_SMS
        ENABLE_NOTIFICATIONS_MONTHLY_EMAIL_DIGEST
        ENABLE_NOTIFICATIONS_WEB_NOTIFICATION
        ENABLE_NOTIFICATIONS_WEEKLY_EMAIL_DIGEST
        ENABLE_NOTIFICATIONS__CHOICE
        ENABLE_NOTIFICATIONS__STATISTICAL
        ENABLE_NOTIFICATION_SOUND
        ENABLE_POPUPS
        ENABLE_WYSIWYG
        END_CHAT
        QUIZ_MISTAKE_COMCODE
        QUIZ_MISTAKE_EXPLAINED_COMCODE
        QUIZ_MISTAKE_EXPLAINED_HTML
        QUIZ_MISTAKE_HTML
        QUIZ_RESULTS
        ENTER
        ENTERED_THE_CHATROOM
        ENTERED_THIS_QUIZ
        ENTER_ADDRESS
        ENTER_ALLOW
        ENTER_ATTACHMENT
        ENTER_BOX_TITLE
        ENTER_CAPTION
        ENTER_CHATROOM
        ENTER_CODE_FROM_EMAIL
        ENTER_FORUM_TO_ANSWER
        ENTER_HIDDEN_TEXT
        ENTER_IMAGE_CAPTION
        ENTER_LINK_NAME
        ENTER_LIST_ENTRY
        ENTER_PAGE_NUMBER
        ENTER_PASSWORD_JS
        ENTER_PASSWORD_JS_2
        ENTER_PROFILE_DETAILS
        ENTER_PROFILE_DETAILS_FINISH
        ENTER_QUOTE_BY
        ENTER_RECIPIENT
        ENTER_TOPIC_ID_MANUALLY
        ENTER_URL
        ENTER_WARNING
        ENTER_YOUR_BIRTHDAY
        ENTRIES
        ENTRY
        EPHEMS
        ERRORS
        ERROR_GUEST_EMAILS_MANDATORY
        ERROR_IMAGE_SAVE
        ERROR_NO_CONTACTS_SELECTED
        ERROR_OCCURRED
        ERROR_UPLOADING_ATTACHMENTS
        EVENT
        EVENTS_THIS_MONTH
        EVENTS_THIS_WEEK
        EVENTS_THIS_YEAR
        EVENT_CONTINUES
        EVENT_ENDS_ON
        EVENT_REMINDER_CONTENT
        EVENT_REMINDER_SUBJECT
        EVENT_TIME_RANGE
        EVENT_TIME_RANGE_WITHIN_DAY
        EVENT_TIME_RANGE_WITHIN_DAY_WITH_TIMEZONE
        EVENT_TIME_RANGE_WITH_TIMEZONE
        EVENT_TYPE
        EXAMPLE
        EXCEEDED_ATTACHMENT_SIZE
        EXCEEDED_TIME_LIMIT
        EXISTING
        EXISTING_BLOCKS
        EXPAND
        EXPLAINED_BLOCK_MEMBER
        EXPLAINED_UNBLOCK_MEMBER
        EXPLANATION
        EXTRACTED_FILE
        EXTRACTED_FILE_PATH
        EXT_NO_PASSWORD_CHANGE
        FAILED
        FAILED_TO_OPEN_POPUP
        FAILURE_UPLOAD_SYNDICATION_AUTH
        FC_MONDAY
        FC_TUESDAY
        FC_WEDNESDAY
        FC_THURSDAY
        FC_FRIDAY
        FC_SATURDAY
        FC_SUNDAY
        MONDAY
        TUESDAY
        WEDNESDAY
        THURSDAY
        FRIDAY
        SATURDAY
        SUNDAY
        MONDAY_SHORT
        TUESDAY_SHORT
        WEDNESDAY_SHORT
        THURSDAY_SHORT
        FRIDAY_SHORT
        SATURDAY_SHORT
        SUNDAY_SHORT
        FEATURES
        FEED
        FEEDBACK
        FEEDS
        FILENAME
        FILE_SIZE
        FILE_TOO_BIG_QUOTA
        FILTER
        FILTER_REGIONS
        FIND_ME
        FINISH_PROFILE
        FINISH_STARTED_ALREADY
        FIRST
        FIRST_POST
        FIRST_POST_IS_UNVALIDATED
        FIXED_COMCODE
        FLOOD_CONTROL_BLOCKED
        FLOOD_CONTROL_RESTRICT
        FOLDER
        FOLLOWING_SUBSCRIBED
        FONT
        FORCE_REMOVE_LOCALLY
        FORENAME
        FORM_AUTO_SUBMITS
        FORM_TIME_SECURITY
        FORUM_AND_TIME_HOVER
        FORUM_FOR_CLUB
        FORUM_INTRO_QUESTION_TEXT
        FORUM_INTRO_QUESTION_TEXT_ALT
        FORUM_NAME
        FORUM_NUM_MEMBERS
        FORUM_NUM_POSTS
        FORUM_NUM_TOPICS
        FORUM_ORDER_BY_FIRST_POST
        FORUM_ORDER_BY_LAST_POST
        FORUM_ORDER_BY_TITLE
        FORUM_PASSWORD_TO_INTRO_QUESTION
        FORUM_POST
        FORUM_POSTS
        FORUM_POST_ISOLATED_RESULT
        FORUM_POST_NUMBERED
        FORUM_RULES
        FORUM_STATISTICS
        FORUM_TOPIC
        FORUM_TOPICS
        FORUM_TOPIC_NUMBERED
        FORWARDING
        FOUND_THIS_ON
        FREE_PURCHASE
        FRIEND
        FRIENDS
        FRIEND_EMAIL_ADDRESS
        FRIEND_LIST
        FROM
        FROM_SIMPLE
        FULL_BLOCK_TUTORIAL
        FULL_COMCODE_TUTORIAL
        FULL_EDITOR
        FULL_NEWSLETTER_UNSUBSCRIBED
        GALLERIES
        GALLERIES_HOME
        GALLERY
        GALLERY_MOBILE_PAGE_OF
        GAMBLE_A
        GAMBLE_COMMISERATIONS
        GAMBLE_CONGRATULATIONS
        GAMBLING
        GAMBLING_DESCRIPTION
        GENERAL
        GIVEN_ANSWER
        GIVEN_POINTS_FOR
        GIVEN_POINTS_FOR_ANON
        GIVE_POINTS
        GIVE_TEXT
        GIVE_TEXT_UNLIMITED
        GIVE_TO
        GOTO_CHAT_LOBBY
        GOTO_CHAT_LOBBY_FOR_MORE
        GO_BACK
        GO_UPWARDS_TO
        GRAND_TOTAL
        GRAY
        GREEN
        GROUP_ACCEPTED_MAIL
        GROUP_ACCEPTED_MAIL_SUBJECT
        GROUP_ADVANCE
        GROUP_DECLINED_MAIL
        GROUP_DECLINED_MAIL_REASON
        GROUP_DECLINED_MAIL_SUBJECT
        GROUP_JOIN_REQUEST_MAIL
        GROUP_JOIN_REQUEST_MAIL_SUBJECT
        GROUP_LED_BY
        GROUP_MEMBERS
        GROUP_MEMBERS_SPECIFIC
        GROUP_NUM_MEMBERS
        GUEST
        GUESTS
        GUEST_NAME
        HAPPY_BIRTHDAY_PERSON
        HAS_NOT_READ
        HAS_RATING
        HAS_READ
        HAVE_COSTED_THEM
        HEIGHT
        HELP
        HELP_OR_ADVICE
        HERE
        HIDE
        HIGHLIGHTED_NAME
        HIGHLIGHT_NAME_A
        HOME
        HOUR
        HOURS_AGO
        HOVER_FOR_FULL
        HOVER_MOUSE_IMPORTANT
        HOW_REMOVE_CHAT_NOTIFICATION
        IDENTIFIER
        ID_NUM
        IF_FORGOTTEN_PASSWORD
        IF_NO_CONFIRM
        IGNORE
        IMAGE
        IMAGES
        IMAGE_ATTACHMENT
        IMAGE_BAD_DIMENSIONS
        IMAGE_NOTIFICATION_MAIL
        IMAGE_NOTIFICATION_MAIL_SUBJECT
        IMAGE_TOO_LARGE_FOR_THUMB
        IMPORTANT
        IMPROPERLY_FILLED_IN
        IM_INFO_CHAT_WITH
        IM_INVITED_MESSAGE
        IM_INVITED_SUBJECT
        IM_MULTI
        IN
        INACTIVE
        INCORRECT_CONFIRMATION
        INCORRECT_CONFIRM_CODE
        INFO_CONFIRM
        INLINE_PERSONAL_POSTS
        INLINE_POSTS_MOVED_MESSAGE
        INPUTSYSTEM_CANCEL
        INPUTSYSTEM_CLOSE
        INPUTSYSTEM_OK
        INPUTSYSTEM_TYPE_EITHER
        INPUT_CHATCODE_invite
        INPUT_CHATCODE_new_room
        INPUT_CHATCODE_private_message
        INPUT_COMCODE_attachment
        INPUT_COMCODE_b
        INPUT_COMCODE_block
        INPUT_COMCODE_box
        INPUT_COMCODE_code
        INPUT_COMCODE_comcode
        INPUT_COMCODE_email
        INPUT_COMCODE_font
        INPUT_COMCODE_hide
        INPUT_COMCODE_html
        INPUT_COMCODE_i
        INPUT_COMCODE_img
        INPUT_COMCODE_list
        INPUT_COMCODE_menu
        INPUT_COMCODE_page
        INPUT_COMCODE_quote
        INPUT_COMCODE_thumb
        INPUT_COMCODE_url
        INSTALL
        INSTANT_MESSAGING
        INSTANT_MESSAGING_CONVO
        INTERESTS
        INTRODUCE_YOURSELF
        INTRO_ANSWER_INCORRECT
        INTRO_NOTICE
        INTRO_POST_DEFAULT
        INTRO_QUESTION
        MODERATED_AS_SPAM
        INVALID_EMAIL_ADDRESS
        INVALID_EMAIL_PREFIX
        INVALID_FILE_TYPE
        INVALID_FILE_TYPE_GENERAL
        INVALID_GAMBLE_AMOUNT
        INVALID_OPERATION
        INVALID_SECURITY_CODE_ENTERED
        INVISIBLE
        INVISIBLE_TEXT
        INVITED_TO_PT
        INVITED_TO_PTS
        INVITED_TO_TOPIC_BODY
        INVITED_TO_TOPIC_SUBJECT
        INVITE_ALREADY_JOINED
        INVITE_CURRENT_IM
        INVITE_MEMBER_MESSAGE
        INVITE_MEMBER_TO_PT
        INVITE_MEMBER_TO_PT_TEXT
        INVITE_ONLY
        INVOICE
        INVOICE_MESSAGE
        INVOICE_OF
        INVOICE_SUBJECT
        INVOLVED_TOPICS
        IN_PRINT_MODE
        IN_PROBATION
        IN_TOPIC_PP
        IP_ADDRESS
        IS_PUBLIC
        I_AGREE
        I_DISAGREE
        JOIN
        JOINED
        JOINED_AGO
        JOINED_DATE_RANGE
        JOINING
        JOIN_DATE
        JOIN_DISALLOW
        JOIN_HERE
        JOIN_OR_LOGIN
        JUMP
        JUMP_TO_FIRST_UNREAD
        JUMP_TO_PAGE
        JUST_NEWS_CATEGORIES
        KEYBOARD_MAP
        KEYWORD
        KEYWORDCHECK
        LABEL
        LACKING_POINTS
        LANGUAGE
        LARGEST_FIRST
        LAST
        LAST_ACTIVITY
        LAST_EDIT_BY
        LAST_POST
        LAST_POST_BY
        LAST_POST_BY_SIMPLE
        LAST_SLIDE
        ORDER_DISPATCHED_MAIL_MESSAGE
        ORDER_DISPATCHED_MAIL_SUBJECT
        LAST_SUBMIT_TIME
        LAST_VIEWED
        LAST_VISIT
        LAST_VISIT_TIME
        LATITUDE
        LEADER_BOARD_ABOUT
        LEADER_BOARD_PAGE_TEXT
        LEFT_CHATROOM
        LESS
        LICENCE_WAS_ACCEPTED
        LIFETIME_POINTS
        LIKED_BY
        LIKES
        QUESTIONS
        COMPETITION
        SURVEY
        LINK_NEW_WINDOW
        LIST_SEP
        LOAD
        LOADING
        LOAD_MORE
        locale_subst
        locale_ctype_hack
        LOCATED_IN
        LOCATION
        LOG
        LOGGED_IN
        LOGGED_IN_AS
        LOGGED_OUT
        LOGIN
        LOGIN_CONCEDED
        LOGIN_FREQUENCY
        LOGOUT
        LONGITUDE
        LOST_PASSWORD
        LOST_PASSWORD_CONFIRM
        LOST_PASSWORD_FINAL
        LOST_PASSWORD_TEXT
        LOST_PASSWORD_TEXT_TEMPORARY
        LOW_STOCK_DISPATCH_FAILED
        MAIL_FORW_A
        MAIL_FORW_B
        MAIL_NEW_PASSWORD
        MAIL_NEW_PASSWORD_EMAIL_LOGIN
        MAIL_POP3_A
        MAIL_POP3_B
        MAIL_QUOTA_A
        MAIL_SENT
        MAIL_SENT_TEXT
        MAINTAINER
        MAKE_ANONYMOUS_POST
        MAKE_ANONYMOUS_POST_DESCRIPTION
        MAKE_COMMENT
        MAKE_FRIEND
        MAKE_PAYMENT
        MAKE_PERSONAL
        MAKE_PERSONAL_DESCRIPTION
        MAKE_POST
        MAKE_POST_SHORT
        MANAGE_BOOKMARKS
        MANUALLY_MARKED
        MAP_POSITION_FIELD
        MAP_TABLE
        MARKED_POST_ACTIONS
        MARKER
        MARKS
        MARK_JAVASCRIPT_DISABLED
        MARK_READ
        MARK_UNREAD
        MARK_UNREAD_TOO_OLD
        MAXIMUM_SELECTIONS
        MAXIMUM_UPLOAD
        MAY_LEAVE_BLANK_ADVANCED
        MEC_NOTIFICATION_MAIL
        MEC_NOTIFICATION_MAIL_SUBJECT
        MEDIA
        MEDIA_FEATURES
        MEDIA_SET
        MEDIA_TYPE_audio_general
        MEDIA_TYPE_audio_microsoft
        MEDIA_TYPE_audio_websafe
        MEDIA_TYPE_code
        MEDIA_TYPE_email
        MEDIA_TYPE_extract
        MEDIA_TYPE_flash
        MEDIA_TYPE_hyperlink
        MEDIA_TYPE_image_websafe
        MEDIA_TYPE_oembed
        MEDIA_TYPE_pdf
        MEDIA_TYPE_quicktime
        MEDIA_TYPE_realaudio
        MEDIA_TYPE_realmedia
        MEDIA_TYPE_svg
        MEDIA_TYPE_video_facebook
        MEDIA_TYPE_video_general
        MEDIA_TYPE_video_microsoft
        MEDIA_TYPE_video_websafe
        MEDIA_TYPE_vimeo
        MEDIA_TYPE_youtube
        SHIPPING_ADDRESS_CITY
        SHIPPING_ADDRESS_COUNTRY
        SHIPPING_ADDRESS_NAME
        SHIPPING_ADDRESS_STATE
        SHIPPING_ADDRESS_STREET
        SHIPPING_ADDRESS_ZIP
        MEMBER
        MEMBERS
        MEMBERSHIP_LENGTH
        MEMBERS_VIEWING
        MEMBER_ACCOUNT
        MEMBER_BAD_PASSWORD
        MEMBER_INVALID_LOGIN
        MEMBER_BLOCKING
        MEMBER_CALENDAR
        MEMBER_CALENDAR_NOTIFICATION_DELETED_EVENT_BODY
        MEMBER_CALENDAR_NOTIFICATION_DELETED_EVENT_SUBJECT
        MEMBER_CALENDAR_NOTIFICATION_EDITED_EVENT_BODY
        MEMBER_CALENDAR_NOTIFICATION_EDITED_EVENT_SUBJECT
        MEMBER_CALENDAR_NOTIFICATION_NEW_EVENT_BODY
        MEMBER_CALENDAR_NOTIFICATION_NEW_EVENT_SUBJECT
        MEMBER_CALENDAR_NOTIFICATION_RESCHEDULED_EVENT_BODY
        MEMBER_CALENDAR_NOTIFICATION_RESCHEDULED_EVENT_SUBJECT
        MEMBER_CATEGORY
        MEMBER_DIRECTORY
        MEMBER_DIRECTORY_FILTERED
        MEMBER_DIRECTORY_FILTERED_NO_RESULTS
        MEMBER_DIRECTORY_UNFILTERED
        MEMBER_DIRECTORY_UNFILTERED_NO_RESULTS
        MEMBER_DIRECTORY_UNFILTERED_NO_RESULTS_GALLERIES
        MEMBER_HAS_BEEN_CHARGED
        MEMBER_INFORMATION
        MEMBER_IS_IN_GROUP
        MEMBER_IS_UNCONFIRMED
        MEMBER_IS_UNVALIDATED
        MEMBER_LEADS_GROUP
        MEMBER_LOGIN_ERROR
        MEMBER_NOT_VALIDATED_EMAIL
        MEMBER_NOT_VALIDATED_STAFF
        MEMBER_NO_EMAIL_ADDRESS_RESET_TO
        MEMBER_POINT_FIND
        MEMBER_TITLE
        MEMBER_TITLE_TOO_BIG
        MEMBER_TO_INVITE
        MEMBER_VALIDATED
        MEMBER_WARNING
        MENU
        MERGE_WIKI_POSTS
        MESSAGE
        MESSAGE_POSTED
        MESSAGE_POSTING_ERROR
        MESSAGE_SENT
        MESSAGE_THEM
        MINIMUM_SELECTIONS
        MINUTE
        MINUTES
        MINUTES_AGO
        MISSING_RESOURCE
        MISSING_RESOURCE_SUBSTRING
        MJG_NOTIFICATION_MAIL
        MJG_NOTIFICATION_MAIL_SUBJECT
        MM_POST_TEXT
        MOBILE_VERSION
        MODERATION_OPTIONS
        MODIFIED
        MODIFIER_announcement
        MODIFIER_closed
        MODIFIER_hot
        MODIFIER_involved
        MODIFIER_pinned
        MODIFIER_poll
        MODIFIER_sunk
        MODIFIER_unvalidated
        MODIFY_POINTS
        MODULE_TRANS_NAME_authors
        MODULE_TRANS_NAME_awards
        MODULE_TRANS_NAME_banners
        MODULE_TRANS_NAME_bookmarks
        MODULE_TRANS_NAME_calendar
        MODULE_TRANS_NAME_catalogues
        MODULE_TRANS_NAME_chat
        MODULE_TRANS_NAME_contact_member
        MODULE_TRANS_NAME_downloads
        MODULE_TRANS_NAME_forums
        MODULE_TRANS_NAME_forumview
        MODULE_TRANS_NAME_galleries
        MODULE_TRANS_NAME_groups
        MODULE_TRANS_NAME_invoices
        MODULE_TRANS_NAME_join
        MODULE_TRANS_NAME_leader_board
        MODULE_TRANS_NAME_login
        MODULE_TRANS_NAME_lost_password
        MODULE_TRANS_NAME_members
        MODULE_TRANS_NAME_news
        MODULE_TRANS_NAME_newsletter
        MODULE_TRANS_NAME_points
        MODULE_TRANS_NAME_pointstore
        MODULE_TRANS_NAME_polls
        MODULE_TRANS_NAME_purchase
        MODULE_TRANS_NAME_quiz
        MODULE_TRANS_NAME_recommend
        MODULE_TRANS_NAME_search
        MODULE_TRANS_NAME_staff
        MODULE_TRANS_NAME_subscriptions
        MODULE_TRANS_NAME_tickets
        MODULE_TRANS_NAME_topics
        MODULE_TRANS_NAME_topicview
        MODULE_TRANS_NAME_users_online
        MODULE_TRANS_NAME_vforums
        MODULE_TRANS_NAME_warnings
        MODULE_TRANS_NAME_wiki
        MONTH
        MONTHLY
        MORE
        MORE_INFO
        MORE_OPTIONS
        MOST_ACTIVE_FORUM
        MOST_DOWNLOADED
        MOVE
        MOVE_OR_DELETE_BOOKMARKS
        MULTIPLE
        MUST_ADD_CONTACTS
        MUST_BE_REAL_ADDRESS
        MUST_LOGIN
        MUTUAL_FRIEND
        MY_INVOICES
        MY_ORDERS
        MY_ORDER_DETAILS
        MY_PROFILE
        MY_SUBSCRIPTIONS
        NA
        NAME
        NAMED_FORUM
        NAMED_PRIVATE_TOPIC
        NAMED_TOPIC
        NAME_HIGHLIGHTING
        NAME_HIGHLIGHTING_DESCRIPTION
        NA_EM
        NC_art
        NC_business
        NC_community
        NC_difficulties
        NC_entertainment
        NC_general
        NC_technology
        NETWORK
        NETWORK_DOWN
        NEW
        NEWEST_FIRST
        NEWEST_MEMBER
        NEWEST_MEMBER_WELCOME
        NEWEST_RESULTS_FIRST
        NEWS
        NEWSLETTER
        NEWSLETTER_0
        NEWSLETTER_1
        NEWSLETTER_2
        NEWSLETTER_3
        NEWSLETTER_4
        NEWSLETTER_CONFIRM
        NEWSLETTER_CONFIRMED
        NEWSLETTER_GENERAL
        NEWSLETTER_JOIN
        NEWSLETTER_PASSWORD_BEEN_RESET
        NEWSLETTER_PASSWORD_CHANGE
        NEWSLETTER_PASSWORD_ENCRYPTED
        NEWSLETTER_PASSWORD_RESET
        NEWSLETTER_SIGNUP
        NEWSLETTER_SIGNUP_TEXT
        NEWSLETTER_SUBSCRIPTIONS
        NEWSLETTER_UNSUBSCRIBED
        NEWSLETTER_UNSUBSCRIBE_MEMBER
        NEWSLETTER_UNSUBSCRIBE_NEWSLETTER
        NEWSLETTER_UPDATE
        NEWS_ARCHIVE
        NEWS_CATEGORIES
        NEWS_CATEGORY
        NEWS_FROM
        NEWS_MAIL_AVAILABLE
        NEWS_NOTIFICATION_MAIL
        NEWS_NOTIFICATION_MAIL_SUBJECT
        NEWS_RELEASED
        NEWS_UNDER_THESE
        NEW_CLUB_NOTIFICATION_MAIL
        NEW_CLUB_NOTIFICATION_MAIL_SUBJECT
        NEW_COMMENT_A
        NEW_COMMENT_B
        NEW_COMMENT_BODY
        NEW_COMMENT_C
        NEW_COMMENT_D
        NEW_COMMENT_E
        NEW_COMMENT_F
        NEW_COMMENT_SUBJECT
        NEW_MEMBER_NOTIFICATION_MAIL
        NEW_MEMBER_NOTIFICATION_MAIL_SUBJECT
        NEW_MESSAGES
        NEW_PASSWORD
        NEW_PASSWORD_MAILED
        NEW_PERSONAL_GALLERY_OF
        NEW_PERSONAL_POST_MESSAGE
        NEW_PERSONAL_POST_SUBJECT
        NEW_POSTS
        NEW_PP_NOTIFICATION
        NEW_PRIVATE_TOPIC
        NEW_PRIVATE_TOPIC_MESSAGE
        NEW_PRIVATE_TOPIC_SHORT
        NEW_PRIVATE_TOPIC_SUBJECT
        NEW_PT_NOTIFICATION
        NEW_PT_NOTIFICATION_DETAILS
        NEW_TICKET_WELCOME
        NEW_TOPICS
        NEW_UGROUP_SUB_FOR
        NEW_WARNING_TO_YOU
        NEW_WINDOW
        NEXT
        NO
        NOBODY_ONLINE
        NONE
        NONE_EM
        NONMOBILE_VERSION
        NOONE_SELECTED_YET
        NORATE
        NOTES
        NOTHING_SELECTED
        NOTHING_SELECTED_YET
        NOTIFICATIONS
        NOTIFICATIONS_COPY_UNDER
        NOTIFICATIONS_INTRO
        NOTIFICATIONS_MARK_READ
        NOTIFICATION_BODY_invited_content
        NOTIFICATION_EMAIL_MESSAGE_WRAP
        NOTIFICATION_EMAIL_MESSAGE_WRAP_DIRECT_REPLY
        NOTIFICATION_EMAIL_SUBJECT_WRAP
        NOTIFICATION_FOR_DAYS
        NOTIFICATION_MEMBER_MENTION_BODY
        NOTIFICATION_MEMBER_MENTION_SUBJECT
        NOTIFICATION_PRIVILEGED
        NOTIFICATION_PT_MESSAGE_WRAP
        NOTIFICATION_PT_MESSAGE_WRAP_DIRECT_REPLY
        NOTIFICATION_PT_SUBJECT_WRAP
        NOTIFICATION_SMS_COMPLETE_WRAP
        NOTIFICATION_SOUND
        NOTIFICATION_SUBJECT_invited_content
        NOTIFICATION_TYPE_calendar_event
        NOTIFICATION_TYPE_calendar_reminder
        NOTIFICATION_TYPE_catalogue_entry
        NOTIFICATION_TYPE_cns_birthday
        NOTIFICATION_TYPE_cns_club
        NOTIFICATION_TYPE_cns_friend_birthday
        NOTIFICATION_TYPE_cns_group_declined
        NOTIFICATION_TYPE_cns_group_join_request
        NOTIFICATION_TYPE_cns_member_joined_group
        NOTIFICATION_TYPE_cns_new_member
        NOTIFICATION_TYPE_cns_new_pt
        NOTIFICATION_TYPE_cns_password_changed
        NOTIFICATION_TYPE_cns_rank_promoted
        NOTIFICATION_TYPE_cns_topic
        NOTIFICATION_TYPE_cns_topic_invite
        NOTIFICATION_TYPE_cns_username_changed
        NOTIFICATION_TYPE_comment_posted
        NOTIFICATION_TYPE_content_validated
        NOTIFICATION_TYPE_download
        NOTIFICATION_TYPE_gallery_entry
        NOTIFICATION_TYPE_im_invited
        NOTIFICATION_TYPE_invited_content
        NOTIFICATION_TYPE_invoice
        NOTIFICATION_TYPE_like
        NOTIFICATION_TYPE_member_calendar_changes
        NOTIFICATION_TYPE_member_entered_chatroom
        NOTIFICATION_TYPE_member_mention
        NOTIFICATION_TYPE_new_friend
        NOTIFICATION_TYPE_new_order
        NOTIFICATION_TYPE_news_entry
        NOTIFICATION_TYPE_order_dispatched
        NOTIFICATION_TYPE_paid_subscription_messages
        NOTIFICATION_TYPE_poll_chosen
        NOTIFICATION_TYPE_quiz_results
        NOTIFICATION_TYPE_received_points
        NOTIFICATION_TYPE_ticket_reply
        NOTIFICATION_TYPE_wiki
        NOTIFICATION_VIEW
        NOT_AUDIO
        NOT_A_DATE
        NOT_A_EMAIL
        NOT_A_SUPPORT_OPERATOR
        NOT_A_URL
        NOT_CURRENTLY_FRIENDS
        NOT_ENOUGH_POINTS
        NOT_FLOAT
        NOT_IMAGE
        NOT_IN_PREVIEW_MODE
        NOT_LOGGED_IN
        NOT_NEWSLETTER_SUBSCRIBER
        NOT_ON_NEWSLETTER
        NOT_OPEN_THIS
        NOT_SPECIFIED
        NOT_USERNAME
        NOT_VALID_ATTACHMENT
        NOT_VIDEO
        NOVOTE
        NOW_ADDED_AS_FRIEND
        NOW_DISABLED_NOTIFICATIONS
        NOW_DISABLED_NOTIFICATIONS_FORUM
        NOW_DISABLED_NOTIFICATIONS_TOPIC
        NOW_ENABLED_NOTIFICATIONS
        NOW_ENABLED_NOTIFICATIONS_FORUM
        NOW_ENABLED_NOTIFICATIONS_TOPIC
        NOW_INVISIBLE
        NOW_ONLINE
        NO_AWARD
        NO_BANNER
        NO_BANNERS
        NO_BLOCK_STAFF
        NO_CATEGORIES
        NO_CHILDREN
        NO_COLOR_SELECTION
        NO_COMMENTS
        NO_DESCRIPTION
        NO_DOUBLE_POST
        NO_DOWNLOADS_FOUND
        NO_ENTRIES
        NO_ENTRIES_AWARDS
        NO_FONT_SELECTED
        NO_FORWARDINGS
        NO_FRIEND_ENTRIES
        NO_HELP_HERE
        NO_IMAGE
        NO_IMAGE_PASTE_SAFARI
        NO_IM_ACTIVE
        NO_IM_CONVERSATIONS
        NO_INBOX
        NO_INVITE
        NO_INVITE_SENSE
        NO_INVITE_SENSE_ALREADY
        NO_JOIN_LOGGED_IN
        NO_LAST_VISIT
        NO_MARKERS_SELECTED
        NO_MEMBERS
        NO_NEWS
        NO_NOTIFICATIONS
        NO_OF_THIS
        NO_PAGE_OUTPUT
        NO_PASSWORD_RESET_HTTPAUTH
        NO_PERMISSION_TO_USE_POINTS
        NO_POP3S
        NO_POSTS
        NO_POSTS_YET
        NO_PT_FROM_ALLOW
        NO_REPORTED_POST_FORUM
        NO_RESET_ACCESS
        NO_RESULTS
        NO_RESULTS_DAYS
        NO_RESULTS_SEARCH
        NO_SUBJECT
        NO_SUCH_AUTHOR
        NO_SUCH_AUTHOR_CONFIGURE_ONE
        NO_TICKETS_GUESTS
        NO_TOPICS
        NO_TOPICS_HOT
        NO_TOPIC_TITLE
        NUM_GUESTS
        NUM_MEMBERS_IMPORTED
        NUM_POINTS
        NUM_RESULTS
        NUM_UNREAD
        NUM_WARNINGS
        OFFLINE
        OLDER_NEWS
        OLDEST_FIRST
        OLDEST_RESULTS_FIRST
        ONLINE
        ONLINE_NOW
        ONLY_TITLES
        ON_PROBATION_UNTIL
        _ON_PROBATION_UNTIL
        OPEN
        OPEN_CHATROOMS_IN_TABS
        OPEN_IM_POPUP
        OPEN_TICKET
        OPEN_TICKETS_ONLY
        OPEN_TOPIC
        OPTIONAL
        OPTIONAL_REASON
        OPTIONS
        OR
        ORANGE
        ORDER_FORWARDER_DONE
        ORDER_GENERAL_DONE
        ORDER_POP3_DONE
        ORDER_PRICE_AMT
        ORDER_QUOTA_DONE
        ORIGINAL_COMCODE
        OTHER
        OTHER_ACTIONS
        OTHER_DETAILS
        OTHER_IMAGES_IN_GALLERY
        OTHER_TICKETS_BY_MEMBER
        OTHER_USERGROUPS
        OUTMODED
        OUT_OF_STOCK
        OVERRIDES_FOR_FRIEND
        OVER_DAILY_QUOTA
        PAGE
        PAGE_LINK
        PAID_SUBSCRIPTION_ENDED
        PAID_SUBSCRIPTION_STARTED
        PARTICIPANTS
        PASSED
        LINKS_STRIPPED
        PASSWORD
        PASSWORD_ALREADY_RESET
        PASSWORD_CHANGED_MAIL_BODY
        PASSWORD_CHANGED_MAIL_BODY_2
        PASSWORD_CHANGED_MAIL_SUBJECT
        PASSWORD_DESIRED
        PASSWORD_EXPIRED
        PASSWORD_MISMATCH
        PASSWORD_NOT_COMPLEX_ENOUGH
        PASSWORD_RESET_BUTTON
        PASSWORD_RESET_ERROR
        PASSWORD_RESET_ERROR_2
        PASSWORD_RESET_ULTRA_BODY
        PASSWORD_RESET_ULTRA_FROM
        PASSWORD_RESET_ULTRA_SUBJECT
        PASSWORD_STRENGTH
        PASSWORD_TOO_LONG
        PASSWORD_TOO_OLD
        PASSWORD_TOO_SHORT
        PATTERN_NOT_MATCHED
        PAUSE
        PAYMENT_HEADING
        PAYMENT_RECEIVED_BODY
        PAYMENT_RECEIVED_SUBJECT
        PEOPLE_NOW_FRIENDS
        PERCENTAGE
        PERFORM_ACTION_ON_PREVIEW
        PERFORM_CHECKS_ON_PREVIEW
        PERSONAL_BLOCK
        PERSONAL_GALLERIES
        PERSONAL_GALLERY_OF
        PERSONAL_POST
        PER_DAILY_VISIT
        PER_DAY
        PER_MONTH
        PER_PAGE
        PER_WEEK
        PER_YEAR
        PE_LACKING_GIFT_POINTS
        PE_NEGATIVE_GIFT
        PE_SELF
        PGE_X
        PGE_A
        PGE_C
        PGE_COMM
        PGE_E
        PGE_I
        PGE_N
        PGE_P
        PGE_S
        PGE_T
        PGE_V
        PHONE_THEM_UP
        PHOTO
        PHOTO_CHANGE
        PHP_CODE
        PINNED
        PIN_TOPIC
        PIN_TOPIC_A
        PLAY_AUDIO_VERSION
        PLEASE_SELECT_SOUND
        PLEASE_WAIT
        PLEASE_WAIT_WHILE_UPLOADING
        PLUPLOAD_COMPLETE
        PLUPLOAD_FAILED
        PLUPLOAD_UPLOADING
        POINTS
        POINTSTORE
        POINTSTORE_INTRO
        POINTSTORE_ITEMS
        POINTSTORE_ONE_PER_MEMBER
        POINTS_COMPLETED_QUIZ
        POINTS_EARNED
        POINTS_GAINED_BETWEEN
        POINTS_GIFTED
        POINTS_IN_ADDITION
        POINTS_LEFT
        POINTS_MUST_LOGIN
        POINTS_NOW
        POINTS_RECEIVED
        POINTS_RSS_LINE
        POINTS_SEARCH_FORM
        POINTS_SPENT
        POINTS_THANKS
        POINTS_TO_GIVE
        POINTS_TO_SPEND
        POINTS_UNIT
        POINTS_USED
        POINT_LEADER_BOARD
        POLL
        POLLS
        POLL_ARCHIVE
        POLL_CHOSEN_NOTIFICATION_MAIL
        POLL_CHOSEN_NOTIFICATION_MAIL_SUBJECT
        POLL_IN_LIST
        POLL_IS_OPEN
        POLL_IS_PRIVATE
        POLL_NOT_ENOUGH_ERROR
        POLL_NOT_ENOUGH_ERROR_2
        POLL_REQUIRES_REPLY
        POLL_RESULTS
        POP3
        POPULARITY
        POSSIBLE_CORRECTIONS
        POSTED_TIME_SIMPLE
        POSTED_TIME_SIMPLE_LOWER
        POSTS_SINCE
        POSTS_SINCE_DAYS
        POSTS_SINCE_HOURS
        POSTS_SINCE_LAST_VISIT
        POSTS_SINCE_MINUTES
        POSTS_SINCE_WEEKS
        POSTS_SPLIT_FROM
        POSTS_WITHIN_TOPIC
        POST_BY
        POST_COMMENT
        POST_IN
        POST_INDICATOR_new_posts
        POST_INDICATOR_new_posts_redirect
        POST_INDICATOR_no_new_posts
        POST_INDICATOR_no_new_posts_redirect
        POST_INDICATOR_redirect
        POST_MAP_RE
        POST_MAP_REPLIES
        POST_NOTIFICATION_MAIL
        POST_NOTIFICATION_MAIL_SUBJECT
        POST_PLU
        POST_REPORTED
        POST_REVIEW
        POST_SEARCH_CLOSED
        POST_SEARCH_OPEN
        POST_SEARCH_PINNED
        POST_SEARCH_STARTER
        POST_SEARCH_UNVALIDATED
        POST_STAFF
        POST_TEMPLATE
        POST_TOO_LONG
        POST_TOO_SHORT
        POST_WARNING
        PP_TO
        PREFER_CMS_ATTACHMENTS
        PRESENT_AWARD
        PREVIEW
        PREVIEW_POSTS
        PREVIOUS
        PRE_STAFF
        PRICE
        PRICE_GIVE
        PRIMARY_GROUP
        PRIMARY_MEMBERS
        PRIMARY_PAGE_FORM
        PRINT
        PRINTER_FRIENDLY
        PRINT_THIS_SCREEN
        PRIORITY
        PRIORITY_na
        PRIVACY
        PRIVACY_SETTINGS
        PRIVACY_SETTINGS_INTRO
        PRIVACY_SETTINGS_INTRO_EXTRA
        PRIVATE_HIDDEN
        PRIVATE_TOPIC
        PRIVATE_TOPICS
        PRIVATE_TOPICS_INBOX
        PRIVATE_TOPICS_OF
        PRIVATE_TOPIC_DOC
        PROBATION
        PROCEED
        PROCEED_SHORT
        PROCEED_TEXT
        PRODUCT
        PRODUCT_NO_SUCH
        PRODUCT_PURCHASE_CANCEL
        PRODUCT_QUANTITY_CHANGED
        PRODUCT_UNAVAILABLE_WARNING
        PROFILE
        PROFILE_VIEWS
        PROJECT_PROGRESS
        PROMOTION_THRESHOLD
        PROMOTION_TO
        PROSPECTIVE_MEMBERS
        PROTECTED_FORUM
        PROTECTED_LAST_POST
        PR_LUCKY
        PR_NORMAL
        PT_ALLOW
        PT_ALLOW_DESCRIPTION
        PT_ALLOW_DESCRIPTION_CHAT
        PT_RULES_PAGE_INTRO
        PT_RULES_TEXT
        PT_RULES_TEXT_DESCRIPTION
        PUNITIVE_HISTORY
        PURCHASED_BANNER
        PURCHASE_CUSTOM_PRODUCT
        PURCHASE_DISABLED
        PURCHASE_FINISHED
        PURCHASE_MAIL
        PURCHASE_MAIL_BODY
        PURCHASE_MAIL_SUBJECT
        PURCHASE_NOT_LOGGED_IN
        PURCHASE_PERMISSION_PRODUCT
        PURCHASE_POP3_QUOTA
        PURCHASE_QUOTA
        PURCHASE_SOME_PRODUCT
        PURCHASE_STAGE_browse
        PURCHASE_STAGE_details
        PURCHASE_STAGE_finish
        PURCHASE_STAGE_message
        PURCHASE_STAGE_pay
        PURCHASE_STAGE_terms
        PURCHASE_STORED_DETAILS
        PURCHASING
        PURCHASING_TITLE
        PURPLE
        PUT_YOUR_REPORT_BELOW
        QUANTITY
        QUESTION
        QUICK_REPLY
        TEST
        QUIZ
        QUIZZES
        QUIZ_AFFIRMATION_HTML
        QUIZ_NOTIFICATION_TITLE
        QUIZ_RESULT
        QUIZ_THIS_WITH
        QUIZ_TWICE
        QUIZ_UNKNOWN
        QUOTA_AMOUNT
        QUOTA_COST
        QUOTA_DESCRIPTION
        QUOTA_INCREASE
        QUOTA_INTRO
        QUOTED_REPLY_MESSAGE
        QUOTES
        QUOTE_POST
        QUOTE_POSTS
        QUOTE_TO_NEW_TICKET
        QUOTE_TO_PT
        Q_SURE
        Q_SURE_LOSE
        Q_SURE_REMOVE_FROM_GROUP
        RADIO_NOTHING_SELECTED
        RANDOM
        RANDOM_CONTENT
        RANDOM_PAGE
        RANK_PROMOTED_MAIL
        RANK_PROMOTED_MAIL_SUBJECT
        RANK_SETS
        RATE
        RATE_DENIED
        RATE_DENIED_OWN
        RATING
        RATING_CONTENT
        READING_PRIVATE_TOPIC
        READ_ERROR
        READ_MORE
        REALNAME
        REASON
        RECENT
        RECENTLY_READ
        RECOMMENDATION_MADE
        RECOMMENDING_TO_LINE
        RECOMMEND_AUTO_TEXT
        RECOMMEND_COMPOSR
        RECOMMEND_LINK
        RECOMMEND_MEMBER_MESSAGE
        RECOMMEND_MEMBER_SUBJECT
        RECOMMEND_MEMBER_SUBJECT_SPECIFIC
        RECOMMEND_SITE
        RECOMMEND_SITE_TEXT
        RECOMMEND_SITE_TEXT_CHOOSE_CONTACTS
        RECOMMEND_SITE_TO
        RECOMMEND_SUP_MESSAGE
        RECURRENCE
        RECURRENCE_ITERATION
        RED
        REDIRECTED_LINK
        REDIRECTING_TO_BIRTHDAY_TOPIC
        REDIRECT_TEXT
        REDIRECT_TEXT_NO_COMPLETE
        REDIRECT_TO_TOPIC
        REDO_TIME
        REFERENCE_SOURCE
        REFUND
        REINSTALL
        RELATED_FIELD
        RELEVANCE
        RELEVANCE_SORT
        RELY_FORUMS
        RELY_SITE_DEFAULT
        REMEMBER_ME
        REMEMBER_ME_COOKIE
        REMEMBER_ME_TEXT
        REMINDERS
        REMINDER_TIME
        REMOVE
        REMOVE_CHAT_NOTIFICATION
        REMOVE_MEMBER_FROM_GROUP
        RENEWAL_STATUS
        REPAIR_HELP
        REPEATING_TOO_SOON
        REPEAT_SUFFIX
        REPLY
        REPORTED_POST
        REPORTED_POST_TITLE
        REPORT_POST
        REQUIRED
        REQUIRED_NOT_FILLED_IN
        REQUIRES_IP_VALIDATION
        RESET_CODE_MAILED
        RESET_FILTER
        RESIGN_FROM_GROUP
        RESOLUTION
        RESTORE_SAVED_FORM_DATA
        RESULTS
        RESULTS_LAUNCHER_ASK
        RESULTS_LAUNCHER_JUMP
        RESULT_OVERVIEW
        REVEAL_AGE
        REVIEWS
        REVISIONS
        RICH_CONTENT
        ROLE
        ROOT
        ROOT_EM
        ROOT_FORUM
        RSS_SOURCE_FROM
        RSS_STREAM
        RSS_XSLT_INTRODUCTION
        RULES
        SAID
        SAVE
        SAVED_SEARCHES
        SAVE_COMPUTER_USING_COOKIE
        SEARCH
        SEARCH_ADMIN
        SEARCH_AND
        SEARCH_DOMAIN
        SEARCH_FOR
        SEARCH_FORUM_POSTS
        SEARCH_HELP
        SEARCH_HOOK_NOT_AVAILABLE
        SEARCH_OPTIONS
        SEARCH_OR
        SEARCH_POSTS
        SEARCH_POSTS_WITHIN_TOPIC
        SEARCH_QUERY_TOO_SLOW
        SEARCH_RESULTS
        SEARCH_RESULTS_ARE
        SEARCH_RESULTS_ARE_UNNAMED
        SEARCH_TITLE
        SEARCH_UNDERNEATH
        SEARCH_USER
        SECOND
        SECONDARY_CATEGORIES
        SECONDARY_GROUP_MEMBERSHIP
        SECONDARY_MEMBERS
        SECONDS
        SECRET_GROUP
        SECTIONS
        SECTION_CHAT
        SECTION_DOWNLOADS
        SECTION_FORUMS
        SECTION_SOCIAL
        SECURITY_IMAGE
        SEE_FULL_IMAGE
        SELECT
        SELECT2_DELETE
        SELECT2_ENTER
        SELECT2_LOADING_FAILED
        SELECT2_LOADING_MORE
        SELECT2_MULTIPLE_RESULTS
        SELECT2_NO_RESULTS
        SELECT2_SEARCHING
        SELECT2_SELECTION_LIMIT
        SELECT2_SINGLE_RESULT
        SELECT_CHATROOM
        SELECT_IMAGE
        SELECT_TARGET_POST_DESTINATION
        SEND
        SEND_AS_ECARD
        SEND_EMAIL
        SEND_MESSAGE
        SEND_NOTIFICATION
        SENT_SIMPLE
        SEPARATOR
        SERVICE_CANCELLED
        SERVICE_PAID_FOR
        SETTINGS
        SET_TICKET_EXTRA_ACCESS
        SHIPPING_COST
        SHIPPING_PRICE
        SHOPPING
        SHOUTBOX
        SHOW
        SHOW_AS_EDITED
        SHOW_COMMENTS
        SHOW_MORE_COMMENTS
        SHOW_SEARCH_FORM
        SIGNATURE
        SIGNATURE_NOW
        SIGNATURE_TOO_BIG
        SINK_TOPIC
        SITE
        SITEMAP
        SITE_META
        SIZE
        SKILLS
        SKIP
        SKIP_NAVIGATION
        SKIP_SIGNATURE
        SLASH_OR
        SLIDESHOW
        SMALLEST_FIRST
        SMART_TOPIC_NOTIFICATION
        SMS
        SOFTWARE_CHAT
        SOME_ERRORS_OCCURRED
        SORT
        SORT_BY
        SOUND_EFFECTS
        SOURCE
        SPACER_POST
        SPACER_POST_MATCHER
        SPACER_TOPIC_TITLE_WRAP
        SPECIAL_CPF__cms_city
        SPECIAL_CPF__cms_country
        SPECIAL_CPF__cms_county
        SPECIAL_CPF__cms_currency
        SPECIAL_CPF__cms_firstname
        SPECIAL_CPF__cms_gift_points_used
        SPECIAL_CPF__cms_lastname
        SPECIAL_CPF__cms_latitude
        SPECIAL_CPF__cms_longitude
        SPECIAL_CPF__cms_mobile_phone_number
        SPECIAL_CPF__cms_mobile_phone_number_DESCRIPTION
        SPECIAL_CPF__cms_payment_card_cv2
        SPECIAL_CPF__cms_payment_card_expiry_date
        SPECIAL_CPF__cms_payment_card_issue_number
        SPECIAL_CPF__cms_payment_card_number
        SPECIAL_CPF__cms_payment_card_start_date
        SPECIAL_CPF__cms_payment_cardholder_name
        SPECIAL_CPF__cms_payment_type
        SPECIAL_CPF__cms_points_gained_chat
        SPECIAL_CPF__cms_points_gained_given
        SPECIAL_CPF__cms_points_gained_rating
        SPECIAL_CPF__cms_points_gained_visiting
        SPECIAL_CPF__cms_points_gained_voting
        SPECIAL_CPF__cms_points_gained_wiki
        SPECIAL_CPF__cms_points_used
        SPECIAL_CPF__cms_post_code
        SPECIAL_CPF__cms_role
        SPECIAL_CPF__cms_sites
        SPECIAL_CPF__cms_smart_topic_notification
        SPECIAL_CPF__cms_state
        SPECIAL_CPF__cms_street_address
        SPECIAL_REGISTRATION_FIELDS
        SPECIFIC_AVATAR
        SPECIFIC_FORUM_POSTS
        SPEED_IN_SECS
        SPELLCHECK
        SPELLCHECKER_DISABLED
        SPELLCHECKER_ENABLED
        SPELLCHECKER_LABEL
        SPELLCHECKER_TOGGLE
        SPLIT_POSTS
        SPOILER_WARNING
        SPREAD_TABLE
        STAFF
        STAFF_ACTIONS
        STAFF_TITLE
        START
        STARTER
        START_IM
        START_NEW_SUBSCRIPTION
        START_SUBSCRIPTION
        STATIC_USERS_ONLINE
        STATISTICS
        STATUS
        STOPPED
        STOPPED_BY_ANTISPAM
        SUBCATEGORIES_HERE
        SUBFORUMS
        SUBGALLERY_BITS
        SUBGALLERY_BITS_IMAGES
        SUBGALLERY_BITS_VIDEOS
        SUBJECT
        SUBJECT_TICKET_REROUTED
        SUBMISSION
        SUBMIT
        SUBMITTED_BY
        SUBMITTED_WITHIN
        SUBMITTER
        SUBMIT_AGE_DAYS
        SUBMIT_AWARD
        SUBMIT_UNVALIDATED_FORUM_POSTS
        SUBMIT_UNVALIDATED_FORUM_TOPICS
        SUBSCRIBE
        SUBSCRIBED_REMINDERS
        SUBSCRIBE_EVENT
        SUBSCRIBE_TO
        SUBSCRIPTION
        SUBSCRIPTIONS_SCREEN
        SUBSCRIPTION_CANCEL
        SUBSCRIPTION_CANCEL_WARNING
        SUBSCRIPTION_CANCEL_WARNING_GENERAL
        SUBSCRIPTION_EXPIRY_DATE
        SUBSCRIPTION_EXPIRY_MESSAGE
        SUBSCRIPTION_EXPIRY_TIME
        SUBSCRIPTION_FOR
        SUBSCRIPTION_LEVEL_FOR
        SUBSCRIPTION_OF
        SUBSCRIPTION_START_TIME
        SUBSCRIPTION_TERM_END_TIME
        SUBSCRIPTION_TERM_START_TIME
        SUCCESS
        SUCCESSFUL_CONFIRM
        SUCCESS_NEWSLETTER_AUTO
        SUCCESS_NEWSLETTER_AUTO_INSTANT
        SUCCESS_SAVE
        SUCCESS_UPLOAD_SYNDICATION_AUTH
        SUGGESTIONS_ONLY
        SUNK
        SUPER_MODERATORS
        SUPPORT_NO_TICKETS
        SUPPORT_SEARCH_FAQ
        SUPPORT_TICKET
        SUPPORT_TICKETS
        SUPPORT_TICKETS_FOR_MEMBER
        SUPPORT_TICKETS_FOR_TYPE
        SURNAME
        SURVEYS
        SURVEY_THANKYOU
        SYSTEM
        TABLE_OF_CONTENTS
        TAGS
        TAG_CLOUD
        TAG_CONTENTS
        TAKEN_RANDOM_WIKI_PAGE
        TAX
        TEMPLATE_SEARCH_INFO
        TESTS
        TEST_FAIL
        TEST_PASS
        TEST_RESULTS
        TEST_SOUND
        TEST_UNKNOWN
        TEXT
        TEXT_OR_COMCODE_GOES_HERE
        THANKYOU_FOR_RATING
        THANKYOU_FOR_RATING_SHORT
        THANKYOU_VOTING
        THEIR_ADDRESS
        THEM
        THEY_DOWNLOADED_THIS
        THIS
        THIS_IS_COMMENT_TOPIC
        THIS_WITH_COMCODE
        THIS_WITH_SIMPLE
        THREADED_REPLY_NOTICE
        THUMBNAIL
        THUMB_OR_IMG_2
        TICKETS_MEMBER
        TICKETS_MERGED_INTO_BODY
        TICKETS_MERGED_INTO_SUBJECT
        TICKETS_MERGED_POST
        TICKETS_MERGED_TITLE
        TICKET_ACTIVITY_SUBJECT
        TICKET_IS_CLOSED
        TICKET_REPLY
        TICKET_REPLY_MESSAGE
        TICKET_REPLY_MESSAGE_FOR_STAFF
        TICKET_SIMPLE_FROM
        TICKET_SIMPLE_MAIL_new
        TICKET_SIMPLE_MAIL_new_regexp
        TICKET_SIMPLE_MAIL_reply
        TICKET_SIMPLE_MAIL_reply_regexp
        TICKET_SIMPLE_SUBJECT_new
        TICKET_SIMPLE_SUBJECT_regexp
        TICKET_SIMPLE_SUBJECT_reply
        TICKET_STARTED
        TICKET_TYPE
        TICK_ANON
        TIME
        TIMEOUT
        TIMEZONE
        TIME_FOR_THEM
        TIME_LEFT_TO_EDIT_POST
        TIME_RANGE_END
        TIME_RANGE_START
        TIME_REMAINING
        TITLE
        TITLE_BANNER_UPGRADE
        TITLE_NEWFORWARDING
        TITLE_NEWPOP3
        TITLE_POP3
        TITLE_QUOTA
        TITLE_TOO_LONG
        TO
        TODAY
        TOGGLE_GROUPING_VISIBILITY
        TOGGLE_SELECTION
        TOGGLE_WYSIWYG_2
        TOGGLE_WYSIWYG_ERROR
        TOO_LONG_ON_SCREEN
        TOO_MANY_ATTACHMENTS
        TOO_MANY_FILES_TO_EXTRACT
        TOO_MANY_USERS_ONLINE
        TOP
        TOPICS_PARTICIPATED_IN
        TOPICS_UNREAD
        TOPIC_ACTIONS
        TOPIC_EMOTICON
        TOPIC_HAS_WHISPER_TO_YOU
        TOPIC_IN
        TOPIC_IS_CLOSED
        TOPIC_MARKED_READ
        TOPIC_MARKED_UNREAD
        TOPIC_MODERATION
        TOPIC_MOVE_MAIL
        TOPIC_MOVE_MAIL_SUBJECT
        TOPIC_NOTIFICATION_MAIL
        TOPIC_NOTIFICATION_MAIL_SUBJECT
        TOPIC_PINNED_ALREADY
        TOPIC_PINNED_MAX_DAYS
        TOPIC_PINNING
        TOPIC_PINNING_DESCRIPTION
        TOPIC_PIN_LACK_POINTS
        TOPIC_PIN_NUMBER_DAYS
        TOPIC_PIN_NUMBER_DAYS_DESCRIPTION
        TOPIC_PIN_POSITIVE_DAYS
        TOPIC_POLLS
        TOPIC_POLL_CLOSED
        TOPIC_POLL_IS_PRIVATE
        TOPIC_STARTED_DATE_TIME
        TOPIC_TITLE_WILL_BE
        TOTAL_EVENTS_AND_HIGHEST_PRIORITY
        TOTAL_PRICE
        TO_DOWNLOAD_VIDEO
        TRACKBACKS
        TRACKBACK_LINK
        TRACKING
        TRANSACTION_ID_WRITTEN
        TRANSACTION_NOT_COMPLETE
        TRANSACT_INFO
        TRANSCODING_IN_PROGRESS
        TREE
        TT_COMPLAINT
        TT_OTHER
        TUTORIALS
        TUTORIAL_ON_THIS
        TYPE
        UNANSWERED_TOPICS
        UNASSIGNED
        UNCASCADE_TOPIC
        UNDECLARE_EVENT_INTEREST
        UNIT_PRICE
        UNKNOWN
        UNKNOWN_FORMAT
        UNLINK_EXISTING_UPLOAD
        UNLINK_EXISTING_UPLOADS
        UNLINK_EXISTING_UPLOAD_IMAGE
        UNLINK_EXISTING_UPLOAD_SPECIFIC
        UNNAMED
        UNRATED
        UNSEEN_PERSONAL_POSTS
        UNSINK_TOPIC
        UNSUBSCRIBE
        UNSUBSCRIBE_EVENT
        UNVALIDATED
        UNVALIDATED_TEXT
        UNVALIDATED_TEXT_NON_DIRECT
        UNVALIDATED_TEXT_STAFF
        UNVALIDATED_TITLE
        UNVALIDATE_POST
        UPDATE_CART
        _UPDATE_CART
        UPGRADE
        UPGRADE_ACCOUNT
        UPLOAD
        UPLOAD_MUST_SYNDICATE
        UPLOAD_SYNDICATION_AUTH
        UPLOAD_TO
        US
        USE
        USERGROUP
        USERGROUPS
        USERGROUP_SUBSCRIPTION
        USERGROUP_SUBSCRIPTIONS
        USERNAME
        USERNAME_ALREADY_EXISTS
        USERNAME_BAD_SUBSTRING
        USERNAME_BAD_SYMBOLS
        USERNAME_CHANGED_MAIL
        USERNAME_CHANGED_MAIL_SUBJECT
        USERNAME_NUMERIC
        USERNAME_PASSWORD_WHITESPACE
        USERNAME_TOO_LONG
        USERNAME_TOO_SHORT
        USERS_IN_CHATROOM
        USERS_ONLINE
        USERS_ONLINE_BLOCK
        USERS_ONLINE_RECORD
        USER_AGENT
        USER_OS
        USE_ALL_EXCEPT_SELECTED
        USE_ALL_SELECTED
        USE_CHAT_RULES
        USE_INVITE
        USE_INVITE_DESCRIPTION
        USE_IN_ALL_FORUMS
        USE_MOUSE_WHEEL_SCROLL
        USE_WEBSITE_RULES
        VALIDATED_MEMBER_SUBJECT
        VALIDATE_POSTS
        VIDEO
        VIDEOS
        VIDEO_DETAILS
        VIDEO_NOTIFICATION_MAIL
        VIDEO_NOTIFICATION_MAIL_SUBJECT
        VIEW
        VIEWING_POINTS_PROFILE_OF
        VIEWING_SLIDE
        VIEWS_SIGNATURES
        VIEWS_SIMPLE
        VIEW_ALL_QUIZZES
        VIEW_ALL_STAFF
        VIEW_ARCHIVE
        VIEW_AS_LINEAR
        VIEW_AS_THREADED
        VIEW_AUTHOR
        VIEW_COMMENT_TOPIC
        VIEW_FULL_QUIZ_RESULTS
        VIEW_IMAGE
        VIEW_IMAGE_IN
        VIEW_MY_AUTHOR_PROFILE
        VIEW_POINTS
        VIEW_PRIVATE_TOPIC
        VIEW_PRODUCT
        VIEW_PROFILE
        VIEW_SLIDESHOW
        VIEW_SUPPORT_TICKET
        VIEW_THIS_GALLERY
        VIEW_TOPIC
        VIEW_USERGROUP
        VIEW_VIDEO
        VIEW_VIDEO_IN
        VIRTUAL_FORUM
        VIRTUAL_ROOT
        VISIBLE
        VISIBLE_TO
        VISIBLE_TO_FRIENDS
        VISIBLE_TO_GUESTS
        VISIBLE_TO_MEMBERS
        VISIBLE_TO_STAFF
        VOTE
        VOTES
        VOTES_ARE_IN
        VOTE_DENIED
        VOTE_FORFEIGHT
        VOTE_IN_POLL
        WANT_TO_DOWNLOAD_LOGS
        WEBSITE
        WEEK
        WEEKLY
        WELCOME
        WELCOME_BACK
        WELCOME_CHAT_LOBBY
        WELCOME_CHAT_LOBBY_PRIVATE_CHATROOMS
        WELCOME_CHAT_LOBBY_USE_IM
        WELCOME_CHAT_LOBBY_USE_IM2_NO_SITEWIDE
        WELCOME_CHAT_LOBBY_USE_IM2_SITEWIDE
        WELCOME_TO_STRIPPABLE
        WHAT_IS_COMCODE
        WHETHER_SAVE_WYSIWYG_SELECTION
        WHISPER
        WHISPER_PP
        WHISPER_PT
        WHISPER_TEXT
        WHO_CAN_SEE_YOUR
        WIDTH
        WIKI
        WIKI_HOME
        WIKI_MAKE_POST
        WIKI_PAGE
        WIKI_PAGES
        WIKI_POST
        WIKI_POST_COUNT
        WILDCARD
        WILL_CONTINUE_AFTER_VIDEO_FINISHED
        WILL_COST
        WITH
        WITH_SIMPLE
        WITH_SIMPLE_LOWER
        WITH_TITLING
        WORDFILTER_YOU
        WORDS
        WYSIWYG_DISABLE_ALWAYS
        WYSIWYG_DISABLE_ONCE
        WYSIWYG_DISABLE_ONCE_AND_DONT_ASK
        YEAR
        YEARLY
        YELLOW
        YES
        YESTERDAY
        YOURE_MY_FRIEND_BODY
        YOURE_MY_FRIEND_SUBJECT
        YOUR_ADDRESS
        YOUR_EMAIL_ADDRESS
        YOUR_MESSAGE_WAS_SENT_BODY
        YOUR_MESSAGE_WAS_SENT_SUBJECT
        YOUR_NAME
        YOUR_PASSWORD
        YOU_ARE_BANNED
        YOU_ARE_HERE
        YOU_CAN_DELETE_PHOTO
        YOU_GIVEN_POINTS
        YOU_HAVE_TEMPORARY_PASSWORD
        _ACTIVITY_GIVE_AWARD
        _ACTIVITY_LIKES
        _ACTIVITY_LIKES_UNTITLED
        _ACTIVITY_LIKES_UNTYPED
        _ADDED_COMMENT_ON
        _ADDED_COMMENT_ON_UNTITLED
        _ADDED_COMMENT_ON_UNTYPED
        _ADD_AS_FRIEND
        _ADD_POST
        _ADD_POST_TOP_LEVEL
        _ADD_POST_UNDER
        _ADD_PRIVATE_TOPIC
        _ADD_PRIVATE_TOPIC_UNKNOWN
        _ADD_TOPIC
        _ALL
        _ALREADY_HAVE
        _APPLY_TO_GROUP
        _ARE_YOU_SURE_DELETE
        _ASCENDING
        _ASSIGNED_TO
        _ATTACHMENT
        _AUTHOR
        _AWARD
        _BACK_TO_TOP
        _BLOCK_HELPER
        _BLOCK_MEMBER
        _CALENDAR_EVENT_VCAL
        _CALENDAR_SPECIFIC
        _CALENDAR_SPECIFIC_WEEK
        _CANT_AFFORD
        _CATEGORISE_PTS
        _CHATROOM
        _COLLABORATION
        _COMCODE
        _COMMENTS
        _COUNT_POSTS
        _DEFAULT
        _DEFINE_AUTHOR
        _DELETE_MEMBER
        _DELETE_MEMBER_SUICIDAL
        _DELETE_TOPIC
        _DESCENDING
        _DOWNLOAD
        _DOWNLOAD_CATEGORY
        _EMAIL_ADDRESS_IN_USE
        _EMAIL_MEMBER
        _ENTER_PROFILE_DETAILS
        _FEEDBACK
        _FEEDS
        _GALLERY
        _HOME
        _IMPORTANCE_MODULUS
        _INSTANT_MESSAGING
        _INVALID_EMAIL_ADDRESS
        _INVITE_MEMBER_TO_PT
        _JOIN
        _LOGGED_OUT
        _LOGIN
        _LOGIN_TEXT
        _LOGOUT
        _MAKE_ANONYMOUS_POST
        _MARKED_POST_ACTIONS
        _MAX_SIG_LENGTH_COMCODE
        _MEMBERS_VIEWING
        _MEMBER_NO_EXIST
        _MISSING_RESOURCE
        _MOST_ACTIVE_FORUM
        _MOVE_TOPIC
        _MULTIPLE
        _MY_ORDER_DETAILS
        _NEW
        _NEWS
        _NEWSLETTER_JOIN
        _NEW_MESSAGES
        _NOT_FILE_TYPE
        _NO_FRIEND_ENTRIES
        _ONLINE_NOW_NO_DAYS
        _ONLINE_NOW_NO_HOURS
        _ONLINE_NOW_NO_MINUTES
        _ONLINE_NOW_NO_MONTHS
        _OVER_DAILY_QUOTA
        _PAID_SUBSCRIPTION_ENDED
        _PAID_SUBSCRIPTION_STARTED
        _PAID_SUBSCRIPTION_UHOH
        _POINTS
        _POINTS_GIFTED
        _POINTS_SPENT
        _POLL_REQUIRES_REPLY
        _PR_LUCKY
        _PURCHASE_FINISHED
        _QUOTE_POST
        _RECOMMEND_SITE
        _REDIRECTING
        _REMOVE_AS_FRIEND
        _REPLY
        _REPORT_POST
        _REQUIRED
        _REVIEWS
        _SEARCH_PRIVATE_TOPICS
        _SEARCH_TITLE
        _SECTION_DOWNLOADS
        _SEND_NOTIFICATION
        _SERVICE_CANCELLED
        _SERVICE_PAID_FOR
        _SLIDESHOW
        _STAFF
        _STATISTICS
        _SUBGALLERY_BITS
        _SUBGALLERY_BITS_IMAGES
        _SUBGALLERY_BITS_VIDEOS
        _SUBMITTED_BY
        _TYPE
        _UNKNOWN
        _UNSET
        VIDEO_LENGTH
        _VIEW_AS_LINEAR
        _VIEW_AS_THREADED
        _VIEW_IMAGE
        _VIEW_SUPPORT_TICKET
        _VIEW_TOPIC
        _VIEW_VIDEO
        _WELCOME
        _WIKI_EDIT_PAGE
        _WIKI_PAGE
        __JOIN
        __OVER_DAILY_QUOTA
        __POINTS_THANKS
        calendar_date
        calendar_date_range_single
        calendar_date_range_single_long
        calendar_date_verbose
        calendar_day_of_month
        calendar_day_of_month_verbose
        calendar_minute
        calendar_minute_ampm_known
        calendar_minute_no_minutes
        calendar_month_in_year
        calendar_month_in_year_verbose
        charset
        date_concise_near_date
        date_no_year
        date_regular_date
        date_regular_joiner
        date_regular_time
        date_verbose_date
        date_verbose_joiner
        date_verbose_time
        date_withinweek_date
        date_withinweek_joiner
        dictionary
        dictionary_variant
        dir
        dont_escape_trick
        en_left
        en_right
        faqs__CATALOGUE_CATEGORY
        faqs__CATALOGUE_ENTRY
        faqs__CATALOGUE_INDEX
        links__CATALOGUE_CATEGORY
        links__CATALOGUE_ENTRY
        links__CATALOGUE_INDEX
        locale
    ')));

    // Start with what we know
    $lang_strings_admin = array_diff($lang_strings_admin_initial, $lang_strings_non_admin_initial);
    $lang_strings_non_admin = array_diff($lang_strings_non_admin_initial, $lang_strings_admin_initial);
    $lang_strings_unknown = array();

    // Also we can make assumptions for config string names if not already set
    if ($do_guessing) {
        $hooks = find_all_hooks('systems', 'config');
        foreach ($hooks as $hook => $place) {
            if ($place == 'sources_custom') {
                continue;
            }

            require_code('hooks/systems/config/' . filter_naughty_harsh($hook));

            $ob = object_factory('Hook_config_' . filter_naughty_harsh($hook));
            if (method_exists($ob, 'get_details')) {
                $details = $ob->get_details();

                $str = $details['human_name'];

                if (in_array($str, $lang_strings_admin) || in_array($str, $lang_strings_non_admin) || in_array($str, $lang_strings_unknown)) {
                    continue;
                }

                $lang_strings_admin[] = $str;
            }
        }
    }

    // What is likely, based on usage within the code?
    $lang_strings_admin_likely = find_strings($source_files_admin);
    $lang_strings_non_admin_likely = find_strings($source_files_non_admin);

    // Go through and make our determinations based on guessing
    $all_strings = array();
    $all_strings_in_lang = array();
    $strings_files = array();
    $d = get_file_base() . '/lang/' . fallback_lang();
    $dh = opendir($d);
    while (($f = readdir($dh)) !== false) {
        if (substr($f, -4) == '.ini') {
            // Load stuff and keep a record...

            $strings = get_lang_file_map(fallback_lang(), basename($f, '.ini'), true);
            $all_strings = array_merge($all_strings, array_keys($strings));

            $strings_lang = get_lang_file_map($lang, basename($f, '.ini'), true);
            $all_strings_in_lang += $strings_lang;

            foreach (array_keys($strings) as $str) {
                $strings_files[$str] = $f;
            }

            // Loop through each string...

            foreach (array_diff(array_keys($strings), $lang_strings_non_admin, $lang_strings_admin) as $str) {
                if (in_array($str, $lang_strings_admin) || in_array($str, $lang_strings_non_admin) || in_array($str, $lang_strings_unknown)) {
                    continue;
                }

                if ($do_guessing) {
                    $is_admin_string = (in_array($f, $admin_files)) || (preg_match('#^(' . implode('|', $admin_string_suffixes) . ')#', $str) != 0) || (in_array($str, $lang_strings_admin_likely));
                    $is_non_admin_string = (in_array($f, $non_admin_files)) || (preg_match('#^(' . implode('|', $non_admin_string_suffixes) . ')#', $str) != 0) || (in_array($str, $lang_strings_non_admin_likely));
                } else {
                    $is_admin_string = false;
                    $is_non_admin_string = false;
                }

                if ($is_admin_string) {
                    $lang_strings_admin[] = $str;
                } elseif ($is_non_admin_string) {
                    $lang_strings_non_admin[] = $str;
                } else {
                    $lang_strings_unknown[] = $str;
                }
            }
        }
    }
    closedir($dh);

    // De-duplicate
    $lang_strings_admin = array_unique($lang_strings_admin);
    $lang_strings_non_admin = array_unique($lang_strings_non_admin);
    $lang_strings_unknown = array_unique($lang_strings_unknown);

    // Return only stuff that actually still exists
    if ($only_exists) {
        $lang_strings_admin = array_intersect($all_strings, $lang_strings_admin);
        $lang_strings_non_admin = array_intersect($all_strings, $lang_strings_non_admin);
        $lang_strings_unknown = array_intersect($all_strings, $lang_strings_unknown);
    }

    // Sort
    sort($lang_strings_admin);
    sort($lang_strings_non_admin);
    sort($lang_strings_unknown);

    // Split out so we also have a shared array
    $just_lang_strings_admin = array_diff($lang_strings_admin, $lang_strings_non_admin);
    $just_lang_strings_non_admin = array_diff($lang_strings_non_admin, $lang_strings_admin);
    $lang_strings_shared = array_intersect($lang_strings_non_admin, $lang_strings_admin);

    return array($just_lang_strings_admin, $just_lang_strings_non_admin, $lang_strings_shared, $lang_strings_unknown, $all_strings_in_lang, $strings_files);
}

function find_files_in_dir($dir)
{
    $files = array();

    $d = get_file_base() . '/' . $dir;

    $dh = opendir($d);
    while (($f = readdir($dh)) !== false) {
        if (is_dir($dir . '/' . $f) && $f != '.' && $f != '..' && strpos($f, 'custom') === false) {
            $files = array_merge($files, find_files_in_dir($dir . '/' . $f));
        } else {
            if (substr($f, -4) == '.php') {
                $files[] = $dir . '/' . $f;
            }
        }
    }
    closedir($dh);

    return $files;
}

function find_strings($from)
{
    $out = array();

    foreach ($from as $file) {
        $c = file_get_contents(get_file_base() . '/' . $file);
        $matches = array();

        $num_matches = preg_match_all('#do_lang(_tempcode)?\(\'([^\']+)\'(?! \.)#', $c, $matches);
        for ($i = 0; $i < $num_matches; $i++) {
            $str = $matches[2][$i];
            $str = preg_replace('#.*:#', '', $str);
            $out[$str] = true;
        }

        $num_matches = preg_match_all('#\{\!(\w+)[\},]#', $c, $matches);
        for ($i = 0; $i < $num_matches; $i++) {
            $str = $matches[1][$i];
            $str = preg_replace('#.*:#', '', $str);
            $out[$str] = true;
        }
    }

    return array_keys($out);
}
